//-----ScriptName=SCRIPT CalculateWaterFlowRate_Primary
/**************************************************************************************
SCRIPT Name:	Calculate water flow rate
Fired On:		Every 100ms by its own timer (WaterFlowRateScript_Timer)

Description:	Update the variable CalculatedWaterFlowRate based on the RawWaterFlow_Pulses
				variable received via CAN from the analog-to-CAN module.  This uses the
				2-point calibration variables f_2PointOffset and f_2PointSlope which are
				saved during on-unit calibration.

***************************************************************************************/
uint TotalWaterFlowTimeBase = 0;
uint[] waterFlowRateBuffer(64);
uint[] waterFlowRateRaw(64);
uint BufferedRawWaterFlow;
uint LastWaterFlowRate = 0;
bool InitializeBuffers = true;
void SCRIPT_CalculateWaterFlowRate_Primary () 
{
  	uint SimulatedWaterFlowEnabled;
	uint SimulatedWaterFlowValue;
	uint RawWaterFlow_Pulses;
	bool RawPulsesReceived;
	uint CalculatedWaterFlowRate;
	uint AveragedWaterFlowRate;
	
	int s16_TempGPMRate;
	double f_2PointSlope;
	double f_2PointOffset;
	
	//Make sure all buffers have an initial 0 value
	if (InitializeBuffers == true)
	{
		for (uint count = 0; count < 64; count++)
		{
			waterFlowRateBuffer[count] = 0;
			waterFlowRateRaw[count] = 0;
			Secondary_waterFlowRateBuffer[count] = 0;
			Secondary_waterFlowRateRaw[count] = 0;
		}
		InitializeBuffers = false;
	}
	
	smRead(VariableIDs.Primary_WaterCal_2PointOffset, f_2PointOffset);
	smRead(VariableIDs.Primary_WaterCal_2PointSlope, f_2PointSlope);
	
	
	smRead(VariableIDs.Primary_SimulatedWaterFlow_Enabled, SimulatedWaterFlowEnabled);
	
	uint RawWaterFlowSource;
	smRead(VariableIDs.FLX_WaterFlowSource, RawWaterFlowSource);
	if (RawWaterFlowSource == 1)	//A2C module with old motor driver
		RawPulsesReceived = smRead(VariableIDs.Primary_RawWaterFlowPulses, RawWaterFlow_Pulses);
	else							//New motor driver
		RawPulsesReceived = smRead(VariableIDs.Primary_RawWaterFlowPulses_NewMotorDriver, RawWaterFlow_Pulses);	
	
	if (SimulatedWaterFlowEnabled == 1)
	{
		//If we are in simulated water flow - set the water flow rate to the simulated value.
		smRead(VariableIDs.Primary_SimulatedFlowGPM, SimulatedWaterFlowValue);
		smWrite(VariableIDs.Primary_WaterFlowRate_GPM, SimulatedWaterFlowValue);
		CalculatedWaterFlowRate = SimulatedWaterFlowValue;
	}
	else
	{
		if (RawPulsesReceived == false)
		{
			CalculatedWaterFlowRate = 0;
		}
		else
		{
			if (RawWaterFlow_Pulses < 1)//was set to 2
			{
				CalculatedWaterFlowRate	= 0;	// if rawflowrate is less than 10 display 0
			}
			else
			{

				//Two point calculation - using line intercept y = (slope x pps) + offset
				s16_TempGPMRate = (f_2PointSlope * BufferedRawWaterFlow) +  f_2PointOffset;

				if (s16_TempGPMRate < 0)
				{
					CalculatedWaterFlowRate = 0;
				}
				else
				{
					CalculatedWaterFlowRate = s16_TempGPMRate;
				}
			}	
		}
		
		AveragedWaterFlowRate = RotateWaterFlowBuffer(CalculatedWaterFlowRate, RawWaterFlow_Pulses);
		smWrite(VariableIDs.Primary_WaterFlowRate_GPM, AveragedWaterFlowRate);
	}
	
	
	/*	Check for auto enable *********************************************************************************/
	uint AutoEnableState;
	smRead(VariableIDs.FLX_AutoEnable, AutoEnableState);
		/*	The current power state should be 1 of 4 values:
		0 = OFF
		1 = ON in Foam A
		2 = ON in Foam B
		3 = ON in FLUSH
	*/
	uint thisPowerState;
	smRead(VariableIDs.Primary_FoamLogix_Enabled, thisPowerState);
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	uint thisTankSelected;
	smRead(VariableIDs.Primary_FoamSystemTankSelection, thisTankSelected);
	
	uint SimulatedFlowActive;
	smRead(VariableIDs.Primary_SimulatedWaterFlow_Enabled, SimulatedFlowActive);
	
	if ((AveragedWaterFlowRate >= 15 && LastWaterFlowRate == 0) && AutoEnableState == 1 && thisTankSelected != 0 && thisPowerState == 0 && SimulatedFlowActive == 0)
	{
		TurnFoamSystem_ON(0);
	}
	
	//Use as a trigger to know when 
	if (AveragedWaterFlowRate >= 15)
		LastWaterFlowRate = AveragedWaterFlowRate;
	else
		LastWaterFlowRate = 0;	
	
	/************************************************************************************************************/
	
	/*	Set the water flow active bit - this is sent out in the 18EFFFD0 message.
	*/
	if (CalculatedWaterFlowRate != 0)
	{
		smWrite(VariableIDs.BIT_WaterFlowActive, 1);
	}
	else
	{
		smWrite(VariableIDs.BIT_WaterFlowActive, 0);
	}
	
	CalculateTotalWaterFlowed(CalculatedWaterFlowRate);  	//Total water flow used the actual flow rate and not the averaged flow rate
}





/*************************************************************************
	Calculate the total water flow.  This function is run every loop cycle
	above - which is every 100ms.  This waits every 10 calls to perform the 
	function in order to get flow totals every second.
**************************************************************************/
void CalculateTotalWaterFlowed(uint CurrentFlowRate)
{
	double ThisNumberOfGallons;
	double TotalGallonsFlowed;
	
	smRead(VariableIDs.Primary_TotalWaterGallons, TotalGallonsFlowed);
	
	TotalWaterFlowTimeBase++;
	if (TotalWaterFlowTimeBase >= 10)	//one second at a .100 second script rate
	{
		TotalWaterFlowTimeBase = 0;
		ThisNumberOfGallons = CurrentFlowRate;
		ThisNumberOfGallons = ThisNumberOfGallons / 60;	//remove the per minute
		TotalGallonsFlowed = TotalGallonsFlowed + ThisNumberOfGallons;
		smWrite(VariableIDs.Primary_TotalWaterGallons, TotalGallonsFlowed);
	}		 
}


/*	Rotate the water flow buffers - the new data will be put into the last slot.
	The buffers will then be added and averaged.
*/
uint RotateWaterFlowBuffer(uint NewWaterFlowRate, uint RawWaterFlow_Pulses)
{
	uint accumulator = 0;
	uint accumulatorRaw = 0;
	
	/*	9/14/2018
		Adding configurable number of water flow buffers like we did for ATP system.
		We used to only use 24 buffers - but now we can use 24, 42, or 64
	*/
	uint NUM_BUFFERS = 42;	//default value (MED)
	uint BufferConfig = 0;
	smRead(VariableIDs.WaterFlow_NumberBuffers, BufferConfig);
	switch (BufferConfig)
	{
		case 0:	//LOW buffering
				NUM_BUFFERS = 24;	break;
				
		case 1:	//MED buffering
				NUM_BUFFERS = 42;	break;
				
		case 2:	//HIGH buffering
				NUM_BUFFERS = 64;	break;
		
		default:	//MED buffering
				NUM_BUFFERS = 42;	break;	
	}
	
	
	for (uint i = 0; i < (NUM_BUFFERS-1); i++)
	{
		waterFlowRateBuffer[i] = waterFlowRateBuffer[i+1];
		waterFlowRateRaw[i] = waterFlowRateRaw[i+1];
	}
	
	waterFlowRateBuffer[(NUM_BUFFERS-1)] = NewWaterFlowRate;
	waterFlowRateRaw[NUM_BUFFERS-1] = RawWaterFlow_Pulses;
	
	
	for (uint i = 0; i < NUM_BUFFERS; i++)
	{
		accumulator += waterFlowRateBuffer[i];
		accumulatorRaw += waterFlowRateRaw[i];
	}
	
	accumulator = accumulator / NUM_BUFFERS;			//Acumulated calibrated water flow	
	accumulator = RoundToFive(accumulator);
	
	accumulatorRaw = accumulatorRaw / NUM_BUFFERS;		//Accumulated raw water flow 
	BufferedRawWaterFlow = accumulatorRaw;
	
	/*	Since this averaged accumulator method is basically 2.5 seconds behind the
		actual water flow changes - there is the possibility that the water flow has been
		shut down, but the average will still contain a value for 2.5 seconds.  This can
		cause foam to flow for up to 2.5 seconds after water flow has stopped.  To keep this
		from happening we will check the 3 newest buffers (23, 22, and 21) and if they are
		0 then we will force the whole buffer to 0 in order to shutdown foam flow.
	*/
	if (waterFlowRateBuffer[(NUM_BUFFERS-1)] == 0 && waterFlowRateBuffer[(NUM_BUFFERS-2)] == 0 && waterFlowRateBuffer[(NUM_BUFFERS-3)] == 0)
	{
		accumulator = 0;
		
		for (uint i = 0; i < NUM_BUFFERS; i++)
		{
			waterFlowRateBuffer[i] = 0;
		}	
	} 
	
	smWrite(VariableIDs.Primary_RawWaterFlowRate_Buffered, accumulatorRaw);
	
	return  accumulator;
}


/*	A simple routine to round a value to the nearest five.
*/
uint RoundToFive(uint ValueToRound)
{
	uint OnesDigit;
	
	OnesDigit = ValueToRound % 10;
	
	ValueToRound = ValueToRound - OnesDigit;
	
	if (OnesDigit > 2 && OnesDigit < 8)	//round up to 5
	{
		ValueToRound = ValueToRound + 5;
	}
	else if (OnesDigit >= 8)			//round up to 10
	{
		ValueToRound = ValueToRound + 10;
	}
	else								//round down to 0
	{
		ValueToRound = ValueToRound;
	}
	
	return 	ValueToRound;	
}



//-----ScriptName=SCRIPT CalculatePrimaryFoamFlowRate
/*	Calculate the foam flow rate from the raw foam flow rate received from the DC motor driver.
	This script is fired by a change in the data of FoamPrimary.RawFoamFlowRate.
*/
double Primary_FoamFlowGPM;

void SCRIPT_CalculatePrimaryFoamFlowRate () 
{
 	uint 	FoamPrimary_RawFoamFlowRate;
	double 	TempScratchPad = 0;
	double  TempScratchPad2 = 0;
	uint8 	CurrentTankSelected;
	uint 	TotalFoamPulses;	
	uint 	Primary_FoamCalFactor_TankA;
	uint 	Primary_FoamCalFactor_TankB;
	uint[]	WaterFlowRateGPM(2);
	double  PulseFactorFudge;
	
	
	smRead(VariableIDs.FLX_PulseFactor, PulseFactorFudge);
	
	
	
	/*	Get access to the multipler value for foam A and B.  This is the value that
		the raw foam pulses from the DC motor driver will be multiplied by to
		get the flow rate of foam in GPM.
	*/ 
	smRead(VariableIDs.Primary_FoamFlowCalculationValue_A, Primary_FoamCalFactor_TankA); 
	smRead(VariableIDs.Primary_FoamFlowCalculationValue_B, Primary_FoamCalFactor_TankB); 
	
	
	/*	Get the primary foam flow rate value - this is received via a CAN message from the DC Motor driver.
	*/
	uint CurrentPowerState;
	smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
	uint PrimePumpRequest;
	smRead(VariableIDs.PrimeFoamPump_Power, PrimePumpRequest);
	uint DebuggingMode;
	smRead(VariableIDs.DebuuginModeActive, DebuggingMode); 
	if (CurrentPowerState != 0 || PrimePumpRequest == 1 || DebuggingMode == 1)
	{
		smRead(VariableIDs.FoamPrimary_RawFoamFlowRate, FoamPrimary_RawFoamFlowRate);
	}
	else
	{
		FoamPrimary_RawFoamFlowRate = 0;
	} 
	
	smWrite(VariableIDs.RawFoamFlow_Primary, FoamPrimary_RawFoamFlowRate);


	/*	Because Ovel required me to use "fudge" factors in the DC Motor driver - the
		frequency we receive is not the actual frequency.  We can re-adjust this value
		back to normal by multiplying by the ratio of the real factor (18432) to
		Ovel's fudge factor (16588).  18432 / 16588 = 1.111165
		The result is the actual frequency multiplied by 10.
	*/
	TempScratchPad = FoamPrimary_RawFoamFlowRate;
	TempScratchPad = TempScratchPad * PulseFactorFudge;
	TempScratchPad = TempScratchPad / 10;
	
	/*	Put the result back into the raw foam flow rate variable
	*/
   	FoamPrimary_RawFoamFlowRate = TempScratchPad; 
	//smWrite(VariableIDs.Primary_FoamFlowRateGPM, FoamPrimary_RawFoamFlowRate);
	//smWrite(VariableIDs.FoamPrimary_RawFoamFlowRate, FoamPrimary_RawFoamFlowRate);
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	
	
	
	if (CurrentFoamPrimeStatus[0] != PRIME_LIMPHOME)
	{
		/*	Calculate the GPM rate of foam from the raw value received from the DC motor driver
			making sure to use the correct calibration multiplier value.
			Put this value into the Primary_FoamFlowGPM.
			Load the total foam flow data from the DC Motor Driver CAN message
			into the appropriate system variable for total foam pulses.
		*/
		switch (CurrentTankSelected)
		{
			case 0:	//none (flush)
					smWrite(VariableIDs.Primary_TotalFoamGallons_A, 0);
					smWrite(VariableIDs.Primary_TotalFoamGallons_B, 0);
					Primary_FoamFlowGPM = 0;
					break;
			case 1:	//tank B
					smRead(VariableIDs.FoamPrimary_TotalFoamPulses, TotalFoamPulses);
					TempScratchPad = TotalFoamPulses;
					/*	We need to adjust the total foam flowed pulses received from the DC motor driver to account for the
						original "fudge" factor.
					*/
					TempScratchPad = TempScratchPad * 0.940767;
					TempScratchPad = TempScratchPad * PulseFactorFudge;
					TempScratchPad = TempScratchPad / Primary_FoamCalFactor_TankB;	
					smWrite(VariableIDs.Primary_TotalFoamGallons_B, TempScratchPad);
					
					TempScratchPad = FoamPrimary_RawFoamFlowRate;					//Get the raw foam flow rate in pulses per second (Hertz).
					TempScratchPad = TempScratchPad * 60;							//Mulitply by 60 to get Pulses Per Minute.
					TempScratchPad = TempScratchPad / Primary_FoamCalFactor_TankB;	//Divide by calibation factor B to get Gallons Per Minute.
					Primary_FoamFlowGPM = TempScratchPad;
					break;
			case 2:	//tank A
					smRead(VariableIDs.FoamPrimary_TotalFoamPulses, TotalFoamPulses);
					TempScratchPad = TotalFoamPulses;
					/*	We need to adjust the total foam flowed pulses received from the DC motor driver to account for the
						original "fudge" factor.
					*/
					TempScratchPad = TempScratchPad * 0.940767;
					TempScratchPad = TempScratchPad * PulseFactorFudge; 	
					TempScratchPad = TempScratchPad / Primary_FoamCalFactor_TankA;
					smWrite(VariableIDs.Primary_TotalFoamGallons_A, TempScratchPad);
					
					TempScratchPad = FoamPrimary_RawFoamFlowRate;					//Get the raw foam flow rate in pulses per second (Hertz).
					TempScratchPad = TempScratchPad * 60;							//Mulitply by 60 to get Pulses Per Minute.
					TempScratchPad = TempScratchPad / Primary_FoamCalFactor_TankA;	//Divide by calibation factor A to get Gallons Per Minute.
					Primary_FoamFlowGPM = TempScratchPad;
					break;
			case 3:	//Motor 1 and motor 2 tank select mismatch
					smWrite(VariableIDs.Primary_TotalFoamGallons_A, 0);
					smWrite(VariableIDs.Primary_TotalFoamGallons_B, 0);
					Primary_FoamFlowGPM = 0;
					break;
		}
	}
	else
	{
		switch (CurrentTankSelected)
		{
			case 0:	//flush
					Primary_FoamFlowGPM = 0;
					break;
			case 1: //tank B
					Primary_FoamFlowGPM = LimpHome_FoamFlowCalculation(0);
					smRead(VariableIDs.Primary_TotalFoamGallons_B, TempScratchPad2);	//Get the current total foam gallons
					TempScratchPad = Primary_FoamFlowGPM;								//Get the current flow rate per minute
					TempScratchPad = TempScratchPad / 600;								//divide it by 600 to convert it from minutes to 0.1 seconds
					TempScratchPad = TempScratchPad + TempScratchPad2;					//Add it to the previous total foam gallons
					smWrite(VariableIDs.Primary_TotalFoamGallons_B, TempScratchPad);	//Put it back into the total
					break;
			case 2: //tank A
					Primary_FoamFlowGPM = LimpHome_FoamFlowCalculation(0);
					smRead(VariableIDs.Primary_TotalFoamGallons_A, TempScratchPad2);	//Get the current total foam gallons
					TempScratchPad = Primary_FoamFlowGPM;								//Get the current flow rate per minute
					TempScratchPad = TempScratchPad / 600;								//divide it by 600 to convert it from minutes to 0.1 seconds
					TempScratchPad = TempScratchPad + TempScratchPad2;					//Add it to the previous total foam gallons
					smWrite(VariableIDs.Primary_TotalFoamGallons_A, TempScratchPad);	//Put it back into the total
					break;
			case 3: //mismatch
					Primary_FoamFlowGPM = 0;
					break;
		}	
	}
	
	
	
	/*	TODO - should we even care to make sure "Primary_FoamFlowGPM" is set to zero if there is no water flow?
		The foam pump shouldn't even be running if there is no foam flow.
		Anyway - a single type system (whether single pump or dual pump) can use both water flow paddlewheel inputs
		for operation - it an either-or situtation.  This is so a builder can run one paddlewheel in a large diamter
		manifold for high flow and a small diameter manifold for low flow.	
		Foam system control (0=single motor, 1=dual motor(as one system), 2=dual motor (as two systems)
	*/
	uint FoamSystemControlType;
	smRead(VariableIDs.FoamSystemControlType, FoamSystemControlType);
	
	/*	Get the current water flow rate.
	*/
	smRead(VariableIDs.Primary_WaterFlowRate_GPM, WaterFlowRateGPM[0]);
	smRead(VariableIDs.Secondary_WaterFlowRate_GPM, WaterFlowRateGPM[1]);
	
	uint PrimeFoamPumpPower;
	smRead(VariableIDs.PrimeFoamPump_Power, PrimeFoamPumpPower);
	
	if (FoamSystemControlType == 0 || FoamSystemControlType == 1)
	{
		//Don't show a foam flow when water flow is less than 5 GPM
		if ((WaterFlowRateGPM[0] < 5 && WaterFlowRateGPM[1] < 5) && DebuggingMode == 0 && PrimeFoamPumpPower == 0)
		{
			Primary_FoamFlowGPM = 0;
		}
	}
	else
	{
		//Don't show a foam flow when water flow is less than 5 GPM
		if (WaterFlowRateGPM[0] < 5 && DebuggingMode == 0 && PrimeFoamPumpPower == 0)
		{
			Primary_FoamFlowGPM = 0;
		}
	}
	
	
	/*	Put the calculated foam flow rate into the system variable.
	*/
	smWrite(VariableIDs.Primary_FoamFlowRateGPM, Primary_FoamFlowGPM);	
}






double LimpHome_FoamFlowCalculation(uint Motor)
{
	double FoamFlowGPM = 0;
	
	sendEvent(EventIDs.SCRIPT_UpdatePerformanceChart);	//Make sure we update the performance chart - this is where the LimpHome_FlowRate[] is set
	/*	We are limping home - no foam pulses detected. Calculate foam flow rate based on look-up table.
		PWM_Percent[] = 0 to 1000 (0% to 100%)
	*/
	double tmpDouble;
	uint LimpHomeArrayElement = 0;
	uint MotorSpeedDutyCycle;
	if (Motor == 0)
		smRead(VariableIDs.Primary_MotorPWM_Request, MotorSpeedDutyCycle);
	else
		smRead(VariableIDs.Secondary_MotorPWM_Request, MotorSpeedDutyCycle);	
	
	if (MotorSpeedDutyCycle >= 50)	//Greater than 5% (50 = 5.0)
	{
		tmpDouble = MotorSpeedDutyCycle;
		tmpDouble = tmpDouble - 50;
		tmpDouble = tmpDouble * 0.02;
		LimpHomeArrayElement = tmpDouble;
		FoamFlowGPM = LimpHome_FlowRate[Motor][LimpHomeArrayElement];
	}
	else
	{
		FoamFlowGPM = 0;
	}
	
	return FoamFlowGPM;
}

//-----ScriptName=SCRIPT CalculateMotorPWM
/**************************************************************************************
SCRIPT Name:	Calculate motor PWM
Fired On:		Every 200ms by its own timer (Calculate Motor PWM Timer)

Description:	This is the main foam logix algorithm which calculates the duty cycle
				of the foam motor based on water flow and the calculated foam flow.
				

---------------------------------------------------------------------------------------
REVISION HISTORY - update system variable "FoamLogix Version" when making changes to 
the configuration.
				
Rev		By	Date		Description
----	---	----------	---------------------------------------------------------------
1.00	AMS	11-13-2014	Initial configuration 
1.01	AMS	11-25-2014	Added 20 second flush timer - to shut down system after 20 seconds in flush mode.

***************************************************************************************/
uint[] 	PWM_Percent(2);				//Variable holds the current motor PWM requested.
bool[]	b_FastStart(2);				//Sets the duty cycle to 50% on first press of power button.
uint[]  LastMotorPWM(2);
uint[]	FastStartTimer(2);			//Used to determine how long the fast start PWM will last.
uint[]	FlushTimer(2);				//Used to shutdown operation in flush mode after 20 seconds.
uint[] 	RampToPWM(2);
uint[]	CalibrationPWM(2);			//Used to set the calibration motor PWM speed - this is set in the individual CalibrateFoamFlow_PowerButton scripts


double  ScriptRate 			= 0.05;	//This is how often this script is run - it is based on the timer (Calculate Motor PWM Timer) in this script's event section
const uint8	PRIME_OKAY 		= 0;	//Foam prime definitions
const uint8 PRIME_LOSS 		= 1;
const uint8 PRIME_SHUTDOWN 	= 2;
const uint8 PRIME_LIMPHOME 	= 3;	


const uint SIXTY_SECONDS 	= 60 / ScriptRate;	//Low foam run time out value.
uint[] FLUSH_TIME(2);		

uint[]	LowFoamTimer(2);			//Low foam timers used in HandleLowFoamOperation().

const uint8	SYSTEM_OFF = 0;
const uint8 SYSTEM_TANK_MISMATCH = 4;

const uint8 FLUSH = 0;				//Foam tank selection defines.
const uint8 TANK_A = 2;
const uint8 TANK_B = 1;
const uint8 TANK_MISMATCH = 3;

int[] 	s16_TempPWM_UpdateWindow(2);
uint[]	u16_PWM_UpdateWindow(2);	//Updated with V1.19 for both motors

bool 	InitializeArrays = true;	//Used to initialize all the of the arrays to 0 upon first run of this script

uint[]	CurrentFoamPrimeStatus(2);	//Status of the foam prime - 0=good, 1=loss of foam detected, 2=shutdown due to loss of foam
uint[]	PrimeLossDetectTimer(2);	//Time used to evaluate how long foam flow has been lost before setting the lost prime variable.
uint[]	PrimeIsLostTimer(2);		//Timer used to determine how long we can stay in prime loss mode before shutting down.

bool	TemporarilyUseSingleMotorOnly = false;	//Use only a single motor if foam flow rate is lower than operation variable "MinimumFoamFlowRate"
bool[] 	EnableLowFlowAlgorithm(2);
bool[] 	PulseMotor(2);
uint[] LowFlowON_Timer(2);
uint[] LowFlowOFF_Timer(2);



void SCRIPT_CalculateMotorPWM () 
{
	uint FoamSystemControlType;
	smRead(VariableIDs.FoamSystemControlType, FoamSystemControlType);
	
	/*	
		Foam system control (0=single motor, 1=dual motor(as one system), 2=dual motor (as two systems).
		The FLX_NumberOfPumps variable is used in the SingleSystem_MotorControl() function in order
		to control two pumps as a single system.
	*/
	if (FoamSystemControlType == 0)
	{
		smWrite(VariableIDs.FLX_NumberOfPumps, 1);
		SingleSystem_MotorControl();
	}
	else if (FoamSystemControlType == 1)
	{
		smWrite(VariableIDs.FLX_NumberOfPumps, 2);
		SingleSystem_MotorControl();
	}
	else
	{
		smWrite(VariableIDs.FLX_NumberOfPumps, 2);
		DualSystem_MotorControl();
	}
	
	
	/*	1.17 - adding autobypass.  This output will be active if the manual-prime button is pressed, the system is in auto-priming (when ramping to regain prime), 
		or when in foam calibration mode.
	*/
	uint[] PrimePumpRequest(2);
	smRead(VariableIDs.PrimeFoamPump_Power, PrimePumpRequest[0]);
	smRead(VariableIDs.PrimeFoamPump_Power_2, PrimePumpRequest[1]);
	uint[] CurrentPowerState(2);
	smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState[0]);
	smRead(VariableIDs.Secondary_FoamLogix_Enabled, CurrentPowerState[1]);
	uint[] FoamCalibrationRequest(2);
	smRead(VariableIDs.Calibration_PrimaryMotorEnable, FoamCalibrationRequest[0]);
	smRead(VariableIDs.Calibration_SecondaryMotorEnable, FoamCalibrationRequest[1]);

	if (PrimePumpRequest[0] == 1 || CurrentFoamPrimeStatus[0] == PRIME_LOSS || FoamCalibrationRequest[0] == 1)
	{
		smWrite(VariableIDs.FreeFormCAN_AutoBypassOutput_1, 1);
	}
	else
	{
		smWrite(VariableIDs.FreeFormCAN_AutoBypassOutput_1, 0);
	}
	
	if (PrimePumpRequest[1] == 1 || CurrentFoamPrimeStatus[1] == PRIME_LOSS || FoamCalibrationRequest[1] == 1)
	{
		smWrite(VariableIDs.FreeFormCAN_AutoBypassOutput_2, 1);
	}
	else
	{
		smWrite(VariableIDs.FreeFormCAN_AutoBypassOutput_2, 0);
	}
}



uint[] LastCurrentPowerState(2);
uint[] PreviousCurrentTankSelected(2);
bool[] JustFinishedFlushing(2);
/*************************************************************************
	Single motor control
	
	This function controls a single motor system, or a dual motor system
	when they are used as a single system to double the capacity - for 
	example, two 6.5 GPM systems would yield 13 GPM.

*************************************************************************/
void SingleSystem_MotorControl()
{
	uint8	CurrentTankSelected;		//The evaluated tank
	double 	FoamPercentage;				//Vurrent foam percentage selected
	double 	f_DesiredFoamRateGPM;		//Calculated foam rate desired - based on current water flow and current foam percentage
	uint	WaterFlowRateGPM;			//Current water flow rate
	uint[] 	RawFoamFlowRate(2);			//Raw foam pulses from the motor drivers
	double[]FoamFlowRate(2);			//Calculated Foam Flow Rate
	uint 	NumberOfPumps;				//Number of pumps selected (dual 6.5 sets this to 2)
	double  AlgorithmDiv;				//Divisor used to change units from GAL (1) to Liters (3.8)	
	
	uint 	CurrentPowerState;
	double	TotalFoamFlowRate_GPM = 0;	//Holds the sum of the foam flow GPM rates from motor 1 and motor 2
	double 	MinimumFoamFlowRate;		//Flow rate which dictates use of a single motor to meet low flow requirement
	double  LowFlowTransitionHysteresis;	
	uint    MotorTemperatureCritical;
	uint 	InitialFastStartSpeed;
	smRead(VariableIDs.FLX_InitialFastStartSpeed, InitialFastStartSpeed);
	InitialFastStartSpeed = InitialFastStartSpeed * 10;	//Multiply by ten because DC Motor driver takes the value without the decimal.
	
	
	/*	Initialize all of the arrays first time into this script.
		These are globals.
	*/
	if (InitializeArrays == true)
	{
		InitializeArrays = false;
		CurrentFoamPrimeStatus[0] 	= PRIME_OKAY;
		PrimeLossDetectTimer[0] 	= 0;
		PrimeIsLostTimer[0]	 		= 0;
		PWM_Percent[0]				= 0;
		RampToPWM[0]				= 0;
		b_FastStart[0]				= false;
		FastStartTimer[0]			= 0;
		PulseMotor[0]				= false;
		EnableLowFlowAlgorithm[0]	= false;
		LastCurrentPowerState[0]	= 0;
		PreviousCurrentTankSelected[0] = 0;
		JustFinishedFlushing[0]		= false;
		CurrentFoamPrimeStatus[1] 	= PRIME_OKAY;
		PrimeLossDetectTimer[1] 	= 0;
		PrimeIsLostTimer[1]	 		= 0;
		PWM_Percent[1]				= 0;
		RampToPWM[1]				= 0;
		b_FastStart[1]				= false;
		FastStartTimer[1]			= 0;	
		PulseMotor[1]				= false;
		EnableLowFlowAlgorithm[1]	= false;
		LastCurrentPowerState[1]	= 0;
		PreviousCurrentTankSelected[1] = 0;
		JustFinishedFlushing[1]		= false;
		LowFlowON_Timer[0]			= 0;
		LowFlowON_Timer[1]			= 0;
		LowFlowOFF_Timer[0]			= 0;
		LowFlowOFF_Timer[1]			= 0;			
	}
	
	
	/*	Get the number of pumps this system is set for (1 or 2)
	*/
	smRead(VariableIDs.FLX_NumberOfPumps, NumberOfPumps);
	
	/*	Get the current state of the FoamLogix system.
	*/
	smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
	
	/*	Get the primary foam flow rate value - this is received via a CAN message from the DC Motor driver 1.
	*/
	smRead(VariableIDs.FoamPrimary_RawFoamFlowRate, RawFoamFlowRate[0]);
	smRead(VariableIDs.Primary_FoamFlowRateGPM, FoamFlowRate[0]);
	
	
	/*	Get the secondary foam flow rate value - this is received via a CAN message from the DC Motor driver 2.
	*/
	smRead(VariableIDs.FoamSecondary_RawFoamFlowRate, RawFoamFlowRate[1]);
	smRead(VariableIDs.Secondary_FoamFlowRateGPM, FoamFlowRate[1]);
	
	/*	Get the current water flow rate.  We will get the flow value from the primary and secondary paddlewheels and then
		determine which one to use based on flow.  This is could be used if the builder used two paddlewheels - one for
		a larger manifold and another for a smaller manifold in an effort to have accurate low water flow detection.  We will
		only use the water flow that is currently higher of the two because one should always be zero. 
	*/
	uint Paddlewheel_Flow_1;
	uint Paddlewheel_Flow_2;
	smRead(VariableIDs.Primary_WaterFlowRate_GPM, Paddlewheel_Flow_1);
	smRead(VariableIDs.Secondary_WaterFlowRate_GPM, Paddlewheel_Flow_2);
	if (Paddlewheel_Flow_2 > Paddlewheel_Flow_1)
	{
		WaterFlowRateGPM = Paddlewheel_Flow_2;
	}
	else
	{
		WaterFlowRateGPM = Paddlewheel_Flow_1;
	}
	smWrite(VariableIDs.Displayed_WaterFlow_GPM, WaterFlowRateGPM);
	
	/*	Update the displayed water flow value variable and the total water flowed value variable
	*/
	uint TotalWaterFlowed_1;
	uint TotalWaterFlowed_2;
	smRead(VariableIDs.Primary_TotalWaterGallons, TotalWaterFlowed_1);
	smRead(VariableIDs.Secondary_TotalWaterGallons, TotalWaterFlowed_2);
	uint TotalWaterFlowed;
	TotalWaterFlowed = TotalWaterFlowed_1 + TotalWaterFlowed_2;
	smWrite(VariableIDs.Displayed_TotalWaterFlowed, TotalWaterFlowed);
	
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT UpdateFoamLogixBackground.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	
	/*	Update foamlogix enabled bit which is transmitted to SPS via 18EFFFD0 CAN message
	*/
	if (CurrentPowerState != SYSTEM_OFF && CurrentPowerState != SYSTEM_TANK_MISMATCH)
	{
		smWrite(VariableIDs.BIT_FoamLogixEnabled, 1);
	}
	else
	{
		smWrite(VariableIDs.BIT_FoamLogixEnabled, 0);
	}
	
	/*	Start or stop the datalogger based on the system state change
	*/
	if (LastCurrentPowerState[0] != CurrentPowerState)
	{
		if (CurrentPowerState == 1)
			sendEvent(EventIDs.Event_DataLoggingStart);
		else
			sendEvent(EventIDs.Event_DataLoggingStop);
		
		LastCurrentPowerState[0] = CurrentPowerState;
	}
	
	/*	Determing which flush timing to use:
		If the system is OFF when the tank selector is set to FLUSH then use 20 seconds.
		If the system is ON when the tank selector is set to flush then use 6 seconds.
	*/
	double tempVanriable;
	uint FlushTimeValue;
	smRead(VariableIDs.FLX_FlushTime, FlushTimeValue);
	tempVanriable = FlushTimeValue;
	tempVanriable = tempVanriable / ScriptRate;
	FlushTimeValue = tempVanriable;
	if (CurrentTankSelected != PreviousCurrentTankSelected[0])
	{
		if (CurrentTankSelected == 0)	//We are in FLUSH mode
		{
			FLUSH_TIME[0] = FlushTimeValue;
		}
		else
		{
			if (JustFinishedFlushing[0] == true)
			{
				if (FLUSH_TIME[0] == FlushTimeValue)	//Only turn the foam system back ON if we are using transition flush
				{
					//When using a manual selector we will turn the system OFF so they can set the new selector position
					sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Primary);

				}
			}	
		}
	}
			
	

	/*	We do not need to control the PWM algorithm if the foamlogix system is not enabled or
		if the pump 1 and pump 2 foam tank selectors do not match.
		Added "PrimePumpRequest" - this is used when the prime pump screen is activated to force the pump to run at 100% for priming purposes.
	*/
	uint PrimePumpRequest;
	smRead(VariableIDs.PrimeFoamPump_Power, PrimePumpRequest);
	uint DebuggingMode;
	smRead(VariableIDs.DebuuginModeActive, DebuggingMode); 
	if (CurrentPowerState != SYSTEM_OFF && CurrentPowerState != SYSTEM_TANK_MISMATCH && PrimePumpRequest == 0 && DebuggingMode == 0)
	{
		JustFinishedFlushing[0] = false;
		/*	Check for a loss of prime in motor 1 and motor 2.  If we are in a loss of prime condition for 4 or more seconds then
			flag the loss of prime so we can start increasing the motor PWM to try and get foam flow back.
			We do not need to ramp the motor or look for missing foam pulses if we are in FLUSH mode.	
		*/
		for (uint Motor = 0; Motor < NumberOfPumps; Motor++)
		{
			/*	If we are temporarily only using one motor then we need keep the primeloss variables clear.
			*/
			bool KeepMotor2Clear = false;
			if (Motor == 1 && TemporarilyUseSingleMotorOnly == true)
				KeepMotor2Clear = true;
				
			/*	Check for motor shutdown due to critical heat
			*/
			if (Motor == 0)
			{
				smRead(VariableIDs.FoamPrimary_TemperatureCritical, MotorTemperatureCritical);
			}
			else
			{
				smRead(VariableIDs.FoamSecondary_TemperatureCritical, MotorTemperatureCritical);
			}
			
			//Removed 2-10-2016, this is not needed because we don't modify Primary_FoamFlowGPM in here so why write it?  It is written at the end of "SCRIPT CalculatePrimaryFoamFlowRate".
			//smWrite(VariableIDs.Primary_FoamFlowRateGPM, Primary_FoamFlowGPM);	
				
			//if (WaterFlowRateGPM > 0 && FoamFlowRate[Motor] == 0 && CurrentTankSelected != FLUSH && CurrentTankSelected != TANK_MISMATCH && KeepMotor2Clear != true && MotorTemperatureCritical == 0)
			if (WaterFlowRateGPM > 0 && RawFoamFlowRate[Motor] < 5 && CurrentTankSelected != FLUSH && CurrentTankSelected != TANK_MISMATCH && KeepMotor2Clear != true && MotorTemperatureCritical == 0)
			{
				if (CurrentFoamPrimeStatus[Motor] == PRIME_OKAY)
				{
					PrimeLossDetectTimer[Motor]++;
					
					/*	Get the user set time for the prime loss.  The user sets the time in seconds and	
						this script runs every 0.2 seconds so divide user time by 0.2 to get the value required
						e.g. 5 seconds / 0.2 = 25;
					*/
					uint PrimeLostStart;
					smRead(VariableIDs.FLX_PrimeLostStart, PrimeLostStart);
					PrimeLostStart = PrimeLostStart / ScriptRate;

					if (PrimeLossDetectTimer[Motor] > PrimeLostStart)
					{
						CurrentFoamPrimeStatus[Motor] 	= PRIME_LOSS;	//Old method which just turned the motor OFF when the foam sensor failed.
						PrimeLossDetectTimer[Motor] 	= 0;
						PrimeIsLostTimer[Motor] 		= 0;
					} 
				}	
			}
			else
			{
				/*	Don't allow the limp home mode to be turned OFF unless we have detected RawFoamFlowRate.
				*/
				if (CurrentFoamPrimeStatus[Motor] != PRIME_LIMPHOME)
				{
					PrimeLossDetectTimer[Motor] 	= 0;
					CurrentFoamPrimeStatus[Motor] 	= PRIME_OKAY;
				}
				else
				{
					if (RawFoamFlowRate[Motor] >= 5)
					{
						PrimeLossDetectTimer[Motor] 	= 0;
						CurrentFoamPrimeStatus[Motor] 	= PRIME_OKAY;
						if (Motor == 0)
							smWrite(VariableIDs.LimpHomeMode_Active_Motor_1, 0);
						else
							smWrite(VariableIDs.LimpHomeMode_Active_Motor_2, 0);
					}
				}
			}
		}	

		/*	Main calculation - takes the current water flow rate and creates the desired foam flow
			rate based on the foam percentage selected.  Calculates the update window and then calls
			the UpdateMotorPWM_Value() function for each motor to control the PWM.
		*/
		if (CurrentTankSelected != FLUSH && CurrentTankSelected != TANK_MISMATCH)
		{
			u16_PWM_UpdateWindow[0]	= 0;				
	
			if(CurrentTankSelected == TANK_B)
			{
				smRead(VariableIDs.Primary_FoamPercentage_TankB, FoamPercentage);
				//smRead(VariableIDs.Foam_B_Percent, FoamPercentage);		//This is what is used in SmartFOAM
			}
			else
			{
				smRead(VariableIDs.Primary_FoamPercentage_TankA, FoamPercentage);
				//smRead(VariableIDs.Foam_A_Percent, FoamPercentage);		//This is what is used in SmartFOAM
			}			

			/*	Calculate the desired foam rate.  This is based from the equation:
				Foam GPM = water GPM * ( %foam / 100).
				e.g What is the desired foam GPM when water flow is 100 GPM and foam % is 1.0?
				
				Foam GPM = 100 GPM * (1.0 / 100)
				Foam GPM = 1
			*/
			f_DesiredFoamRateGPM	= FoamPercentage;							//Percentage foam 
			f_DesiredFoamRateGPM	= f_DesiredFoamRateGPM / 100;				//Percentage foam / 100
			f_DesiredFoamRateGPM 	= f_DesiredFoamRateGPM * WaterFlowRateGPM;	//Multiply by the water flow rate GPM
			
			smWrite(VariableIDs.Primary_FoamFlowRateGPM_Requested, f_DesiredFoamRateGPM);
			
			
smWrite(VariableIDs.DEBUG_1, (Primary_FoamFlowGPM * 100));

smWrite(VariableIDs.DEBUG_3, (f_DesiredFoamRateGPM * 100));



			
			smRead(VariableIDs.FLX_MinimumFoamFlowRate, MinimumFoamFlowRate);
			smRead(VariableIDs.FLX_LowFlowTransitionHysteresis, LowFlowTransitionHysteresis);
			
			/*	Determine if only one of the motors should be used.  If the desired foam flow rate is less than the
				configured operating variable "MinimumFoamFlowRate" then it will use only one motor.
			*/
			if (f_DesiredFoamRateGPM <= MinimumFoamFlowRate)
			{
				TemporarilyUseSingleMotorOnly = true;
			}
			/*	Allow some hysteresis before allowing the second motor to come back on. 
			*/ 
			else if (f_DesiredFoamRateGPM > (MinimumFoamFlowRate + LowFlowTransitionHysteresis))
			{
				TemporarilyUseSingleMotorOnly = false;
			}
			
			/*	Get the flow rate for motor 1 and motor 2.  These variables are defined and updated in
				SCRIPT CalculatePrimaryFoamFlowRate and SCRIPT CalculateSecondaryFoamFlowRate.  Add them
				together to get the total flow rate.
			*/
			if (NumberOfPumps == 2 &&  TemporarilyUseSingleMotorOnly == false)
				TotalFoamFlowRate_GPM = Primary_FoamFlowGPM + Secondary_FoamFlowGPM;
			else
				TotalFoamFlowRate_GPM = Primary_FoamFlowGPM;
			
			/*	Calculate the update window.  This is the size of the difference between the desired flow rate
				and the current flow rate.  Larger differentials yield larger changes in motor speed to accomodate.
			*/
			s16_TempPWM_UpdateWindow[0]	= ((f_DesiredFoamRateGPM * 100) - (TotalFoamFlowRate_GPM * 100));
smWrite(VariableIDs.DEBUG_5, s16_TempPWM_UpdateWindow[0]);
			
			/*	When using GPM calibration values the AlgorithmDiv was 1 (no division), but when using LPM calibration
				values the s16_TempPWM_UpdateWindow would get very large so we had to divide it by 3.8 (lpm to gpm)
				in order for the PWM calculation to come out right.  Now we can use a custom variable to fine tune
				the operation.
				7-30-2015 - since we have a "unit of measure" value use it to determine if we use
					AlgorithmDiv = 1 (english units)
					AlgorithmDiv = 3.785412 (metric units)
				This is basically just converting gallons to liters.	
			*/
			uint UnitOfMeasure;
			smRead(VariableIDs.FLX_UnitOfMeasure, UnitOfMeasure);
			if (UnitOfMeasure == 0)
			{
				AlgorithmDiv = s16_TempPWM_UpdateWindow[0];		//English - no conversion
			}
			else
			{
				AlgorithmDiv = s16_TempPWM_UpdateWindow[0] / 3.785412;	//Metric - divide by 3.785412
			}
			s16_TempPWM_UpdateWindow[0] 	= AlgorithmDiv;		
			u16_PWM_UpdateWindow[0]		= abs(s16_TempPWM_UpdateWindow[0]);
						
			
			/*	If we have a positive water flow rate then we can start normal operation and creation
				of the DC Motor driver PWM value request.
			*/				
			if (WaterFlowRateGPM > 0)
			{
				/*	Check for low foam flow in order to enable the low flow algorithm.
					EnableLowFlowAlgorithm is a global variable and is used in the UpdateMotorPWM_Value() function below.
				*/
				uint Check_LowFlowTime_OFF;
				uint Check_LowFlowTime_ON;
				smRead(VariableIDs.FLX_LowFoamTime_OFF, Check_LowFlowTime_OFF);
				smRead(VariableIDs.FLX_LowFoamTime_ON, Check_LowFlowTime_ON);
				//If any of low flow time variables (ON or OFF) is set to 0 then we will not use the low flow pulsing).
				if (Check_LowFlowTime_ON > 0 && Check_LowFlowTime_OFF > 0)
				{
					if (f_DesiredFoamRateGPM <= MinimumFoamFlowRate) // was static 0.5 gpm)
					{
						EnableLowFlowAlgorithm[0] = true;
					}
					else if (f_DesiredFoamRateGPM >= (MinimumFoamFlowRate + LowFlowTransitionHysteresis)) // was static 0.7 gpm)
					{
						EnableLowFlowAlgorithm[0] = false;
					}
				}
				else
				{
					EnableLowFlowAlgorithm[0] = false;
				}

				/*	Fast start is enabled on the first press of the power button.  It sets the PWM value
					to 50% for the first 1.6 seconds and then turns fast start back off to allow the
					standard algorithm to control the PWM value.
				*/
				/*	Get the user set time for fast start.  The user sets the time in 0.2 second increments and	
					this script runs every 0.2 seconds so divide user time by 0.2 to get the value required
					e.g. 1.6 seconds / 0.2 = 8;
				*/
				double InitialFastStartTime;
				smRead(VariableIDs.FLX_InitialFastStartTime, InitialFastStartTime);
				InitialFastStartTime = InitialFastStartTime / ScriptRate;
				
				if (b_FastStart[0] == false)
				{
					FastStartTimer[0] = 0;
				}
				else
				{
					FastStartTimer[0]++;
					
					if (FastStartTimer[0] >= InitialFastStartTime)
					{
						b_FastStart[0] = false;	
					} 	
				}
				
				if (b_FastStart[1] == false)
				{
					FastStartTimer[1] = 0;
				}
				else
				{
					FastStartTimer[1]++;
								
					if (FastStartTimer[1] >= InitialFastStartTime)
					{
						b_FastStart[1] = false;	
					} 	
				}
			
				/*	Handle low foam operation.  If there is low foam then it will only allow running for 60 seconds
					the first time and then 30 seconds thereafter.
				*/
				HandleLowFoamOperation();
				
				UpdateMotorPWM_Value(0);
				smWrite(VariableIDs.Primary_MotorPWM_Request, PWM_Percent[0]);
smWrite(VariableIDs.DEBUG_7, PWM_Percent[0]);
				
				if (TemporarilyUseSingleMotorOnly == true || NumberOfPumps == 1)
				{
					PWM_Percent[1] = 0;
					smWrite(VariableIDs.Secondary_MotorPWM_Request, PWM_Percent[1]);	
				}
				else
				{	
					UpdateMotorPWM_Value(1);	
					smWrite(VariableIDs.Secondary_MotorPWM_Request, PWM_Percent[1]);
				}
			}
			else
			{
				PWM_Percent[0] = 0; //InitialFastStartSpeed;
				PWM_Percent[1] = 0; //InitialFastStartSpeed;
				smWrite(VariableIDs.Primary_MotorPWM_Request, 0);
				smWrite(VariableIDs.Secondary_MotorPWM_Request, 0);	
				TemporarilyUseSingleMotorOnly = false;	//Reset the temporary use of one pump when water flow comes back
			}
			
			FlushTimer[0] = 0; //Keep the flush timer clear when in a normal tank.
		}
		/*	If we are in FLUSH mode then run the motor at 100% until flush mode is exited OR the FoamLogix enable is turned to OFF
			Or when the flush timer meets 20 seconds.
		*/
		else
		{
			uint FlushTimeRemaining = 0;
			
			FlushTimeRemaining = FLUSH_TIME[0] - FlushTimer[0];
			FlushTimeRemaining = FlushTimeRemaining * ScriptRate;
			FlushTimeRemaining = FlushTimeRemaining + 1;
			smWrite(VariableIDs.FlushTimeRemaining_Primary, FlushTimeRemaining);
			
			if (WaterFlowRateGPM > 0)
			{	
				b_FastStart[0]	= false;
				b_FastStart[1]	= false;
				RampToPWM[0]	= 1000;
				RampToPWM[1]	= 1000;
				if (PWM_Percent[0] < RampToPWM[0])
				{
					PWM_Percent[0] = PWM_Percent[0] + 100;	
				}
				if (PWM_Percent[1] < RampToPWM[1])
				{
					PWM_Percent[1] = PWM_Percent[1] + 100;	
				}

				smWrite(VariableIDs.Primary_MotorPWM_Request, PWM_Percent[0]);
				smWrite(VariableIDs.Secondary_MotorPWM_Request, PWM_Percent[1]);
				
				/*	Increment the flush timer - when it gets to 20 seconds turn OFF the system.
					This task runs every 0.2 seconds so a value of 100 is 20 seconds.	
				*/
				FlushTimer[0]++;
				if (FlushTimer[0] > FLUSH_TIME[0])
				{
					JustFinishedFlushing[0] = true;
					if (EDT_ADT_Request == 1)
					{	//Set EDT/ADT to tank A
						smWrite(VariableIDs.FreeFormCAN_TankA_Select, 1);
						smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
						EDT_ADT_Request = 0;
						sendEvent(EventIDs.Event_TransmitOutputControlMessage);
					}
					else if (EDT_ADT_Request == 2)
					{	//Set EDT/ADT to tank B
						smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
						smWrite(VariableIDs.FreeFormCAN_TankB_Select, 1);
						EDT_ADT_Request = 0;
						sendEvent(EventIDs.Event_TransmitOutputControlMessage);
					}
					else
					{
						sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Primary);
						sendEvent(EventIDs.Warning_FlushIsComplete);	//Popup a warning that the flush is complete
					}
					
					TypeFoamInLines = NO_FOAM_INLINE;			//The lines are now cleared.
					smWrite(VariableIDs.CurrentFoamTypeInLine, TypeFoamInLines);
					InformedUser_FoamFlushRequired = false;	//clear the foam flush requirement
					smWrite(VariableIDs.FlushTimeRemaining_Primary, 0);
				}
			}
			else
			{
				FlushTimer[0] 	= 0;
				PWM_Percent[0] 	= 0;
				PWM_Percent[1] 	= 0;
				smWrite(VariableIDs.Primary_MotorPWM_Request, PWM_Percent[0]);
				smWrite(VariableIDs.Secondary_MotorPWM_Request, PWM_Percent[1]);
			}
		}	
	}
	else
	{
		/*	Check to see if the system is in calibration mode - if it is then we let the calibration script handle the
			motor speed, otherwise we make sure to turn the motors OFF.
		*/
		uint Calibration_PrimaryMotorEnable;
		uint Calibration_SecondaryMotorEnable;
		smRead(VariableIDs.Calibration_PrimaryMotorEnable, Calibration_PrimaryMotorEnable);
		smRead(VariableIDs.Calibration_SecondaryMotorEnable, Calibration_SecondaryMotorEnable);
		
		/*	If we are here because the Prime button is being pressed then handle in here.
			If we aren't priming then check if we are in calibration or debugging modes.
		*/
		if (PrimePumpRequest == 1)
		{
			RampToPWM[0]	= 1000;
			RampToPWM[1]	= 1000;
			if (PWM_Percent[0] < RampToPWM[0])
			{
				PWM_Percent[0] = PWM_Percent[0] + 100;	
			}
			if (PWM_Percent[1] < RampToPWM[1])
			{
				PWM_Percent[1] = PWM_Percent[1] + 100;	
			}

			smWrite(VariableIDs.Primary_MotorPWM_Request, PWM_Percent[0]);
			smWrite(VariableIDs.Secondary_MotorPWM_Request, PWM_Percent[1]);
			JustFinishedFlushing[0] = false;	
		}
		else
		{
			/*	Calibration control of the PWM value.  The CalibrateFoamFlow_PowerButton scripts for primary and secondary motors
				Sets/clears the _PrimaryMotorEnable variable and sets the CalibrationPWM[] value.  If we are turning on the motor then
				we will ramp up to the defined value.
			*/
			if (Calibration_PrimaryMotorEnable == 0)
			{
				PWM_Percent[0] = 0;
			}
			else
			{
				JustFinishedFlushing[0] = false;
				if (PWM_Percent[0] < CalibrationPWM[0])
				{
					PWM_Percent[0] = PWM_Percent[0] + 100;	
				}
				if (PWM_Percent[0] > CalibrationPWM[0])
				{
					PWM_Percent[0] = CalibrationPWM[0];
				}	
			}
			smWrite(VariableIDs.Primary_MotorPWM_Request, PWM_Percent[0]);
			
			if (Calibration_SecondaryMotorEnable == 0)
			{
				PWM_Percent[1] = 0;
			}
			else
			{
				JustFinishedFlushing[0] = false;
				if (PWM_Percent[1] < CalibrationPWM[1])
				{
					PWM_Percent[1] = PWM_Percent[1] + 100;	
				}
				if (PWM_Percent[1] > CalibrationPWM[1])
				{
					PWM_Percent[1] = CalibrationPWM[1];
				}
			}
			smWrite(VariableIDs.Secondary_MotorPWM_Request, PWM_Percent[1]);	
		}
			
		if (DebuggingMode == 1)
		{
			uint Primary_DebugMotorPWM;
			uint Secondary_DebugMotorPWM;
			smRead(VariableIDs.Secondary_DebugMotorPWM, Secondary_DebugMotorPWM);
			smRead(VariableIDs.Primary_DebugMotorPWM, Primary_DebugMotorPWM);
			smWrite(VariableIDs.Primary_MotorPWM_Request, Primary_DebugMotorPWM);
			smWrite(VariableIDs.Secondary_MotorPWM_Request, Secondary_DebugMotorPWM);
		}

		
		/*	Clear any warning for PRIME_LOSS or PRIME_SHUTDOWN we may have had - do not clear if it was
			for PRIME_LIMPHOME
		*/
		if (CurrentFoamPrimeStatus[0] != PRIME_LIMPHOME)
		{
			PrimeLossDetectTimer[0] 	= 0;
			CurrentFoamPrimeStatus[0] 	= PRIME_OKAY;
		}
		if (CurrentFoamPrimeStatus[1] != PRIME_LIMPHOME)
		{
			PrimeLossDetectTimer[1] 	= 0;
			CurrentFoamPrimeStatus[1] 	= PRIME_OKAY;
		}
	}
	
	
	/*	Now that we have calculated foam flow rate we can update the "foam is flowing" variable
		once foam has flowed for 3 or more seconds.  This is used for CAFS.
	*/
	if (TotalFoamFlowRate_GPM > 0)
	{
		FoamFlowingTimer++;	//This script runs every 200ms so 15 = 3 seconds
		if (FoamFlowingTimer > 15)	//test if (FoamFlowingTimer > 15)
		{
			FoamFlowingForAtLeastThreeSeconds = true;
			FoamFlowingTimer = 15;
			smWrite(VariableIDs.HasSystemBeenRun, 1);	//Set the flag used to evaluate the maintenance minder since foam is flowing
		}
	}
	else
	{
		/*	Only turn OFF the FoamFlowingForAtLeastThreeSeconds variable if there is no
			foam detected in the lines.  Mike Laskaris says that the foam flowing 3 seconds
			requirement is only for the first instance to make sure that foam is in the
			line. 
		*/
		if (TypeFoamInLines == NO_FOAM_INLINE)
		{
			/*	In low flow mode the motor is being pulsed ON/OFF.  When the motor is OFF we won't have any foam flow rate GPM so
				we need to not clear the FoamFlowingTimer variable - otherwise the foam type in lines variable will not be set.
			*/
			if (EnableLowFlowAlgorithm[0] == false)
			{
				FoamFlowingForAtLeastThreeSeconds = false;
				FoamFlowingTimer = 0;
			}
		}
	}
	

	/*	I created the BarGraph_MotorPWM_Value for use on the operation screen's foam motor bargraph so that
		it could show a steady value when in the low flow algorithm.  The low flow algorithm pulses the motor
		between the calculated PWM value and 0.5% (5) every 200ms (this scripts update rate).  This is in an
		effort to reduce the foam injection rate for low flow since the motor can only turn at a minimum of
		about 5% (50) which would be too rich.  The side-affect of using this algorithm is that the bar graph
		would bounce between the running percent and almost nothing (0.5%) so I created this variable which
		will not update with a value less than 1% (10) - unless it is at 0.
	*/
	if (PWM_Percent[0] == 0 || PWM_Percent[0] > 10)
	{
		smWrite(VariableIDs.BarGraph_PWM_Value_1, PWM_Percent[0]); 
	}
	if (PWM_Percent[1] == 0 || PWM_Percent[1] > 10)
	{
		smWrite(VariableIDs.BarGraph_PWM_Value_2, PWM_Percent[1]); 
	}
	
	smWrite(VariableIDs.FoamPrimeStatus, CurrentFoamPrimeStatus[0]);
	
	/*	Update the sum of motor 1 and 2 foam gpm rate - this is transmitted via CAN
		message 18EFFFD0.
	*/
	smWrite(VariableIDs.SumFoamFlowRate_GPM, TotalFoamFlowRate_GPM);
	
	PreviousCurrentTankSelected[0] = CurrentTankSelected;
}





/*************************************************************************
	Dual motor control
	
	This function is used to individually control two foam motors as 
	inividual systems - for example, a primary and auxilliary pump system.

*************************************************************************/
void DualSystem_MotorControl()
{
	uint8[]	CurrentTankSelected(2);		//The evaluated tank
	double 	FoamPercentage;				//Vurrent foam percentage selected
	double 	f_DesiredFoamRateGPM;		//Calculated foam rate desired - based on current water flow and current foam percentage
	uint[]	WaterFlowRateGPM(2);			//Current water flow rate
	uint[] 	RawFoamFlowRate(2);			//Raw foam pulses from the motor drivers
	double[] 	FoamFlowRate(2);			//Calculated Foam Flow Rate
	uint 	NumberOfPumps;				//Number of pumps selected (dual 6.5 sets this to 2)
	double  AlgorithmDiv;				//Divisor used to change units from GAL (1) to Liters (3.8)
	
	uint[] 	CurrentPowerState(2);
	double	TotalFoamFlowRate_GPM = 0;	//Holds the sum of the foam flow GPM rates from motor 1 and motor 2
	double 	MinimumFoamFlowRate = 0;		//Flow rate which dictates use of a single motor to meet low flow requirement
	double  LowFlowTransitionHysteresis = 0;	
	uint    MotorTemperatureCritical;
	uint 	InitialFastStartSpeed;
	uint    thisCurrentPowerState;
	
	smRead(VariableIDs.FLX_InitialFastStartSpeed, InitialFastStartSpeed);
	InitialFastStartSpeed = InitialFastStartSpeed * 10;	//Multiply by ten because DC Motor driver takes the value without the decimal.
	
	
	/*	Initialize all of the arrays first time into this script.
		These are globals.
	*/
	if (InitializeArrays == true)
	{
		InitializeArrays = false;
		CurrentFoamPrimeStatus[0] 	= PRIME_OKAY;
		PrimeLossDetectTimer[0] 	= 0;
		PrimeIsLostTimer[0]	 		= 0;
		PWM_Percent[0]				= 0;
		RampToPWM[0]				= 0;
		b_FastStart[0]				= false;
		PulseMotor[0]				= false;
		LastCurrentPowerState[0]	= 0;
		PreviousCurrentTankSelected[0] = 0;
		JustFinishedFlushing[0]		= false;
		CurrentFoamPrimeStatus[1] 	= PRIME_OKAY;
		PrimeLossDetectTimer[1] 	= 0;
		PrimeIsLostTimer[1]	 		= 0;
		PWM_Percent[1]				= 0;
		RampToPWM[1]				= 0;
		b_FastStart[1]				= false;	
		PulseMotor[1]				= false;
		LastCurrentPowerState[1]	= 0;
		PreviousCurrentTankSelected[1] = 0;
		JustFinishedFlushing[1]		= false;
		LowFlowON_Timer[0]			= 0;
		LowFlowON_Timer[1]			= 0;
		LowFlowOFF_Timer[0]			= 0;
		LowFlowOFF_Timer[1]			= 0;
	}
		
	/*	Get the current state of the FoamLogix system.
	*/
	smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState[0]);
	smRead(VariableIDs.Secondary_FoamLogix_Enabled, CurrentPowerState[1]);
	
	/*	Get the primary and secondary foam flow rate values - this is received via a CAN message from the DC Motor driver 1.
	*/
	smRead(VariableIDs.FoamPrimary_RawFoamFlowRate, RawFoamFlowRate[0]);
	smRead(VariableIDs.FoamSecondary_RawFoamFlowRate, RawFoamFlowRate[1]);
	smRead(VariableIDs.Primary_FoamFlowRateGPM, FoamFlowRate[0]);
	smRead(VariableIDs.Secondary_FoamFlowRateGPM, FoamFlowRate[1]);
	
	/*	Get the current water flow rates.
	*/
	smRead(VariableIDs.Primary_WaterFlowRate_GPM, WaterFlowRateGPM[0]);
	smRead(VariableIDs.Secondary_WaterFlowRate_GPM, WaterFlowRateGPM[1]);
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT UpdateFoamLogixBackground.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected[0]);
	smRead(VariableIDs.FoamSecondary_FoamTankSelected, CurrentTankSelected[1]);
	
	/*	Update foamlogix enabled bit which is transmitted to SPS via 18EFFFD0 CAN message
	*/
	if ((CurrentPowerState[0] != SYSTEM_OFF && CurrentPowerState[0] != SYSTEM_TANK_MISMATCH) || (CurrentPowerState[1] != SYSTEM_OFF && CurrentPowerState[1] != SYSTEM_TANK_MISMATCH))
	{
		smWrite(VariableIDs.BIT_FoamLogixEnabled, 1);
	}
	else
	{
		smWrite(VariableIDs.BIT_FoamLogixEnabled, 0);
	}


	/*	Start or stop the datalogger based on the system state change
	*/
	if (CurrentPowerState[0] == 0 && CurrentPowerState[1] == 0)
		thisCurrentPowerState = 0;
	else
		thisCurrentPowerState = 1;
		
	if (LastCurrentPowerState[0] != thisCurrentPowerState)
	{
		if (thisCurrentPowerState == 1)
			sendEvent(EventIDs.Event_DataLoggingStart);
		else
			sendEvent(EventIDs.Event_DataLoggingStop);
		
		LastCurrentPowerState[0] = thisCurrentPowerState;
	}
	
	/*	Determing which flush timing to use:
		If the system is OFF when the tank selector is set to FLUSH then use 20 seconds.
		If the system is ON when the tank selector is set to flush then use 6 seconds.
	*/
	double tempVanriable;
	uint FlushTimeValue;
	smRead(VariableIDs.FLX_FlushTime, FlushTimeValue);
	tempVanriable = FlushTimeValue;
	tempVanriable = tempVanriable / ScriptRate;
	FlushTimeValue = tempVanriable;
			
	if (CurrentTankSelected[0] != PreviousCurrentTankSelected[0])
	{
		if (CurrentTankSelected[0] == 0)	//We are in FLUSH mode
		{	
			FLUSH_TIME[0] = FlushTimeValue;
		}
		else
		{
			if (JustFinishedFlushing[0] == true)
			{
				if (FLUSH_TIME[0] == FlushTimeValue)	//Only turn the foam system back ON if we are using transition flush
				{
					sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Primary);
				}
			}	
		}
	}
	
	if (CurrentTankSelected[1] != PreviousCurrentTankSelected[1])
	{
		if (CurrentTankSelected[1] == 0)	//We are in FLUSH mode
		{

			FLUSH_TIME[1] = FlushTimeValue;
		}
		else
		{
			if (JustFinishedFlushing[1] == true)
			{
				if (FLUSH_TIME[1] == FlushTimeValue)	//Only turn the foam system back ON if we are using transition flush
				{
					sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Secondary);
				}
			}	
		}
	}

	
	
	for (uint Motor = 0; Motor < 2; Motor++)
	{
		/*	We do not need to control the PWM algorithm if the foamlogix system is not enabled or
			if the pump 1 and pump 2 foam tank selectors do not match.
			Added "PrimePumpRequest" - this is used when the prime pump screen is activated to force the pump to run at 100% for priming purposes.
		*/
		uint[] PrimePumpRequest(2);
		smRead(VariableIDs.PrimeFoamPump_Power, PrimePumpRequest[0]);
		smRead(VariableIDs.PrimeFoamPump_Power_2, PrimePumpRequest[1]);
		uint DebuggingMode;
		smRead(VariableIDs.DebuuginModeActive, DebuggingMode);
		if (CurrentPowerState[Motor] != SYSTEM_OFF && PrimePumpRequest[Motor] == 0 && DebuggingMode == 0)
		{
			JustFinishedFlushing[Motor] = false;
			/*	Check for a loss of prime in motor 1 and motor 2.  If we are in a loss of prime condition for 4 or more seconds then
				flag the loss of prime so we can start increasing the motor PWM to try and get foam flow back.
				We do not need to ramp the motor or look for missing foam pulses if we are in FLUSH mode.	
			*/
		
			/*	Check for motor shutdown due to critical heat
			*/
			if (Motor == 0)
			{
				smRead(VariableIDs.FoamPrimary_TemperatureCritical, MotorTemperatureCritical);
			}
			else
			{
				smRead(VariableIDs.FoamSecondary_TemperatureCritical, MotorTemperatureCritical);
			}
			

			//if (WaterFlowRateGPM[Motor] > 0 && FoamFlowRate[Motor] == 0 && CurrentTankSelected[Motor] != FLUSH && MotorTemperatureCritical == 0)
			if (WaterFlowRateGPM[Motor] > 0 && RawFoamFlowRate[Motor] < 5 && CurrentTankSelected[Motor] != FLUSH && MotorTemperatureCritical == 0)
			{
				if (CurrentFoamPrimeStatus[Motor] == PRIME_OKAY)
				{
					PrimeLossDetectTimer[Motor]++;
					
					/*	Get the user set time for the prime loss.  The user sets the time in seconds and	
						this script runs every 0.2 seconds so divide user time by 0.2 to get the value required
						e.g. 5 seconds / 0.2 = 25;
					*/
					uint PrimeLostStart;
					smRead(VariableIDs.FLX_PrimeLostStart, PrimeLostStart);
					PrimeLostStart = PrimeLostStart / ScriptRate;
				
					if (PrimeLossDetectTimer[Motor] > PrimeLostStart)
					{
						/*	If we have detected that the foam sensor is not working then we will set the status to "limp home"
							mode which will use a look-up table for the pump size.  The data in this table is detemined by running
							dutycycles every 5% and noting the flowrate at that duty cycle - all at 150 PSI.
						*/
						if (Motor == 0)
							smWrite(VariableIDs.LimpHomeMode_Active_Motor_1, 1);
						else
							smWrite(VariableIDs.LimpHomeMode_Active_Motor_2, 1);
			
						CurrentFoamPrimeStatus[Motor] 	= PRIME_LOSS;	
						PrimeLossDetectTimer[Motor] 	= 0;
						PrimeIsLostTimer[Motor] 		= 0;
					} 
				}	
			}
			else
			{
				/*	Don't allow the limp home mode to be turned OFF unless we have detected RawFoamFlowRate.
				*/
				if (CurrentFoamPrimeStatus[Motor] != PRIME_LIMPHOME)
				{
					PrimeLossDetectTimer[Motor] 	= 0;
					CurrentFoamPrimeStatus[Motor] 	= PRIME_OKAY;
				}
				else
				{
					if (RawFoamFlowRate[Motor] >= 5)
					{
						PrimeLossDetectTimer[Motor] 	= 0;
						CurrentFoamPrimeStatus[Motor] 	= PRIME_OKAY;
						if (Motor == 0)
							smWrite(VariableIDs.LimpHomeMode_Active_Motor_1, 0);
						else
							smWrite(VariableIDs.LimpHomeMode_Active_Motor_2, 0);
					}
				}
			}
			



			
//MOVED	ALL OF THIS INSIDE THE POWER ON SECTION		
			/*	Main calculation - takes the current water flow rate and creates the desired foam flow
				rate based on the foam percentage selected.  Calculates the update window and then calls
				the UpdateMotorPWM_Value() function for each motor to control the PWM.
			*/
			if (CurrentTankSelected[Motor] != FLUSH)
			{
				u16_PWM_UpdateWindow[Motor]	= 0;				
		
				if(CurrentTankSelected[Motor] == TANK_B)
				{
					if (Motor == 0)
						smRead(VariableIDs.Primary_FoamPercentage_TankB, FoamPercentage);
					else
						smRead(VariableIDs.Secondary_FoamPercentage_TankB, FoamPercentage);
				}
				else
				{
					if (Motor == 0)
						smRead(VariableIDs.Primary_FoamPercentage_TankA, FoamPercentage);
					else
						smRead(VariableIDs.Secondary_FoamPercentage_TankA, FoamPercentage);
				}
				
				/*	Calculate the desired foam rate.  This is based from the equation:
					Foam GPM = water GPM * ( %foam / 100).
					e.g What is the desired foam GPM when water flow is 100 GPM and foam % is 1.0?
					
					Foam GPM = 100 GPM * (1.0 / 100)
					Foam GPM = 1
				*/
				f_DesiredFoamRateGPM	= FoamPercentage;							//Percentage foam 
				f_DesiredFoamRateGPM	= f_DesiredFoamRateGPM / 100;				//Percentage foam / 100
				f_DesiredFoamRateGPM 	= f_DesiredFoamRateGPM * WaterFlowRateGPM[Motor];	//Multiply by the water flow rate GPM
				
				if (Motor == 0)
					smWrite(VariableIDs.Primary_FoamFlowRateGPM_Requested, f_DesiredFoamRateGPM);
				else
					smWrite(VariableIDs.Secondary_FoamFlowRateGPM_Requested, f_DesiredFoamRateGPM);
				
				/*	Get the flow rate for motor 1 or motor 2.  These variables are defined and updated in
					SCRIPT CalculatePrimaryFoamFlowRate and SCRIPT CalculateSecondaryFoamFlowRate.
				*/
				double FoamFlowRate_GPM = 0;
				if (Motor == 0) 
					FoamFlowRate_GPM = Primary_FoamFlowGPM;
				else
					FoamFlowRate_GPM = Secondary_FoamFlowGPM;
					
smWrite(VariableIDs.DEBUG_1, Primary_FoamFlowGPM);
smWrite(VariableIDs.DEBUG_2, Secondary_FoamFlowGPM);

double desiredPrimaryFoam;
double desiredSecondaryFoam;
smRead(VariableIDs.Primary_FoamFlowRateGPM_Requested, desiredPrimaryFoam);
smRead(VariableIDs.Secondary_FoamFlowRateGPM_Requested, desiredSecondaryFoam);

smWrite(VariableIDs.DEBUG_3, desiredPrimaryFoam);
smWrite(VariableIDs.DEBUG_4, desiredSecondaryFoam);


 
				
				/*	Calculate the update window.  This is the size of the difference between the desired flow rate
					and the current flow rate.  Larger differentials yield larger changes in motor speed to accomodate.
				*/
				s16_TempPWM_UpdateWindow[Motor]	= ((f_DesiredFoamRateGPM * 100) - (FoamFlowRate_GPM * 100));

smWrite(VariableIDs.DEBUG_5, s16_TempPWM_UpdateWindow[0]);
smWrite(VariableIDs.DEBUG_6, s16_TempPWM_UpdateWindow[1]);
				
				/*	When using GPM calibration values the AlgorithmDiv was 1 (no division), but when using LPM calibration
					values the s16_TempPWM_UpdateWindow would get very large so we had to divide it by 3.8 (lpm to gpm)
					in order for the PWM calculation to come out right.  Now we can use a custom variable to fine tune
					the operation.
					7-30-2015 - since we have a "unit of measure" value use it to determine if we use
					AlgorithmDiv = 1 (english units)
					AlgorithmDiv = 3.785412 (metric units)
					This is basically just converting gallons to liters.
				*/
				uint UnitOfMeasure;
				smRead(VariableIDs.FLX_UnitOfMeasure, UnitOfMeasure);
				if (UnitOfMeasure == 0)
				{
					AlgorithmDiv = s16_TempPWM_UpdateWindow[Motor];		//English - no conversion
				}
				else
				{
					AlgorithmDiv = s16_TempPWM_UpdateWindow[Motor] / 3.785412;	//Metric - divide by 3.785412
				}
				s16_TempPWM_UpdateWindow[Motor] 	= AlgorithmDiv;	
				u16_PWM_UpdateWindow[Motor]	= abs(s16_TempPWM_UpdateWindow[Motor]);
							
				
				/*	If we have a positive water flow rate then we can start normal operation and creation
					of the DC Motor driver PWM value request.
				*/			
				if (WaterFlowRateGPM[Motor] > 0)
				{
					/*	Check for low foam flow in order to enable the low flow algorithm.
					EnableLowFlowAlgorithm is a global variable and is used in the UpdateMotorPWM_Value() function below.
					*/
					uint Check_LowFlowTime_OFF;
					uint Check_LowFlowTime_ON;
					smRead(VariableIDs.FLX_LowFoamTime_OFF, Check_LowFlowTime_OFF);
					smRead(VariableIDs.FLX_LowFoamTime_ON, Check_LowFlowTime_ON);
					//If any of low flow time variables (ON or OFF) is set to 0 then we will not use the low flow pulsing).
					if (Check_LowFlowTime_ON > 0 && Check_LowFlowTime_OFF > 0)
					{
						if (f_DesiredFoamRateGPM <= MinimumFoamFlowRate) // was static 0.5 gpm)
						{
							EnableLowFlowAlgorithm[Motor] = true;
						}
						else if (f_DesiredFoamRateGPM >= (MinimumFoamFlowRate + LowFlowTransitionHysteresis)) // was static 0.7 gpm)
						{
							EnableLowFlowAlgorithm[Motor] = false;
						}
					}
					else
					{
						EnableLowFlowAlgorithm[Motor] = false;
					}
				
					/*	Fast start is enabled on the first press of the power button.  It sets the PWM value
						to 50% for the first 1.6 seconds and then turns fast start back off to allow the
						standard algorithm to control the PWM value.
					*/
					if (b_FastStart[Motor] == false)
					{
						FastStartTimer[Motor] = 0;
					}
					else
					{
						FastStartTimer[Motor]++;
						
						/*	Get the user set time for fast start.  The user sets the time in 0.2 second increments and	
							this script runs every 0.2 seconds so divide user time by 0.2 to get the value required
							e.g. 1.6 seconds / 0.2 = 8;
						*/
						double InitialFastStartTime;
						smRead(VariableIDs.FLX_InitialFastStartTime, InitialFastStartTime);
						InitialFastStartTime = InitialFastStartTime / ScriptRate;
									
						if (FastStartTimer[Motor] >= InitialFastStartTime)
						{
							b_FastStart[Motor] = false;	
						} 	
					}
				
					/*	Handle low foam operation.  If there is low foam then it will only allow running for 60 seconds
						the first time and then 30 seconds thereafter.
					*/
					HandleLowFoamOperation();
					UpdateMotorPWM_Value(Motor);

					

					
					smWrite(VariableIDs.HasSystemBeenRun, 1);	//Set the flag used to evaluate the maintenance minder since foam is flowing

					if (Motor == 0)
						smWrite(VariableIDs.Primary_MotorPWM_Request, PWM_Percent[0]);
					else	
						smWrite(VariableIDs.Secondary_MotorPWM_Request, PWM_Percent[1]);	
				}
				else
				{
					PWM_Percent[Motor] = 0; //InitialFastStartSpeed;
					if (Motor == 0)
						smWrite(VariableIDs.Primary_MotorPWM_Request, 0);
					else
						smWrite(VariableIDs.Secondary_MotorPWM_Request, 0);	

				}
			}
			/*	If we are in FLUSH mode then run the motor at 100% until flush mode is exited OR the FoamLogix enable is turned to OFF
				Or when the flush timer meets 20 seconds.
			*/
			else
			{
				uint FlushTimeRemaining = 0;
				
				FlushTimeRemaining = FLUSH_TIME[Motor] - FlushTimer[Motor];
				FlushTimeRemaining = FlushTimeRemaining * ScriptRate;
				FlushTimeRemaining = FlushTimeRemaining + 1;
				if (Motor == 0)
					smWrite(VariableIDs.FlushTimeRemaining_Primary, FlushTimeRemaining);
				else
					smWrite(VariableIDs.FlushTimeRemaining_Secondary, FlushTimeRemaining);
			
				if (WaterFlowRateGPM[Motor] > 0)
				{
					JustFinishedFlushing[Motor] = false;	
					b_FastStart[Motor]	= false;				
					PWM_Percent[Motor] = 1000;

					if (Motor == 0)
						smWrite(VariableIDs.Primary_MotorPWM_Request, PWM_Percent[0]);
					else
						smWrite(VariableIDs.Secondary_MotorPWM_Request, PWM_Percent[1]);
					
					/*	Increment the flush timer - when it gets to 20 seconds turn OFF the system.
						This task runs every 0.2 seconds so a value of 100 is 20 seconds.	
					*/
					FlushTimer[Motor]++;
					if (FlushTimer[Motor] >= FLUSH_TIME[Motor])
					{
						JustFinishedFlushing[Motor] = true;
						FlushTimer[Motor] = 0;
						if (Motor == 0)
							sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Primary);
						else
							sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Secondary);	
					}
				}
				else
				{
					FlushTimer[Motor] = 0;
					PWM_Percent[Motor] = 0;
					if (Motor == 0)
						smWrite(VariableIDs.Primary_MotorPWM_Request, PWM_Percent[0]);
					else
						smWrite(VariableIDs.Secondary_MotorPWM_Request, PWM_Percent[1]);


				}
			}
		}
		else
		{
			//PrimeLossDetectTimer[Motor] 	= 0;
			//CurrentFoamPrimeStatus[Motor] 	= PRIME_OKAY;
			
			/*	Check to see if the system is in calibration mode - if it is then we let the calibration script handle the
				motor speed, otherwise we make sure to turn the motors OFF.
			*/
			uint Calibration_PrimaryMotorEnable;
			uint Calibration_SecondaryMotorEnable;
			smRead(VariableIDs.Calibration_PrimaryMotorEnable, Calibration_PrimaryMotorEnable);
			smRead(VariableIDs.Calibration_SecondaryMotorEnable, Calibration_SecondaryMotorEnable);
			
			
			/*	If we are here because the Prime button is being pressed then handle in here.
			*/
			if (PrimePumpRequest[Motor] == 1)
			{
				JustFinishedFlushing[Motor] = false;
				RampToPWM[Motor]	= 1000;
				PWM_Percent[Motor] 	= 1000;
				
				if (PWM_Percent[Motor] < RampToPWM[Motor])
				{
					PWM_Percent[Motor] = PWM_Percent[Motor] + 100;	
				}
				
				if (Motor == 0)
					smWrite(VariableIDs.Primary_MotorPWM_Request, PWM_Percent[0]);
				else
					smWrite(VariableIDs.Secondary_MotorPWM_Request, PWM_Percent[1]);	
			}
			else
			{
				if (Motor == 0)
				{	
					if (Calibration_PrimaryMotorEnable == 0)
					{
						PWM_Percent[0] = 0;	
					}
					else
					{
						JustFinishedFlushing[0] = false;
						if (PWM_Percent[0] < CalibrationPWM[0])
						{
							PWM_Percent[0] = PWM_Percent[0] + 100;	
						}
						if (PWM_Percent[0] > CalibrationPWM[0])
						{
							PWM_Percent[0] = CalibrationPWM[0];
						}	
					}
					
					smWrite(VariableIDs.Primary_MotorPWM_Request, PWM_Percent[0]);
				}
				else
				{		
					if (Calibration_SecondaryMotorEnable == 0)
					{
						PWM_Percent[1] = 0;
						
					}
					else
					{
						JustFinishedFlushing[1] = false;
						if (PWM_Percent[1] < CalibrationPWM[1])
						{
							PWM_Percent[1] = PWM_Percent[1] + 100;	
						}
						if (PWM_Percent[1] > CalibrationPWM[1])
						{
							PWM_Percent[1] = CalibrationPWM[1];
						}
					}
					smWrite(VariableIDs.Secondary_MotorPWM_Request, PWM_Percent[1]);
				}
			}
			
			if (DebuggingMode == 1)
			{
				uint Primary_DebugMotorPWM;
				uint Secondary_DebugMotorPWM;
				smRead(VariableIDs.Secondary_DebugMotorPWM, Secondary_DebugMotorPWM);
				smRead(VariableIDs.Primary_DebugMotorPWM, Primary_DebugMotorPWM);
				smWrite(VariableIDs.Primary_MotorPWM_Request, Primary_DebugMotorPWM);
				smWrite(VariableIDs.Secondary_MotorPWM_Request, Secondary_DebugMotorPWM);
			}
			
			/*	Clear any warning for PRIME_LOSS or PRIME_SHUTDOWN we may have had - do not clear if it was
				for PRIME_LIMPHOME
			*/
			if (CurrentFoamPrimeStatus[Motor] != PRIME_LIMPHOME)
			{
				PrimeLossDetectTimer[Motor] 	= 0;
				CurrentFoamPrimeStatus[Motor] 	= PRIME_OKAY;
			}
		}		
	}
	smWrite(VariableIDs.FoamPrimeStatus, CurrentFoamPrimeStatus[0]);

	
	/*	I created the BarGraph_MotorPWM_Value for use on the operation screen's foam motor bargraph so that
		it could show a steady value when in the low flow algorithm.  The low flow algorithm pulses the motor
		between the calculated PWM value and 0.5% (5) every 200ms (this scripts update rate).  This is in an
		effort to reduce the foam injection rate for low flow since the motor can only turn at a minimum of
		about 5% (50) which would be too rich.  The side-affect of using this algorithm is that the bar graph
		would bounce between the running percent and almost nothing (0.5%) so I created this variable which
		will not update with a value less than 1% (10) - unless it is at 0.
	*/
	if (PWM_Percent[0] == 0 || PWM_Percent[0] > 10)
	{
		smWrite(VariableIDs.BarGraph_PWM_Value_1, PWM_Percent[0]); 
	}
	if (PWM_Percent[1] == 0 || PWM_Percent[1] > 10)
	{
		smWrite(VariableIDs.BarGraph_PWM_Value_2, PWM_Percent[1]); 
	}
	
	PreviousCurrentTankSelected[0] = CurrentTankSelected[0];
	PreviousCurrentTankSelected[1] = CurrentTankSelected[1];	
}


/*************************************************************************
	Low foam operation
	
	When the system is in a low foam condition then we only allow the
	pump to run for 60 seconds and then 30 seconds there after.
	This function is called by the above main script only when the system
	is currently running.  The timers are incremented here every 200 ms
	(main script run time) and when one of the timers reach 60 seconds
	it fires the power button script which will turn the system OFF and
	reset the timer values.
*************************************************************************/
bool Warning_NoFoam_A = false;
bool Warning_NoFoam_B = false;
void HandleLowFoamOperation()
{	
	uint CurrentTankSelected;
	uint LowFoamA;
	uint LowFoamB;
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT UpdateFoamLogixBackground.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	smRead(VariableIDs.FoamPrimary_Foam_A_Low, LowFoamA);
	smRead(VariableIDs.FoamPrimary_Foam_B_Low, LowFoamB);
	
	
	/*	Increment the low foam timers.  This script is run every 200ms.
	*/
	if (LowFoamA != 0 && CurrentTankSelected == TANK_A)
	{ 	
		LowFoamTimer[0]++;
	}
	else	//Not low foam or this tank is not selected - keep the timer clear.
	{
		LowFoamTimer[0] = 0;
		Warning_NoFoam_A = false;
	}
	
	if (LowFoamB != 0 && CurrentTankSelected == TANK_B)
	{
		LowFoamTimer[1]++;
	}
	else
	{
		LowFoamTimer[1] = 0;
		Warning_NoFoam_B = false;
	} 
	
	/*	If a timer times out - fire the power button script to turn OFF the system
	*/
	if (LowFoamTimer[0] >= SIXTY_SECONDS)
	{
		sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Primary);
		
		/*	Pop up the "no foam" warning.
		*/
		Warning_NoFoam_A = true;
		sendEvent(EventIDs.WARNING_Popup_No_Foam_A);
	}
	else if (LowFoamTimer[1] >= SIXTY_SECONDS)
	{
		sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Secondary);
		
		/*	Pop up the "no foam" warning.
		*/
		Warning_NoFoam_B = true;
		sendEvent(EventIDs.WARNING_Popup_No_Foam_B);
	}		
}



/*************************************************************************
	Update Motor PWM Value
	
	Increment or decrement the motor(s) PWM value based on prime loss, fast
	start condition, or normal operation.
*************************************************************************/
void UpdateMotorPWM_Value(uint MotorNumber)
{
	uint 	THIRTY_SECONDS = 150;	//The main script runs this function every .2 seconds so 150 * .2 = 30 seconds	
	uint8 	u8_MIN_PWM  = 55;		//Minimum PWM value is 5.5%
	uint8 	u8_PWM_Step = 0;
	uint 	NumberOfPumps;				//Number of pumps selected (dual 6.5 sets this to 2)
	smRead(VariableIDs.FLX_NumberOfPumps, NumberOfPumps);
	
	uint MotorTemperatureCritical;

	if (MotorNumber == 0)
	{
		smRead(VariableIDs.FoamPrimary_TemperatureCritical, MotorTemperatureCritical);
	}
	else
	{
		smRead(VariableIDs.FoamSecondary_TemperatureCritical, MotorTemperatureCritical);
	}
	
	
	
	
	/*	Order of operation.  This if, else-if, else below is our standard order of operation - 
		first: 	determine if we have lost prime (foam flow) and handle accordingly.
		second:	determine if this is a fast start condition (power button was just pressed).  This gets foam up and flowing quickly.
		third:	Standard PWM creation. 
	*/
	
	/*	Loss of foam prime - ramp fairly quickly to 100 % in order to get foam flow back.
		If we are still in this condition for more than 30 seconds then we need to stop and flag an error.
	*/
	if ((CurrentFoamPrimeStatus[MotorNumber] == PRIME_LOSS || CurrentFoamPrimeStatus[MotorNumber] == PRIME_SHUTDOWN) && MotorTemperatureCritical == 0)
	{
		/*	If we have detected that we have lost pulses, but not yet gone to PRIME_SHUTDOWN then we work in here to ramp the motor duty cycle to try and
			regain foam prime.
		*/
		if (CurrentFoamPrimeStatus[MotorNumber] == PRIME_LOSS)
		{
		
			/*	Get the user's prime lost ramp speed.  The user sets the speed in % per second as 5, 10, 15, 20, 25,
				30, 35, 40, 45, 50
			*/
			uint PrimeRampSpeed;
			smRead(VariableIDs.FLX_PrimeRampSpeed, PrimeRampSpeed);
			
			uint PrimeLostTime;
			smRead(VariableIDs.FLX_PrimeLostTime, PrimeLostTime);
			
			uint FoamSystemControlType;
			smRead(VariableIDs.FoamSystemControlType, FoamSystemControlType);
			
			/*	Get the user's prime lost time out.  The user sets this in seconds.
				this script runs every 0.2 seconds so divide user time by 0.2 to get the value required
				e.g. 5 seconds / 0.2 = 25;
			*/
			PrimeLostTime = PrimeLostTime / ScriptRate;
		
			PWM_Percent[MotorNumber] += PrimeRampSpeed;
			
			if (PWM_Percent[MotorNumber] > 1000)
			{
				PWM_Percent[MotorNumber] = 1000;	
			}
			if (PWM_Percent[MotorNumber] > 900 || EnableLowFlowAlgorithm[MotorNumber] == true)	//Keep the counter incrementing even if we are in low flow algorithm mode - toggling between actual percent and 0.5%
			{
				PrimeIsLostTimer[MotorNumber]++;							
				if (PrimeIsLostTimer[MotorNumber] > PrimeLostTime)
				{
					PrimeIsLostTimer[MotorNumber] = PrimeLostTime;
					//PWM_Percent[MotorNumber] = 0;	//Shut down the motor.
					
					/*	If we have detected that the foam sensor is not working then we will set the status to "limp home"
						mode which will use a look-up table for the pump size.  The data in this table is detemined by running
						dutycycles every 5% and noting the flowrate at that duty cycle - all at 150 PSI.
					*/
					if (MotorNumber == 0)
						smWrite(VariableIDs.LimpHomeMode_Active_Motor_1, 1);
					else
						smWrite(VariableIDs.LimpHomeMode_Active_Motor_2, 1);
					
					sendEvent(EventIDs.WARNING_Popup_Limp_Home_Mode_1);
		
					CurrentFoamPrimeStatus[MotorNumber] = PRIME_LIMPHOME;		
				}
			}
			
			if (MotorNumber == 0)
				smWrite(VariableIDs.FoamPrimeLoss_CountdownTimer_Motor_1, ((PrimeLostTime - PrimeIsLostTimer[MotorNumber]) * ScriptRate));
			else
				smWrite(VariableIDs.FoamPrimeLoss_CountdownTimer_Motor_2, ((PrimeLostTime - PrimeIsLostTimer[MotorNumber]) * ScriptRate));
		}
		else
		{
			/*	If we have set the system to PRIME_SHUTDOWN then just set this motor to OFF.
				This will not be used if we are using the "limp home" method below.
			*/
			PWM_Percent[MotorNumber] = 0;	//Shut down the motor.
		}
	}
	
	/*	Fast start on initial start for 1.5 seconds.  This is used to get the foam flowing quickly - it is basically priming the foam.
	*/
	else if (b_FastStart[MotorNumber] == true)
	{
		/*	Get the user set time for fast start speed.  The user sets this in percent.
		*/
		uint InitialFastStartSpeed;
		smRead(VariableIDs.FLX_InitialFastStartSpeed, InitialFastStartSpeed);
		InitialFastStartSpeed = InitialFastStartSpeed * 10;	//Multiply by ten because DC Motor driver takes the value without the decimal.
		PWM_Percent[MotorNumber] = InitialFastStartSpeed;	
	}
	/*	Standard motor control operation.
	*/
	else
	{
		/*	The update window is the sensitivity.  Once we get outside of this window we will try and modify the pump speed
			to match the actual flow with the desired flow.	
			We allow the user to specify a sensitivity from 1 to 10 which is 0.01 gpm to 0.10 gpm.  The default was
			5 which is 0.05 gpm
		*/
		uint FLX_Sensitivity;
		smRead(VariableIDs.FLX_Sensitivity, FLX_Sensitivity);
		
		if (u16_PWM_UpdateWindow[MotorNumber] >= FLX_Sensitivity)	
		{
			/*	Create PWM step - bigger steps for larger differential between desired foam and actual foam.
				This step is called the gain.
				The default was - 
						u8_PWM_Step = (u16_PWM_UpdateWindow / 15) + 1; 
						u8_PWM_Step = 25 if (u16_PWM_UpdateWindow > 300)
				Gain gain be 0.5 to 2.0 in 0.1 increments.  The new equation is u8_PWM_Step = (u16_PWM_UpdateWindow * (Gain / 10)) + 1
			*/
			double FLX_Gain;
			smRead(VariableIDs.FLX_Gain, FLX_Gain);
			FLX_Gain = FLX_Gain / 10;
			
			u8_PWM_Step = (u16_PWM_UpdateWindow[MotorNumber] * FLX_Gain) + 1;
		
		
			if (u8_PWM_Step > 100)
			{
				u8_PWM_Step = 100;
			}
			
			/*	If the signed window is negative then we need do decrease the motor speed.
				Make sure that we cannot subtract more PWM than is available - DO NOT GO NEGATIVE!
			*/
			if ((s16_TempPWM_UpdateWindow[MotorNumber] & 0x8000) != 0)
			{
				if (PWM_Percent[MotorNumber] >= u8_PWM_Step)
				{
					PWM_Percent[MotorNumber] -= u8_PWM_Step;
				}
				else
				{
					PWM_Percent[MotorNumber] = 0;	
				}	
			}
			else
			{
				PWM_Percent[MotorNumber] += u8_PWM_Step;
			}

			// keep pwm in limits
			if(PWM_Percent[MotorNumber] > 1000)
			{
				PWM_Percent[MotorNumber] = 1000;
			}
							
			if(PWM_Percent[MotorNumber] < u8_MIN_PWM)
			{
				PWM_Percent[MotorNumber] = u8_MIN_PWM;
			}
		}
	}
	
	/*	Low flow motor pulse mode - the FoamLogix head determined low flow at 0.5 GPM and turned OFF low flow algorithm at 0.7 GPM or greater.
		When it is on the FoamLogix would pulse the motor 100ms normal PWM and then 30ms at 0.5% PWM (which turned OFF the motor without causing
		the DC motor driver to stop counting foam pulses).
		
		10-30-2015 - Added two new variables in the Factory Menu that allows the ON and OFF pulse times to be set (from 0.05 to 1.00 seconds).
			
	*/
	uint LowFlowTime_OFF;
	uint LowFlowTime_ON;
	smRead(VariableIDs.FLX_LowFoamTime_OFF, LowFlowTime_OFF);
	smRead(VariableIDs.FLX_LowFoamTime_ON, LowFlowTime_ON);
	
	if (EnableLowFlowAlgorithm[MotorNumber] == true)
	{
		/*	Pulse is active - this means that we want to send 0.5% to the DC Motor driver so that it will not spin the pump.
			We can't send 0% because the DC Motor driver doesn't count foam flow pulses when it is at 0%.  So we send 0.5%
			which isn't enough to spin the pump, but is enough to make the motor driver count any foam pulses it sees.
			This statement block will set the PulseMotor[MotorNumber] variable back to false once its timer has met the
			configured OFF time.
		*/
		if (PulseMotor[MotorNumber] == true)
		{
			LowFlowON_Timer[MotorNumber] = 0;
			LowFlowOFF_Timer[MotorNumber]++;
			if (LowFlowOFF_Timer[MotorNumber] > LowFlowTime_OFF)
			{
				PulseMotor[MotorNumber] = false;	
			}
			PWM_Percent[MotorNumber] = 5;	
		}
		/*	Pulse is inactive - send the normally calculated motor PWM to the DC motor driver.
			This statement block will set the PulseMotor[MotorNumber] variable back to true once its timer has met the
			configured ON time.
		*/
		else
		{
			LowFlowOFF_Timer[MotorNumber] = 0;
			LowFlowON_Timer[MotorNumber]++;
			if (LowFlowON_Timer[MotorNumber] > LowFlowTime_ON)
			{
				PulseMotor[MotorNumber] = true;	
				LastMotorPWM[MotorNumber] = PWM_Percent[MotorNumber];	//Save the last good PWM value for when we return here.
			}
			PWM_Percent[MotorNumber] = LastMotorPWM[MotorNumber];
		}	
	}
		
	//Handle motor temperature issues - shutdown the motor if it is critical temperature (message from DC Motor driver)
	if (MotorTemperatureCritical == 1)
	{
		PWM_Percent[MotorNumber] = 0;
	}
	
}

//-----ScriptName=SCRIPT FoamLogixPowerButton_Primary
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_FoamLogixPowerButton_Primary for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_FoamLogixPowerButton_Primary () 
{
 	uint CurrentPowerState;
	uint CurrentTankSelected;
	
	/*	Foam system control type can be one of three:
		Single system - one pump (primary)
		Dual system (as single) - two pumps controlled as a single system
		Dual system (as two) - two pumps individually controlled
	*/
	uint FoamSystemControlType;
	smRead(VariableIDs.FoamSystemControlType, FoamSystemControlType);
	
	smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	
	/*	The current power state should be 1 of 4 values:
		0 = OFF
		1 = ON in Foam A
		2 = ON in Foam B
		3 = ON in FLUSH
	*/

	if (CurrentPowerState == 0)	//Currently OFF - we are requesting it to be ON
	{
		/*	Clear the foam prime status (0=good, 1=loss of foam detected, 2=shutdown due to loss of foam, 3 = limp home mode) so its ready and fresh.
			But not if we already determined that we were in limp home mode.  The only thing that will turn OFF limp home is if we start detecting pulses
			again.
		*/
		if (CurrentFoamPrimeStatus[0] != 3)
		{
			CurrentFoamPrimeStatus[0] = 0;
		}

		if (FoamSystemControlType == 0 || FoamSystemControlType == 1)	//If we are using single system types then clear the other pump.
		{
			if (CurrentFoamPrimeStatus[1] != 3)
			{
				CurrentFoamPrimeStatus[1] = 0;
			}
		}
		
		sendEvent(EventIDs.Warnings_ClearMenuPageWarnings);
		
		switch (CurrentTankSelected)
		{
			case 0:	//FLUSH
					smWrite(VariableIDs.Primary_FoamLogix_Enabled, 3);
					break;
			case 1:	//Tank B
					if (TypeFoamInLines != FOAM_A_INLINE)
					{
						smWrite(VariableIDs.Primary_FoamLogix_Enabled, 2);
						//Set the fast start variable to TRUE.  This runs the motor at 50% when the foam logix system is turned on in order to prime the foam.	
						b_FastStart[0] = true;
						b_FastStart[1] = true;
					}
					else
					{
						sendEvent(EventIDs.Warning_ShowFlushRequired);
					}
					break;
			case 2:	//Tank A
					if (TypeFoamInLines != FOAM_B_INLINE)
					{
						smWrite(VariableIDs.Primary_FoamLogix_Enabled, 1);
						//Set the fast start variable to TRUE.  This runs the motor at 50% when the foam logix system is turned on in order to prime the foam.	
						b_FastStart[0] = true;
						b_FastStart[1] = true;
					}
					else
					{
						sendEvent(EventIDs.Warning_ShowFlushRequired);
					}
					break;
			case 3:	//motor 1 and motor 2 tank select mismatch
					smWrite(VariableIDs.Primary_FoamLogix_Enabled, 4);
					break;
		}
	}
	else //Currently ON - we are requesting it to turn OFF
	{
		//Set the FoamLogix enable system variable to OFF (0)
		smWrite(VariableIDs.Primary_FoamLogix_Enabled, 0);
		b_FastStart[0] = false;
		b_FastStart[1] = false;
		
		//Make sure the foam pumps are turned OFF when the power button is off.
		smWrite(VariableIDs.Primary_MotorPWM_Request, 0);
		smWrite(VariableIDs.Primary_FoamFlowRateGPM_Requested, 0);
		
		//Reset the low foam run timer - if it had timed out before then set it for half of 60 seconds.
		if (LowFoamTimer[0] >= SIXTY_SECONDS)
		{
			LowFoamTimer[0] = SIXTY_SECONDS / 2;
		}
		else
		{
			LowFoamTimer[0] = 0;
		}
		
		if (FoamSystemControlType == 0 || FoamSystemControlType == 1)
		{
			smWrite(VariableIDs.Secondary_MotorPWM_Request, 0);
			
			//Reset the low foam run timer - if it had timed out before then set it for half of 60 seconds.
			if (LowFoamTimer[1] >= SIXTY_SECONDS)
			{
				LowFoamTimer[1] = SIXTY_SECONDS / 2;
			}
			else
			{
				LowFoamTimer[1] = 0;
			}
		}		
	} 	 
}



void TurnFoamSystem_OFF(uint pump)
{
	uint CurrentPowerState;
	
	if (pump == 0)
	{
		smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
		if (CurrentPowerState != 0)
		{
			sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Primary);
		}
	}
	else
	{
		smRead(VariableIDs.Secondary_FoamLogix_Enabled, CurrentPowerState);
		if (CurrentPowerState != 0)
		{
			sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Secondary);
		}
	}
}


void TurnFoamSystem_ON(uint pump)
{
	uint CurrentPowerState;
	
	if (pump == 0)
	{
		smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
		if (CurrentPowerState == 0)
		{
			sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Primary);
		}
	}
	else
	{
		smRead(VariableIDs.Secondary_FoamLogix_Enabled, CurrentPowerState);
		if (CurrentPowerState == 0)
		{
			sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Secondary);
		}
	}
}

//-----ScriptName=SCRIPT PrimaryCalibrateFoamFlow_PowerButton
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_PrimaryCalibrateFoamFlow_PowerButton for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_PrimaryCalibrateFoamFlow_PowerButton () 
{
	uint CurrentTankSelected;
	uint MotorRequestON;
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	
	smRead(VariableIDs.Calibration_PrimaryMotorEnable, MotorRequestON); 
	
	
	if (MotorRequestON == 0 && /*CurrentTankSelected != FLUSH &&*/ CurrentTankSelected != TANK_MISMATCH)	//we are going from OFF to ON
	{
		/*	Get the desired calibration motor speed from the factory menu item.
			Then write it to the variable packed into the CAN transmit message to the DC Motor driver.
		*/
		uint MotorSpeed;
		smRead(VariableIDs.FLX_FoamCalibrationSpeed, MotorSpeed);
		MotorSpeed = MotorSpeed * 10;
		
		CalibrationPWM[0] = MotorSpeed;
		//smWrite(VariableIDs.Primary_MotorPWM_Request, MotorSpeed);
		
		/*	Set the clear total foam command - which is also transmitted in the same message
			as the motor speed.
		*/
		smWrite(VariableIDs.FoamPrimary_Command, 3);	//clear total foam flow value.
		
		/*	Start the event which starts the 250ms timer - when that timer times out it sets the 
			command value back to 0.  This is important because the DC motor driver won't run
			if we are constantly sending it the clear command.
		*/
		sendEvent(EventIDs.Event_StartPrimaryFoamClearCommandTimer);
		
		/*	Update the variable used to determine if this script should turn OFF/On the motor request.
			It is basically a toggle.  It is used in the CalculateMotorPWM script.
		*/
		smWrite(VariableIDs.Calibration_PrimaryMotorEnable, 1);
	}
	else
	{
		/*	Turn OFF the motor request.
		*/
		smWrite(VariableIDs.Calibration_PrimaryMotorEnable, 0);
		/*	Update the variable used to determine if this script should turn OFF/On the motor request.
			It is basically a toggle.
		*/
		smWrite(VariableIDs.Primary_MotorPWM_Request, 0);
	}
}

//-----ScriptName=SCRIPT ZeroizeFoamTotal
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_ZeroizeFoamTotal for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_ZeroizeFoamTotal () 
{
 	smWrite(VariableIDs.FoamPrimary_Command, 3);	//clear total foam flow value.
	sendEvent(EventIDs.Event_StartPrimaryFoamClearCommandTimer);  
	
	smWrite(VariableIDs.FoamSecondary_Command, 3);	//clear total foam flow value.
	sendEvent(EventIDs.Event_StartSecondaryFoamClearCommandTimer); 
	
	/*	If we are in limphome mode then we have to clear the actual system variable since
		we aren't receiving the total from the sensor anymore.
	*/
	smWrite(VariableIDs.Primary_TotalFoamGallons_A, 0);
	smWrite(VariableIDs.Primary_TotalFoamGallons_B, 0);
	
	sendEvent(EventIDs.TR_Show_Saving_Widget);
	
	//DEMO values
	Primary_TotalFoamPulses = 0;
	Secondary_TotalFoamPulses = 0;
}

//-----ScriptName=SCRIPT Primary_FoamPercentage_Increase
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_Primary_FoamPercentage_Increase for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Primary_FoamPercentage_Increase () 
{
	uint CurrentTankSelected;
	
 	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	
	switch (CurrentTankSelected)
	{
		case 1:	//Tank B foam percentage
				sendEvent(EventIDs.Calculate_Primary_FoamPercentage_TankB_Equals_INCR);
				break;
				
		case 2:	//Tank A foam percentage
				sendEvent(EventIDs.Calculate_Primary_FoamPercentage_TankA_Equals_INCR);
				break;
	} 
	
	/*	For use with the FoamLogix display type to show the percent screen and then come back
	*/ 
	uint FoamLogixActiveDisplay;
	if (TemporarilyDisplayFoamPercent == false)
	{ 
		TemporarilyDisplayFoamPercent = true;	
		smRead(VariableIDs.FoamLogix_Active_Display, FoamLogixActiveDisplay);
		Last_FoamLogixActiveDisplay = FoamLogixActiveDisplay;
		smWrite(VariableIDs.FoamLogix_Active_Display, 1);	//Go to % display	
	}
	TemporarilyDisplayFoamPercentTimer = 0;
	
}

//-----ScriptName=SCRIPT Primary_FoamPercentage_Decrease
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_Primary_FoamPercentage_Decrease for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Primary_FoamPercentage_Decrease () 
{
	uint CurrentTankSelected;
	
 	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	
	switch (CurrentTankSelected)
	{
		case 1:	//Tank B foam percentage
				sendEvent(EventIDs.Calculate_Primary_FoamPercentage_TankB_Equals_DECR);
				break;
				
		case 2:	//Tank A foam percentage
				sendEvent(EventIDs.Calculate_Primary_FoamPercentage_TankA_Equals_DECR);
				break;
	}
	
	/*	For use with the FoamLogix display type to show the percent screen and then come back
	*/ 
	uint FoamLogixActiveDisplay;
	if (TemporarilyDisplayFoamPercent == false)
	{ 
		TemporarilyDisplayFoamPercent = true;	
		smRead(VariableIDs.FoamLogix_Active_Display, FoamLogixActiveDisplay);
		Last_FoamLogixActiveDisplay = FoamLogixActiveDisplay;
		smWrite(VariableIDs.FoamLogix_Active_Display, 1);	//Go to % display	
	}
	TemporarilyDisplayFoamPercentTimer = 0;  
}

//-----ScriptName=SCRIPT CalibrateWaterFlow
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_CalibrateWaterFlow for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_CalibrateWaterFlow () 
{
  	uint Low_WaterCalValue;
	uint High_WaterCalValue;
	uint Low_WaterRawPulses;
	uint High_WaterRawPulses;
	
	uint DifferenceOfCalValues;
	double f_2PointSlope;
	double f_2PointOffset;
	uint AllowSave = 0;
	
	/*	Only calibrate if we have values that are in range.
		For example - High_WaterCalValue must be greater than Low_WaterCalValue.
					- High_WaterRawPulses and Low_WaterRawPulses must be greater than 0
					- High_WaterRawPulses must be greater than Low_WaterRawPulses.
	*/
	
	smRead(VariableIDs.WaterFlowCalibration_HighValue, High_WaterCalValue);
	smRead(VariableIDs.WaterFlowCalibration_LowValue, Low_WaterCalValue);
	
	smRead(VariableIDs.WaterFlowCalibration_HighRawPulses, High_WaterRawPulses);
	smRead(VariableIDs.WaterFlowCalibration_LowRawPulses, Low_WaterRawPulses);
	
	/*	The system variable AllowSaveOfWaterFlowVariables has a calculation event that evaluates the
		variable conditions:
							- High_WaterCalValue must be greater than Low_WaterCalValue.
							- High_WaterCalValue must be >= 50
							- Low_WaterCalValue must be >= 10
							- High_WaterRawPulses and Low_WaterRawPulses must be greater than 0
							- High_WaterRawPulses must be greater than Low_WaterRawPulses.
	*/
	smRead(VariableIDs.AllowSaveOfWaterFlowVariables, AllowSave);
	
	if (AllowSave != 0)
	{
		DifferenceOfCalValues = High_WaterCalValue - Low_WaterCalValue;
 
		f_2PointSlope = High_WaterRawPulses;
		f_2PointSlope = f_2PointSlope - Low_WaterRawPulses;
		f_2PointSlope = DifferenceOfCalValues / f_2PointSlope;		//subtraction of the two point value: High GPM - Low GPM
	
		f_2PointOffset = f_2PointSlope;
		f_2PointOffset = f_2PointOffset * High_WaterRawPulses;
		f_2PointOffset = High_WaterCalValue - f_2PointOffset;
		
		smWrite(VariableIDs.Primary_WaterCal_2PointOffset, f_2PointOffset);	 
		smWrite(VariableIDs.Primary_WaterCal_2PointSlope, f_2PointSlope);
		
		sendEvent(EventIDs.Event_Save_Water_Flow_Variables);
	}
	else
	{
		//report error
	}
}

//-----ScriptName=SCRIPT_Password_9
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_Password_9 for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Password_9 () 
{
   	uint handlePasswordValue = VariableIDs.PasswordValue;
   	uint PasswordValue;
   	smRead(handlePasswordValue, PasswordValue);
	
	uint eventClearPasswordWarning = EventIDs.TR_Clear_Information_Text;
	sendEvent(eventClearPasswordWarning);
   
   	if (NumPasswordDigits < 5)
   	{
   		PasswordValue = PasswordValue * 10;
		PasswordValue = PasswordValue + 9;
		NumPasswordDigits++;	
   	}
	
	smWrite(handlePasswordValue, PasswordValue);
}

//-----ScriptName=SCRIPT_Password_0
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as $SCRIPT_Password_9$ for main script method
//---------------------------------------------------------------------------------------------
uint NumPasswordDigits = 0;

void SCRIPT_Password_0 () 
{
   	uint handlePasswordValue = VariableIDs.PasswordValue;
   	uint PasswordValue;
   	smRead(handlePasswordValue, PasswordValue);
	
	uint eventClearPasswordWarning = EventIDs.TR_Clear_Information_Text;
	sendEvent(eventClearPasswordWarning);
   
   	if (NumPasswordDigits < 5)
   	{
   		PasswordValue = PasswordValue * 10;
		PasswordValue = PasswordValue + 0;
		
		//Don't increment the number of digits if they are entering zeros.
		if (PasswordValue > 0)
		{
			NumPasswordDigits++;
		}	
   	}
	
	smWrite(handlePasswordValue, PasswordValue);
	
}

//-----ScriptName=SCRIPT_Password_1
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as $SCRIPT_Password_9$ for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Password_1 () 
{
   	uint handlePasswordValue = VariableIDs.PasswordValue;
   	uint PasswordValue;
   	smRead(handlePasswordValue, PasswordValue);
	
	uint eventClearPasswordWarning = EventIDs.TR_Clear_Information_Text;
	sendEvent(eventClearPasswordWarning);
   
   	if (NumPasswordDigits < 5)
   	{
   		PasswordValue = PasswordValue * 10;
		PasswordValue = PasswordValue + 1;
		NumPasswordDigits++;	
   	}
	
	smWrite(handlePasswordValue, PasswordValue);
}

//-----ScriptName=SCRIPT_Password_2
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as $SCRIPT_Password_9$ for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Password_2 () 
{
   	uint handlePasswordValue = VariableIDs.PasswordValue;
   	uint PasswordValue;
   	smRead(handlePasswordValue, PasswordValue);
	
	uint eventClearPasswordWarning = EventIDs.TR_Clear_Information_Text;
	sendEvent(eventClearPasswordWarning);
   
   	if (NumPasswordDigits < 5)
   	{
   		PasswordValue = PasswordValue * 10;
		PasswordValue = PasswordValue + 2;
		NumPasswordDigits++;	
   	}
	
	smWrite(handlePasswordValue, PasswordValue);
}

//-----ScriptName=SCRIPT_Password_3
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as $SCRIPT_Password_9$ for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Password_3 () 
{
   	uint handlePasswordValue = VariableIDs.PasswordValue;
   	uint PasswordValue;
   	smRead(handlePasswordValue, PasswordValue);
	
	uint eventClearPasswordWarning = EventIDs.TR_Clear_Information_Text;
	sendEvent(eventClearPasswordWarning);
   
   	if (NumPasswordDigits < 5)
   	{
   		PasswordValue = PasswordValue * 10;
		PasswordValue = PasswordValue + 3;
		NumPasswordDigits++;	
   	}
	
	smWrite(handlePasswordValue, PasswordValue);
}

//-----ScriptName=SCRIPT_Password_4
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as $SCRIPT_Password_9$ for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Password_4 () 
{
   	uint handlePasswordValue = VariableIDs.PasswordValue;
   	uint PasswordValue;
   	smRead(handlePasswordValue, PasswordValue);
	
	uint eventClearPasswordWarning = EventIDs.TR_Clear_Information_Text;
	sendEvent(eventClearPasswordWarning);
   
   	if (NumPasswordDigits < 5)
   	{
   		PasswordValue = PasswordValue * 10;
		PasswordValue = PasswordValue + 4;
		NumPasswordDigits++;	
   	}
	
	smWrite(handlePasswordValue, PasswordValue);
}

//-----ScriptName=SCRIPT_Password_5
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as $SCRIPT_Password_9$ for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Password_5 () 
{
   	uint handlePasswordValue = VariableIDs.PasswordValue;
   	uint PasswordValue;
   	smRead(handlePasswordValue, PasswordValue);
	
	uint eventClearPasswordWarning = EventIDs.TR_Clear_Information_Text;
	sendEvent(eventClearPasswordWarning);
   
   	if (NumPasswordDigits < 5)
   	{
   		PasswordValue = PasswordValue * 10;
		PasswordValue = PasswordValue + 5;
		NumPasswordDigits++;	
   	}
	
	smWrite(handlePasswordValue, PasswordValue);
}

//-----ScriptName=SCRIPT_Password_6
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as $SCRIPT_Password_9$ for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Password_6 () 
{
   	uint handlePasswordValue = VariableIDs.PasswordValue;
   	uint PasswordValue;
   	smRead(handlePasswordValue, PasswordValue);
	
	uint eventClearPasswordWarning = EventIDs.TR_Clear_Information_Text;
	sendEvent(eventClearPasswordWarning);
   
   	if (NumPasswordDigits < 5)
   	{
   		PasswordValue = PasswordValue * 10;
		PasswordValue = PasswordValue + 6;
		NumPasswordDigits++;	
   	}
	
	smWrite(handlePasswordValue, PasswordValue);
}

//-----ScriptName=SCRIPT_Password_7
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as $SCRIPT_Password_9$ for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Password_7 () 
{
   	uint handlePasswordValue = VariableIDs.PasswordValue;
   	uint PasswordValue;
   	smRead(handlePasswordValue, PasswordValue);
	
	uint eventClearPasswordWarning = EventIDs.TR_Clear_Information_Text;
	sendEvent(eventClearPasswordWarning);
   
   	if (NumPasswordDigits < 5)
   	{
   		PasswordValue = PasswordValue * 10;
		PasswordValue = PasswordValue + 7;
		NumPasswordDigits++;	
   	}
	
	smWrite(handlePasswordValue, PasswordValue);
}

//-----ScriptName=SCRIPT_Password_8
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as $SCRIPT_Password_9$ for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Password_8 () 
{
   	uint handlePasswordValue = VariableIDs.PasswordValue;
   	uint PasswordValue;
   	smRead(handlePasswordValue, PasswordValue);
	
	uint eventClearPasswordWarning = EventIDs.TR_Clear_Information_Text;
	sendEvent(eventClearPasswordWarning);
   
   	if (NumPasswordDigits < 5)
   	{
   		PasswordValue = PasswordValue * 10;
		PasswordValue = PasswordValue + 8;
		NumPasswordDigits++;	
   	}
	
	smWrite(handlePasswordValue, PasswordValue);
}

//-----ScriptName=SCRIPT_Password_CLEAR
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_Password_CLEAR for main script method
//---------------------------------------------------------------------------------------------


void SCRIPT_Password_CLEAR () 
{
   	uint handlePasswordValue = VariableIDs.PasswordValue;
	smWrite(handlePasswordValue, 0);
	NumPasswordDigits = 0;
	
	uint eventClearPasswordWarning = EventIDs.TR_Clear_Information_Text;
	sendEvent(eventClearPasswordWarning);
}

//-----ScriptName=SCRIPT_Password_BACK
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_Password_BACK for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Password_BACK () 
{
   	uint handlePasswordValue = VariableIDs.PasswordValue;
   	uint PasswordValue;
   	smRead(handlePasswordValue, PasswordValue);
	
	uint eventClearPasswordWarning = EventIDs.TR_Clear_Information_Text;
	sendEvent(eventClearPasswordWarning);
	
	PasswordValue = PasswordValue / 10;
	if (NumPasswordDigits > 0)
	{
		NumPasswordDigits--;
	}
	
	smWrite(handlePasswordValue, PasswordValue);
}

//-----ScriptName=SCRIPT_PasswordEvaluate
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_PasswordEvaluate for main script method
//---------------------------------------------------------------------------------------------

uint NumberOfUnsuccessfulTries = 0;

void SCRIPT_PasswordEvaluate () 
{
 	uint handlePasswordValue = VariableIDs.PasswordValue;
   	uint PasswordValue;
   	smRead(handlePasswordValue, PasswordValue);
	

	
	//uint handleWarningText = ScreenObjectIDs.Text_PasswordAttemptWarning;
	//string WarningText = "";
	
	//uint handleText_Informational = ScreenObjectIDs.Text_Informational;
	
	uint NumberOfPumps;
	smRead(VariableIDs.FLX_NumberOfPumps, NumberOfPumps);
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation); 

	
	/**** Evaluate the entered password *******/
	switch(PasswordValue)
	{
		case 66867:	//Go to motor controller 1 set up
					if (DisplayOrientation == 0)	//horizontal
						sendEvent(EventIDs.Event_GOTO_MotorControllerSetup_HORIZONTAL);
					else
						sendEvent(EventIDs.Event_GOTO_MotorControllerSetup_VERTICAL);
					break;
					
		case 66868:	//Go to motor controller 2 set up
					if (DisplayOrientation == 0)	//horizontal
						sendEvent(EventIDs.Event_GOTO_MotorControllerSetup2_HORIZONTAL);
					else
						sendEvent(EventIDs.Event_GOTO_MotorControllerSetup2_VERTICAL);
					break;
					
					
		case 52591:	//Go to the motor driver debugging screen.
					sendEvent(EventIDs.Event_GOTO_MotorControllerDebug);
					break;
					
		case 5804:	//Load all defaults - in case things get screwed up.
					sendEvent(EventIDs.Event_LoadAllDefaults);
					break;
		
		case 9999:	//Go to maintenance interval screen
					if (DisplayOrientation == 0)	//horizontal
						SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Menu_Layer_Default_Maintenance_Interval_View);
					else
						SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Menu_Layer_Default_VERTICAL_Maintenance_Interval_View);
					
					SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Popup_Layer_Popup_Page_CLEAR_View);
					smWrite(VariableIDs.HasSystemBeenRun, 0);	//reset the flag used to evaluate the maintenance minder
					sendEvent(EventIDs.Calculate_MiantenancePasswordEntered_Equals_TRUE);
					NumberOfUnsuccessfulTries = 0;
					break;
											
					//Go to foam calibration screen
		case 1560:	if (NumberOfPumps == 1)
					{
						if (DisplayOrientation == 0)	//horizontal
							sendEvent(EventIDs.Event_GOTO_Foam_Calibration_Minus_Single_Motor);
						else
							sendEvent(EventIDs.Event_GOTO_Foam_Calibration_Minus_Single_Motor_VERT);
					}
					else
					{
						if (DisplayOrientation == 0)	//horizontal	
							sendEvent(EventIDs.Event_GOTO_Foam_Calibration_Minus_Dual_Motor_1);
						else
							sendEvent(EventIDs.Event_GOTO_Foam_Calibration_Minus_Dual_Motor_1_VERT);	
					}
					NumberOfUnsuccessfulTries = 0;	
					break;
					
					//Go to water calibration screen 1
		case 6679:	if (DisplayOrientation == 0)
						sendEvent(EventIDs.Event_GOTO_Water_Calibration);
					else
						sendEvent(EventIDs.Event_GOTO_Water_Calibration_VERT);	
					NumberOfUnsuccessfulTries = 0;	
					break;
					
					//Go to water calibration screen 2
		case 6681:	if (DisplayOrientation == 0)
						sendEvent(EventIDs.Event_GOTO_Water_Calibration_2);
					else
						sendEvent(EventIDs.Event_GOTO_Water_Calibration_2_VERT);	
					NumberOfUnsuccessfulTries = 0;	
					break;
					
					//Go to direct water calibration screen 1
		case 9766:	if (DisplayOrientation == 0)
						sendEvent(EventIDs.Event_GOTO_Direct_Water_Calibration);
					else
						sendEvent(EventIDs.Event_GOTO_Direct_Water_Calibration_VERT);
					NumberOfUnsuccessfulTries = 0;	
					break;
					
					//Go to direct water calibration screen 2
		case 1866:	if (DisplayOrientation == 0)
						sendEvent(EventIDs.Event_GOTO_Direct_Water_Calibration_2);
					else
						sendEvent(EventIDs.Event_GOTO_Direct_Water_Calibration_2_VERT);	
					NumberOfUnsuccessfulTries = 0;	
					break;
								
					//Go to Factory menu
		case 57643:	if (DisplayOrientation == 0)
						sendEvent(EventIDs.Event_GOTO_Operation_Variables_Menu_Start);
					else
						sendEvent(EventIDs.Event_GOTO_Operation_Variables_Menu_Start_VERT);
					NumberOfUnsuccessfulTries = 0;
					smWrite(VariableIDs.SelectedOperationVariable, 0);	
					break;
					
					//Go to User Menu
		case 1849:	if (DisplayOrientation == 0)
						sendEvent(EventIDs.Event_GOTO_User_Menu);
					else
						sendEvent(EventIDs.Event_GOTO_User_Menu_VERT);	
					NumberOfUnsuccessfulTries = 0;	
					smWrite(VariableIDs.SelectedUserVariable, 0);
					break;
					
					//Go to OEM menu
		case 2314:	if (DisplayOrientation == 0)
						sendEvent(EventIDs.Event_GOTO_OEM_Menu_1);
					else
						sendEvent(EventIDs.Event_GOTO_OEM_Menu_1_VERT);
					NumberOfUnsuccessfulTries = 0;	
					smWrite(VariableIDs.SelectedOEMVariable, 0);
					break;
					
					//Go to preset configuration
		case 1023:	if (DisplayOrientation == 0)
						sendEvent(EventIDs.Event_GOTO_Preset_Configuration);
					else
						sendEvent(EventIDs.Event_GOTO_Preset_Configuration_VERT);
					NumberOfUnsuccessfulTries = 0;	
					break;
							
					//Go to datalogger
		case 3564:	if (DisplayOrientation == 0)
						sendEvent(EventIDs.Event_GOTO_Datalogger);
					else
						sendEvent(EventIDs.Event_GOTO_Datalogger_VERT);	
					NumberOfUnsuccessfulTries = 0;	
					break;
					
					//Toggle Demo Mode
		case 83749:	sendEvent(EventIDs.Calculate_DemoModeActive_Equals_TOGGLE);
					NumberOfUnsuccessfulTries = 0;	
					break;
						
		default:	sendEvent(EventIDs.eventShowInvalidPassword);
					NumberOfUnsuccessfulTries++;
					if (NumberOfUnsuccessfulTries > 5)
					{
						//sendEvent(eventPasswordAttemptWarning);
						//WarningText = ("You have attempted to gain access to protected screens " + NumberOfUnsuccessfulTries + " Times.  Please seek assistance before continuing.");
						//setTextWidgetText(handleWarningText, WarningText);
						processScreenApi(); 	
					}
					break;
	}
	
	//Clear the password before exiting.
	smWrite(handlePasswordValue, 0);
	NumPasswordDigits = 0;   
}

//-----ScriptName=SCRIPT CalculateSecondaryFoamFlowRate
/*	Calculate the foam flow rate from the raw foam flow rate received from the DC motor driver.
	This script is fired by a change in the data of FoamSecondary.RawFoamFlowRate.
*/
double Secondary_FoamFlowGPM;

void SCRIPT_CalculateSecondaryFoamFlowRate () 
{
 	uint 	FoamSecondary_RawFoamFlowRate;
	double 	TempScratchPad = 0;
	double  TempScratchPad2 = 0;
	uint8 	CurrentTankSelected;
	uint 	TotalFoamPulses;	
	uint 	Secondary_FoamCalFactor_TankA;
	uint 	Secondary_FoamCalFactor_TankB;
	uint	WaterFlowRateGPM;
	double  PulseFactorFudge;
	
	
	smRead(VariableIDs.FLX_PulseFactor, PulseFactorFudge);
	
	
	/*	Get access to the multipler value for foam A and B.  This is the value that
		the raw foam pulses from the DC motor driver will be multiplied by to
		get the flow rate of foam in GPM.
	*/ 
	smRead(VariableIDs.Secondary_FoamFlowCalculationValue_A, Secondary_FoamCalFactor_TankA); 
	smRead(VariableIDs.Secondary_FoamFlowCalculationValue_B, Secondary_FoamCalFactor_TankB); 
	
	
	/*	Get the primary foam flow rate value - this is received via a CAN message from the DC Motor driver.
		Also check to see what type of motor control system we are using - because if we are a dual system
		acting as a single then we need to use the CurrentPowerState of the primary foamlogix button.
	*/
	/*	Foam system control type can be one of three:
		0 = Single system - one pump (primary)
		1 = Dual system (as single) - two pumps controlled as a single system
		2 = Dual system (as two) - two pumps individually controlled
	*/
	uint FoamSystemControlType;
	smRead(VariableIDs.FoamSystemControlType, FoamSystemControlType);
	
	uint CurrentPowerState;
	if (FoamSystemControlType < 2)	//If this is a single pump system - or a dual pump as a single system use primary
	{
		smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
		/*	Get the current water flow rate.
		*/
		smRead(VariableIDs.Primary_WaterFlowRate_GPM, WaterFlowRateGPM);
	}
	else 	//Otherwise we are a dual pump as individual systems - use secondary.
	{	
		smRead(VariableIDs.Secondary_FoamLogix_Enabled, CurrentPowerState);
		/*	Get the current water flow rate.
		*/
		smRead(VariableIDs.Secondary_WaterFlowRate_GPM, WaterFlowRateGPM);
	}
	
	uint PrimePumpRequest;	
	if (FoamSystemControlType == 1)
		smRead(VariableIDs.PrimeFoamPump_Power, PrimePumpRequest);
	else if (FoamSystemControlType == 2)
		smRead(VariableIDs.PrimeFoamPump_Power_2, PrimePumpRequest);
		
	uint DebuggingMode;
	smRead(VariableIDs.DebuuginModeActive, DebuggingMode);
	if (CurrentPowerState != 0 || PrimePumpRequest == 1 || DebuggingMode == 1)
	{
		smRead(VariableIDs.FoamSecondary_RawFoamFlowRate, FoamSecondary_RawFoamFlowRate);
	}
	else
	{
		FoamSecondary_RawFoamFlowRate = 0;
	}
	
	smWrite(VariableIDs.RawFoamFlow_Secondary, FoamSecondary_RawFoamFlowRate);
	 
	
	/*	Because Ovel required me to use "fudge" factors in the DC Motor driver - the
		frequency we receive is not the actual frequency.  We can re-adjust this value
		back to normal by multiplying by the ratio of the real factor (18432) to
		Ovel's fudge factor (16588).  18432 / 16588 = 1.111165
		The result is the actual frequency multiplied by 10.
	*/
	TempScratchPad = FoamSecondary_RawFoamFlowRate;
	TempScratchPad = TempScratchPad * PulseFactorFudge;
	TempScratchPad = TempScratchPad / 10;
	
	/*	Put the result back into the raw foam flow rate variable
	*/
   	FoamSecondary_RawFoamFlowRate = TempScratchPad; 
	

	//smWrite(VariableIDs.Secondary_FoamFlowRateGPM, FoamSecondary_RawFoamFlowRate);
	//smWrite(VariableIDs.FoamSecondary_RawFoamFlowRate, FoamSecondary_RawFoamFlowRate);
	
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	
	/*	Calculate the GPM rate of foam from the raw value received from the DC motor driver
		making sure to use the correct calibration multiplier value.
		Put this value into the Primary_FoamFlowGPM.
		Load the total foam flow data from the DC Motor Driver CAN message
		into the appropriate system variable for total foam pulses.
	*/
	if (CurrentFoamPrimeStatus[1] != PRIME_LIMPHOME)
	{
		switch (CurrentTankSelected)
		{
			case 0:	//none (flush)
					smWrite(VariableIDs.Secondary_TotalFoamGallons_A, 0);
					smWrite(VariableIDs.Secondary_TotalFoamGallons_B, 0);
					Secondary_FoamFlowGPM = 0;
					break;
			case 1:	//tank B
					smRead(VariableIDs.FoamSecondary_TotalFoamPulses, TotalFoamPulses);	//Calculate the total foam B gallons flowed
					TempScratchPad = TotalFoamPulses;
					/*	We need to adjust the total foam flowed pulses received from the DC motor driver to account for the
						original "fudge" factor.
					*/
					TempScratchPad = TempScratchPad * 0.940767;
					TempScratchPad = TempScratchPad * PulseFactorFudge;
					TempScratchPad = TempScratchPad / Secondary_FoamCalFactor_TankB;	
					smWrite(VariableIDs.Secondary_TotalFoamGallons_B, TempScratchPad);
					
					
					TempScratchPad = FoamSecondary_RawFoamFlowRate;						//Get the raw foam flow rate in pulses per second (Hertz).
					TempScratchPad = TempScratchPad * 60;								//Mulitply by 60 to get Pulses Per Minute.
					TempScratchPad = TempScratchPad / Secondary_FoamCalFactor_TankB;	//Divide by calibation factor B to get Gallons Per Minute.
					Secondary_FoamFlowGPM = TempScratchPad;
					break;
			case 2:	//tank A
					smRead(VariableIDs.FoamSecondary_TotalFoamPulses, TotalFoamPulses);	//Calculate the total foam A gallons flowed
					TempScratchPad = TotalFoamPulses;
					/*	We need to adjust the total foam flowed pulses received from the DC motor driver to account for the
						original "fudge" factor.
					*/
					TempScratchPad = TempScratchPad * 0.940767;
					TempScratchPad = TempScratchPad * PulseFactorFudge;
					TempScratchPad = TempScratchPad / Secondary_FoamCalFactor_TankA;
					smWrite(VariableIDs.Secondary_TotalFoamGallons_A, TempScratchPad);
					
					TempScratchPad = FoamSecondary_RawFoamFlowRate;						//Get the raw foam flow rate in pulses per second (Hertz).
					TempScratchPad = TempScratchPad * 60;								//Mulitply by 60 to get Pulses Per Minute.
					TempScratchPad = TempScratchPad / Secondary_FoamCalFactor_TankA;	//Divide by calibation factor A to get Gallons Per Minute.
					Secondary_FoamFlowGPM = TempScratchPad;
					break;
			case 3:	//Motor 1 and motor 2 tank select mismatch
					smWrite(VariableIDs.Secondary_TotalFoamGallons_A, 0);
					smWrite(VariableIDs.Secondary_TotalFoamGallons_B, 0);
					Secondary_FoamFlowGPM = 0;
					break;
		}
	}
	else
	{
		switch (CurrentTankSelected)
		{
			case 0:	//flush
					Secondary_FoamFlowGPM = 0;
					break;
			case 1: //tank B
					Secondary_FoamFlowGPM = LimpHome_FoamFlowCalculation(1);
					smRead(VariableIDs.Secondary_TotalFoamGallons_B, TempScratchPad2);	//Get the current total foam gallons
					TempScratchPad = Secondary_FoamFlowGPM;								//Get the current flow rate per minute
					TempScratchPad = TempScratchPad / 600;								//divide it by 600 to convert it from minutes to 0.1 seconds
					TempScratchPad = TempScratchPad + TempScratchPad2;					//Add it to the previous total foam gallons
					smWrite(VariableIDs.Secondary_TotalFoamGallons_B, TempScratchPad);	//Put it back into the total
					break;
			case 2: //tank A
					Secondary_FoamFlowGPM = LimpHome_FoamFlowCalculation(1);
					smRead(VariableIDs.Secondary_TotalFoamGallons_A, TempScratchPad2);	//Get the current total foam gallons
					TempScratchPad = Secondary_FoamFlowGPM;								//Get the current flow rate per minute
					TempScratchPad = TempScratchPad / 600;								//divide it by 600 to convert it from minutes to 0.1 seconds
					TempScratchPad = TempScratchPad + TempScratchPad2;					//Add it to the previous total foam gallons
					smWrite(VariableIDs.Secondary_TotalFoamGallons_A, TempScratchPad);	//Put it back into the total
					break;
			case 3: //mismatch
					Secondary_FoamFlowGPM = 0;
					break;
		}	
	}
	
	//Don't show a foam flow when water flow is less than 5 GPM
	uint PrimeFoamPumpPower;
	if (FoamSystemControlType == 1)
		smRead(VariableIDs.PrimeFoamPump_Power, PrimeFoamPumpPower);
	else if (FoamSystemControlType == 2)
		smRead(VariableIDs.PrimeFoamPump_Power_2, PrimeFoamPumpPower);
	
	if (WaterFlowRateGPM < 5 && DebuggingMode == 0 && PrimeFoamPumpPower == 0)
	{
		Secondary_FoamFlowGPM = 0;
	}
	
	/*	Put the calculated foam flow rate into the system variable.
	*/
	smWrite(VariableIDs.Secondary_FoamFlowRateGPM, Secondary_FoamFlowGPM);	
}

//-----ScriptName=SCRIPT UpdateFoamLogixMainBackground
/*	This script is used to update the SmartFOAM background displayed - foam A, foam B, or FLUSH.

	This script is fired by a change in state of the received variables: FoamPrimary.FoamTankSelected
	and FoamSecondary.FoamTankSelected which are received via CAN messages from the motor driver module(s).
*/
void SCRIPT_UpdateFoamLogixMainBackground () 
{
	uint[] TankSelected(2);		//The tank selected value coming from each of the motor drivers.
	uint CurrentPowerState;
	uint CurrentTankSelected;
	uint NumberOfPumps;
	
	
	
	uint FoamSystemControlType;
	smRead(VariableIDs.FoamSystemControlType, FoamSystemControlType);
	
	/*	
		Foam system control (0=single motor, 1=dual motor(as one system), 2=dual motor (as two systems).
		
	*/
	if (FoamSystemControlType == 0 || FoamSystemControlType == 2)
	{
		NumberOfPumps = 1;
	}
	else if (FoamSystemControlType == 1)
	{
	}
	

//START - PRIMARY FOAM SYSTEM EVALUATION ********************************************************************************	
	smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
	
	/*	Evaluate the foam tank selection.  Get this value from each of the motor drivers and then evaluate
		them together.  If they do not match it means that they are not selecting the same tank and we
		need to flag the current tank selection as ERROR - tank select mismatch.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
		
		Don't even check for mismatch if there is only one pump in the system.
	*/
	smRead(VariableIDs.FoamPrimary_FoamTankSelected, TankSelected[0]);
	smRead(VariableIDs.FoamSecondary_FoamTankSelected, TankSelected[1]);
	if (FoamSystemControlType == 1)	//Dual pumps as a single system
	{	
		if (TankSelected[0] == TankSelected[1]) 
		{
			CurrentTankSelected = TankSelected[0];
		}
		else
		{
			CurrentTankSelected = TANK_MISMATCH;
		}
		//Write it out to the system variable for use in the page designer and other scripts.	
	}
	else
	{
		CurrentTankSelected = TankSelected[0];
	}
	
	smWrite(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	
	/*	The current power state should be 1 of 4 values:
		0 = OFF
		1 = ON in Foam A
		2 = ON in Foam B
		3 = ON in FLUSH
	*/
	if (CurrentPowerState != 0)	//Only update the FoamA, FoamB, Flush background if the system is ON.
	{
		switch (CurrentTankSelected)
		{
			case 0:	//FLUSH
					smWrite(VariableIDs.Primary_FoamLogix_Enabled, 3);
					break;
			case 1:	//Tank B
					smWrite(VariableIDs.Primary_FoamLogix_Enabled, 2);
					//Set the fast start variable to TRUE.  This runs the motor at 50% when the foam logix system is turned on in order to prime the foam.	
					break;
			case 2:	//Tank A
					smWrite(VariableIDs.Primary_FoamLogix_Enabled, 1);
					//Set the fast start variable to TRUE.  This runs the motor at 50% when the foam logix system is turned on in order to prime the foam.	
					break;
			case 3:	//motor 1 and motor 2 tank select mismatch
					smWrite(VariableIDs.Primary_FoamLogix_Enabled, 4);
					break;
					
		}
	}
	
//END - PRIMARY FOAM SYSTEM EVALUATION ********************************************************************************	


//START - SECONDARY FOAM SYSTEM EVALUATION ********************************************************************************	
	smRead(VariableIDs.Secondary_FoamLogix_Enabled, CurrentPowerState);
	
	/*	Evaluate the foam tank selection.  Get this value from each of the motor drivers and then evaluate
		them together.  If they do not match it means that they are not selecting the same tank and we
		need to flag the current tank selection as ERROR - tank select mismatch.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
		
		Don't even check for mismatch if there is only one pump in the system.
	*/
	smRead(VariableIDs.FoamSecondary_FoamTankSelected, TankSelected[1]);
	CurrentTankSelected = TankSelected[1];
	smWrite(VariableIDs.Secondary_FoamSystemTankSelection, CurrentTankSelected);

	
	/*	The current power state should be 1 of 4 values:
		0 = OFF
		1 = ON in Foam A
		2 = ON in Foam B
		3 = ON in FLUSH
	*/
	if (CurrentPowerState != 0)	//Only update the FoamA, FoamB, Flush background if the system is ON.
	{
		switch (CurrentTankSelected)
		{
			case 0:	//FLUSH
					smWrite(VariableIDs.Secondary_FoamLogix_Enabled, 3);
					break;
			case 1:	//Tank B
					smWrite(VariableIDs.Secondary_FoamLogix_Enabled, 2);
					//Set the fast start variable to TRUE.  This runs the motor at 50% when the foam logix system is turned on in order to prime the foam.	
					break;
			case 2:	//Tank A
					smWrite(VariableIDs.Secondary_FoamLogix_Enabled, 1);
					//Set the fast start variable to TRUE.  This runs the motor at 50% when the foam logix system is turned on in order to prime the foam.	
					break;
			case 3:	//motor 1 and motor 2 tank select mismatch
					smWrite(VariableIDs.Secondary_FoamLogix_Enabled, 4);
					break;
					
		}
	}
	
//END - SECONDARY FOAM SYSTEM EVALUATION ********************************************************************************	    
}


//-----ScriptName=SCRIPT FoamLogix_WarningHandler
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_FoamLogix_WarningHandler for main script method
//---------------------------------------------------------------------------------------------

uint[] InhibitTimer(32);
uint[] WarningInhibited(32);

uint WarningBitSelected = 0;
uint FoamLogixWarningStatus_ForTX = 0;


/**************************************************************************

How to add a new warning:
	Update the TOTAL_DEFINED_WARNINGS with the new number.
	Add the InhibitTimeLimit[x] variable and time limit.
	Add a new evaluation in the GET THE WARNING DATA section.
	Add a new case to the ShowCurrentWarning() function.  
		You will have to add a new warning page view on the Settings Layer.
		You will have to add a new event to show that view on the settings layer
			- it will be like the Event "WARNING Popup Low Foam A
**************************************************************************/
void SCRIPT_FoamLogix_WarningHandler () 
{

 	/*	FoamLogixWarningStatus has 32 bits which can be used to show warnings, etc.
	
		bit 0 = Low Foam A
		bit 1 = Low Foam B
		bit 2 = FLUSH 
		bit 3 = Foam Motor 1 increasing to regain prime
		bit 4 = Foam Motor 1 lost prime - motor shut down
		bit 5 = Foam Motor 2 increasing to regain prime
		bit 6 = Foam Motor 2 lost prime - motor shut down
		bit 7 	= Communication loss pump 1
		bit 8 	= Communication loss pump 2
		bit 9	= Tank select mismatch - pump 1 and pump 2 tank selections do not match
		bit 10	= Foam Motor 1 temperature HIGH
		bit 11	= Foam Motor 2 temperature HIGH 
		bit 12  = Foam Motor 1 shutdown due to heat
		bit 13  = Foam Motor 2 shutdown due to heat
		bit 14 	= No Foam tank A
		bit 15 	= No Foam tank B
		
	*/
	uint FoamLogixWarningStatus = 0;	//Make sure the warning variable is CLEAR.  It will be updated below in the GET THE WARNING DATA section.
	uint8 TOTAL_DEFINED_WARNINGS = 16;	//**** Make sure to UPDATE to the number of variables defined.
	uint8 LAST_DEFINED_WARNING = (TOTAL_DEFINED_WARNINGS - 1);
	uint8 LowFoamA;
	uint8 LowFoamB;
	uint8 CurrentTankSelected;
	bool NoWarningsFound = true;
	uint8 InCalibrationMode;
	bool CommPump1_Okay = false;
	bool CommPump2_Okay = false;
	uint MotorDriverTemperature_1;
	uint MotorDriverTemperature_2;
	uint NumberOfPumps;
	
	uint Motor1TemperatureWarning;
	uint Motor2TemperatureWarning;
	uint Motor1TemperatureCritical;
	uint Motor2TemperatureCritical;
	
	smRead(VariableIDs.FoamPrimary_TemperatureWarning, Motor1TemperatureWarning);
	smRead(VariableIDs.FoamPrimary_TemperatureCritical, Motor1TemperatureCritical);
	smRead(VariableIDs.FoamSecondary_TemperatureWarning, Motor2TemperatureWarning);
	smRead(VariableIDs.FoamSecondary_TemperatureCritical, Motor2TemperatureCritical);
	
	//Turn OFF heat warning if temperature is now critical
	if (Motor1TemperatureCritical == 1)
		Motor1TemperatureWarning = 0;
		
	if (Motor2TemperatureCritical == 1)
		Motor2TemperatureWarning = 0;	
	
	
	/*	Get the number of pumps this system is set for (1 or 2)
	*/
	smRead(VariableIDs.FLX_NumberOfPumps, NumberOfPumps);
	
	/*	Get the inhibit time limit from the operation value menu
	*/
	uint InhibitTimeLimit;
	smRead(VariableIDs.FLX_WarningInhibitTime, InhibitTimeLimit);

	
	/*	Increment the inhibit timers which keep a warning from being shown.
		If they meet the time limit then disable the inhibit.
	*/
	for (uint ThisWarning = 0; ThisWarning < TOTAL_DEFINED_WARNINGS; ThisWarning++)
	{
		InhibitTimer[ThisWarning]++;
		if (InhibitTimer[ThisWarning] > InhibitTimeLimit)
		{
			InhibitTimer[ThisWarning] 		= 0;
			WarningInhibited[ThisWarning] 	= 0;
		}
	}
	
	/**********************************************************************************************************************
		GET THE WARNING DATA:
		Get the warning bit data and then set the appropriate bit in the
		FoamLogixWarningStatus variable.
	*/
	smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	smRead(VariableIDs.FoamPrimary_Foam_A_Low, LowFoamA);
	if (LowFoamA != 0 && CurrentTankSelected == TANK_A)
	{ 	
		if (WarningInhibited[1] == 0)
			FoamLogixWarningStatus |= 0x00000001;
			
		FoamLogixWarningStatus_ForTX |= 0x00000001;
	}
	else
	{
		WarningInhibited[1] = 0;
		InhibitTimer[1] = 0;
		FoamLogixWarningStatus_ForTX &= ~0x00000001;
	}
	smRead(VariableIDs.FoamPrimary_Foam_B_Low, LowFoamB);
	if (LowFoamB != 0 && CurrentTankSelected == TANK_B)
	{
		if (WarningInhibited[2] == 0)
			FoamLogixWarningStatus |= 0x00000002;
			
		FoamLogixWarningStatus_ForTX |= 0x00000002;
	}
	else
	{
		WarningInhibited[2] = 0;
		InhibitTimer[2] = 0;
		FoamLogixWarningStatus_ForTX &= ~0x00000002;
	} 
	
	/* Foam Tank selected will equal 0 (FLUSH), 1 (Tank B), 2 (Tank A), 3 (tank mismatch) - so we check for 0 and if so set to ON */
	if (CurrentTankSelected == FLUSH)
	{
		if (WarningInhibited[3] == 0)
			FoamLogixWarningStatus |= 0x00000004;
			
		FoamLogixWarningStatus_ForTX |= 0x00000004;
	}
	else
	{
		WarningInhibited[3] = 0;
		InhibitTimer[3] = 0;
		FoamLogixWarningStatus_ForTX &= ~0x00000004;
	}
	//Trying to regain prime warning - motor 1
	if (CurrentFoamPrimeStatus[0] == PRIME_LOSS)
	{
		if (WarningInhibited[4] == 0) 
			FoamLogixWarningStatus |= 0x00000008;
			
		FoamLogixWarningStatus_ForTX |= 0x00000008;
	} 
	else
	{
		WarningInhibited[4] = 0;
		InhibitTimer[4] = 0;
		FoamLogixWarningStatus_ForTX &= ~0x00000008;
	}
	//Shutdown due to loss of prime warning - motor 1
	if (CurrentFoamPrimeStatus[0] == PRIME_SHUTDOWN)
	{
		smWrite(VariableIDs.FoamPump_1_LostPrime, 1);
		if (WarningInhibited[5] == 0) 
			FoamLogixWarningStatus |= 0x00000010;
			
		FoamLogixWarningStatus_ForTX |= 0x00000010;
	}
	else
	{
		smWrite(VariableIDs.FoamPump_1_LostPrime, 0);
		WarningInhibited[5] = 0;
		InhibitTimer[5] = 0;
		FoamLogixWarningStatus_ForTX &= ~0x00000010;
	}
	
	//Don't check warnings for prime loss issues for motor 2 unless the system is configured for 2 motors AND the system is not trying to
	//use a single motor because the requested flow rate is less than the minimum flow transition operating variable.
	if (NumberOfPumps == 2 && TemporarilyUseSingleMotorOnly == false)
	{
		//Trying to regain prime warning - motor 2
		if (CurrentFoamPrimeStatus[1] == PRIME_LOSS)
		{
			if (WarningInhibited[6] == 0) 
				FoamLogixWarningStatus |= 0x00000020; 
				
			FoamLogixWarningStatus_ForTX |= 0x00000020;
		}
		else
		{
			WarningInhibited[6] = 0;
			InhibitTimer[6] = 0;
			FoamLogixWarningStatus_ForTX &= ~0x00000020;
		}

		//Shutdown due to loss of prime warning - motor 2
		if (CurrentFoamPrimeStatus[1] == PRIME_SHUTDOWN)
		{
			smWrite(VariableIDs.FoamPump_2_LostPrime, 1);
			if( WarningInhibited[7] == 0) 
				FoamLogixWarningStatus |= 0x00000040;
				
			FoamLogixWarningStatus_ForTX |= 0x00000040;
		}
		else
		{
			smWrite(VariableIDs.FoamPump_2_LostPrime, 0);
			WarningInhibited[7] = 0;
			InhibitTimer[7] = 0;
			FoamLogixWarningStatus_ForTX &= ~0x00000040;
		}
	}
	
	//Communication loss with pump 1
	CommPump1_Okay = smRead(VariableIDs.FoamPrimary_DriverTemperature, MotorDriverTemperature_1); 
	if (CommPump1_Okay == false)
	{
		if( WarningInhibited[8] == 0) 
			FoamLogixWarningStatus |= 0x00000080;
			
		FoamLogixWarningStatus_ForTX |= 0x00000080;
	}
	else
	{
		WarningInhibited[8] = 0;
		InhibitTimer[8] = 0;
		FoamLogixWarningStatus_ForTX &= ~0x00000080;
	}
	
	//Communication loss with pump 2
	if (NumberOfPumps == 2)
	{
		CommPump2_Okay = smRead(VariableIDs.FoamSecondary_DriverTemperature, MotorDriverTemperature_2); 
		if (CommPump2_Okay == false)
		{
			if( WarningInhibited[9] == 0) 
				FoamLogixWarningStatus |= 0x00000100;
				
			FoamLogixWarningStatus_ForTX |= 0x00000100;
		}
		else
		{
			WarningInhibited[9] = 0;
			InhibitTimer[9] = 0;
			FoamLogixWarningStatus_ForTX &= ~0x00000100;
		}
	}
	
	//Check for tank select mismatch - Foam Tank selected will  = 0 (FLUSH), 1 (Tank B), 2 (Tank A), 3 (tank mismatch)
	if (NumberOfPumps == 2)
	{
		if (CurrentTankSelected == TANK_MISMATCH)
		{
			if (WarningInhibited[10] == 0)
				FoamLogixWarningStatus |= 0x00000200;
				
			FoamLogixWarningStatus_ForTX |= 0x00000200;
		}
		else
		{
			WarningInhibited[10] = 0;
			InhibitTimer[10] = 0;
			FoamLogixWarningStatus_ForTX &= ~0x00000200;
		}
	}
	
	
	//Motor 1 temperature warning
	if (Motor1TemperatureWarning == 1)
	{
		if (WarningInhibited[11] == 0) 
			FoamLogixWarningStatus |= 0x00000400;
			
		FoamLogixWarningStatus_ForTX |= 0x00000400;
	} 
	else
	{
		WarningInhibited[11] = 0;
		InhibitTimer[11] = 0;
		FoamLogixWarningStatus_ForTX &= ~0x00000400;
	}
	
	//Motor 2 temperature warning
	if (Motor2TemperatureWarning == 1)
	{
		if (WarningInhibited[12] == 0) 
			FoamLogixWarningStatus |= 0x00000800;
		
		FoamLogixWarningStatus_ForTX |= 0x00000800;
	} 
	else
	{
		WarningInhibited[12] = 0;
		InhibitTimer[12] = 0;
		FoamLogixWarningStatus_ForTX &= ~0x00000800;
	}
	
	
	//Motor 1 temperature warning
	if (Motor1TemperatureCritical == 1)
	{
		if (WarningInhibited[13] == 0) 
			FoamLogixWarningStatus |= 0x00001000;
			
		FoamLogixWarningStatus_ForTX |= 0x00001000;
	} 
	else
	{
		WarningInhibited[13] = 0;
		InhibitTimer[13] = 0;
		FoamLogixWarningStatus_ForTX &= ~0x00001000;
	}
	
	//Motor 2 temperature warning
	if (Motor2TemperatureCritical == 1)
	{
		if (WarningInhibited[14] == 0) 
			FoamLogixWarningStatus |= 0x00002000;
			
		FoamLogixWarningStatus_ForTX |= 0x00002000;
	} 
	else
	{
		WarningInhibited[14] = 0;
		InhibitTimer[14] = 0;
		FoamLogixWarningStatus_ForTX &= ~0x00002000;
	}
	
	//Tank A - no foam warning
	if (Warning_NoFoam_A == true)
	{
		if (WarningInhibited[15] == 0) 
			FoamLogixWarningStatus |= 0x00004000;
			
		FoamLogixWarningStatus_ForTX |= 0x00004000;
	} 
	else
	{
		WarningInhibited[15] = 0;
		InhibitTimer[15] = 0;
		FoamLogixWarningStatus_ForTX &= ~0x00004000;
	}
	
	//Tank B - no foam warning
	if (Warning_NoFoam_B == true)
	{
		if (WarningInhibited[16] == 0) 
			FoamLogixWarningStatus |= 0x00008000;
			
		FoamLogixWarningStatus_ForTX |= 0x00008000;
	} 
	else
	{
		WarningInhibited[16] = 0;
		InhibitTimer[16] = 0;
		FoamLogixWarningStatus_ForTX &= ~0x00008000;
	}
	

	//Update the warning status variable which is transmitted via CAN to 3rd party displays.
	smWrite(VariableIDs.RemoteOperation_FoamLogixWarningStatus, FoamLogixWarningStatus_ForTX);
	
	/*
		END of GET THE WARNING DATA:
	**********************************************************************************************************************/
	
	
	/*	Update selected foam tank is low bit - this is transmitted via CAN message
		18EFFFD0 for the CAFS SPS.
	*/	
	if ((CurrentTankSelected == TANK_A && LowFoamA != 0) || (CurrentTankSelected == TANK_B && LowFoamB != 0))
	{
		smWrite(VariableIDs.BIT_SelectedFoamTankIsLow, 1);
	}
	else
	{
		smWrite(VariableIDs.BIT_SelectedFoamTankIsLow, 0);
	}
	
	if (CurrentTankSelected == FLUSH)
	{
		smWrite(VariableIDs.BIT_FlushIsActive, 1);
	}
	else
	{
		smWrite(VariableIDs.BIT_FlushIsActive, 0);
	}
	
	if (NumberOfPumps == 2 && TemporarilyUseSingleMotorOnly == false)
	{
		if (CurrentFoamPrimeStatus[0] == PRIME_SHUTDOWN && CurrentFoamPrimeStatus[1] == PRIME_SHUTDOWN)
		{
			smWrite(VariableIDs.BIT_NoPrimeIsActive, 1);
		}
		else
		{
			smWrite(VariableIDs.BIT_NoPrimeIsActive, 0);
		}
	}
	else
	{
		if (CurrentFoamPrimeStatus[0] == PRIME_SHUTDOWN)
		{
			smWrite(VariableIDs.BIT_NoPrimeIsActive, 1);
		}
		else
		{
			smWrite(VariableIDs.BIT_NoPrimeIsActive, 0);
		}
	}
	
		
	smRead(VariableIDs.CalibrationMode, InCalibrationMode);
	
	/*	We don't want to popup any of these warnings if we are in calibration mode.  
	*/
	if (InCalibrationMode == 0)
	{	
		/*	We will loop through up to a maximum of the defined total number of warnings OR until
			the next warning is identified as ON.
		*/	
		for (uint BitCheck = 0; BitCheck < TOTAL_DEFINED_WARNINGS; BitCheck++)
		{
			if (((FoamLogixWarningStatus >> WarningBitSelected) & 0x00000001) != 0 )	//If this bit position is active then we flag the warning
			{
				//Update the system variable with the current warning value.  We add 1 because 0 should be NO ERRORS.
				smWrite(VariableIDs.ShowingCurrentWarning, (WarningBitSelected + 1));
				ShowCurrentWarning((WarningBitSelected + 1));
				NoWarningsFound = false;
				break;
			}
	
			FocusNextWarningBit(LAST_DEFINED_WARNING);			
		}
		
		//We need to increment the warning bit before we leave so that we are ready to test the next position.
		FocusNextWarningBit(LAST_DEFINED_WARNING);
		
		/*	If there are no warnings found then make sure the system variable ShowCurrentWarning is
			cleared so that we don't show any warning popups.
		*/
		if (NoWarningsFound == true)
		{
			smWrite(VariableIDs.ShowingCurrentWarning, 0);
			ShowCurrentWarning(0);
		}
	}
		
			 
}



/*	This function simply increments the WarningBitSelected variable
	to its next position (or allows it to wrap).
*/
void FocusNextWarningBit(uint LastDefinedWarning)
{
	if (WarningBitSelected < LastDefinedWarning)
	{
		WarningBitSelected++;	
	}
	else
	{
		WarningBitSelected = 0;	
	}
}


/*	This function calls the event to show the correct warning.
*/
void ShowCurrentWarning(uint ThisWarning)
{
	switch (ThisWarning)
	{
		case 0:	//No warnings
				sendEvent(EventIDs.WARNING_Popups_Cleared);
				break;
		case 1:	//Low A foam
				sendEvent(EventIDs.WARNING_Popup_Low_Foam_A);
				break;
		case 2:	//Low B Foam
				sendEvent(EventIDs.WARNING_Popup_Low_Foam_B);
				break;
		case 3:	//Tank is in FLUSH
				//sendEvent(EventIDs.WARNING_Popup_Tank_In_FLUSH);
				break;
		case 4:	//Motor 1 trying to regain prime
				sendEvent(EventIDs.WARNING_Popup_Motor_1_regain_prime);
				break;
		case 5:	//Motor 1 lost prime - shutdown
				sendEvent(EventIDs.WARNING_Popup_Motor_1_lost_prime);
				break;
		case 6:	//Motor 2 trying to regain prime
				sendEvent(EventIDs.WARNING_Popup_Motor_2_regain_prime);
				break;
		case 7:	//Motor 2 lost prime - shutdown
				sendEvent(EventIDs.WARNING_Popup_Motor_2_lost_prime);
				break;
		case 8:	//Motor 1 comm loss
				sendEvent(EventIDs.WARNING_Popup_Comm_Loss_Pump_1);
				break;
		case 9:	//Motor 2 commm loss
				sendEvent(EventIDs.WARNING_Popup_Comm_Loss_Pump_2);
				break;
		case 10:	//Motor 1 and motor 2 foam tank select mismatch
				sendEvent(EventIDs.WARNING_Popup_Tank_Select_Mismatch);
				break;
				
		case 11:	//Motor 1 temperature warning
				sendEvent(EventIDs.WARNING_Popup_Motor_1_getting_hot);
				break;			
		case 12:	//Motor 2 temperature warning
				sendEvent(EventIDs.WARNING_Popup_Motor_2_getting_hot);
				break;
		case 13:	//Motor 1 temperature critical - shutdown
				sendEvent(EventIDs.WARNING_Popup_Motor_1_shutdown_due_to_heat);
				break;
		case 14:	//Motor 2 temperature critical - shutdown
				sendEvent(EventIDs.WARNING_Popup_Motor_2_shutdown_due_to_heat);
				break;
		case 15:	//Foam tank A - no foam
				sendEvent(EventIDs.WARNING_Popup_No_Foam_A);
				break;
		case 16:	//Foam tank B - no foam
				sendEvent(EventIDs.WARNING_Popup_No_Foam_B);
				break;
		
		default://No warnings
				sendEvent(EventIDs.WARNING_Popups_Cleared);
				break;		
				
	}
}


/*	This function calls the event to show the correct warning.
*/
void InhibitCurrentWarning(uint ThisWarning)
{
	InhibitTimer[ThisWarning] 		= 0;
	WarningInhibited[ThisWarning] 	= 1;
	sendEvent(EventIDs.WARNING_Popups_Cleared);
}

//-----ScriptName=SCRIPT InhibitCurrentWarning
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_InhibitCurrentWarning for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_InhibitCurrentWarning () 
{
	uint ThisWarning;
	smRead(VariableIDs.ShowingCurrentWarning, ThisWarning);
	
	//This fires the function in the SCRIPT FoamLogix_WarningHandler.
 	InhibitCurrentWarning(ThisWarning);   
}

//-----ScriptName=SCRIPT OperationVariable_INCREASE
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_OperationVariable_INCREASE for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_OperationVariable_INCREASE () 
{
  	uint ThisSelectedVariable;
	smRead(VariableIDs.SelectedOperationVariable, ThisSelectedVariable);
	
	double ThisVariableValue;
	
	switch (ThisSelectedVariable)
	{
		case 0:	//Gain
				smRead(VariableIDs.tmp_FLX_Gain, ThisVariableValue);
				if (ThisVariableValue < 2.0)
				{
					ThisVariableValue = ThisVariableValue + 0.1;
					smWrite(VariableIDs.tmp_FLX_Gain, ThisVariableValue);
				}
				break;
				
		case 1:	//Sensitivity
				smRead(VariableIDs.tmp_FLX_Sensitivity, ThisVariableValue);
				if (ThisVariableValue < 10)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_Sensitivity, ThisVariableValue);
				}
				break;
				
		case 2:	//algorithm div
				smRead(VariableIDs.tmp_FLX_AlgorithmDiv, ThisVariableValue);
				if (ThisVariableValue < 10)
				{
					ThisVariableValue = ThisVariableValue + 0.1;
					smWrite(VariableIDs.tmp_FLX_AlgorithmDiv, ThisVariableValue);
				}
				break;
				
		case 3:	//Prime lost start
				smRead(VariableIDs.tmp_FLX_PrimeLostStart, ThisVariableValue);
				if (ThisVariableValue < 10)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_PrimeLostStart, ThisVariableValue);
				}
				break;	
				
		case 4:	//Prime lost ramp speed
				smRead(VariableIDs.tmp_FLX_PrimeRampSpeed, ThisVariableValue);
				if (ThisVariableValue < 100)
				{
					ThisVariableValue = ThisVariableValue + 5;
					smWrite(VariableIDs.tmp_FLX_PrimeRampSpeed, ThisVariableValue);
				}
				break;
				
		case 5:	//Prime lost fail time
				smRead(VariableIDs.tmp_FLX_PrimeLostTime, ThisVariableValue);
				if (ThisVariableValue < 60)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_PrimeLostTime, ThisVariableValue);
				}
				break;
				
		case 6:	//Foam Cal speed
				smRead(VariableIDs.tmp_FLX_FoamCalibrationSpeed, ThisVariableValue);
				if (ThisVariableValue < 100)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_FoamCalibrationSpeed, ThisVariableValue);
				}
				break;
				
		case 7:	//Fast start speed
				smRead(VariableIDs.tmp_FLX_InitialFastStartSpeed, ThisVariableValue);
				if (ThisVariableValue < 100)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_InitialFastStartSpeed, ThisVariableValue);
				}
				break;
				
		case 8:	//Fast start time
				smRead(VariableIDs.tmp_FLX_InitialFastStartTime, ThisVariableValue);
				if (ThisVariableValue < 5)
				{
					ThisVariableValue = ThisVariableValue + 0.2;
					smWrite(VariableIDs.tmp_FLX_InitialFastStartTime, ThisVariableValue);
				}
				break;
	
		case 9:	//Number of pumps in the system
				smRead(VariableIDs.tmp_FLX_NumberOfPumps, ThisVariableValue);
				if (ThisVariableValue < 2)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_NumberOfPumps, ThisVariableValue);
				}
				break;
				
		case 10:	//Low flow transition - under this value and only one pump runs
				smRead(VariableIDs.tmp_FLX_MinimumFoamFlowRate, ThisVariableValue);
				if (ThisVariableValue < 40.0)
				{
					ThisVariableValue = ThisVariableValue + 0.01;
					smWrite(VariableIDs.tmp_FLX_MinimumFoamFlowRate, ThisVariableValue);
				}
				break;
				
		case 11:	//Minimum flow rate - hysteresis.  
				smRead(VariableIDs.tmp_FLX_LowFlowTransitionHysteresis, ThisVariableValue);
				if (ThisVariableValue < 6)
				{
					ThisVariableValue = ThisVariableValue + 0.1;
					smWrite(VariableIDs.tmp_FLX_LowFlowTransitionHysteresis, ThisVariableValue);
				}
				break;
				
		case 12:	//Warning inhibit time
				smRead(VariableIDs.tmp_FLX_WarningInhibitTime, ThisVariableValue);
				if (ThisVariableValue < 300)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_WarningInhibitTime, ThisVariableValue);
				}
				break;
				
		case 13:	//Voltage Tweak
				smRead(VariableIDs.tmp_FLX_VoltageTweak, ThisVariableValue);
				if (ThisVariableValue < 1.5)
				{
					ThisVariableValue = ThisVariableValue + 0.1;
					smWrite(VariableIDs.tmp_FLX_VoltageTweak, ThisVariableValue);
				}
				break;
				
		case 14:	//Low foam time OFF - this is the OFF time when pulsing the motor for low flow algorithm
				smRead(VariableIDs.tmp_FLX_LowFoamTime_OFF, ThisVariableValue);
				if (ThisVariableValue < 40)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_LowFoamTime_OFF, ThisVariableValue);
				}
				break;
				
		case 15:	//Low foam time ON - this is the ON time when pulsing the motor for low flow algorithm
				smRead(VariableIDs.tmp_FLX_LowFoamTime_ON, ThisVariableValue);
				if (ThisVariableValue < 20)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_LowFoamTime_ON, ThisVariableValue);
				}
				break;
				
		case 16:	//Ovel's fudge factor
				smRead(VariableIDs.tmp_FLX_PulseFactor, ThisVariableValue);
				if (ThisVariableValue < 1.200)
				{
					ThisVariableValue = ThisVariableValue + 0.001;
					smWrite(VariableIDs.tmp_FLX_PulseFactor, ThisVariableValue);
				}
				break;
				
		case 17:	//Number of buffers used with the water flow calculation array (4 to 128)
				smRead(VariableIDs.tmpWaterFlow_NumberBuffers, ThisVariableValue);
				if (ThisVariableValue < 2)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmpWaterFlow_NumberBuffers, ThisVariableValue);
				}
				break;
	}  
}

//-----ScriptName=SCRIPT OperationVariable_DECREASE
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_OperationVariable_DECREASE for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_OperationVariable_DECREASE () 
{
	uint ThisSelectedVariable;
	smRead(VariableIDs.SelectedOperationVariable, ThisSelectedVariable);
	
	double ThisVariableValue;
	
	switch (ThisSelectedVariable)
	{
		case 0:	//Gain
				smRead(VariableIDs.tmp_FLX_Gain, ThisVariableValue);
				if (ThisVariableValue > 0.5)
				{
					ThisVariableValue = ThisVariableValue - 0.1;
					smWrite(VariableIDs.tmp_FLX_Gain, ThisVariableValue);
				}
				break;
				
		case 1:	//Sensitivity
				smRead(VariableIDs.tmp_FLX_Sensitivity, ThisVariableValue);
				if (ThisVariableValue > 1)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_Sensitivity, ThisVariableValue);
				}
				break;
				
		case 2:	//algorithm div
				smRead(VariableIDs.tmp_FLX_AlgorithmDiv, ThisVariableValue);
				if (ThisVariableValue > 0.1)
				{
					ThisVariableValue = ThisVariableValue - 0.1;
					smWrite(VariableIDs.tmp_FLX_AlgorithmDiv, ThisVariableValue);
				}
				break;
				
		case 3:	//Prime lost start
				smRead(VariableIDs.tmp_FLX_PrimeLostStart, ThisVariableValue);
				if (ThisVariableValue > 1)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_PrimeLostStart, ThisVariableValue);
				}
				break;	
				
		case 4:	//Prime lost ramp speed
				smRead(VariableIDs.tmp_FLX_PrimeRampSpeed, ThisVariableValue);
				if (ThisVariableValue > 5)
				{
					ThisVariableValue = ThisVariableValue - 5;
					smWrite(VariableIDs.tmp_FLX_PrimeRampSpeed, ThisVariableValue);
				}
				break;
				
		case 5:	//Prime lost fail time
				smRead(VariableIDs.tmp_FLX_PrimeLostTime, ThisVariableValue);
				if (ThisVariableValue > 5)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_PrimeLostTime, ThisVariableValue);
				}
				break;
				
		case 6:	//Foam Cal speed
				smRead(VariableIDs.tmp_FLX_FoamCalibrationSpeed, ThisVariableValue);
				if (ThisVariableValue > 25)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_FoamCalibrationSpeed, ThisVariableValue);
				}
				break;
				
		case 7:	//Fast start speed
				smRead(VariableIDs.tmp_FLX_InitialFastStartSpeed, ThisVariableValue);
				if (ThisVariableValue > 10)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_InitialFastStartSpeed, ThisVariableValue);
				}
				break;
				
		case 8:	//Fast start time
				smRead(VariableIDs.tmp_FLX_InitialFastStartTime, ThisVariableValue);
				if (ThisVariableValue > 0.2)
				{
					ThisVariableValue = ThisVariableValue - 0.2;
					smWrite(VariableIDs.tmp_FLX_InitialFastStartTime, ThisVariableValue);
				}
				break;
		
		case 9:	//Number of pumps in the system
				smRead(VariableIDs.tmp_FLX_NumberOfPumps, ThisVariableValue);
				if (ThisVariableValue > 1)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_NumberOfPumps, ThisVariableValue);
				}
				break;
				
		case 10:	//Minimum flow rate - under this value and only one pump runs
				smRead(VariableIDs.tmp_FLX_MinimumFoamFlowRate, ThisVariableValue);
				if (ThisVariableValue > 0.01)
				{
					ThisVariableValue = ThisVariableValue - 0.01;
					smWrite(VariableIDs.tmp_FLX_MinimumFoamFlowRate, ThisVariableValue);
				}
				break;
				
		case 11:	//Minimum flow rate - hysteresis.  
				smRead(VariableIDs.tmp_FLX_LowFlowTransitionHysteresis, ThisVariableValue);
				if (ThisVariableValue > 0.1)
				{
					ThisVariableValue = ThisVariableValue - 0.1;
					smWrite(VariableIDs.tmp_FLX_LowFlowTransitionHysteresis, ThisVariableValue);
				}
				break;
							
		case 12:	//Warning inhibit time
				smRead(VariableIDs.tmp_FLX_WarningInhibitTime, ThisVariableValue);
				if (ThisVariableValue > 30)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_WarningInhibitTime, ThisVariableValue);
				}
				break;
				
		case 13:	//Voltage Tweak
				smRead(VariableIDs.tmp_FLX_VoltageTweak, ThisVariableValue);
				if (ThisVariableValue > -1.5)
				{
					ThisVariableValue = ThisVariableValue - 0.1;
					smWrite(VariableIDs.tmp_FLX_VoltageTweak, ThisVariableValue);
				}
				break;
	
		case 14:	//Low foam time OFF - this is the OFF time when pulsing the motor for low flow algorithm (0 = low flow OFF)
				smRead(VariableIDs.tmp_FLX_LowFoamTime_OFF, ThisVariableValue);
				if (ThisVariableValue >= 1)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_LowFoamTime_OFF, ThisVariableValue);
				}
				break;
				
		case 15:	//Low foam time ON - this is the ON time when pulsing the motor for low flow algorithm (0 = low flow off)
				smRead(VariableIDs.tmp_FLX_LowFoamTime_ON, ThisVariableValue);
				if (ThisVariableValue >= 1)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_LowFoamTime_ON, ThisVariableValue);
				}
				break;
				
		case 16:	//Ovel's fudge factor
				smRead(VariableIDs.tmp_FLX_PulseFactor, ThisVariableValue);
				if (ThisVariableValue > 0.800)
				{
					ThisVariableValue = ThisVariableValue - 0.001;
					smWrite(VariableIDs.tmp_FLX_PulseFactor, ThisVariableValue);
				}
				break;
				
		case 17:	//Number of buffers used with the water flow calculation array (4 to 128)
				smRead(VariableIDs.tmpWaterFlow_NumberBuffers, ThisVariableValue);
				if (ThisVariableValue > 0)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmpWaterFlow_NumberBuffers, ThisVariableValue);
				}
	}  
}

//-----ScriptName=SCRIPT Saving Widget
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_Saving_Widget for main script method
//---------------------------------------------------------------------------------------------
float SavingDegrees = 0;
void SCRIPT_Saving_Widget () 
{
 	uint SaveEnabled;
	smRead(VariableIDs.SavingStep, SaveEnabled);
	
	if (SaveEnabled != 0)
	{
		screenApiRotate(ScreenObjectIDs.Image_Saving_disk, SavingDegrees, 240, 136);
		SavingDegrees = SavingDegrees - 22.5;
		
		if (abs(SavingDegrees) >= 360)
			SavingDegrees = 0;
			
		processScreenApi();
			
	}   
}

//-----ScriptName=SCRIPT SecondaryCalibrateFoamFlow_PowerButton
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_SecondaryCalibrateFoamFlow_PowerButton for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_SecondaryCalibrateFoamFlow_PowerButton () 
{
 	uint CurrentTankSelected;
	uint MotorRequestON;
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Secondary_FoamSystemTankSelection, CurrentTankSelected);
	
	smRead(VariableIDs.Calibration_SecondaryMotorEnable, MotorRequestON); 

	if (MotorRequestON == 0 && /*CurrentTankSelected != FLUSH &&*/ CurrentTankSelected != TANK_MISMATCH)	//we are going from OFF to ON
	{
		/*	Get the desired calibration motor speed from the factory menu item.
			Then write it to the variable packed into the CAN transmit message to the DC Motor driver.
		*/
		uint MotorSpeed;
		smRead(VariableIDs.FLX_FoamCalibrationSpeed, MotorSpeed);
		MotorSpeed = MotorSpeed * 10;
		
		CalibrationPWM[1] = MotorSpeed;
		//smWrite(VariableIDs.Primary_MotorPWM_Request, MotorSpeed);
		
		/*	Set the clear total foam command - which is also transmitted in the same message
			as the motor speed.
		*/
		smWrite(VariableIDs.FoamSecondary_Command, 3);	//clear total foam flow value.
		
		/*	Start the event which starts the 250ms timer - when that timer times out it sets the 
			command value back to 0.  This is important because the DC motor driver won't run
			if we are constantly sending it the clear command.
		*/
		sendEvent(EventIDs.Event_StartSecondaryFoamClearCommandTimer);
		
		/*	Update the variable used to determine if this script should turn OFF/On the motor request.
			It is basically a toggle.  It is used in the CalculateMotorPWM script.
		*/
		smWrite(VariableIDs.Calibration_SecondaryMotorEnable, 1);
		

	}
	else
	{
		/*	Turn OFF the motor request.
		*/
		smWrite(VariableIDs.Calibration_SecondaryMotorEnable, 0);
		/*	Update the variable used to determine if this script should turn OFF/On the motor request.
			It is basically a toggle.
		*/
		smWrite(VariableIDs.Secondary_MotorPWM_Request, 0);
	}   
}



//-----ScriptName=SCRIPT OperationVariable PREVIOUS
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_OperationVariable_PREVIOUS for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_OperationVariable_PREVIOUS () 
{
 	uint ThisVariable;
	sendEvent(EventIDs.Calculate_SelectedOperationVariable_Equals_PREVIOUS);
	
	smRead(VariableIDs.SelectedOperationVariable, ThisVariable); 
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);

	//If we have gone from 9 to 8 then we need to go back to page 1	
	if (ThisVariable == 9)
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_Operation_Variables_Menu_1);
		else
			sendEvent(EventIDs.Event_GOTO_Operation_Variables_Menu_1_VERT);
	}  
}

//-----ScriptName=SCRIPT OperationVariable NEXT
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_OperationVariable_NEXT for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_OperationVariable_NEXT () 
{
 	uint ThisVariable;
	sendEvent(EventIDs.Calculate_SelectedOperationVariable_Equals_NEXT);
	
	smRead(VariableIDs.SelectedOperationVariable, ThisVariable); 
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation); 

	//If we have gone from 8 to 9 then we need to go back to page 2	
	if (ThisVariable == 8)
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_Operation_Variables_Menu_2); 
		else
		  	sendEvent(EventIDs.Event_GOTO_Operation_Variables_Menu_2_VERT);
	}  
}

//-----ScriptName=SCRIPT UpdateRemoteActivationLED
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_UpdateRemoteActivationLED for main script method
//---------------------------------------------------------------------------------------------
bool RemoteLED_ON = false;
void SCRIPT_UpdateRemoteActivationLED () 
{
	uint CurrentPowerState;
	uint WaterFlowRateGPM;
	
  	/*	Get the current state of the FoamLogix system.
	*/
	smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState); 
	
	/*	Get the current water flow rate.
	*/
	smRead(VariableIDs.Primary_WaterFlowRate_GPM, WaterFlowRateGPM);
	
	if (CurrentPowerState != SYSTEM_OFF && CurrentPowerState != SYSTEM_TANK_MISMATCH)
	{
		if (WaterFlowRateGPM > 0)
		{
			if (RemoteLED_ON == false)
			{
				sendEvent(EventIDs.Event_Remote_LED_OFF);
				RemoteLED_ON = true;
			}
			else
			{
				sendEvent(EventIDs.Event_Remote_LED_ON);
				RemoteLED_ON = false;
			}
		}
		else
		{
			sendEvent(EventIDs.Event_Remote_LED_ON);
			RemoteLED_ON = false;
		}
		
	}
	else
	{
		sendEvent(EventIDs.Event_Remote_LED_OFF);
		RemoteLED_ON = false;
	}	 
}

//-----ScriptName=SCRIPT RemoteActivationSwitch
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_RemoteActivationSwitch for main script method
//---------------------------------------------------------------------------------------------
bool RemoteSwitchState = true;
void SCRIPT_RemoteActivationSwitch () 
{
  	
	uint RemoteSwitchAnalogValue = 0;
	uint RemoteActivationPreset;
	smRead(VariableIDs.RemoteActivationPreset, RemoteActivationPreset); 
	smRead(VariableIDs.IO_RemoteActivationRequest, RemoteSwitchAnalogValue);
	
	/*	Get the current state of the FoamLogix system.
	*/
	uint CurrentPowerState;
	smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
	
	//If the remote switch went from off to on (this is a momentary switch)
	if (RemoteSwitchAnalogValue == 0 && RemoteSwitchState == false)	
	{
		RemoteSwitchState = true;
		/*	When there were no presets this function fired the power button.
			sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Primary);
		*/
		if (CurrentPowerState == SYSTEM_OFF)
		{ 	
			/*	Now we check which preset has been set as the remote activation preset 
				and start operation using that rate.  Otherwise we use whatever the current
				rate is.
			*/
			if (RemoteActivationPreset >= 1 && RemoteActivationPreset <= 6)
				EnterOperationScreen(RemoteActivationPreset-1);	//Realign 1-6 into 0-5
			else
				TurnFoamSystem_ON(0);
		}
		else
		{
			TurnFoamSystem_OFF(0);
		}
	}
	
	if (RemoteSwitchAnalogValue == 1)
	{
		RemoteSwitchState = false;
	}  
}

//-----ScriptName=SCRIPT CheckSmartCAFS_SPS_RemoteOn
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_CheckSmartCAFS_SPS_RemoteOn for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_CheckSmartCAFS_SPS_RemoteOn () 
{
	uint RemoteActivationFromSPS;
	uint CurrentPowerState;
	
	 /*	The current power state should be 1 of 4 values:
		0 = OFF
		1 = ON in Foam A
		2 = ON in Foam B
		3 = ON in FLUSH
	*/
	smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
	smRead(VariableIDs.RemoteActivationFromSPS, RemoteActivationFromSPS);
	
	if (RemoteActivationFromSPS == 1 && CurrentPowerState == 0)
	{
		sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Primary);
	}
	

 	/*	Clear the RemoteActivationFromSPS variable.  This variable is used to
		fire this script on change.  The SPS sends a 1 in byte 0 of the received
		CAN message in order to activate the foam logix system and the message
		is only sent once when a preset button is pressed.
	*/	
	smWrite(VariableIDs.RemoteActivationFromSPS, 0);   
}

//-----ScriptName=SCRIPT SmoothBoreNozzleCalculation GPM
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_SmoothBoreNozzleCalculation_GPM for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_SmoothBoreNozzleCalculation_GPM () 
{
 	double NozzleSize;
	double NozzlePressure;
	double NozzleGPM;
	
	smRead(VariableIDs.LookupChart_NozzleSize_INCHES, NozzleSize);  
	smRead(VariableIDs.LookupChart_NozzlePressure_PSI, NozzlePressure); 
	
	NozzleGPM = (29.71 * (NozzleSize * NozzleSize)) * sqrt(NozzlePressure);
	
	smWrite(VariableIDs.LookupChart_GPM, NozzleGPM);
}

//-----ScriptName=SCRIPT SmoothBoreNozzleCalculation LPM
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_SmoothBoreNozzleCalculation_LPM for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_SmoothBoreNozzleCalculation_LPM () 
{
  	double NozzleSize;
	double NozzlePressure;
	double NozzleLPM;
	
	smRead(VariableIDs.LookupChart_NozzleSize_MILLIMETERS, NozzleSize);  
	smRead(VariableIDs.LookupChart_NozzlePressure_BAR, NozzlePressure); 
	
	NozzleLPM = (0.664026 * (NozzleSize * NozzleSize)) * sqrt(NozzlePressure);
	
	smWrite(VariableIDs.LookupChart_LPM, NozzleLPM);  
}

//-----ScriptName=SCRIPT VoltageEvaluation
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_VoltageEvaluation for main script method
//---------------------------------------------------------------------------------------------
uint LowVoltageDetectedTimer = 0;
uint LowVoltageFlasher = 0;
void SCRIPT_VoltageEvaluation () 
{
	/*	Evaluate the battery voltage for a low condition
		
	*/ 
	
	double BatteryVoltage;	//From on board voltage detection
	double SystemVoltage;	//Onboard voltage modified with voltage tweak
	double VoltageTweak;
	double LowVoltage_12V;
	double LowVoltage_24V;
	
	smRead(VariableIDs.IO_Battery_Voltage, BatteryVoltage);
	smRead(VariableIDs.FLX_VoltageTweak, VoltageTweak);
	smRead(VariableIDs.FLX_LowVoltage_12V, LowVoltage_12V);
	smRead(VariableIDs.FLX_LowVoltage_24V, LowVoltage_24V);
	
	SystemVoltage = BatteryVoltage + VoltageTweak;
	
	smWrite(VariableIDs.SystemVoltage, SystemVoltage); 
	
	//Check for 12V system low voltage
	if (SystemVoltage < LowVoltage_12V)
	{
		LowVoltageDetectedTimer++;
	} 
	//Check for 24V system low voltage
	else if (SystemVoltage > 16.0 && SystemVoltage < LowVoltage_24V)
	{
		LowVoltageDetectedTimer++;
	} 
	//Not low in 12V or 24V so clear the timer.
	else
	{
		LowVoltageDetectedTimer = 0;
	}
	
	/*	This script uses its own 500ms timer - se we check for low voltage
		having been active for 2 seconds before we flag it.
	*/
	if (LowVoltageDetectedTimer >= 4)
	{
		LowVoltageDetectedTimer = 4;	//maintain the time
		if (LowVoltageFlasher == 0)
		{
			LowVoltageFlasher = 1;
		}
		else
		{
			LowVoltageFlasher = 0;
		}	
	}
	else
	{
		LowVoltageFlasher = 0;
	}
	
	smWrite(VariableIDs.VoltageIsLow, LowVoltageFlasher);
	
	 
}

//-----ScriptName=SCRIPT DemoModeOnly
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_DemoModeOnly for main script method
//---------------------------------------------------------------------------------------------
double Primary_TotalFoamPulses = 0;
uint Primary_RawFoamFlowRate = 0;
double Secondary_TotalFoamPulses = 0;
uint Secondary_RawFoamFlowRate = 0;
uint demo_FoamFlowingTimer = 0;
void SCRIPT_DemoModeOnly () 
{

	uint DemoModeActive;
	smRead(VariableIDs.DemoModeActive, DemoModeActive);
	
	uint DemoTankSelected;
	smRead(VariableIDs.Demo_TankSelect, DemoTankSelected);
	
	if (DemoModeActive == 1)
	{		 
		double tmp_foamPulses;
	 	smWrite(VariableIDs.Primary_RawWaterFlowPulses, 75);
		smWrite(VariableIDs.Secondary_RawWaterFlowPulses, 75); 
		smWrite(VariableIDs.Primary_RawWaterFlowPulses_NewMotorDriver, 75);
		smWrite(VariableIDs.Secondary_RawWaterFlowPulses_NewMotorDriver, 75);
		
		smWrite(VariableIDs.FoamPrimary_Foam_A_Low, 0);
		
		uint Primary_RequestedMotorSpeed;
		smRead(VariableIDs.Primary_MotorPWM_Request, Primary_RequestedMotorSpeed); 
		
		if (Primary_RequestedMotorSpeed > 0)
		{
			smWrite(VariableIDs.FoamPrimary_MotorDutyCycle, Primary_RequestedMotorSpeed);
			
			tmp_foamPulses = Primary_RequestedMotorSpeed;
			//tmp_foamPulses = tmp_foamPulses * 0.25;
			Primary_RawFoamFlowRate = tmp_foamPulses;
			
			tmp_foamPulses = Primary_RawFoamFlowRate;
			tmp_foamPulses = (tmp_foamPulses / 100);	//convert pulses per second to pulses per .1 second
			Primary_TotalFoamPulses = Primary_TotalFoamPulses + tmp_foamPulses;
			
			demo_FoamFlowingTimer++;
			if (demo_FoamFlowingTimer > 30)
			{
				FoamFlowingForAtLeastThreeSeconds = true;
				demo_FoamFlowingTimer = 30;
			}
			
		} 
		else
		{
			Primary_RawFoamFlowRate = 0;
			demo_FoamFlowingTimer = 0;
		}
		
		uint Primary_PowerState;
		smRead(VariableIDs.Calibration_PrimaryMotorEnable, Primary_PowerState);
		if (Primary_PowerState == 0)
		{
			smWrite(VariableIDs.FoamPrimary_MotorDutyCycle, 0);
		}
		
		uint FoamSelectorType;
		smRead(VariableIDs.FLX_FoamTankSelectorType, FoamSelectorType); 
	
		if (FoamSelectorType < 2)
		{
			switch (DemoTankSelected)
			{
				case 0:	//tank A	
						smWrite(VariableIDs.FoamPrimary_FoamTankSelected, 2);
						break;
				case 1:	//tank B	
						smWrite(VariableIDs.FoamPrimary_FoamTankSelected, 1);
						break;
				case 2:	//flush	
						smWrite(VariableIDs.FoamPrimary_FoamTankSelected, 0);
						break;
			}
		}
		else	//ADT/EDT
		{		
			uint MotorDriverOutput_TankA;
			uint MotorDriverOutput_TankB;
			smRead(VariableIDs.FreeFormCAN_TankA_Select, MotorDriverOutput_TankA);
			smRead(VariableIDs.FreeFormCAN_TankB_Select, MotorDriverOutput_TankB);
			
			if (MotorDriverOutput_TankA == 0 && MotorDriverOutput_TankB == 0)
			{
				smWrite(VariableIDs.FoamPrimary_FoamTankSelected, 0);	//FLUSH	
				smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
				smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
			}
			if (MotorDriverOutput_TankA == 1 && MotorDriverOutput_TankB == 0)
			{
				smWrite(VariableIDs.FoamPrimary_FoamTankSelected, 2);	//Tank A
				smWrite(VariableIDs.FreeFormCAN_TankA_Select, 1);
				smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);	
			}
			if (MotorDriverOutput_TankA == 0 && MotorDriverOutput_TankB == 1)
			{
				smWrite(VariableIDs.FoamPrimary_FoamTankSelected, 1);	//Tank B
				smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
				smWrite(VariableIDs.FreeFormCAN_TankB_Select, 1);	
			}
		}
		
			
		smWrite(VariableIDs.FoamPrimary_RawFoamFlowRate, Primary_RawFoamFlowRate);
		smWrite(VariableIDs.FoamPrimary_TotalFoamPulses, Primary_TotalFoamPulses); 	
		
		//for comm
		smWrite(VariableIDs.FoamPrimary_DriverTemperature, 10); 
		
		
		
		uint Secondary_RequestedMotorSpeed;
		smRead(VariableIDs.Secondary_MotorPWM_Request, Secondary_RequestedMotorSpeed); 
		
		if (Secondary_RequestedMotorSpeed > 0)
		{
			smWrite(VariableIDs.FoamSecondary_MotorDutyCycle, Secondary_RequestedMotorSpeed);
			
			tmp_foamPulses = Secondary_RequestedMotorSpeed;
			//tmp_foamPulses = tmp_foamPulses * 0.25;
			Secondary_RawFoamFlowRate = tmp_foamPulses;
			
			tmp_foamPulses = Secondary_RawFoamFlowRate;
			tmp_foamPulses = (tmp_foamPulses / 100);	//convert pulses per second to pulses per .1 second
			Secondary_TotalFoamPulses = Secondary_TotalFoamPulses + tmp_foamPulses;
			
		} 
		else
		{
			Secondary_RawFoamFlowRate = 0;
		}
		
		
		uint Secondary_PowerState;
		smRead(VariableIDs.Calibration_SecondaryMotorEnable, Secondary_PowerState);
		if (Secondary_PowerState == 0)
		{
			smWrite(VariableIDs.FoamSecondary_MotorDutyCycle, 0);
		}
	
		//Foamtank
		switch (DemoTankSelected)
		{
			case 0:	//tank A	
					smWrite(VariableIDs.FoamSecondary_FoamTankSelected, 2);
					break;
			case 1:	//tank B	
					smWrite(VariableIDs.FoamSecondary_FoamTankSelected, 1);
					break;
			case 2:	//flush	
					smWrite(VariableIDs.FoamSecondary_FoamTankSelected, 0);
					break;
		}

		smWrite(VariableIDs.FoamSecondary_RawFoamFlowRate, Secondary_RawFoamFlowRate);
		smWrite(VariableIDs.FoamSecondary_TotalFoamPulses, Secondary_TotalFoamPulses); 	
		
		//for comm
		smWrite(VariableIDs.FoamSecondary_DriverTemperature, 10);
	}
}

//-----ScriptName=SCRIPT PID_Calculation
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_PID_Calculation for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_PID_Calculation () 
{
    
}


const double Epsilon = 0.02;
const double Delta_T = 0.02;
const uint Maximum = 1000;
const uint Minimum = 0;
double Kp = 1; //0.1;
double Kd = 1 ; //0.01;
double Ki = 0.5;

double PreviousError = 0;
double Integral = 0;

uint PID_LoopCalculation(double DesiredFlowRate, double ActualFlowRate)
{
	double thisError = 0;
	double thisDerivative = 0;
	double thisOutput = 0;
	uint returnOutput = 0;
	
	//Calculate P,I,D
	thisError = DesiredFlowRate - ActualFlowRate;
	
	if ( abs(thisError) > Epsilon)
	{
		Integral = Integral + (thisError * Delta_T);	
	}
	
	thisDerivative = (thisError - PreviousError) / Delta_T;
	thisOutput = Kp * thisError + Ki * Integral + Kd * thisDerivative;
	
	//Saturation filter
	if (thisOutput > Maximum)
	{
		thisOutput = Maximum;	
	}
	else if (thisOutput < Minimum)
	{
		thisOutput = Minimum;
	}
	
	PreviousError = thisError;
	
	returnOutput = thisOutput;
	
	return returnOutput;
}

//-----ScriptName=SCRIPT OEMVariable_NEXT
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_OEMVariable_NEXT for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_OEMVariable_NEXT () 
{
  	uint ThisVariable;
	sendEvent(EventIDs.Calculate_SelectedOEMVariableEquals_NEXT);
	
	//If we have gone from 8 to 9 then we need to go back to page 2	
	smRead(VariableIDs.SelectedOEMVariable, ThisVariable);
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);
	
	if (ThisVariable == 8)
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_OEM_Menu_2);  
		else
			sendEvent(EventIDs.Event_GOTO_OEM_Menu_2_VERT);
		 
	}
}

//-----ScriptName=SCRIPT OEMVariable_PREVIOUS
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_OEMVariable_PREVIOUS for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_OEMVariable_PREVIOUS () 
{
  	 uint ThisVariable;
	sendEvent(EventIDs.Calculate_SelectedOEMVariableEquals_PREVIOUS);
	
	//If we have gone from 9 to 8 then we need to go back to page 1
	smRead(VariableIDs.SelectedOperationVariable, ThisVariable);
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);
		
	if (ThisVariable == 9)
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_OEM_Menu_1);  
		else
		 	sendEvent(EventIDs.Event_GOTO_OEM_Menu_1_VERT);  
	} 
}

//-----ScriptName=SCRIPT OEMVariable_INCREASE
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_OEMVariable_INCREASE for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_OEMVariable_INCREASE () 
{
 	uint ThisSelectedVariable;
	smRead(VariableIDs.SelectedOEMVariable, ThisSelectedVariable);
	
	double ThisVariableValue;
	
	switch (ThisSelectedVariable)
	{
		case 0:	//Foam system control (0=single motor, 1=dual motor(as one system), 2=dual motor (as two systems).
				smRead(VariableIDs.tmp_FoamSystemControlType, ThisVariableValue);
				if (ThisVariableValue < 2)
				{
					ThisVariableValue = ThisVariableValue + 1;	
				}
				else
				{
					ThisVariableValue = 0;
				}
				smWrite(VariableIDs.tmp_FoamSystemControlType, ThisVariableValue);
				break;
				
		case 1:	//Pump model
				//0 = 1.7,  1 = 2.1, 2 = 3.3, 3 = 5.0, 4 = 6.5, 5 = 13        (12V)
				//6 = 1.7,  7 = 2.1, 8 = 3.3, 9 = 5.0, 10 = 6.5, 11 = 13      (24V)
				smRead(VariableIDs.tmp_FLX_PumpModel, ThisVariableValue);
				if (ThisVariableValue < 11)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_PumpModel, ThisVariableValue);
				}
				else
				{
					ThisVariableValue = 0;
					smWrite(VariableIDs.tmp_FLX_PumpModel, ThisVariableValue);
				}
				break;
				
		case 2:	//Pump model
				//0 = 1.7,  1 = 2.1, 2 = 3.3, 3 = 5.0, 4 = 6.5, 5 = 13        (12V)
				//6 = 1.7,  7 = 2.1, 8 = 3.3, 9 = 5.0, 10 = 6.5, 11 = 13      (24V)
				smRead(VariableIDs.tmp_FLX_PumpModel_2, ThisVariableValue);
				if (ThisVariableValue < 11)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_PumpModel_2, ThisVariableValue);
				}
				else
				{
					ThisVariableValue = 0;
					smWrite(VariableIDs.tmp_FLX_PumpModel_2, ThisVariableValue);
				}
				break;
				
		case 3:	//Voltage Tweak
				smRead(VariableIDs.tmp_FLX_VoltageTweak, ThisVariableValue);
				if (ThisVariableValue< 1.5)
				{
					ThisVariableValue = ThisVariableValue + 0.1;
					smWrite(VariableIDs.tmp_FLX_VoltageTweak, ThisVariableValue);
				}
				break;
	
		case 4:	//12V system low voltage point
				smRead(VariableIDs.tmp_FLX_LowVoltage_12V, ThisVariableValue);
				if (ThisVariableValue < 12.0)
				{
					ThisVariableValue = ThisVariableValue + 0.1;
					smWrite(VariableIDs.tmp_FLX_LowVoltage_12V, ThisVariableValue);
				}
				break;
				
		case 5:	//24V system low voltage point
				smRead(VariableIDs.tmp_FLX_LowVoltage_24V, ThisVariableValue);
				if (ThisVariableValue < 24.0)
				{
					ThisVariableValue = ThisVariableValue + 0.1;
					smWrite(VariableIDs.tmp_FLX_LowVoltage_24V, ThisVariableValue);
				}
				break;
				
		case 6:	//Warning inhibit time
				smRead(VariableIDs.tmp_FLX_WarningInhibitTime, ThisVariableValue);
				if (ThisVariableValue < 300)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_WarningInhibitTime, ThisVariableValue);
				}
				break;
				
		case 7:	//Foam Calibration speed
				smRead(VariableIDs.tmp_FLX_FoamCalibrationSpeed, ThisVariableValue);
				if (ThisVariableValue < 100)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_FoamCalibrationSpeed, ThisVariableValue);
				}
				break;
				
		case 8:	//Allow home button (FoamLogix display)
				smRead(VariableIDs.tmp_FoamLogix_AllowHomeButton, ThisVariableValue);
				if (ThisVariableValue == 0)
					ThisVariableValue = 1;
				else
					ThisVariableValue = 0;
				smWrite(VariableIDs.tmp_FoamLogix_AllowHomeButton, ThisVariableValue);
				break;
				
		case 9:	//Foam selector type (0 = none, 1 = mst, 2 = mdtII, 3 = adt
				smRead(VariableIDs.tmp_FLX_FoamTankSelectorType, ThisVariableValue);
				if (ThisVariableValue < 3)
				{
					ThisVariableValue = ThisVariableValue + 1;	
				}
				else
				{
					ThisVariableValue = 0;
				}
				smWrite(VariableIDs.tmp_FLX_FoamTankSelectorType, ThisVariableValue);
				break;
				
		case 10:	//tank A maximum %
				smRead(VariableIDs.tmp_FLX_MaxFoamPercent_A, ThisVariableValue);
				if (ThisVariableValue < 10.0)
				{
					ThisVariableValue = ThisVariableValue + 0.1;
					smWrite(VariableIDs.tmp_FLX_MaxFoamPercent_A, ThisVariableValue);
				}
				break;
		case 11://tank B maximum %
				smRead(VariableIDs.tmp_FLX_MaxFoamPercent_B, ThisVariableValue);
				if (ThisVariableValue < 10.0)
				{
					ThisVariableValue = ThisVariableValue + 0.1;
					smWrite(VariableIDs.tmp_FLX_MaxFoamPercent_B, ThisVariableValue);
				}
				break;
				
		case 12:	//Water flow pulses source - 0=new motor driver, 1 = A2C module (old motor driver)
				smRead(VariableIDs.tmp_FLX_WaterFlowSource, ThisVariableValue);
				if (ThisVariableValue == 0)
					ThisVariableValue = 1;
				else
					ThisVariableValue = 0;
				smWrite(VariableIDs.tmp_FLX_WaterFlowSource, ThisVariableValue);
				break;
				
		case 13://Flush time
				smRead(VariableIDs.tmp_FLX_FlushTime, ThisVariableValue);
				if (ThisVariableValue < 60)
				{
					ThisVariableValue = ThisVariableValue + 1;
					smWrite(VariableIDs.tmp_FLX_FlushTime, ThisVariableValue);
				}
				break;
				
		case 14:	//Auto enable active/inactive
				smRead(VariableIDs.tmp_FLX_AutoEnable, ThisVariableValue);
				if (ThisVariableValue == 0)
					ThisVariableValue = 1;
				else
					ThisVariableValue = 0;
				smWrite(VariableIDs.tmp_FLX_AutoEnable, ThisVariableValue);
				break;
	}   
}

//-----ScriptName=SCRIPT OEMVariable_DECREASE
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_OEMVariable_DECREASE for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_OEMVariable_DECREASE () 
{
	uint ThisSelectedVariable;
	smRead(VariableIDs.SelectedOEMVariable, ThisSelectedVariable);
	
	double ThisVariableValue;
	
	switch (ThisSelectedVariable)
	{
		case 0:	//Foam system control (0=single motor, 1=dual motor(as one system), 2=dual motor (as two systems).
				smRead(VariableIDs.tmp_FoamSystemControlType, ThisVariableValue);
				if (ThisVariableValue > 0)
				{
					ThisVariableValue = ThisVariableValue - 1;	
				}
				else
				{
					ThisVariableValue = 2;
				}
				smWrite(VariableIDs.tmp_FoamSystemControlType, ThisVariableValue);
				break;
		
		case 1:	//Pump model
				//0 = 1.7,  1 = 2.1, 2 = 3.3, 3 = 5.0, 4 = 6.5, 5 = 13        (12V)
				//6 = 1.7,  7 = 2.1, 8 = 3.3, 9 = 5.0, 10 = 6.5, 11 = 13      (24V)
				smRead(VariableIDs.tmp_FLX_PumpModel, ThisVariableValue);
				if (ThisVariableValue > 0)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_PumpModel, ThisVariableValue);
				}
				else
				{
					ThisVariableValue = 11;
					smWrite(VariableIDs.tmp_FLX_PumpModel, ThisVariableValue);
				}
				break;
				
		case 2:	//Pump model
				//0 = 1.7,  1 = 2.1, 2 = 3.3, 3 = 5.0, 4 = 6.5, 5 = 13        (12V)
				//6 = 1.7,  7 = 2.1, 8 = 3.3, 9 = 5.0, 10 = 6.5, 11 = 13      (24V)
				smRead(VariableIDs.tmp_FLX_PumpModel_2, ThisVariableValue);
				if (ThisVariableValue > 0)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_PumpModel_2, ThisVariableValue);
				}
				else
				{
					ThisVariableValue = 11;
					smWrite(VariableIDs.tmp_FLX_PumpModel_2, ThisVariableValue);
				}
				break;
						
		case 3:	//Voltage Tweak
				smRead(VariableIDs.tmp_FLX_VoltageTweak, ThisVariableValue);
				if (ThisVariableValue > -1.5)
				{
					ThisVariableValue = ThisVariableValue - 0.1;
					smWrite(VariableIDs.tmp_FLX_VoltageTweak, ThisVariableValue);
				}
				break;
	
		case 4:	//12V system low voltage point
				smRead(VariableIDs.tmp_FLX_LowVoltage_12V, ThisVariableValue);
				if (ThisVariableValue > 9.0)
				{
					ThisVariableValue = ThisVariableValue - 0.1;
					smWrite(VariableIDs.tmp_FLX_LowVoltage_12V, ThisVariableValue);
				}
				break;
				
		case 5:	//24V system low voltage point
				smRead(VariableIDs.tmp_FLX_LowVoltage_24V, ThisVariableValue);
				if (ThisVariableValue > 18.0)
				{
					ThisVariableValue = ThisVariableValue - 0.1;
					smWrite(VariableIDs.tmp_FLX_LowVoltage_24V, ThisVariableValue);
				}
				break;
				
		case 6:	//Warning inhibit time
				smRead(VariableIDs.tmp_FLX_WarningInhibitTime, ThisVariableValue);
				if (ThisVariableValue > 30)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_WarningInhibitTime, ThisVariableValue);
				}
				break;
				
		case 7:	//Foam Calibration speed
				smRead(VariableIDs.tmp_FLX_FoamCalibrationSpeed, ThisVariableValue);
				if (ThisVariableValue > 5)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_FoamCalibrationSpeed, ThisVariableValue);
				}
				break;
		
		case 8:	//Allow home button (FoamLogix display)
				smRead(VariableIDs.tmp_FoamLogix_AllowHomeButton, ThisVariableValue);
				if (ThisVariableValue == 0)
					ThisVariableValue = 1;
				else
					ThisVariableValue = 0;
				smWrite(VariableIDs.tmp_FoamLogix_AllowHomeButton, ThisVariableValue);
				break;
				
		case 9:	//Foam selector type (0 = none, 1 = mst, 2 = mdtII, 3 = adt
				smRead(VariableIDs.tmp_FLX_FoamTankSelectorType, ThisVariableValue);
				if (ThisVariableValue > 0)
				{
					ThisVariableValue = ThisVariableValue - 1;	
				}
				else
				{
					ThisVariableValue = 3;
				}
				smWrite(VariableIDs.tmp_FLX_FoamTankSelectorType, ThisVariableValue);
				break;
				
		case 10:	//tank A maximum %
				smRead(VariableIDs.tmp_FLX_MaxFoamPercent_A, ThisVariableValue);
				if (ThisVariableValue > 0.1)
				{
					ThisVariableValue = ThisVariableValue - 0.1;
					smWrite(VariableIDs.tmp_FLX_MaxFoamPercent_A, ThisVariableValue);
				}
				break;
		case 11://tank B maximum %
				smRead(VariableIDs.tmp_FLX_MaxFoamPercent_B, ThisVariableValue);
				if (ThisVariableValue > 0.1)
				{
					ThisVariableValue = ThisVariableValue - 0.1;
					smWrite(VariableIDs.tmp_FLX_MaxFoamPercent_B, ThisVariableValue);
				}
				break;
				
		case 12:	//Water flow pulses source - 0=new motor driver, 1 = A2C module (old motor driver)
				smRead(VariableIDs.tmp_FLX_WaterFlowSource, ThisVariableValue);
				if (ThisVariableValue == 0)
					ThisVariableValue = 1;
				else
					ThisVariableValue = 0;
				smWrite(VariableIDs.tmp_FLX_WaterFlowSource, ThisVariableValue);
				break;
				
		case 13://Flush time
				smRead(VariableIDs.tmp_FLX_FlushTime, ThisVariableValue);
				if (ThisVariableValue > 4)
				{
					ThisVariableValue = ThisVariableValue - 1;
					smWrite(VariableIDs.tmp_FLX_FlushTime, ThisVariableValue);
				}
				break;
				
		case 14:	//Auto enable active/inactive
				smRead(VariableIDs.tmp_FLX_AutoEnable, ThisVariableValue);
				if (ThisVariableValue == 0)
					ThisVariableValue = 1;
				else
					ThisVariableValue = 0;
				smWrite(VariableIDs.tmp_FLX_AutoEnable, ThisVariableValue);
				break;
	}    
}

//-----ScriptName=SCRIPT_GOTO_OperatingScreen
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_GOTO_OperatingScreen for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_GOTO_OperatingScreen () 
{
	const uint HORIZONTAL = 0;
	
 	uint FoamSystemControlType;
	smRead(VariableIDs.FoamSystemControlType, FoamSystemControlType);
	
	uint UnitOfMeasure;
	smRead(VariableIDs.FLX_UnitOfMeasure, UnitOfMeasure);
	
	uint FoamSystemDisplayBackground;
	smRead(VariableIDs.FLX_Display_Type, FoamSystemDisplayBackground);
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation); 
	
	SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Popup_Layer_Popup_Page_CLEAR_View);
	
	if (FoamSystemDisplayBackground == 0)	//SmartFOAM type
	{
		switch(FoamSystemControlType)
		{
			case 0:		if (DisplayOrientation == HORIZONTAL)
							SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Main_Layer_Default_SmartFOAM_Minus_Single_View);
						else
							SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Main_Layer_Default_VERTICAL_SmartFOAM_Minus_Single_View);	
						break;
						
			case 1:		if (DisplayOrientation == HORIZONTAL)
							SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Main_Layer_Default_SmartFOAM_Minus_Dual_as_Single_View);
						else
							SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Main_Layer_Default_VERTICAL_SmartFOAM_Minus_Dual_as_Single_View);
						break;
						
			case 2:		if (DisplayOrientation == HORIZONTAL)
							SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Main_Layer_Default_SmartFOAM_Minus_Dual_Seperate_View);
						else
							SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Main_Layer_Default_SmartFOAM_Minus_Dual_Seperate_View);	//TODO make vertical dual	
						break;			
		} 
	}
	else	//FoamLogix, analog, or Chase Thailand operating screen.
	{
		if (FoamSystemDisplayBackground == 1)
		{
			if (DisplayOrientation == HORIZONTAL)
				SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Main_Layer_Default_FoamLogix_View);
			else
				SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Main_Layer_Default_VERTICAL_FoamLogix_View);
				
		}
		if (FoamSystemDisplayBackground == 2)
		{
			if (DisplayOrientation == HORIZONTAL)
				SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Main_Layer_Default_AnalogFoam_View);
			else
				SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Main_Layer_Default_VERTICAL_AnalogFoam_View);
				
		}
		if (FoamSystemDisplayBackground == 3)
		{
			if (UnitOfMeasure == 0)
			{
				SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Main_Layer_Default_Chase_Thailand_Screen_Minus_ENGLISH_View);
			}
			else
			{
				SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Main_Layer_Default_Chase_Thailand_Screen_Minus_METRIC_View);
			}
				
		}
	} 
	
	smWrite(VariableIDs.OnPresetStartPage, 0);	//Indicate that we are not on the preset page. 
}

//-----ScriptName=SCRIPT FoamLogixPowerButton_Secondary
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_FoamLogixPowerButton_Secondary for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_FoamLogixPowerButton_Secondary () 
{
 	uint CurrentPowerState;
	uint CurrentTankSelected;
	
	/*	Foam system control type can be one of three:
		Single system - one pump (primary)
		Dual system (as single) - two pumps controlled as a single system
		Dual system (as two) - two pumps individually controlled
	*/
	uint FoamSystemControlType;
	smRead(VariableIDs.FoamSystemControlType, FoamSystemControlType);
	
	smRead(VariableIDs.Secondary_FoamLogix_Enabled, CurrentPowerState);
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.FoamSecondary_FoamTankSelected, CurrentTankSelected);
	
	/*	The current power state should be 1 of 4 values:
		0 = OFF
		1 = ON in Foam A
		2 = ON in Foam B
		3 = ON in FLUSH
	*/
	if (CurrentPowerState == 0)	//Currently OFF - we are requesting it to be ON
	{
		switch (CurrentTankSelected)
		{
			case 0:	//FLUSH
					smWrite(VariableIDs.Secondary_FoamLogix_Enabled, 3);
					break;
			case 1:	//Tank B
					smWrite(VariableIDs.Secondary_FoamLogix_Enabled, 2);
					//Set the fast start variable to TRUE.  This runs the motor at 50% when the foam logix system is turned on in order to prime the foam.	
					b_FastStart[1] = true;
					break;
			case 2:	//Tank A
					smWrite(VariableIDs.Secondary_FoamLogix_Enabled, 1);
					//Set the fast start variable to TRUE.  This runs the motor at 50% when the foam logix system is turned on in order to prime the foam.	
					b_FastStart[1] = true;
					break;
			case 3:	//motor 1 and motor 2 tank select mismatch
					smWrite(VariableIDs.Secondary_FoamLogix_Enabled, 4);
					break;
		}
	}
	else //Currently ON - we are requesting it to turn OFF
	{
		//Set the FoamLogix enable system variable to OFF (0)
		smWrite(VariableIDs.Secondary_FoamLogix_Enabled, 0);
		b_FastStart[1] = false;
		
		/*	Clear the foam prime status (0=good, 1=loss of foam detected, 2=shutdown due to loss of foam, 3 = limp home mode) so its ready and fresh.
			But not if we already determined that we were in limp home mode.  The only thing that will turn OFF limp home is if we start detecting pulses
			again.
		*/
		if (CurrentFoamPrimeStatus[1] != 3)
		{
			CurrentFoamPrimeStatus[1] = 0;
		}
		//Make sure the foam pumps are turned OFF when the power button is off.
		smWrite(VariableIDs.Secondary_MotorPWM_Request, 0);
		smWrite(VariableIDs.Secondary_FoamFlowRateGPM_Requested, 0);
		
		//Reset the low foam run timer - if it had timed out before then set it for half of 60 seconds.
		if (LowFoamTimer[1] >= SIXTY_SECONDS)
		{
			LowFoamTimer[1] = SIXTY_SECONDS / 2;
		}
		else
		{
			LowFoamTimer[1] = 0;
		}		
	} 	    
}

//-----ScriptName=SCRIPT CalculateWaterFlowRate_Secondary
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_CalculateWaterFlowRate_Secondary for main script method
//---------------------------------------------------------------------------------------------
uint TotalWaterFlowTimeBase_Secondary = 0;
uint[] Secondary_waterFlowRateBuffer(64);
uint[] Secondary_waterFlowRateRaw(64);
uint Secondary_BufferedRawWaterFlow;
void SCRIPT_CalculateWaterFlowRate_Secondary () 
{
	uint SimulatedWaterFlowEnabled;
	uint SimulatedWaterFlowValue;
	uint RawWaterFlow_Pulses;
	bool RawPulsesReceived;
	uint CalculatedWaterFlowRate;
	uint AveragedWaterFlowRate;
	
	int s16_TempGPMRate;
	double f_2PointSlope;
	double f_2PointOffset;
	
	smRead(VariableIDs.Secondary_WaterCal_2PointOffset, f_2PointOffset);
	smRead(VariableIDs.Secondary_WaterCal_2PointSlope, f_2PointSlope);
	
	
	smRead(VariableIDs.Secondary_SimulatedWaterFlow_Enabled, SimulatedWaterFlowEnabled);
	
	
	uint RawWaterFlowSource;
	smRead(VariableIDs.FLX_WaterFlowSource, RawWaterFlowSource);
	if (RawWaterFlowSource == 1)	//A2C module with old motor driver
		RawPulsesReceived = smRead(VariableIDs.Secondary_RawWaterFlowPulses, RawWaterFlow_Pulses);
	else							//New motor driver
		RawPulsesReceived = smRead(VariableIDs.Secondary_RawWaterFlowPulses_NewMotorDriver, RawWaterFlow_Pulses);
	
	
	if (SimulatedWaterFlowEnabled == 1)
	{
		//If we are in simulated water flow - set the water flow rate to the simulated value.
		smRead(VariableIDs.Primary_SimulatedFlowGPM, SimulatedWaterFlowValue);
		smWrite(VariableIDs.Secondary_WaterFlowRate_GPM, SimulatedWaterFlowValue);
		CalculatedWaterFlowRate = SimulatedWaterFlowValue;
	}
	else
	{
		if (RawPulsesReceived == false)
		{
			CalculatedWaterFlowRate = 0;
		}
		else
		{
			if (RawWaterFlow_Pulses < 2)
			{
				CalculatedWaterFlowRate	= 0;	// if rawflowrate is less than 10 display 0
			}
			else
			{

				//Two point calculation - using line intercept y = (slope x pps) + offset
				s16_TempGPMRate = (f_2PointSlope * Secondary_BufferedRawWaterFlow) +  f_2PointOffset;
				if (s16_TempGPMRate < 0)
				{
					CalculatedWaterFlowRate = 0;
				}
				else
				{
					CalculatedWaterFlowRate = s16_TempGPMRate;
				}

			}	
		}
		
		AveragedWaterFlowRate = Secondary_RotateWaterFlowBuffer(CalculatedWaterFlowRate, RawWaterFlow_Pulses);
		smWrite(VariableIDs.Secondary_WaterFlowRate_GPM, AveragedWaterFlowRate);
	}
	
	
	/*	Set the water flow active bit - this is sent out in the 18EFFFD0 message.
	*/
	if (CalculatedWaterFlowRate != 0)
	{
		//smWrite(VariableIDs.BIT_WaterFlowActive, 1);
	}
	else
	{
		//smWrite(VariableIDs.BIT_WaterFlowActive, 0);
	}
	
	Secondary_CalculateTotalWaterFlowed(CalculatedWaterFlowRate);  	//Total water flow used the actual flow rate and not the averaged flow rate
}



/*	Rotate the water flow buffers - the new data will be put into the last slot.
	The buffers will then be added and averaged.
*/
uint Secondary_RotateWaterFlowBuffer(uint NewWaterFlowRate, uint RawWaterFlow_Pulses)
{
	uint accumulator = 0;
	uint accumulatorRaw = 0;
	
	/*	9/14/2018
		Adding configurable number of water flow buffers like we did for ATP system.
		We used to only use 24 buffers - but now we can use 24, 42, or 64
	*/
	uint NUM_BUFFERS = 42;	//default value (MED)
	uint BufferConfig = 0;
	smRead(VariableIDs.WaterFlow_NumberBuffers, BufferConfig);
	switch (BufferConfig)
	{
		case 0:	//LOW buffering
				NUM_BUFFERS = 24;	break;
				
		case 1:	//MED buffering
				NUM_BUFFERS = 42;	break;
				
		case 2:	//HIGH buffering
				NUM_BUFFERS = 64;	break;
		
		default:	//MED buffering
				NUM_BUFFERS = 42;	break;	
	}
	
	for (uint i = 0; i < (NUM_BUFFERS-1); i++)
	{
		Secondary_waterFlowRateBuffer[i] = Secondary_waterFlowRateBuffer[i+1];
		Secondary_waterFlowRateRaw[i] = Secondary_waterFlowRateRaw[i+1];
	}
	
	Secondary_waterFlowRateBuffer[(NUM_BUFFERS-1)] = NewWaterFlowRate;
	Secondary_waterFlowRateRaw[NUM_BUFFERS-1] = RawWaterFlow_Pulses;
	
	for (uint i = 0; i < NUM_BUFFERS; i++)
	{
		accumulator += Secondary_waterFlowRateBuffer[i];
		accumulatorRaw += Secondary_waterFlowRateRaw[i];
	}
	
	accumulator = accumulator / NUM_BUFFERS;
	accumulator = RoundToFive(accumulator);
	
	accumulatorRaw = accumulatorRaw / NUM_BUFFERS;		//Accumulated raw water flow 
	Secondary_BufferedRawWaterFlow = accumulatorRaw;

	
	
	/*	Since this averaged accumulator method is basically 2.5 seconds behind the
		actual water flow changes - there is the possibility that the water flow has been
		shut down, but the average will still contain a value for 2.5 seconds.  This can
		cause foam to flow for up to 2.5 seconds after water flow has stopped.  To keep this
		from happening we will check the 3 newest buffers (23, 22, and 21) and if they are
		0 then we will force the whole buffer to 0 in order to shutdown foam flow.
	*/
	if (Secondary_waterFlowRateBuffer[(NUM_BUFFERS-1)] == 0 && Secondary_waterFlowRateBuffer[(NUM_BUFFERS-2)] == 0 && Secondary_waterFlowRateBuffer[(NUM_BUFFERS-3)] == 0)
	{
		accumulator = 0;
		
		for (uint i = 0; i < NUM_BUFFERS; i++)
		{
			Secondary_waterFlowRateBuffer[i] = 0;
		}	
	} 
	
	smWrite(VariableIDs.Secondary_RawWaterFlowRate_Buffered, accumulatorRaw);
	
	return  accumulator;
}





/*************************************************************************
	Calculate the total water flow.  This function is run every loop cycle
	above - which is every 100ms.  This waits every 10 calls to perform the 
	function in order to get flow totals every second.
**************************************************************************/
void Secondary_CalculateTotalWaterFlowed(uint CurrentFlowRate)
{
	double ThisNumberOfGallons;
	double TotalGallonsFlowed;
	
	smRead(VariableIDs.Secondary_TotalWaterGallons, TotalGallonsFlowed);
	
	TotalWaterFlowTimeBase_Secondary++;
	if (TotalWaterFlowTimeBase_Secondary >= 10)	//one second at a .100 second script rate
	{
		TotalWaterFlowTimeBase_Secondary = 0;
		ThisNumberOfGallons = CurrentFlowRate;
		ThisNumberOfGallons = ThisNumberOfGallons / 60;	//remove the per minute
		TotalGallonsFlowed = TotalGallonsFlowed + ThisNumberOfGallons;
		smWrite(VariableIDs.Secondary_TotalWaterGallons, TotalGallonsFlowed);
	}		 
}	    


//-----ScriptName=SCRIPT Secondary_FoamPercentage_Decrease
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_Secondary_FoamPercentage_Decrease for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Secondary_FoamPercentage_Decrease () 
{
	uint CurrentTankSelected;
	
 	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Secondary_FoamSystemTankSelection, CurrentTankSelected);
	
	switch (CurrentTankSelected)
	{
		case 1:	//Tank B foam percentage
				sendEvent(EventIDs.Calculate_Secondary_FoamPercentage_TankB_Equals_DECR);
				break;
				
		case 2:	//Tank A foam percentage
				sendEvent(EventIDs.Calculate_Secondary_FoamPercentage_TankA_Equals_DECR);
				break;
	}    
}

//-----ScriptName=SCRIPT Secondary_FoamPercentage_Increase
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_Secondary_FoamPercentage_Increase for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Secondary_FoamPercentage_Increase () 
{
	uint CurrentTankSelected;
	
 	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Secondary_FoamSystemTankSelection, CurrentTankSelected);
	
	switch (CurrentTankSelected)
	{
		case 1:	//Tank B foam percentage
				sendEvent(EventIDs.Calculate_Secondary_FoamPercentage_TankB_Equals_INCR);
				break;
				
		case 2:	//Tank A foam percentage
				sendEvent(EventIDs.Calculate_Secondary_FoamPercentage_TankA_Equals_INCR);
				break;
	}   
}

//-----ScriptName=SCRIPT_ShowUSBSaving
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_ShowUSBSaving for main script method
//---------------------------------------------------------------------------------------------
float USBSavingDegrees = 0;
void SCRIPT_ShowUSBSaving () 
{
 	uint SaveEnabled;
	smRead(VariableIDs.ShowUSBSaving, SaveEnabled);
	
	int Screen_X_Position = 180;	//X position where the widget is placed on the screen (pixels)
	int Screen_Y_Position = 134;	//Y position where the widget is placed on the screen (pixels)
	int ImageSize_Height = 134;		//Height of image widget (pixels)
	int ImageSize_Length = 134;		//Length of image widget (pixels)
	
	int RotationCenter_X;
	int RotationCenter_Y;
	
	RotationCenter_X = Screen_X_Position + (ImageSize_Length / 2);
	RotationCenter_Y = Screen_Y_Position + (ImageSize_Height / 2);
	
	/*	
	*/
	if (SaveEnabled == 1)
	{
		screenApiRotate(ScreenObjectIDs.Image_USB_Saving_disk, USBSavingDegrees, RotationCenter_X, RotationCenter_Y);
		USBSavingDegrees = USBSavingDegrees - 45;
		
		if (abs(USBSavingDegrees) >= 360)
			USBSavingDegrees = 0;
			
		processScreenApi();		
	}   
}

//-----ScriptName=SCRIPT Check Preset Button Presses
/*---------------------------------------------------------------------------------------------
	Script rate 100 ms.
---------------------------------------------------------------------------------------------*/
const uint NUM_BUTTONS = 6;
const uint ENTRY_TIME = 1;
uint[] PresetButton_Timer(NUM_BUTTONS);
uint[] PresetButton_State(NUM_BUTTONS);
uint[] Preset_ButtonDefined(6);
uint[] ForcePresetAfterTankChange(6);

bool InitializeManualMode = true;
uint InitializationTimer = 0;
uint ActivePresetFlasher = 0;

void SCRIPT_Check_Preset_Button_Presses () 
{
	/*	Get the value of the bar color.  We only care that the value is
		non-zero which means that the bar color has been defined.  A zero
		indicates that the user has configured the preset as DISABLED.
		Below in the for loop we evaluate the button press and if the
		bar color is non-zero then the preset has been defined.	
	*/
	smRead(VariableIDs.BarColor_Preset_1, Preset_ButtonDefined[0]);
	smRead(VariableIDs.BarColor_Preset_2, Preset_ButtonDefined[1]);
	smRead(VariableIDs.BarColor_Preset_3, Preset_ButtonDefined[2]);
	smRead(VariableIDs.BarColor_Preset_4, Preset_ButtonDefined[3]);
	smRead(VariableIDs.BarColor_Preset_5, Preset_ButtonDefined[4]);
	smRead(VariableIDs.BarColor_Preset_6, Preset_ButtonDefined[5]);
	
	
 	/*	Get the current state of the preset buttons and put it into our script array.
		A '1' indicates the button is currently pressed, and a '0' indicates that the
		button is released.
		
		When the button is pressed we will increment its button timer.  When it meets
		the time limit we will set the operating state based on its configured values.
		When the button is released we clear its timer.
		**This script runs every 100 ms.
	*/
	smRead(VariableIDs.ButtonPreset_1, PresetButton_State[0]);
	smRead(VariableIDs.ButtonPreset_2, PresetButton_State[1]);
	smRead(VariableIDs.ButtonPreset_3, PresetButton_State[2]);
	smRead(VariableIDs.ButtonPreset_4, PresetButton_State[3]);
	smRead(VariableIDs.ButtonPreset_5, PresetButton_State[4]);
	smRead(VariableIDs.ButtonPreset_6, PresetButton_State[5]);

	
	for (uint thisButton = 0; thisButton < NUM_BUTTONS; thisButton++)
	{
		if (PresetButton_State[thisButton] == 1 && Preset_ButtonDefined[thisButton] != 0)
		{
			PresetButton_Timer[thisButton]++;
			
			/*	Show the popup warning which shows how much time the user must continuye holding the button to enter the preset mode.
			*/
			smWrite(VariableIDs.PresetButtonHoldTimeRemaining, ((ENTRY_TIME - PresetButton_Timer[thisButton]) + 1));	//For displaying the time remaining to hold the preset button.
			//removed for SmartFOAM - sendEvent(EventIDs.SHOW_Popup_Continue_Holding_Preset_Button);
			//Set bit in variable so Warning Handler knows this is active.
			//removed for SmartFOAM - ShowButtonPressWarning = ShowButtonPressWarning | 0x04;
			
			/*	Once the preset button has been held long enough we enter the operation screen.
			*/
			if (PresetButton_Timer[thisButton] > ENTRY_TIME)
			{
				sendEvent(EventIDs.WARNING_Popups_Cleared);
				//Clear bit in variable so Warning Handler knows this is active.
				//removed for SmartFOAM - ShowButtonPressWarning = ShowButtonPressWarning & 0xFFFB;
				EnterOperationScreen(thisButton);
				/*	Set the system variable "OnPresetStartPage" to 0 to indicate that we are not on the preset page anymore
				*/
				smWrite(VariableIDs.OnPresetStartPage, 0);
			}	
		}
		else
		{	
			/*	If the user has just released the button then remove the popup warning screen showing the amount of time reminaing.
				We can do this by evaluating how much time is in the timer - anything other than 0 indicates that the button was
				held before coming here.
			*/
			if (PresetButton_Timer[thisButton] != 0)
			{
				sendEvent(EventIDs.WARNING_Popups_Cleared);
				//Clear bit in variable so Warning Handler knows this is active.
				//removed for SmartFOAM - ShowButtonPressWarning = ShowButtonPressWarning & 0xFFFB;
				PresetButton_Timer[thisButton] = 0;
			}
		}
	}

	//CheckForRunningTankChange(); - check SmartCAFS to see if we need to use this
	
	
	/*	As a method to get attention for the currently selected preset we will flash the
		ActivePreset highlight bar.
	*/
	if (ActivePresetFlasher++ >= 3)
	{
		sendEvent(EventIDs.Calculate_FlashActivePresetBar_Equals_TOGGLE);
		ActivePresetFlasher = 0;
	}	  
}



/*
	This function sets the operating items to the configured values for the preset and the
	starts the operation screen.
*/
void EnterOperationScreen(uint presetButton)
{
	const uint TANKA 		= 0;
	const uint TANKB 		= 1;
	const uint POWER_OFF 	= 0;
	const uint POWER_ON 	= 1;
	
	uint[] Preset_FoamSystem(6);
	uint[] Preset_FoamTank(6);
	uint[] Preset_Foam_PowerState(6);
	double[] Preset_FoamA_Percent(6);
	double[] Preset_FoamB_Percent(6);
	uint FoamCurrentPowerState;
	
	int thisStringID;
	string thisPresetString;
	uint thisPresetIcon;
	
	
	/*	Set the active preset area of the operation page with the text of this preset.
	*/
//	switch (presetButton)
//	{
//		case 0:	thisStringID = StringIDs.PresetButtonText_1;	smRead(VariableIDs.BarIcon_Preset_1, thisPresetIcon);  	break;
//		case 1:	thisStringID = StringIDs.PresetButtonText_2;	smRead(VariableIDs.BarIcon_Preset_2, thisPresetIcon);	break;
//		case 2:	thisStringID = StringIDs.PresetButtonText_3;	smRead(VariableIDs.BarIcon_Preset_3, thisPresetIcon);	break;
//		case 3:	thisStringID = StringIDs.PresetButtonText_4;	smRead(VariableIDs.BarIcon_Preset_4, thisPresetIcon);	break;
//		case 4:	thisStringID = StringIDs.PresetButtonText_5;	smRead(VariableIDs.BarIcon_Preset_5, thisPresetIcon);	break;
//		case 5:	thisStringID = StringIDs.PresetButtonText_6;	smRead(VariableIDs.BarIcon_Preset_6, thisPresetIcon);	break;
//		default: thisPresetString = "Whoops"; thisPresetIcon = 3; break;
//	}
//	if (presetButton < 6)
//	{
//		cultureGetString(thisStringID, thisPresetString, 0);
//	}
//	
//	cultureSetString(StringIDs.ActivePreset_Text, thisPresetString, true);
//	smWrite(VariableIDs.ActivePreset_Icon, thisPresetIcon);
	
	/*	Set ActivePreset variable - 0 = none, 1-6 = preset.  Add one to the
		PresetButton variable that brought us here.
		ActivePreset is set to 0 when the foam system is turned OFF.
	*/
	smWrite(VariableIDs.ActivePreset, (presetButton+1));
	
	
	//Foam is always on for the presets - but not for manual (manual is selected by the user)
	Preset_Foam_PowerState[0] = POWER_ON;
	Preset_Foam_PowerState[1] = POWER_ON;
	Preset_Foam_PowerState[2] = POWER_ON;
	Preset_Foam_PowerState[3] = POWER_ON;
	Preset_Foam_PowerState[4] = POWER_ON;
	Preset_Foam_PowerState[5] = POWER_ON;

		
	
	//Get all of the PRESET 1 states.
	smRead(VariableIDs.FoamTank_Preset_1, Preset_FoamTank[0]);
	if (Preset_FoamTank[0] == TANKA)
		smRead(VariableIDs.FoamPercentage_Preset_1, Preset_FoamA_Percent[0]);
	else 
		smRead(VariableIDs.FoamPercentage_Preset_1, Preset_FoamB_Percent[0]);
	smRead(VariableIDs.FoamSystem_Preset_1, Preset_FoamSystem[0]);

	//Get all of the PRESET 2 states.
	smRead(VariableIDs.FoamTank_Preset_2, Preset_FoamTank[1]);
	if (Preset_FoamTank[1] == TANKA)
		smRead(VariableIDs.FoamPercentage_Preset_2, Preset_FoamA_Percent[1]);
	else 
		smRead(VariableIDs.FoamPercentage_Preset_2, Preset_FoamB_Percent[1]);
	smRead(VariableIDs.FoamSystem_Preset_2, Preset_FoamSystem[1]);

	//Get all of the PRESET 3 states.
	smRead(VariableIDs.FoamTank_Preset_3, Preset_FoamTank[2]);
	if (Preset_FoamTank[2] == TANKA)
		smRead(VariableIDs.FoamPercentage_Preset_3, Preset_FoamA_Percent[2]);
	else 
		smRead(VariableIDs.FoamPercentage_Preset_3, Preset_FoamB_Percent[2]);
	smRead(VariableIDs.FoamSystem_Preset_3, Preset_FoamSystem[2]);
	
	//Get all of the PRESET 4 states.
	smRead(VariableIDs.FoamTank_Preset_4, Preset_FoamTank[3]);
	if (Preset_FoamTank[3] == TANKA)
		smRead(VariableIDs.FoamPercentage_Preset_4, Preset_FoamA_Percent[3]); 
	else
		smRead(VariableIDs.FoamPercentage_Preset_4, Preset_FoamB_Percent[3]);
	smRead(VariableIDs.FoamSystem_Preset_4, Preset_FoamSystem[3]);
	
	//Get all of the PRESET 5 states.
	smRead(VariableIDs.FoamTank_Preset_5, Preset_FoamTank[4]);
	if (Preset_FoamTank[4] == TANKA)
		smRead(VariableIDs.FoamPercentage_Preset_5, Preset_FoamA_Percent[4]); 
	else
		smRead(VariableIDs.FoamPercentage_Preset_5, Preset_FoamB_Percent[4]);
	smRead(VariableIDs.FoamSystem_Preset_5, Preset_FoamSystem[4]);
	
	//Get all of the PRESET 6 states.
	smRead(VariableIDs.FoamTank_Preset_6, Preset_FoamTank[5]);
	if (Preset_FoamTank[5] == TANKA)
		smRead(VariableIDs.FoamPercentage_Preset_6, Preset_FoamA_Percent[5]); 
	else
		smRead(VariableIDs.FoamPercentage_Preset_6, Preset_FoamB_Percent[5]);
	smRead(VariableIDs.FoamSystem_Preset_6, Preset_FoamSystem[5]);
		
	//Read the current state of the foamlogix system - 0 = OFF
	/*	If this is a single tank or a dual as single system then the FoamCurrentPowerState is based on the Primary_FoamLogix_Enabled variable.
		If this is a dual as two systems then we need to see what system this preset is for.
		Foam system control (0=single motor, 1=dual motor(as one system), 2=dual motor (as two systems).
		The FLX_NumberOfPumps variable is used in the SingleSystem_MotorControl() function in order
		to control two pumps as a single system.
	*/
	uint FoamSystemControlType;
	smRead(VariableIDs.FoamSystemControlType, FoamSystemControlType);
	
	if (FoamSystemControlType == 0 || FoamSystemControlType == 1)
	{
		smRead(VariableIDs.Primary_FoamLogix_Enabled, FoamCurrentPowerState);
		/*	Set the foam percents.
		*/
		if (Preset_FoamTank[presetButton] == TANKA)
			smWrite(VariableIDs.Primary_FoamPercentage_TankA, Preset_FoamA_Percent[presetButton]);
		else
			smWrite(VariableIDs.Primary_FoamPercentage_TankB, Preset_FoamB_Percent[presetButton]);
	}
	else
	{
		if (Preset_FoamSystem[presetButton] == 0)
		{
			smRead(VariableIDs.Primary_FoamLogix_Enabled, FoamCurrentPowerState);
			/*	Set the foam percents.
			*/
			if (Preset_FoamTank[presetButton] == TANKA)
				smWrite(VariableIDs.Primary_FoamPercentage_TankA, Preset_FoamA_Percent[presetButton]);
			else
				smWrite(VariableIDs.Primary_FoamPercentage_TankB, Preset_FoamB_Percent[presetButton]);
		}
		else
		{
			smRead(VariableIDs.Secondary_FoamLogix_Enabled, FoamCurrentPowerState);
			/*	Set the foam percents.
			*/
			if (Preset_FoamTank[presetButton] == TANKA)
				smWrite(VariableIDs.Secondary_FoamPercentage_TankA, Preset_FoamA_Percent[presetButton]);
			else
				smWrite(VariableIDs.Secondary_FoamPercentage_TankB, Preset_FoamB_Percent[presetButton]);
		}
	}
	
	

	
	uint FoamTankSelectorType;
	smRead(VariableIDs.FLX_FoamTankSelectorType, FoamTankSelectorType);
	
		
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	uint CurrentTankSelected;
	if (Preset_FoamSystem[presetButton] == 0)
		smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	else
		smRead(VariableIDs.Secondary_FoamSystemTankSelection, CurrentTankSelected);
	
	
	/*	Above we have set the tank selection to the desired value.
		Now we check to see what the desired state of the foam system is - if it is to be ON
		then we fire the "SCRIPT_FoamLogixPowerButton" script to enable the foam system, 
		but only if the system is currently OFF (otherwise calling this script will turn
		the foam system OFF - it's a toggle).
		Lastly, we show the desired foam screen (A or B).
	*/
	if (Preset_FoamTank[presetButton] == TANKA)
	{			
		if (FoamTankSelectorType == 3)	//If this is an ADT system - fire the tank A selection script which sets the transmit variables and the screen
		{
			//MST_MDT_CheckSelectorHandle(TANKA, Preset_FoamSystem[presetButton]); //Use temporarily until we get ADT outputs in new motor driver

			if (TypeFoamInLines == 1)	//IF there is B foam then we have to do a flush - otherwise we can just select tank A
			{
				EDT_ADT_Request = 1;
				smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
				smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
			}
			else
			{
				//Just select the deired tank
				smWrite(VariableIDs.FreeFormCAN_TankA_Select, 1);
				smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
			}
			//ForceFoamLogix_ON(Preset_FoamSystem[presetButton]);
			smWrite(VariableIDs.Primary_FoamLogix_Enabled, 1);	//Turn ON in tank A
			sendEvent(EventIDs.Event_TransmitOutputControlMessage);
			sendEvent(EventIDs.Event_SaveEDT_Positions);
			sendEvent(EventIDs.SCRIPT_GOTO_OperatingScreen);
		}
		else	//MST/MDT systems, just go to the operating screen based on the current tank selection
		{
			MST_MDT_CheckSelectorHandle(TANKA, Preset_FoamSystem[presetButton]);
			if (FoamCurrentPowerState == 0)
			{
				if (CurrentTankSelected == 2)
				{
					ForceFoamLogix_ON(Preset_FoamSystem[presetButton]);
				}
				else
				{
					ForceFoamLogix_OFF(Preset_FoamSystem[presetButton]);
				}
			}
			else
			{
				if (CurrentTankSelected == 1)
				{
					ForceFoamLogix_OFF(Preset_FoamSystem[presetButton]);
				}
			}
		}
		
		
	}
	else
	{	
		if (FoamTankSelectorType == 3)	//If this is an ADT system - fire the tank B selection script which sets the transmit variables and the screen
		{
			//MST_MDT_CheckSelectorHandle(TANKB, Preset_FoamSystem[presetButton]); //Use temporarily until we get ADT outputs in new motor driver
			if (TypeFoamInLines == 2)	//IF there is A foam then we have to do a flush
			{
				EDT_ADT_Request = 2;
				smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
				smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
			}
			else
			{
				//Just select the deired tank
				smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
				smWrite(VariableIDs.FreeFormCAN_TankB_Select, 1);
			}
			//ForceFoamLogix_ON(Preset_FoamSystem[presetButton]);
			smWrite(VariableIDs.Primary_FoamLogix_Enabled, 2);	//Turn ON in tank B
			sendEvent(EventIDs.Event_TransmitOutputControlMessage);
			sendEvent(EventIDs.Event_SaveEDT_Positions);
			sendEvent(EventIDs.SCRIPT_GOTO_OperatingScreen);
		}
		else	//MST/MDT systems, just go to the operating screen based on the current tank selection
		{
			MST_MDT_CheckSelectorHandle(TANKB, Preset_FoamSystem[presetButton]);
			if (FoamCurrentPowerState == 0)
			{
				if (CurrentTankSelected == 1)
					ForceFoamLogix_ON(Preset_FoamSystem[presetButton]);
				else
					ForceFoamLogix_OFF(Preset_FoamSystem[presetButton]);
			}
			else
			{
				if (CurrentTankSelected == 2)
					ForceFoamLogix_OFF(Preset_FoamSystem[presetButton]);
			}
		}	
	}

	
	/*	Force OFF the preset button that brought us here so that it doesn't continue to interfere with operation.
	*/
	switch (presetButton)
	{
		case 0:		smWrite(VariableIDs.ButtonPreset_1, 0);			break;
		case 1:		smWrite(VariableIDs.ButtonPreset_2, 0);			break;
		case 2:		smWrite(VariableIDs.ButtonPreset_3, 0);			break;
		case 3:		smWrite(VariableIDs.ButtonPreset_4, 0);			break;
		case 4:		smWrite(VariableIDs.ButtonPreset_5, 0);			break;
		case 5:		smWrite(VariableIDs.ButtonPreset_6, 0);			break;
	}		
}


void ForceFoamLogix_ON(uint FoamSystem)
{
	uint CurrentPowerState;	
	/*	Check to see which system we need to turn ON - 0 = primary, 1 = secondary
	*/
	if (FoamSystem == 0)
	{
		smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
	
		if (CurrentPowerState == 0)	//if it is OFF then we will fire the power button script
		{
			sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Primary);
		}
	}
	else
	{
		smRead(VariableIDs.Secondary_FoamLogix_Enabled, CurrentPowerState);
		if (CurrentPowerState == 0)	//if it is OFF then we will fire the power button script
		{
			sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Secondary);
		}
	}
}

void ForceFoamLogix_OFF(uint FoamSystem)
{
	uint CurrentPowerState;
	/*	Check to see which system we need to turn ON - 0=primary, 1 = secondary
	*/
	if (FoamSystem == 0)
	{
		smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
	
		if (CurrentPowerState != 0)	//if it is ON then we will fire the power button script
		{
			sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Primary);
		}
	}
	else
	{
		smRead(VariableIDs.Secondary_FoamLogix_Enabled, CurrentPowerState);
		if (CurrentPowerState != 0)	//if it is ON then we will fire the power button script
		{
			sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Secondary);
		}
	}
}





/*	This function sets the operating screen for MST/MDT manual selectors.  
*/
void MST_MDT_CheckSelectorHandle(const uint DesiredTank, uint FoamSystem)
{
	uint thisPresetTank = 0;
	uint CurrentTankSelected;
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	if (FoamSystem == 0)
		smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	else
		smRead(VariableIDs.Secondary_FoamSystemTankSelection, CurrentTankSelected);
		
	sendEvent(EventIDs.SCRIPT_GOTO_OperatingScreen);
	
	//Only 
	if (DesiredTank != 255)
	{
		/*	When using the manual selector MST/MDT then we have to make sure the selector handle is in the
			proper position for the currently selected preset.  If not then we will popup a warning
			and turn the foam system OFF.
		*/
		if (DesiredTank == 0)	//Tank A
			thisPresetTank = 2;
		if (DesiredTank == 1)	//Tank B
			thisPresetTank = 1;
					
		if (thisPresetTank != CurrentTankSelected)
		{
			//ForceFoamLogix_OFF(FoamSystem);
			
			if (thisPresetTank == 2)
			{
				sendEvent(EventIDs.Event_WarningSelectTankA);
			}
			if (thisPresetTank == 1)
			{
				sendEvent(EventIDs.Event_WarningSelectTankB);
			}
		}
	}	
}





//-----ScriptName=SCRIPT InitializePresetButtonText
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_InitializePresetButtonText for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_InitializePresetButtonText () 
{
	file f;
	int fileStatus;
	int fileSize = 0; 
	string thisString = "";
	string thisFileName = "";
	int thisStringID;
	string thisDefaultText;
	
	bool FORCE_TEXT_DEFAULTS = false;
	
	for (int i = 0; i < 6; i++)
	{
		switch (i)
		{
			case 0:	thisFileName = "PresetText_01.txt";		thisStringID = StringIDs.PresetButtonText_1;	thisDefaultText = "FOAM A 0.1%";		break;
			case 1:	thisFileName = "PresetText_02.txt";		thisStringID = StringIDs.PresetButtonText_2;	thisDefaultText = "FOAM A 0.2%";		break;
			case 2:	thisFileName = "PresetText_03.txt";		thisStringID = StringIDs.PresetButtonText_3;	thisDefaultText = "FOAM A 0.3%";		break;
			case 3:	thisFileName = "PresetText_04.txt";		thisStringID = StringIDs.PresetButtonText_4;	thisDefaultText = "FOAM B 1.0%";		break;
			case 4:	thisFileName = "PresetText_05.txt";		thisStringID = StringIDs.PresetButtonText_5;	thisDefaultText = "FOAM B 2.5%";		break;
			case 5:	thisFileName = "PresetText_06.txt";		thisStringID = StringIDs.PresetButtonText_6;	thisDefaultText = "FOAM B 3.0%";		break;
		}
	
		//Try to open the file - if it exists.  If it doesn't then create it.
		if ( f.open( thisFileName, "r" ) >= 0 && FORCE_TEXT_DEFAULTS == false)
		{
			fileSize = f.getSize();
			if (fileSize > 0)
			{	
				fileStatus = f.readString(fileSize, thisString);	
			}
			else
			{
				thisString = " ";
			}
			
			f.close();
			cultureSetString(thisStringID, thisString, true);
		}
		else
		{
			
			f.open( thisFileName, "w" );
			f.writeString(thisDefaultText);
			f.close();
			cultureSetString(thisStringID, thisDefaultText, true);		
		}		
	}
}


//-----ScriptName=SCRIPT UpdatePresetText
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_UpdatePresetText for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_UpdatePresetText () 
{
	string PresetString;	
	uint PresetTextStringID;
	
	switch (CurrentPresetToConfigure)
	{
		case 0:	PresetTextStringID = StringIDs.PresetButtonText_1;	break;
		case 1:	PresetTextStringID = StringIDs.PresetButtonText_2;	break;
		case 2:	PresetTextStringID = StringIDs.PresetButtonText_3;	break;
		case 3:	PresetTextStringID = StringIDs.PresetButtonText_4;	break;
		case 4:	PresetTextStringID = StringIDs.PresetButtonText_5;	break;
		case 5:	PresetTextStringID = StringIDs.PresetButtonText_6;	break;
	}
		
	//Get the text that was just typed into the keyboard...	
 	cultureGetString(StringIDs.test_string, PresetString, 0);
	//...and put it into the preset button text...
	cultureSetString(StringIDs.PresetCurrentText, PresetString, true);	//Local to the preset configure screen
	cultureSetString(PresetTextStringID, PresetString, true);			//The actual text on the preset button bar.
	//...then save the text into its local file... 
	SavePresetText(CurrentPresetToConfigure);
	//...and finish by clearing the keyboard text...
	cultureSetString(StringIDs.test_string, "", true);
	//...and set the keyboard visible value to 0 to hide the keyboard.
	smWrite(VariableIDs.TypePresetText, 0);	   
}


void SavePresetText(int thisPreset)
{
	file f;
	string thisString = "";
	string thisFileName = "";
	int thisStringID;
	
	//Get the name of the local file to save the custom preset text into - and get the string ID for the 
	//string variable used on the preset button bar.
	switch (thisPreset)
	{
		case 0:	thisFileName = "PresetText_01.txt";		thisStringID = StringIDs.PresetButtonText_1;	break;
		case 1:	thisFileName = "PresetText_02.txt";		thisStringID = StringIDs.PresetButtonText_2;	break;
		case 2:	thisFileName = "PresetText_03.txt";		thisStringID = StringIDs.PresetButtonText_3;	break;
		case 3:	thisFileName = "PresetText_04.txt";		thisStringID = StringIDs.PresetButtonText_4;	break;
		case 4:	thisFileName = "PresetText_05.txt";		thisStringID = StringIDs.PresetButtonText_5;	break;
		case 5:	thisFileName = "PresetText_06.txt";		thisStringID = StringIDs.PresetButtonText_6;	break;
	}
	

	if ( f.open( thisFileName, "w" ) >= 0 )
	{
		cultureGetString(thisStringID, thisString, 0);
		f.writeString(thisString);
	}
	
	f.close();
}

//-----ScriptName=SCRIPT SelectPresetConfig_1
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as $SCRIPT SelectPresetConfig_3$ for main script method
//---------------------------------------------------------------------------------------------
uint CurrentPresetToConfigure = 0;
void SCRIPT_SelectPresetConfig_1 () 
{
	//Set the current preset variable to the requested preset to configure.
 	CurrentPresetToConfigure = 0;
	
	string PresetButtonText = "";
	
	uint CurrentFoamTank;
	double CurrentFoamPercent;
	uint CurrentBarColor;
	uint CurrentBarIcon;
	uint CurrentFoamSystem;
	
	//Get this presets current configured values.
	smRead(VariableIDs.BarColor_Preset_1, CurrentBarColor);
	smRead(VariableIDs.BarIcon_Preset_1, CurrentBarIcon);
	smRead(VariableIDs.FoamTank_Preset_1, CurrentFoamTank);
	smRead(VariableIDs.FoamPercentage_Preset_1, CurrentFoamPercent); 
	smRead(VariableIDs.FoamSystem_Preset_1, CurrentFoamSystem); 
	
	//Load the temp variables of the preset configure page with this presets current values.
	smWrite(VariableIDs.tmp_BarColor_Preset, CurrentBarColor);
	smWrite(VariableIDs.tmp_BarIcon_Preset, CurrentBarIcon);
	smWrite(VariableIDs.tmp_FoamTank_Preset, CurrentFoamTank);
	smWrite(VariableIDs.tmp_FoamPercentage_Preset, CurrentFoamPercent);
	smWrite(VariableIDs.tmp_FoamSystem_Preset, CurrentFoamSystem);
	
	//Update the title of the configureation page
	cultureSetString(StringIDs.PresetConfigureTitle, "[Preset 1 Configuration]", true);
	
	//update the current preset button text
	cultureGetString(StringIDs.PresetButtonText_1, PresetButtonText, 0);
	cultureSetString(StringIDs.PresetCurrentText, PresetButtonText, true);
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);
	
	if (CurrentBarColor == 0)	//preset is disabled
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled_VERT);
	}
	else
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled_VERT);
	}
	
	
	//Determine what should be shown for this configurations remote activation value.
	//If the RemoteActivationPreset value (1 through 6) is equal to this CurrentPresetToConfigure
	//(0 through 5) then we set the tmp_RemotePresetActivation_state to non-zero.
	//When we evaluate we have to make sure that we re-align RemoteActivationPreset from (1 to 6)
	//into (0 to 5) for a proper evaluation.
	uint RemoteActivationPreset;
	smRead(VariableIDs.RemoteActivationPreset, RemoteActivationPreset); 
	if (RemoteActivationPreset-1 == CurrentPresetToConfigure)
		smWrite(VariableIDs.tmp_RemotePresetActivation_state, RemoteActivationPreset);
	else
		smWrite(VariableIDs.tmp_RemotePresetActivation_state, 0); 
	   
}

//-----ScriptName=SCRIPT SavePreset
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_SavePreset for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_SavePreset () 
{
   	switch(CurrentPresetToConfigure)
	{
		case 0:	sendEvent(EventIDs.SAVE_Preset_1);	break;
		case 1:	sendEvent(EventIDs.SAVE_Preset_2);	break;
		case 2:	sendEvent(EventIDs.SAVE_Preset_3);	break;
		case 3:	sendEvent(EventIDs.SAVE_Preset_4);	break;
		case 4:	sendEvent(EventIDs.SAVE_Preset_5);	break;
		case 5:	sendEvent(EventIDs.SAVE_Preset_6);	break;
	} 
}

//-----ScriptName=SCRIPT SetProperConfigurationPage
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_SetProperConfigurationPage for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_SetProperConfigurationPage () 
{
	uint CurrentBarColor;
	
	//Get the value of the bar color text being modified in the preset configuration page.
	//This script is fired by tmp_BarColor_Preset changing
	
	smRead(VariableIDs.tmp_BarColor_Preset, CurrentBarColor);
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);
	
	if (CurrentBarColor == 0)	//preset is disabled
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled_VERT);
	}
	else
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled_VERT);
	}  
}

//-----ScriptName=SCRIPT SelectPresetConfig_2
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_SelectPresetConfig_2 for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_SelectPresetConfig_2 () 
{
 	//Set the current preset variable to the requested preset to configure.
 	CurrentPresetToConfigure = 1;
	
	string PresetButtonText = "";
	
	uint CurrentFoamTank;
	double CurrentFoamPercent;
	uint CurrentCAFSIcon;
	uint CurrentCAFS_ARCPosition;
	uint CurrentBarColor;
	uint CurrentBarIcon;
	uint CurrentFoamSystem;
	
	//Get this presets current configured values.
	smRead(VariableIDs.BarColor_Preset_2, CurrentBarColor);
	smRead(VariableIDs.BarIcon_Preset_2, CurrentBarIcon);
	smRead(VariableIDs.FoamTank_Preset_2, CurrentFoamTank);
	smRead(VariableIDs.FoamPercentage_Preset_2, CurrentFoamPercent); 
	smRead(VariableIDs.FoamSystem_Preset_2, CurrentFoamSystem);
	
	//Load the temp variables of the preset configure page with this presets current values.
	smWrite(VariableIDs.tmp_BarColor_Preset, CurrentBarColor);
	smWrite(VariableIDs.tmp_BarIcon_Preset, CurrentBarIcon);
	smWrite(VariableIDs.tmp_FoamTank_Preset, CurrentFoamTank);
	smWrite(VariableIDs.tmp_FoamPercentage_Preset, CurrentFoamPercent);
	smWrite(VariableIDs.tmp_FoamSystem_Preset, CurrentFoamSystem);
	
	//Update the title of the configureation page
	cultureSetString(StringIDs.PresetConfigureTitle, "[Preset 2 Configuration]", true);
	
	//update the current preset button text
	cultureGetString(StringIDs.PresetButtonText_2, PresetButtonText, 0);
	cultureSetString(StringIDs.PresetCurrentText, PresetButtonText, true);
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);
	
	if (CurrentBarColor == 0)	//preset is disabled
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled_VERT);
	}
	else
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled_VERT);
	} 
	
	//Determine what should be shown for this configurations remote activation value.
	//If the RemoteActivationPreset value (1 through 6) is equal to this CurrentPresetToConfigure
	//(0 through 5) then we set the tmp_RemotePresetActivation_state to non-zero.
	//When we evaluate we have to make sure that we re-align RemoteActivationPreset from (1 to 6)
	//into (0 to 5) for a proper evaluation.
	uint RemoteActivationPreset;
	smRead(VariableIDs.RemoteActivationPreset, RemoteActivationPreset); 
	if (RemoteActivationPreset-1 == CurrentPresetToConfigure)
		smWrite(VariableIDs.tmp_RemotePresetActivation_state, RemoteActivationPreset);
	else
		smWrite(VariableIDs.tmp_RemotePresetActivation_state, 0);   
}

//-----ScriptName=SCRIPT SelectPresetConfig_3
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_SelectPresetConfig_3 for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_SelectPresetConfig_3 () 
{
 	//Set the current preset variable to the requested preset to configure.
 	CurrentPresetToConfigure = 2;
	
	string PresetButtonText = "";
	
	uint CurrentFoamTank;
	double CurrentFoamPercent;
	uint CurrentCAFSIcon;
	uint CurrentCAFS_ARCPosition;
	uint CurrentBarColor;
	uint CurrentBarIcon;
	uint CurrentFoamSystem;
	
	//Get this presets current configured values.
	smRead(VariableIDs.BarColor_Preset_3, CurrentBarColor);
	smRead(VariableIDs.BarIcon_Preset_3, CurrentBarIcon);
	smRead(VariableIDs.FoamTank_Preset_3, CurrentFoamTank);
	smRead(VariableIDs.FoamPercentage_Preset_3, CurrentFoamPercent); 
	smRead(VariableIDs.FoamSystem_Preset_3, CurrentFoamSystem);
	
	//Load the temp variables of the preset configure page with this presets current values.
	smWrite(VariableIDs.tmp_BarColor_Preset, CurrentBarColor);
	smWrite(VariableIDs.tmp_BarIcon_Preset, CurrentBarIcon);
	smWrite(VariableIDs.tmp_FoamTank_Preset, CurrentFoamTank);
	smWrite(VariableIDs.tmp_FoamPercentage_Preset, CurrentFoamPercent);
	smWrite(VariableIDs.tmp_FoamSystem_Preset, CurrentFoamSystem);
	
	//Update the title of the configureation page
	cultureSetString(StringIDs.PresetConfigureTitle, "[Preset 3 Configuration]", true);
	
	//update the current preset button text
	cultureGetString(StringIDs.PresetButtonText_3, PresetButtonText, 0);
	cultureSetString(StringIDs.PresetCurrentText, PresetButtonText, true);
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);
	
	if (CurrentBarColor == 0)	//preset is disabled
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled_VERT);
	}
	else
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled_VERT);
	} 
	
	//Determine what should be shown for this configurations remote activation value.
	//If the RemoteActivationPreset value (1 through 6) is equal to this CurrentPresetToConfigure
	//(0 through 5) then we set the tmp_RemotePresetActivation_state to non-zero.
	//When we evaluate we have to make sure that we re-align RemoteActivationPreset from (1 to 6)
	//into (0 to 5) for a proper evaluation.
	uint RemoteActivationPreset;
	smRead(VariableIDs.RemoteActivationPreset, RemoteActivationPreset); 
	if (RemoteActivationPreset-1 == CurrentPresetToConfigure)
		smWrite(VariableIDs.tmp_RemotePresetActivation_state, RemoteActivationPreset);
	else
		smWrite(VariableIDs.tmp_RemotePresetActivation_state, 0);  
}

//-----ScriptName=SCRIPT SelectPresetConfig_4
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_SelectPresetConfig_4 for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_SelectPresetConfig_4 () 
{
 	//Set the current preset variable to the requested preset to configure.
 	CurrentPresetToConfigure = 3;
	
	string PresetButtonText = "";
	
	uint CurrentFoamTank;
	double CurrentFoamPercent;
	uint CurrentCAFSIcon;
	uint CurrentCAFS_ARCPosition;
	uint CurrentBarColor;
	uint CurrentBarIcon;
	uint CurrentFoamSystem;
	
	//Get this presets current configured values.
	smRead(VariableIDs.BarColor_Preset_4, CurrentBarColor);
	smRead(VariableIDs.BarIcon_Preset_4, CurrentBarIcon);
	smRead(VariableIDs.FoamTank_Preset_4, CurrentFoamTank);
	smRead(VariableIDs.FoamPercentage_Preset_4, CurrentFoamPercent); 
	smRead(VariableIDs.FoamSystem_Preset_4, CurrentFoamSystem);
	
	//Load the temp variables of the preset configure page with this presets current values.
	smWrite(VariableIDs.tmp_BarColor_Preset, CurrentBarColor);
	smWrite(VariableIDs.tmp_BarIcon_Preset, CurrentBarIcon);
	smWrite(VariableIDs.tmp_FoamTank_Preset, CurrentFoamTank);
	smWrite(VariableIDs.tmp_FoamPercentage_Preset, CurrentFoamPercent);
	smWrite(VariableIDs.tmp_FoamSystem_Preset, CurrentFoamSystem);
	
	//Update the title of the configureation page
	cultureSetString(StringIDs.PresetConfigureTitle, "[Preset 4 Configuration]", true);
	
	//update the current preset button text
	cultureGetString(StringIDs.PresetButtonText_4, PresetButtonText, 0);
	cultureSetString(StringIDs.PresetCurrentText, PresetButtonText, true);
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);
	
	if (CurrentBarColor == 0)	//preset is disabled
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled_VERT);
	}
	else
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled_VERT);
	} 
	
	//Determine what should be shown for this configurations remote activation value.
	//If the RemoteActivationPreset value (1 through 6) is equal to this CurrentPresetToConfigure
	//(0 through 5) then we set the tmp_RemotePresetActivation_state to non-zero.
	//When we evaluate we have to make sure that we re-align RemoteActivationPreset from (1 to 6)
	//into (0 to 5) for a proper evaluation.
	uint RemoteActivationPreset;
	smRead(VariableIDs.RemoteActivationPreset, RemoteActivationPreset); 
	if (RemoteActivationPreset-1 == CurrentPresetToConfigure)
		smWrite(VariableIDs.tmp_RemotePresetActivation_state, RemoteActivationPreset);
	else
		smWrite(VariableIDs.tmp_RemotePresetActivation_state, 0);   
}

//-----ScriptName=SCRIPT SelectPresetConfig_5
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_SelectPresetConfig_5 for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_SelectPresetConfig_5 () 
{
 	//Set the current preset variable to the requested preset to configure.
 	CurrentPresetToConfigure = 4;
	
	string PresetButtonText = "";
	
	uint CurrentFoamTank;
	double CurrentFoamPercent;
	uint CurrentCAFSIcon;
	uint CurrentCAFS_ARCPosition;
	uint CurrentBarColor;
	uint CurrentBarIcon;
	uint CurrentFoamSystem;
	
	//Get this presets current configured values.
	smRead(VariableIDs.BarColor_Preset_5, CurrentBarColor);
	smRead(VariableIDs.BarIcon_Preset_5, CurrentBarIcon);
	smRead(VariableIDs.FoamTank_Preset_5, CurrentFoamTank);
	smRead(VariableIDs.FoamPercentage_Preset_5, CurrentFoamPercent);
	smRead(VariableIDs.FoamSystem_Preset_5, CurrentFoamSystem); 
	
	//Load the temp variables of the preset configure page with this presets current values.
	smWrite(VariableIDs.tmp_BarColor_Preset, CurrentBarColor);
	smWrite(VariableIDs.tmp_BarIcon_Preset, CurrentBarIcon);
	smWrite(VariableIDs.tmp_FoamTank_Preset, CurrentFoamTank);
	smWrite(VariableIDs.tmp_FoamPercentage_Preset, CurrentFoamPercent);
	smWrite(VariableIDs.tmp_FoamSystem_Preset, CurrentFoamSystem);
	
	//Update the title of the configureation page
	cultureSetString(StringIDs.PresetConfigureTitle, "[Preset 5 Configuration]", true);
	
	//update the current preset button text
	cultureGetString(StringIDs.PresetButtonText_5, PresetButtonText, 0);
	cultureSetString(StringIDs.PresetCurrentText, PresetButtonText, true);
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);
	
	if (CurrentBarColor == 0)	//preset is disabled
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled_VERT);
	}
	else
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled_VERT);
	}
	
	//Determine what should be shown for this configurations remote activation value.
	//If the RemoteActivationPreset value (1 through 6) is equal to this CurrentPresetToConfigure
	//(0 through 5) then we set the tmp_RemotePresetActivation_state to non-zero.
	//When we evaluate we have to make sure that we re-align RemoteActivationPreset from (1 to 6)
	//into (0 to 5) for a proper evaluation.
	uint RemoteActivationPreset;
	smRead(VariableIDs.RemoteActivationPreset, RemoteActivationPreset); 
	if (RemoteActivationPreset-1 == CurrentPresetToConfigure)
		smWrite(VariableIDs.tmp_RemotePresetActivation_state, RemoteActivationPreset);
	else
		smWrite(VariableIDs.tmp_RemotePresetActivation_state, 0);  
}

//-----ScriptName=SCRIPT SelectPresetConfig_6
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_SelectPresetConfig_6 for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_SelectPresetConfig_6 () 
{
 	//Set the current preset variable to the requested preset to configure.
 	CurrentPresetToConfigure = 5;
	
	string PresetButtonText = "";
	
	uint CurrentFoamTank;
	double CurrentFoamPercent;
	uint CurrentCAFSIcon;
	uint CurrentCAFS_ARCPosition;
	uint CurrentBarColor;
	uint CurrentBarIcon;
	uint CurrentFoamSystem;
	
	//Get this presets current configured values.
	smRead(VariableIDs.BarColor_Preset_6, CurrentBarColor);
	smRead(VariableIDs.BarIcon_Preset_6, CurrentBarIcon);
	smRead(VariableIDs.FoamTank_Preset_6, CurrentFoamTank);
	smRead(VariableIDs.FoamPercentage_Preset_6, CurrentFoamPercent); 
	smRead(VariableIDs.FoamSystem_Preset_6, CurrentFoamSystem);
	
	//Load the temp variables of the preset configure page with this presets current values.
	smWrite(VariableIDs.tmp_BarColor_Preset, CurrentBarColor);
	smWrite(VariableIDs.tmp_BarIcon_Preset, CurrentBarIcon);
	smWrite(VariableIDs.tmp_FoamTank_Preset, CurrentFoamTank);
	smWrite(VariableIDs.tmp_FoamPercentage_Preset, CurrentFoamPercent);
	smWrite(VariableIDs.tmp_FoamSystem_Preset, CurrentFoamSystem);
	
	//Update the title of the configureation page
	cultureSetString(StringIDs.PresetConfigureTitle, "[Preset 6 Configuration]", true);
	
	//update the current preset button text
	cultureGetString(StringIDs.PresetButtonText_6, PresetButtonText, 0);
	cultureSetString(StringIDs.PresetCurrentText, PresetButtonText, true);
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);
	
	if (CurrentBarColor == 0)	//preset is disabled
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Disabled_VERT);
	}
	else
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled);
		else
			sendEvent(EventIDs.Event_GOTO_PresetConfiguration_Enabled_VERT);
	}  
	
	//Determine what should be shown for this configurations remote activation value.
	//If the RemoteActivationPreset value (1 through 6) is equal to this CurrentPresetToConfigure
	//(0 through 5) then we set the tmp_RemotePresetActivation_state to non-zero.
	//When we evaluate we have to make sure that we re-align RemoteActivationPreset from (1 to 6)
	//into (0 to 5) for a proper evaluation.
	uint RemoteActivationPreset;
	smRead(VariableIDs.RemoteActivationPreset, RemoteActivationPreset); 
	if (RemoteActivationPreset-1 == CurrentPresetToConfigure)
		smWrite(VariableIDs.tmp_RemotePresetActivation_state, RemoteActivationPreset);
	else
		smWrite(VariableIDs.tmp_RemotePresetActivation_state, 0); 
}

//-----ScriptName=SCRIPT ClearKeyboardText
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_ClearKeyboardText for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_ClearKeyboardText () 
{
 	//Clear the text being typed into the keyboard.	
 	cultureSetString(StringIDs.test_string, "", true);   
}

//-----ScriptName=SCRIPT ExitKeyboard
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_ExitKeyboard for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_ExitKeyboard () 
{
  	string PresetString;	
	uint PresetTextStringID;
	
	switch (CurrentPresetToConfigure)
	{
		case 0:	PresetTextStringID = StringIDs.PresetButtonText_1;	break;
		case 1:	PresetTextStringID = StringIDs.PresetButtonText_2;	break;
		case 2:	PresetTextStringID = StringIDs.PresetButtonText_3;	break;
		case 3:	PresetTextStringID = StringIDs.PresetButtonText_4;	break;
		case 4:	PresetTextStringID = StringIDs.PresetButtonText_5;	break;
		case 5:	PresetTextStringID = StringIDs.PresetButtonText_6;	break;
	}
		
	//Get the original preset button text	
 	cultureGetString(PresetTextStringID, PresetString, 0);
	//...and put it into the preset button text...
	cultureSetString(StringIDs.PresetCurrentText, PresetString, true);	//Local to the preset configure screen
	cultureSetString(PresetTextStringID, PresetString, true);			//The actual text on the preset button bar.
	//...then save the text into its local file... 
	SavePresetText(CurrentPresetToConfigure);
	//...and finish by clearing the keyboard text...
	cultureSetString(StringIDs.test_string, "", true);
	//...and set the keyboard visible value to 0 to hide the keyboard.
	smWrite(VariableIDs.TypePresetText, 0);	  
}

//-----ScriptName=SCRIPT UserVariable_NEXT
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_UserVariable_NEXT for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_UserVariable_NEXT () 
{
 	uint ThisVariable;
	sendEvent(EventIDs.Calculate_SelectedUserVariableEquals_NEXT);  
	
		//If we have gone from 8 to 9 then we need to go back to page 2	
//	smRead(VariableIDs.SelectedOEMVariable, ThisVariable);
//	if (ThisVariable == 8)
//		sendEvent(EventIDs.Event_GOTO_Operation_Variables_Menu_2);    
}

//-----ScriptName=SCRIPT UserVariable_PREVIOUS
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_UserVariable_PREVIOUS for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_UserVariable_PREVIOUS () 
{
 	uint ThisVariable;
	sendEvent(EventIDs.Calculate_SelectedUserVariableEquals_PREVIOUS);
	
	//If we have gone from 9 to 8 then we need to go back to page 1
//	smRead(VariableIDs.SelectedOperationVariable, ThisVariable);	
//	if (ThisVariable == 9)
//		sendEvent(EventIDs.Event_GOTO_Operation_Variables_Menu_1);    
}

//-----ScriptName=SCRIPT UserVariable_INCREASE
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_UserVariable_INCREASE for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_UserVariable_INCREASE () 
{
 	uint ThisSelectedVariable;
	smRead(VariableIDs.SelectedUserVariable, ThisSelectedVariable);
	
	uint NumLanguages;
	smRead(VariableIDs.Num_Languages_Available, NumLanguages);
	
	double ThisVariableValue;
	
	switch (ThisSelectedVariable)
	{
		case 0:	//Language 
				smRead(VariableIDs.LanguageSelection, ThisVariableValue);
				if (ThisVariableValue < (NumLanguages-1))
				{
					ThisVariableValue = ThisVariableValue + 1;	
				}
				else
				{
					ThisVariableValue = 0;
				}
				smWrite(VariableIDs.LanguageSelection, ThisVariableValue);
				break;
				
		case 1:	//Unit of measure - 0=english, 1=metric
				smRead(VariableIDs.tmp_FLX_UnitOfMeasure, ThisVariableValue);
				if (ThisVariableValue == 0)
					ThisVariableValue = 1;
				else
					ThisVariableValue = 0;
				
				smWrite(VariableIDs.tmp_FLX_UnitOfMeasure, ThisVariableValue);	
				break;
	
		case 2:	//Foam A color - 0=Green, 1=Red
				smRead(VariableIDs.tmp_FLX_FoamA_Color, ThisVariableValue);
				if (ThisVariableValue == 0)
					ThisVariableValue = 1;
				else
					ThisVariableValue = 0;
				
				smWrite(VariableIDs.tmp_FLX_FoamA_Color, ThisVariableValue);
				break;
		
		case 3:	//Foam B color - 0=red, 1=yellow
				smRead(VariableIDs.tmp_FLX_FoamB_Color, ThisVariableValue);
				if (ThisVariableValue < 2)
					ThisVariableValue = ThisVariableValue + 1;
				else
					ThisVariableValue = 0;
				
				smWrite(VariableIDs.tmp_FLX_FoamB_Color, ThisVariableValue);
				break;
				
		case 4:	//Display background type (0=SmartFOAM, 1=FoamLogix, 2=Analog, 3=Chase Thailand) 
				smRead(VariableIDs.tmp_FLX_Display_Type, ThisVariableValue);
				if (ThisVariableValue < 3)
				{
					ThisVariableValue = ThisVariableValue + 1;	
				}
				else
				{
					ThisVariableValue = 0;
				}
				smWrite(VariableIDs.tmp_FLX_Display_Type, ThisVariableValue);
				break;
				
		case 5:	//Display Orientation - 0=horizontal, 1=vertical
				smRead(VariableIDs.tmp_FLX_DisplayOrientation, ThisVariableValue);
				if (ThisVariableValue == 0)
					ThisVariableValue = 1;
				else
					ThisVariableValue = 0;
					
				smWrite(VariableIDs.tmp_LogoChoice, ThisVariableValue);	//Update temp logoChoice to set the splash screen to the correct orientation
				
				smWrite(VariableIDs.tmp_FLX_DisplayOrientation, ThisVariableValue);
				break;
				
				
	}     
}

//-----ScriptName=SCRIPT UserVariable_DECREASE
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_UserVariable_DECREASE for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_UserVariable_DECREASE () 
{
 	uint ThisSelectedVariable;
	smRead(VariableIDs.SelectedUserVariable, ThisSelectedVariable);
	
	uint NumLanguages;
	smRead(VariableIDs.Num_Languages_Available, NumLanguages);
	
	double ThisVariableValue;
	
	switch (ThisSelectedVariable)
	{
		case 0:	//Language 
				smRead(VariableIDs.LanguageSelection, ThisVariableValue);
				if (ThisVariableValue > 0)
				{
					ThisVariableValue = ThisVariableValue - 1;	
				}
				else
				{
					ThisVariableValue = (NumLanguages-1);
				}
				smWrite(VariableIDs.LanguageSelection, ThisVariableValue);
				break;
				
		case 1:	//Unit of measure - 0=english, 1=metric
				smRead(VariableIDs.tmp_FLX_UnitOfMeasure, ThisVariableValue);
				if (ThisVariableValue == 0)
					ThisVariableValue = 1;
				else
					ThisVariableValue = 0;
				
				smWrite(VariableIDs.tmp_FLX_UnitOfMeasure, ThisVariableValue);	
				break;
	
		case 2:	//Foam A color - 0=Green, 1=Red
				smRead(VariableIDs.tmp_FLX_FoamA_Color, ThisVariableValue);
				if (ThisVariableValue == 0)
					ThisVariableValue = 1;
				else
					ThisVariableValue = 0;
				
				smWrite(VariableIDs.tmp_FLX_FoamA_Color, ThisVariableValue);
				break;
				
		case 3:	//Foam B color - 0=red, 1=yellow, Green
				smRead(VariableIDs.tmp_FLX_FoamB_Color, ThisVariableValue);
				if (ThisVariableValue > 0)
					ThisVariableValue = ThisVariableValue - 1;
				else
					ThisVariableValue = 2;
				
				smWrite(VariableIDs.tmp_FLX_FoamB_Color, ThisVariableValue);
				break;
				
		case 4:	//Display background type (0=SmartFOAM, 1=FoamLogix, 2=Analog, 3=Chase Thailand) 
				smRead(VariableIDs.tmp_FLX_Display_Type, ThisVariableValue);
				if (ThisVariableValue > 0)
				{
					ThisVariableValue = ThisVariableValue - 1;	
				}
				else
				{
					ThisVariableValue = 3;
				}
				smWrite(VariableIDs.tmp_FLX_Display_Type, ThisVariableValue);
				break;
				
		case 5:	//Display Orientation - 0=horizontal, 1=vertical
				smRead(VariableIDs.tmp_FLX_DisplayOrientation, ThisVariableValue);
				if (ThisVariableValue == 0)
					ThisVariableValue = 1;
				else
					ThisVariableValue = 0;
					
				smWrite(VariableIDs.tmp_LogoChoice, ThisVariableValue);	//Update temp logoChoice to set the splash screen to the correct orientation
				
				smWrite(VariableIDs.tmp_FLX_DisplayOrientation, ThisVariableValue);
				break;
				
	}   
}

//-----ScriptName=SCRIPT UpdateDateAndTime
/*	This script takes the realtime clock values and converts them into a text string that can
	be updated on the screen in the format:
	
		JAN 23, 2015   12:25.05
		
	The only thing required on the display screen is a text widget that uses the override string
	"DateTime_TextString" (this string variable is created and in this folder).
	(or change the handleDateTime_Text within the script below).
	This script is triggered to run every 500 ms with a timer in the event part of this script.
*/
void SCRIPT_UpdateDateAndTime () 
{
 	//Define the strings used locally 
	string[] def_Months = {"JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};
	string Date_Time_String = "";
	string str_Minute;
	string str_Seconds;
	string str_Day;
	
	//Get the handles of the real time clock variables - these are predefined System Manager variables
	uint handleSPN_Hour		= VariableIDs.RealTimeClock_Hour;
	uint handleSPN_Minute	= VariableIDs.RealTimeClock_Minute;
	uint handleSPN_Seconds	= VariableIDs.RealTimeClock_Second;
	uint handleSPN_Year		= VariableIDs.RealTimeClock_Year;
	uint handleSPN_Month	= VariableIDs.RealTimeClock_Month;
	uint handleSPN_Day		= VariableIDs.RealTimeClock_Day;
	
	//Create local variables to hold the real time clock variables
	uint Hour;
	uint Minute;
	uint Seconds;
	uint Year;
	uint Month;
	uint Day;
	
	//Get the real time clock variables into our local variables
	smRead(handleSPN_Hour, Hour);
	smRead(handleSPN_Minute, Minute);
	smRead(handleSPN_Seconds, Seconds);
	smRead(handleSPN_Year, Year);
	smRead(handleSPN_Month, Month);
	smRead(handleSPN_Day, Day);

	//Convert the day, minute, second variables into leading zero when the value was less than 10.
	str_Minute 	= ConvertToString(Minute);
	str_Seconds = ConvertToString(Seconds);
	str_Day 	= ConvertToString(Day);	

	//Format the date/time string with the data - this sets the format as: JAN 23, 2015   12:25.05		
	Date_Time_String = def_Months[Month-1] + " " + str_Day + ", " + Year + "     " + Hour + ":" + str_Minute + "." + str_Seconds; 

	//Update the override string with the newly formated text
	cultureSetString(StringIDs.DateTime_TextString, Date_Time_String, true);   
}



/*
	This function returns a string equivilent of a two digit number
	passed to it.  If that value is less than 10 it adds a leading
	zero.
*/
string ConvertToString(uint Value)
{
	string str_ToReturn;
	
	if (Value < 10)
	{
		str_ToReturn = "0" + Value;	
	}
	else
	{
		str_ToReturn = Value;
	}
	
	return str_ToReturn;
}


//-----ScriptName=SCRIPT ToggleRemotePresetActivation
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_ToggleRemotePresetActivation for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_ToggleRemotePresetActivation () 
{
	uint CurrentState;
	
	/*	The tmp_RemotePresetActivation_state is used on the preset configuration screen to toggle the state of the remote activation variable.
		The tmp_RemotePresetActivation_state is set initially when the user presses the configure preset X button which fires the SCRIPT_SelectPresetConfig_X.
		Inside the configure preset screen this script is fired when the user presses the "Remote Activated?" button.  Here we want to toggle the state of
		the tmp_RemotePresetActivation_state and also
	*/
	
	smRead(VariableIDs.tmp_RemotePresetActivation_state, CurrentState);	

 	switch(CurrentPresetToConfigure)
	{
		case 0:	if (CurrentState == 1)
					CurrentState = 0;
				else
					CurrentState = 1;	
				break;
		case 1:	if (CurrentState == 2)
					CurrentState = 0;
				else
					CurrentState = 2;	
				break;
		case 2:	if (CurrentState == 3)
					CurrentState = 0;
				else
					CurrentState = 3;	
				break;
		case 3:	if (CurrentState == 4)
					CurrentState = 0;
				else
					CurrentState = 4;	
				break;
		case 4:	if (CurrentState == 5)
					CurrentState = 0;
				else
					CurrentState = 5;	
				break;
		case 5:	if (CurrentState == 6)
					CurrentState = 0;
				else
					CurrentState = 6;	
				break;
								
	}
	
	//Take the state of the updated RemoteActivationPreset and load it into the tmp variable that is used on the preset configuration page
	smWrite(VariableIDs.RemoteActivationPreset, CurrentState); 
	smWrite(VariableIDs.tmp_RemotePresetActivation_state, CurrentState);   
}

//-----ScriptName=SCRIPT FoamLogix_Display_Update
/*	This script is fired every 100ms, but will only do anything if the display type is set to FoamLogix (1)
*/
bool InitFoamLogix = true;
uint EnabledFlashingBarTimer = 0;
uint TemporarilyDisplayFoamPercentTimer = 0;
bool TemporarilyDisplayFoamPercent = false;
uint Last_FoamLogixActiveDisplay = 0;
void SCRIPT_FoamLogix_Display_Update () 
{
 	uint FoamSystemDisplayBackground;
	smRead(VariableIDs.FLX_Display_Type, FoamSystemDisplayBackground);
	
	uint FoamLogixActiveDisplay;
	smRead(VariableIDs.FoamLogix_Active_Display, FoamLogixActiveDisplay);
	
	uint FoamLogixPowerEnabled;
	smRead(VariableIDs.BIT_FoamLogixEnabled, FoamLogixPowerEnabled);
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT UpdateFoamLogixBackground.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	uint CurrentTankSelected;
	smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	
	
	uint WaterFlowRate;
	smRead(VariableIDs.Primary_WaterFlowRate_GPM, WaterFlowRate);
	
	uint SimulatedWaterFlowEnabled;
	smRead(VariableIDs.Primary_SimulatedWaterFlow_Enabled, SimulatedWaterFlowEnabled); 
	
	double float_FoamPercentage;
	uint FoamPercentage;
	if (CurrentTankSelected == 2)	//tankA
		smRead(VariableIDs.Primary_FoamPercentage_TankA, float_FoamPercentage);
	if (CurrentTankSelected == 1)	//tankB
		smRead(VariableIDs.Primary_FoamPercentage_TankB, float_FoamPercentage);
	float_FoamPercentage = float_FoamPercentage * 10;	//remove the decimal point
	FoamPercentage = float_FoamPercentage;
		
	uint TotalWaterFlowed;
	smRead(VariableIDs.Primary_TotalWaterGallons,TotalWaterFlowed);
	
	uint TotalFoamFlowed;
	double float_TotalFoamFlowed;
	if (CurrentTankSelected == 2)	//tankA
		smRead(VariableIDs.Primary_TotalFoamGallons_A, float_TotalFoamFlowed);
	if (CurrentTankSelected == 1)	//tankB
		smRead(VariableIDs.Primary_TotalFoamGallons_B, float_TotalFoamFlowed);
	float_TotalFoamFlowed = float_TotalFoamFlowed * 100;
	TotalFoamFlowed = float_TotalFoamFlowed;
			
	uint Digit_0;
	uint Digit_1;
	uint Digit_2;
	uint Digit_3;
	uint Digit_4;
	/*
		Digit to display	  value to set
		 0 - 9  			= 0 - 9
		 OFF				= 255
		 A					= 15
		 B					= 16
		 F					= 17
		 L					= 18
		 U					= 19
		 S					= 20
		 H					= 21
	*/
	
	if (FoamSystemDisplayBackground == 1)	//Background 1 is the Classic FoamLogix display look
	{
	
		//Pressing the INCR or DECR button sets the TemporarilyDisplayFoamPercent to TRUE.  After the timer times out this sets the active display back to the original
		if (TemporarilyDisplayFoamPercent == true)
		{
			TemporarilyDisplayFoamPercentTimer++;
			if (TemporarilyDisplayFoamPercentTimer >= 20)
			{
				smWrite(VariableIDs.FoamLogix_Active_Display, Last_FoamLogixActiveDisplay);
				TemporarilyDisplayFoamPercent = false;	
			}
		}
		else
		{
			TemporarilyDisplayFoamPercentTimer = 0;
		}
	
		
		//First time through - make sure that we don't show the preset screen
		if (InitFoamLogix == true)
		{
			sendEvent(EventIDs.SCRIPT_GOTO_OperatingScreen);
			InitFoamLogix = false;
		}
		
		uint FoamSystemControlType;
		smRead(VariableIDs.FoamSystemControlType, FoamSystemControlType);
		if (FoamSystemControlType == 2)	//Two pumps controlled individually.
		{
			smWrite(VariableIDs.FoamSystemControlType, 0);	//Since this is using old FoamLogix display - we can't have two individual pumps - set to single system.
			sendEvent(EventIDs.Event_SaveFoamSystemControlType);
		}
		
		
		EnabledFlashingBarTimer++;
		if (EnabledFlashingBarTimer > 9)
			EnabledFlashingBarTimer = 0;
		
		if (FoamLogixPowerEnabled != 0)
		{
			if (EnabledFlashingBarTimer < 6)
			{
				smWrite(VariableIDs.FoamLogix_Enabled_Bar_ind, 1);
			}
			else 
			{
				smWrite(VariableIDs.FoamLogix_Enabled_Bar_ind, 0);	 
			}
		}
		else
		{
			EnabledFlashingBarTimer = 0;
			smWrite(VariableIDs.FoamLogix_Enabled_Bar_ind, 0);
			smWrite(VariableIDs.BarGraph_PWM_Value_1, 0);	
		}
		
		
		
		switch (FoamLogixActiveDisplay)
		{
			case 0:	//water flow
					Digit_0 = WaterFlowRate % 10;
					Digit_1 = (WaterFlowRate / 10) % 10;
					Digit_2 = (WaterFlowRate / 100) % 10;
					Digit_3 = (WaterFlowRate / 1000) % 10;
					Digit_4 = (WaterFlowRate / 10000) % 10;
					
					smWrite(VariableIDs.FoamLogix_Decimal_0, 0);
					smWrite(VariableIDs.FoamLogix_Decimal_1, 0);
					
					if (CurrentTankSelected == 0 && EnabledFlashingBarTimer < 6)
					{
						smWrite(VariableIDs.FoamLogix_Digit_4, 17);
						smWrite(VariableIDs.FoamLogix_Digit_3, 18);
						smWrite(VariableIDs.FoamLogix_Digit_2, 19);
						smWrite(VariableIDs.FoamLogix_Digit_1, 20);
						smWrite(VariableIDs.FoamLogix_Digit_0, 21);
					}
					else
					{
						smWrite(VariableIDs.FoamLogix_Digit_0, Digit_0);
						
						if (WaterFlowRate > 9)
							smWrite(VariableIDs.FoamLogix_Digit_1, Digit_1);
						else
							smWrite(VariableIDs.FoamLogix_Digit_1, 255);
						
						if (WaterFlowRate > 99)
							smWrite(VariableIDs.FoamLogix_Digit_2, Digit_2);
						else
							smWrite(VariableIDs.FoamLogix_Digit_2, 255);
							
						if (WaterFlowRate > 999)
							smWrite(VariableIDs.FoamLogix_Digit_3, Digit_3);
						else
							smWrite(VariableIDs.FoamLogix_Digit_3, 255);
							
						if (SimulatedWaterFlowEnabled != 0)
							smWrite(VariableIDs.FoamLogix_Digit_4, 20);
						else
							smWrite(VariableIDs.FoamLogix_Digit_4, 255);
					}				
					break;
					
					
			case 1: //Foam Percentage 
			
					Digit_0 = FoamPercentage % 10;
					Digit_1 = (FoamPercentage / 10) % 10;
					Digit_2 = (FoamPercentage / 100) % 10;
					Digit_3 = 255;
				
					
					
					if (CurrentTankSelected != 0)
					{
						smWrite(VariableIDs.FoamLogix_Decimal_0, 1);
						smWrite(VariableIDs.FoamLogix_Decimal_1, 0);	
						smWrite(VariableIDs.FoamLogix_Digit_0, Digit_0);
						smWrite(VariableIDs.FoamLogix_Digit_1, Digit_1);
	
						if (FoamPercentage > 99)
							smWrite(VariableIDs.FoamLogix_Digit_2, Digit_2);
						else
							smWrite(VariableIDs.FoamLogix_Digit_2, 255);
						
						smWrite(VariableIDs.FoamLogix_Digit_3, 255);
						
						if (CurrentTankSelected == 2) //Tank A
						{
							smWrite(VariableIDs.FoamLogix_Digit_4, 15);		//Put an A in the 10000 digit place
						}
						else
						{
							smWrite(VariableIDs.FoamLogix_Digit_4, 16);		//Put a B in the 10000 digit place
						}
					}
					else
					{
						smWrite(VariableIDs.FoamLogix_Decimal_0, 0);
						smWrite(VariableIDs.FoamLogix_Decimal_1, 0);
						smWrite(VariableIDs.FoamLogix_Digit_4, 17);
						smWrite(VariableIDs.FoamLogix_Digit_3, 18);
						smWrite(VariableIDs.FoamLogix_Digit_2, 19);
						smWrite(VariableIDs.FoamLogix_Digit_1, 20);
						smWrite(VariableIDs.FoamLogix_Digit_0, 21);
					}
					break;
					
					
			case 2:	//total water flowed
					Digit_0 = TotalWaterFlowed % 10;
					Digit_1 = (TotalWaterFlowed / 10) % 10;
					Digit_2 = (TotalWaterFlowed / 100) % 10;
					Digit_3 = (TotalWaterFlowed / 1000) % 10;
					Digit_4 = (TotalWaterFlowed / 10000) % 10;
					
					smWrite(VariableIDs.FoamLogix_Decimal_0, 0);
					smWrite(VariableIDs.FoamLogix_Decimal_1, 0);
						
					if (CurrentTankSelected == 0 && EnabledFlashingBarTimer < 6)
					{
						smWrite(VariableIDs.FoamLogix_Digit_4, 17);
						smWrite(VariableIDs.FoamLogix_Digit_3, 18);
						smWrite(VariableIDs.FoamLogix_Digit_2, 19);
						smWrite(VariableIDs.FoamLogix_Digit_1, 20);
						smWrite(VariableIDs.FoamLogix_Digit_0, 21);
					}
					else
					{
						smWrite(VariableIDs.FoamLogix_Digit_0, Digit_0);
						
						if (TotalWaterFlowed > 9)
							smWrite(VariableIDs.FoamLogix_Digit_1, Digit_1);
						else
							smWrite(VariableIDs.FoamLogix_Digit_1, 255);
						
						if (TotalWaterFlowed > 99)
							smWrite(VariableIDs.FoamLogix_Digit_2, Digit_2);
						else
							smWrite(VariableIDs.FoamLogix_Digit_2, 255);
							
						if (TotalWaterFlowed > 999)
							smWrite(VariableIDs.FoamLogix_Digit_3, Digit_3);
						else
							smWrite(VariableIDs.FoamLogix_Digit_3, 255);
							
						if (TotalWaterFlowed > 9999)
							smWrite(VariableIDs.FoamLogix_Digit_4, Digit_4);
						else
							smWrite(VariableIDs.FoamLogix_Digit_4, 255);
					}
					break;
					
					
			case 3:	//Total foam flowed
					Digit_0 = TotalFoamFlowed % 10;
					Digit_1 = (TotalFoamFlowed / 10) % 10;
					Digit_2 = (TotalFoamFlowed / 100) % 10;
					Digit_3 = (TotalFoamFlowed / 1000) % 10;
					Digit_4 = (TotalFoamFlowed / 10000) % 10;
					
					
					if (CurrentTankSelected != 0)
					{
						smWrite(VariableIDs.FoamLogix_Decimal_0, 0);
						smWrite(VariableIDs.FoamLogix_Decimal_1, 1);
						
						smWrite(VariableIDs.FoamLogix_Digit_0, Digit_0);
						smWrite(VariableIDs.FoamLogix_Digit_1, Digit_1);
						smWrite(VariableIDs.FoamLogix_Digit_2, Digit_2);
	
							
						if (TotalFoamFlowed > 999)
							smWrite(VariableIDs.FoamLogix_Digit_3, Digit_3);
						else
							smWrite(VariableIDs.FoamLogix_Digit_3, 255);
							
						if (CurrentTankSelected == 2) //Tank A
						{
							smWrite(VariableIDs.FoamLogix_Digit_4, 15);		//Put an A in the 10000 digit place
						}
						else
						{
							smWrite(VariableIDs.FoamLogix_Digit_4, 16);		//Put a B in the 10000 digit place
						}
					}
					else
					{
						smWrite(VariableIDs.FoamLogix_Decimal_0, 0);
						smWrite(VariableIDs.FoamLogix_Decimal_1, 0);
						smWrite(VariableIDs.FoamLogix_Digit_4, 17);
						smWrite(VariableIDs.FoamLogix_Digit_3, 18);
						smWrite(VariableIDs.FoamLogix_Digit_2, 19);
						smWrite(VariableIDs.FoamLogix_Digit_1, 20);
						smWrite(VariableIDs.FoamLogix_Digit_0, 21);
					}
					break;
		}	
	}	   
}



//-----ScriptName=SCRIPT_FoamLogixHomeButton
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_FoamLogixHomeButton for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_FoamLogixHomeButton () 
{
  	uint AllowHomeButton;
	smRead(VariableIDs.FoamLogix_AllowHomeButton, AllowHomeButton); 
	
	if (AllowHomeButton == 1)
	{
		sendEvent(EventIDs.Event_GOTO_Preset_Page);
	}
}

//-----ScriptName=SCRIPT_AnalogKnobSelector
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_AnalogKnobSelector for main script method
//---------------------------------------------------------------------------------------------
double Last_FoamA_Percentage = 0.1;
uint Touch_X_Last = 999;
bool InitFoamLogixAnalog = true;
void SCRIPT_AnalogKnobSelector () 
{
	uint FoamSystemDisplayBackground;
	smRead(VariableIDs.FLX_Display_Type, FoamSystemDisplayBackground);
	
	int KnobDegrees;
	uint Touch_X_Start;
	uint Touch_X_Current;
	uint Touch_Y_Current;
	uint Direction = 0;	//1 = CW, 2 = CCW
	
	uint Delta_X = 47;
	uint Delta_Y = 36;
	
	uint KnobIsTouched;
	smRead(VariableIDs.KnobIsTouched, KnobIsTouched);
	
	if (FoamSystemDisplayBackground == 2)
	{
		double FoamA_Percentage;
		smRead(VariableIDs.Primary_FoamPercentage_TankA, FoamA_Percentage);
	
		if (FoamA_Percentage > 1.0)
		{
			FoamA_Percentage = 1.0;
			smWrite(VariableIDs.Primary_FoamPercentage_TankA, FoamA_Percentage);
		}
			
		//First time through - make sure that we don't show the preset screen
		if (InitFoamLogixAnalog == true)
		{
			sendEvent(EventIDs.SCRIPT_GOTO_OperatingScreen);
			InitFoamLogixAnalog = false;
		}
		
		
	
		if (KnobIsTouched == 1)
		{
			
			smRead(VariableIDs.Touch_Current_X, Touch_X_Current);
			smRead(VariableIDs.Touch_Current_Y, Touch_Y_Current);
		
			if (Touch_X_Current >= 124 && Touch_X_Current < (124 + Delta_X) &&  Touch_Y_Current >= 19 && Touch_Y_Current < (19 + Delta_Y))
			{
				FoamA_Percentage = 0.1;	
				KnobDegrees = 0;
			}
			if (Touch_X_Current >= 114 && Touch_X_Current < (114 + Delta_X) &&  Touch_Y_Current >= 64 && Touch_Y_Current < (64 + Delta_Y))
			{
				FoamA_Percentage = 0.2;	
				KnobDegrees = -27;
			}
			if (Touch_X_Current >= 116 && Touch_X_Current < (116 + Delta_X) &&  Touch_Y_Current >= 112 && Touch_Y_Current < (112 + Delta_Y))
			{
				FoamA_Percentage = 0.3;	
				KnobDegrees = -54;
			}
			if (Touch_X_Current >= 148 && Touch_X_Current < (148 + Delta_X) &&  Touch_Y_Current >= 156 && Touch_Y_Current < (156 + Delta_Y))
			{
				FoamA_Percentage = 0.4;	
				KnobDegrees = -81;
			}
			if (Touch_X_Current >= 193 && Touch_X_Current < (193 + Delta_X) &&  Touch_Y_Current >= 170 && Touch_Y_Current < (170 + Delta_Y))
			{
				FoamA_Percentage = 0.5;	
				KnobDegrees = -108;
			}
			if (Touch_X_Current >= 241 && Touch_X_Current < (241 + Delta_X) &&  Touch_Y_Current >= 171 && Touch_Y_Current < (171 + Delta_Y))
			{
				FoamA_Percentage = 0.6;
				KnobDegrees = -135;	
			}
			if (Touch_X_Current >= 286 && Touch_X_Current < (286 + Delta_X) &&  Touch_Y_Current >= 151 && Touch_Y_Current < (151 + Delta_Y))
			{
				FoamA_Percentage = 0.7;
				KnobDegrees = -162;	
			}
			if (Touch_X_Current >= 316 && Touch_X_Current < (316 + Delta_X) &&  Touch_Y_Current >= 112 && Touch_Y_Current < (112 + Delta_Y))
			{
				FoamA_Percentage = 0.8;
				KnobDegrees = -189;	
			}
			if (Touch_X_Current >= 318 && Touch_X_Current < (318 + Delta_X) &&  Touch_Y_Current >= 63 && Touch_Y_Current < (63 + Delta_Y))
			{
				FoamA_Percentage = 0.9;
				KnobDegrees = -216;	
			}
			if (Touch_X_Current >= 307 && Touch_X_Current < (307 + Delta_X) &&  Touch_Y_Current >= 18 && Touch_Y_Current < (18 + Delta_Y))
			{
				FoamA_Percentage = 1.0;	
				KnobDegrees = -243;
			}
				
			if (Last_FoamA_Percentage != FoamA_Percentage)
			{
				smWrite(VariableIDs.Primary_FoamPercentage_TankA, FoamA_Percentage);
			

							
				screenApiRotate(ScreenObjectIDs.AnalogKnobImage, KnobDegrees, 241, 90);
				screenApiRotate(ScreenObjectIDs.AnalogKnobImage_VERT, KnobDegrees, 228, 135);
				processScreenApi();
				
				
				Last_FoamA_Percentage = FoamA_Percentage;
			}
		}
		else
		{
			if (FoamA_Percentage != Last_FoamA_Percentage)
			{
				if (FoamA_Percentage == 0.1)
				{
					KnobDegrees = 0;
				}
				if (FoamA_Percentage == 0.2)
				{
					KnobDegrees = -27;
				}
				if (FoamA_Percentage == 0.3)
				{
					KnobDegrees = -54;
				}
				if (FoamA_Percentage == 0.4)
				{
					KnobDegrees = -81;
				}
				if (FoamA_Percentage == 0.5)
				{
					KnobDegrees = -108;
				}
				if (FoamA_Percentage == 0.6)
				{
					KnobDegrees = -135;	
				}
				if (FoamA_Percentage == 0.7)
				{
					KnobDegrees = -162;	
				}
				if (FoamA_Percentage == 0.8)
				{
					KnobDegrees = -189;	
				}
				if (FoamA_Percentage == 0.9)
				{
					KnobDegrees = -216;	
				}
				if (FoamA_Percentage == 1.0)
				{
					KnobDegrees = -243;
				}
				
				screenApiRotate(ScreenObjectIDs.AnalogKnobImage_VERT, KnobDegrees, 228, 135);
				screenApiRotate(ScreenObjectIDs.AnalogKnobImage, KnobDegrees, 241, 90);	
				processScreenApi();
				
				Last_FoamA_Percentage = FoamA_Percentage;
			}
		}
	}
}

//-----ScriptName=SCRIPT_RemoteOperations_CAN
/*
	This script is fired by a reception of the CAN message 0x18EFD058.
	Byte 0 is the command:
		1 = turn ON foam
		2 = turn OFF foam
		3 = increase foam percentage
		4 = decrease foam percentage
		5 = zero water flow total
		6 = zero foam flow total
		13 = set simulated flow (byte 4 - 1 = ON, 0 = OFF)
		15 = zero water and foam totals
*/
void SCRIPT_RemoteOperations_CAN () 
{
 	uint RemoteCommand_byte0;
	uint RemoteCommand_byte4;
	
	smRead(VariableIDs.RemoteOperation_Byte_0, RemoteCommand_byte0);
	smRead(VariableIDs.RemoteOperation_Byte_4, RemoteCommand_byte4); 
	
		 /*	The current power state should be 1 of 4 values:
		0 = OFF
		1 = ON in Foam A
		2 = ON in Foam B
		3 = ON in FLUSH
	*/
	uint CurrentPowerState;
	smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
	
	

	switch(RemoteCommand_byte0)
	{
		case 1:		// turn ON foam
					if (CurrentPowerState == 0)
					{
						sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Primary);
					}
					break;
		case 2: 	// turn OFF foam
					if (CurrentPowerState != 0)
					{
						sendEvent(EventIDs.SCRIPT_FoamLogixPowerButton_Primary);
					}
					break;
		case 3: 	// increase foam percentage
					sendEvent(EventIDs.SCRIPT_Primary_FoamPercentage_Increase);
					break;
		case 4:		// decrease foam percentage
					sendEvent(EventIDs.SCRIPT_Primary_FoamPercentage_Decrease);
					break;
		case 5:		// zero water flow total
					sendEvent(EventIDs.Event_Zeroize_Primary_Total_Water_Flowed);
					break;
		case 6:		// zero foam flow total
					sendEvent(EventIDs.SCRIPT_ZeroizeFoamTotal);
					break;
		case 13:	// set simulated flow (byte 4 - 1 = ON, 0 = OFF)
					if (RemoteCommand_byte4 == 1)
						sendEvent(EventIDs.Calculate_Primary_SimulatedWaterFlow_Enabled_Equals_ON);
					else
						sendEvent(EventIDs.Calculate_Primary_SimulatedWaterFlow_Enabled_Equals_OFF);
					break;
		case 15:	// zero water and foam totals
					sendEvent(EventIDs.SCRIPT_ZeroizeFoamTotal);
					sendEvent(EventIDs.Event_Zeroize_Primary_Total_Water_Flowed);
					break;
	} 
}

//-----ScriptName=SCRIPT_MaintenanceIntervals
/**********************************************************************************
Maintenance Interval Script
This script is run every 360 seconds (6 minutes, 1/10 hour).
**********************************************************************************/
uint[] DaysInMonth = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
uint CurrentMonth;
uint CurrentYear;
uint CurrentDay;
bool AlreadyResetThisPowerCycle = false;
void SCRIPT_MaintenanceIntervals () 
{

	uint MaintenanceItemsDue;
	smRead(VariableIDs.MaintenanceItemsDue, MaintenanceItemsDue);	//get the state of the current maintenance items due variable
	
	uint HasSystemBeenRun;
	smRead(VariableIDs.HasSystemBeenRun, HasSystemBeenRun);

	
	//**********************************************************************************
	//	Items that are based on real time.
	//********************************************************************************** 
	smRead(VariableIDs.RealTimeClock_Year, CurrentYear);
	smRead(VariableIDs.RealTimeClock_Month, CurrentMonth); 
	smRead(VariableIDs.RealTimeClock_Day, CurrentDay);
	
	
	
	/*
		Make sure that the 3M and 12M intervals have a date loaded.  When the configuration
		is first loaded these will contain 0's.  This means that they have not been set - so
		set them 12M and 3M from the current date.
	*/
	uint _1M_Interval_Year;
	uint _1M_Interval_Day;
	uint _1M_Interval_Month;
	uint _2M_Interval_Year;
	uint _2M_Interval_Day;
	uint _2M_Interval_Month;
	uint _12M_Interval_Year;
	uint _12M_Interval_Day;
	uint _12M_Interval_Month;

	
	smRead(VariableIDs.DateInterval_1M_Day, _1M_Interval_Day);
	smRead(VariableIDs.DateInterval_1M_Month, _1M_Interval_Month);
	smRead(VariableIDs.DateInterval_1M_Year, _1M_Interval_Year);
	
	smRead(VariableIDs.DateInterval_2M_Day, _2M_Interval_Day);
	smRead(VariableIDs.DateInterval_2M_Month, _2M_Interval_Month);
	smRead(VariableIDs.DateInterval_2M_Year, _2M_Interval_Year);
	
	smRead(VariableIDs.DateInterval_12M_Day, _12M_Interval_Day);
	smRead(VariableIDs.DateInterval_12M_Month, _12M_Interval_Month);
	smRead(VariableIDs.DateInterval_12M_Year, _12M_Interval_Year);



	uint NumDaysRemaining = 0;
	
	//Update the 1Month text - Convert the day, month, year variables into strings
	NumDaysRemaining = CalculateDaysRemaining(_1M_Interval_Day, _1M_Interval_Month, _1M_Interval_Year);
	smWrite(VariableIDs.DateInterval_1M_NumDays, NumDaysRemaining);
	if (NumDaysRemaining == 0)
	{
		MaintenanceItemsDue = 1;
	}
	
	//Update the 3Month text - Convert the day, month, year variables into strings
	NumDaysRemaining = CalculateDaysRemaining(_2M_Interval_Day, _2M_Interval_Month, _2M_Interval_Year);
	smWrite(VariableIDs.DateInterval_2M_NumDays, NumDaysRemaining);
	if (NumDaysRemaining == 0)
	{
		MaintenanceItemsDue = 1;
	}
	
	
	//Update the 12Month text - Convert the day, month, year variables into strings
	NumDaysRemaining = CalculateDaysRemaining(_12M_Interval_Day, _12M_Interval_Month, _12M_Interval_Year);
	smWrite(VariableIDs.DateInterval_12M_NumDays, NumDaysRemaining);
	if (NumDaysRemaining == 0)
	{
		MaintenanceItemsDue = 1;
	}
	
	//Update the maintenance items due system variable so that the USER can see there needs some action
	smWrite(VariableIDs.MaintenanceItemsDue, MaintenanceItemsDue);	
	
	/*
		If the system has not been run - then reset all of the maintenance minder dates.
		To minimize the potential number of memory writes - I am using a timer "DailyTimer" which
		is increment each time through this script.  This script is called once every 6 minutes (360,000 ms)
		therefore if I wait for the timer to meet 240 then that will be once per day.
		240 * 6 minutes = 1,440 minutes (1440 / 60 minutes = 24 hours).
		
		4-26-2019 - I changed this from using the 24 hour timer since no system would be idle for that long and it 
		never cleared the timers.  Now I will do it on a power cycle - if the system has never been run (variable still 0) then
		we will reset the interval timers once per power cycle.
	*/
	if (HasSystemBeenRun == 0 && AlreadyResetThisPowerCycle == false)
	{
		SCRIPT_Reset_1M_Interval();
		SCRIPT_Reset_2M_Interval();
		SCRIPT_Reset_12M_Interval();
		AlreadyResetThisPowerCycle = true;		
	}
}









/*	Take the interval date and return the number of days until that date is due.  This is mainly
	used to update the system variable that is used to show the pie chart on the maintenance screen.
*/
uint CalculateDaysRemaining(uint EndDay, uint EndMonth, uint EndYear)
{

	uint thisCurrentYear;
	uint thisCurrentMonth;
	uint thisCurrentDay;
	
	smRead(VariableIDs.RealTimeClock_Year, thisCurrentYear);
	smRead(VariableIDs.RealTimeClock_Month, thisCurrentMonth); 
	smRead(VariableIDs.RealTimeClock_Day, thisCurrentDay);
	
	//Make certain that we can't have a bad input into this function!
	if (EndDay < 1 || EndDay > 31)
		EndDay = 1;
	if (EndMonth < 1 || EndMonth > 12)
		EndMonth = 1;
	if (EndYear < 1970)
		EndYear = 2015;
	
	uint[][] shortDays = {  {   0,  31,  60,  91, 121, 152, 182, 213, 244, 274, 305, 335},
						    { 366, 397, 425, 456, 486, 517, 547, 578, 609, 639, 670, 700},
						    { 731, 762, 790, 821, 851, 882, 912, 943, 974,1004,1035,1065},
						    {1096,1127,1155,1186,1216,1247,1277,1308,1339,1369,1400,1430} };
							
	uint NumberOfDays	= 0;
	uint Epoch_Current;
	uint Epoch_Interval;
	
	
	Epoch_Current = (thisCurrentYear/4*(365*4+1)+ shortDays[thisCurrentYear%4][thisCurrentMonth-1] + thisCurrentDay);
	Epoch_Interval = (EndYear/4*(365*4+1)+ shortDays[EndYear%4][EndMonth-1] + EndDay);
	

	if (Epoch_Interval > Epoch_Current)
	{
		NumberOfDays = Epoch_Interval - Epoch_Current;
	}
	else
	{
		NumberOfDays = 0;
	}

	return NumberOfDays;		
}











void UpdateMaintenanceIntervalTextDates()
{

	uint UnitOfMeasure;
	smRead(VariableIDs.FLX_UnitOfMeasure, UnitOfMeasure);
	
	//**********************************************************************************
	//	Items that are based on real time.
	//********************************************************************************** 
	smRead(VariableIDs.RealTimeClock_Year, CurrentYear);
	smRead(VariableIDs.RealTimeClock_Month, CurrentMonth); 
	smRead(VariableIDs.RealTimeClock_Day, CurrentDay);
	
	/*
		Make sure that the 3M and 12M intervals have a date loaded.  When the configuration
		is first loaded these will contain 0's.  This means that they have not been set - so
		set them 12M and 3M from the current date.
	*/
	uint _1M_Interval_Year;
	uint _1M_Interval_Day;
	uint _1M_Interval_Month;
	uint _2M_Interval_Year;
	uint _2M_Interval_Day;
	uint _2M_Interval_Month;
	uint _12M_Interval_Year;
	uint _12M_Interval_Day;
	uint _12M_Interval_Month;
	
	smRead(VariableIDs.DateInterval_1M_Day, _1M_Interval_Day);
	smRead(VariableIDs.DateInterval_1M_Month, _1M_Interval_Month);
	smRead(VariableIDs.DateInterval_1M_Year, _1M_Interval_Year);
	
	smRead(VariableIDs.DateInterval_2M_Day, _2M_Interval_Day);
	smRead(VariableIDs.DateInterval_2M_Month, _2M_Interval_Month);
	smRead(VariableIDs.DateInterval_2M_Year, _2M_Interval_Year);
	
	smRead(VariableIDs.DateInterval_12M_Day, _12M_Interval_Day);
	smRead(VariableIDs.DateInterval_12M_Month, _12M_Interval_Month);
	smRead(VariableIDs.DateInterval_12M_Year, _12M_Interval_Year);

	
//******CHECK FOR INTERVALS THAT HAVE NEVER BEEN SET - AND SET THEM TO THE CURRENT TIME********************	
	if (_1M_Interval_Year == 0 && _1M_Interval_Day == 0 && _1M_Interval_Month == 0)
	{
		sendEvent(EventIDs.SCRIPT_Reset_1M_Interval);
	}
	
	if (_2M_Interval_Year == 0 && _2M_Interval_Day == 0 && _2M_Interval_Month == 0)
	{
		sendEvent(EventIDs.SCRIPT_Reset_2M_Interval);
	}
	
	if (_12M_Interval_Year == 0 && _12M_Interval_Day == 0 && _12M_Interval_Month == 0)
	{
		sendEvent(EventIDs.SCRIPT_Reset_12M_Interval);	
	}

//*********************************************************************************************************	
	
	
	
	//Update the string that will be displayed on the maintenance screen
	string tempDate = "";
	string str_Day;
	string str_Month;
	string str_Year;
	uint NumDaysRemaining = 0;
	uint MaintenanceItemsDue = 0;
	
	

	//Update the 1Month text - Convert the day, month, year variables into strings
	str_Year 	= _1M_Interval_Year;
	str_Month 	= _1M_Interval_Month;
	str_Day 	= _1M_Interval_Day;

	NumDaysRemaining = CalculateDaysRemaining(_1M_Interval_Day, _1M_Interval_Month, _1M_Interval_Year);
	smWrite(VariableIDs.DateInterval_1M_NumDays, NumDaysRemaining);
	if (NumDaysRemaining == 0)
	{
		MaintenanceItemsDue = 1;
	}
	
	if (UnitOfMeasure == 0)	//english
		tempDate = str_Month + "/" + str_Day + "/" + str_Year;	
	else
		tempDate = str_Day + "/" + str_Month + "/" + str_Year;	
	cultureSetString(StringIDs._1MonthIntervalDate, tempDate, true);
	
	
	//Update the 2Month text - Convert the day, month, year variables into strings
	str_Year 	= _2M_Interval_Year;
	str_Month 	= _2M_Interval_Month;
	str_Day 	= _2M_Interval_Day;

	NumDaysRemaining = CalculateDaysRemaining(_2M_Interval_Day, _2M_Interval_Month, _2M_Interval_Year);
	smWrite(VariableIDs.DateInterval_2M_NumDays, NumDaysRemaining);
	if (NumDaysRemaining == 0)
	{
		MaintenanceItemsDue = 1;
	}
	
	if (UnitOfMeasure == 0)	//english
		tempDate = str_Month + "/" + str_Day + "/" + str_Year;	
	else
		tempDate = str_Day + "/" + str_Month + "/" + str_Year;	
	cultureSetString(StringIDs._2MonthIntervalDate, tempDate, true);
	
	
	//Update the 12Month text - Convert the day, month, year variables into strings
	str_Year 	= _12M_Interval_Year;
	str_Month 	= _12M_Interval_Month;
	str_Day 	= _12M_Interval_Day;

	NumDaysRemaining = CalculateDaysRemaining(_12M_Interval_Day, _12M_Interval_Month, _12M_Interval_Year);
	smWrite(VariableIDs.DateInterval_12M_NumDays, NumDaysRemaining);
	if (NumDaysRemaining == 0)
	{
		MaintenanceItemsDue = 1;
	}
	
	if (UnitOfMeasure == 0)	//english
		tempDate = str_Month + "/" + str_Day + "/" + str_Year;
	else
		tempDate = str_Day + "/" + str_Month + "/" + str_Year;	
	cultureSetString(StringIDs._12MonthIntervalDate , tempDate, true);
	
	
	
	//Update the maintenance items due system variable so that the USER can see there needs some action
	smWrite(VariableIDs.MaintenanceItemsDue, MaintenanceItemsDue);
}


//-----ScriptName=SCRIPT_Reset_2M_Interval
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_Reset_2M_Interval for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Reset_2M_Interval () 
{
 	//**********************************************************************************
	//	Items that are based on real time.
	//********************************************************************************** 
	smRead(VariableIDs.RealTimeClock_Year, CurrentYear);
	smRead(VariableIDs.RealTimeClock_Month, CurrentMonth); 
	smRead(VariableIDs.RealTimeClock_Day, CurrentDay);
	
	uint _2M_Interval_Year;
	uint _2M_Interval_Day;
	uint _2M_Interval_Month;

	_2M_Interval_Day 	= CurrentDay;
	_2M_Interval_Month 	= CurrentMonth;
	_2M_Interval_Year 	= CurrentYear;
	
	_2M_Interval_Month 	= _2M_Interval_Month - 1;			//Months are 1-12, but we have to format it as 0-11 for use with the MODULUS in the next step.
	_2M_Interval_Month 	= (_2M_Interval_Month + 2) % 12;	//Add two months and then modulus by 12 for wrapping to the next years month.
	_2M_Interval_Month 	= _2M_Interval_Month + 1;			//Re-format back to 1-12
	
	if (CurrentDay > DaysInMonth[_2M_Interval_Month])		//If we happened to be in a month with less days than the current day then move to the next month.
	{
		_2M_Interval_Month++;
		if (_2M_Interval_Month > 12)
			_2M_Interval_Month = 1;
			
		_2M_Interval_Day = 1;
	}
	
	if (_2M_Interval_Month < CurrentMonth)					//If the new 2M interval month is less than the current month it means we wrapped to the next year
	{
		_2M_Interval_Year++;
	}
	
	uint NumDaysRemaining;
	NumDaysRemaining = CalculateDaysRemaining(_2M_Interval_Day, _2M_Interval_Month, _2M_Interval_Year);
	smWrite(VariableIDs.DateInterval_2M_NumDays, NumDaysRemaining);
	
	smWrite(VariableIDs.DateInterval_2M_Day, _2M_Interval_Day);
	smWrite(VariableIDs.DateInterval_2M_Month, _2M_Interval_Month);
	smWrite(VariableIDs.DateInterval_2M_Year, _2M_Interval_Year); 
	
	sendEvent(EventIDs.Event_Save_2M_Interval);	
	
	UpdateMaintenanceIntervalTextDates();
}

//-----ScriptName=SCRIPT_Reset_12M_Interval
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_Reset_12M_Interval for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Reset_12M_Interval () 
{
 	//**********************************************************************************
	//	Items that are based on real time.
	//********************************************************************************** 
	smRead(VariableIDs.RealTimeClock_Year, CurrentYear);
	smRead(VariableIDs.RealTimeClock_Month, CurrentMonth); 
	smRead(VariableIDs.RealTimeClock_Day, CurrentDay);
	
	uint _12M_Interval_Year;
	uint _12M_Interval_Day;
	uint _12M_Interval_Month;
	
	_12M_Interval_Day 	= CurrentDay;
	_12M_Interval_Month = CurrentMonth;
	_12M_Interval_Year 	= CurrentYear + 1;					//For the 12 month interval we just need to increase the year
	
	uint NumDaysRemaining;
	NumDaysRemaining = CalculateDaysRemaining(_12M_Interval_Day, _12M_Interval_Month, _12M_Interval_Year);
	smWrite(VariableIDs.DateInterval_12M_NumDays, NumDaysRemaining);
	
	smWrite(VariableIDs.DateInterval_12M_Day, _12M_Interval_Day);
	smWrite(VariableIDs.DateInterval_12M_Month, _12M_Interval_Month);
	smWrite(VariableIDs.DateInterval_12M_Year, _12M_Interval_Year);
	
	sendEvent(EventIDs.Event_Save_12M_Interval);   
	
	UpdateMaintenanceIntervalTextDates();
}

//-----ScriptName=SCRIPT_ShowMaintenanceMinderAtInit
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_ShowMaintenanceMinderAtInit for main script method
//---------------------------------------------------------------------------------------------

void SCRIPT_ShowMaintenanceMinderAtInit () 
{

	UpdateMaintenanceIntervalTextDates();
	
}

//-----ScriptName=SCRIPT_Reset_1M_Interval
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_Reset_1M_Interval for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_Reset_1M_Interval () 
{


  	//**********************************************************************************
	//	Items that are based on real time.
	//********************************************************************************** 
	smRead(VariableIDs.RealTimeClock_Year, CurrentYear);
	smRead(VariableIDs.RealTimeClock_Month, CurrentMonth); 
	smRead(VariableIDs.RealTimeClock_Day, CurrentDay);
	
	uint _1M_Interval_Year;
	uint _1M_Interval_Day;
	uint _1M_Interval_Month;

	_1M_Interval_Day 	= CurrentDay;
	_1M_Interval_Month 	= CurrentMonth;
	_1M_Interval_Year 	= CurrentYear;
	
	_1M_Interval_Month 	= _1M_Interval_Month - 1;			//Months are 1-12, but we have to format it as 0-11 for use with the MODULUS in the next step.
	_1M_Interval_Month 	= (_1M_Interval_Month + 1) % 12;	//Add one months and then modulus by 12 for wrapping to the next years month.
	_1M_Interval_Month 	= _1M_Interval_Month + 1;			//Re-format back to 1-12

	if (CurrentDay > DaysInMonth[_1M_Interval_Month])		//If we happened to be in a month with less days than the current day then move to the next month.
	{
		_1M_Interval_Month++;
		if (_1M_Interval_Month > 12)
			_1M_Interval_Month = 1;
			
		_1M_Interval_Day = 1;
	}
	
	if (_1M_Interval_Month < CurrentMonth)					//If the new 1M interval month is less than the current month it means we wrapped to the next year
	{
		_1M_Interval_Year++;
	}
	

	uint NumDaysRemaining;
	NumDaysRemaining = CalculateDaysRemaining(_1M_Interval_Day, _1M_Interval_Month, _1M_Interval_Year);
	

	smWrite(VariableIDs.DateInterval_1M_NumDays, NumDaysRemaining);	
	smWrite(VariableIDs.DateInterval_1M_Day, _1M_Interval_Day);
	smWrite(VariableIDs.DateInterval_1M_Month, _1M_Interval_Month);
	smWrite(VariableIDs.DateInterval_1M_Year, _1M_Interval_Year);
	
	sendEvent(EventIDs.Event_Save_1M_Interval);	

	UpdateMaintenanceIntervalTextDates();  
}

//-----ScriptName=SCRIPT_EvaluateFlushRequired
/***********************************************************************************************
	EvaluateFlushRequired()
	
	This function sets the script variable "TypeFoamInLines" to the appropriate value:
		FOAM_A_INLINE
		FOAM_B_INLINE
		NO_FOAM_INLINE
	These let other functions know what foam is currently in the plumbing so that a 
	"must flush" warning may be issued if the user attempts to change tanks.
	
***********************************************************************************************/
const uint FOAM_A_INLINE 	= 2;	//Used to set which foam is currently in the plumbing.	
const uint FOAM_B_INLINE 	= 1;
const uint NO_FOAM_INLINE 	= 0;
uint TypeFoamInLines;	//0 = none, 1 = B, 2 = A
uint FlushPerfomingTimer 	= 0;				//Automatic flush must happen for 3 seconds - this is its timer.
bool InformedUser_FoamFlushRequired = false;	//Let's us know if we have popped up a warning to the user that FLUSH is required before changing tanks.
bool FoamFlowingForAtLeastThreeSeconds = false;	//Used to determine when foam has been flowing for 3 or more seconds.
uint FoamFlowingTimer		= 0;	//Updated in calculate motor PWM script

void SCRIPT_EvaluateFlushRequired () 
{
 	uint Primary_WaterFlowRateGPM = 0;
	uint Secondary_WaterFlowRateGPM = 0;
	smRead(VariableIDs.Primary_WaterFlowRate_GPM, Primary_WaterFlowRateGPM);
	//smRead(VariableIDs.Secondary_WaterFlowRate_GPM, Secondary_WaterFlowRateGPM);
	
	
	smRead(VariableIDs.CurrentFoamTypeInLine, TypeFoamInLines);

	CheckForRunningTankChange();	
	
		
	/*	The current power state should be 1 of 4 values:
		0 = OFF
		1 = ON in Foam A
		2 = ON in Foam B
		3 = ON in FLUSH
	*/
	uint Primary_CurrentPowerState 		= 0;
	uint Secondary_CurrentPowerState 	= 0;
	smRead(VariableIDs.Primary_FoamLogix_Enabled, Primary_CurrentPowerState);
	smRead(VariableIDs.Secondary_FoamLogix_Enabled, Secondary_CurrentPowerState);
	
	
	/* 	The foam selector type is used to determine how we should handle the flush requirement.
		With the MST/MDT we have to inform the user to select flush first.
		With the ADT we can automatically set the system to flush before changing the tank.	
		
		Currently the SmartFOAM does not control the ADT system - so handle them like it was an MDT - manually.
	*/
	uint FoamTankSelectorType;
	smRead(VariableIDs.FLX_FoamTankSelectorType, FoamTankSelectorType);

	/*	Update the TypeFoamInLines.  This will let us keep track of what foam may be in the lines based
		on the last run by using the value of the CurrentPowerState:
			0 = OFF				(don't care)
			1 = ON in Foam A	
			2 = ON in Foam B
			3 = ON in FLUSH		(we can clear the TypeFoamInLines if it has been run for 3 seconds)
			
		FoamFlowingForAtLeastThreeSeconds is set in SCRIPT CalculateMotorPWM when the foam system
		has been flowing foam for 3 seconds.  We can use this to update the TypeFoamInLines variable.
		This variable is only updated for foam A or B flowing - it does not get set when FLUSH has
		been run for at least three seconds.
	*/
	if (FoamFlowingForAtLeastThreeSeconds == true)
	{
		//Only update the foam in the lines if it has been cleared previosly.
		if (TypeFoamInLines == NO_FOAM_INLINE)
		{
			switch(Primary_CurrentPowerState)
			{
				case 1:	//foam A
						TypeFoamInLines = FOAM_A_INLINE;
						break;
				case 2:	//foam B
						TypeFoamInLines = FOAM_B_INLINE;
						break;
			}	
		}		
	}
	
	
	smWrite(VariableIDs.CurrentFoamTypeInLine, TypeFoamInLines);

	

	/*	FLUSH condition satisfied checker.  Once the FLUSH operation has been run for 3 seconds then it
		will allow the reset of the InformedUser_FoamFlushRequired variable and clear the warning.
	
	uint FLUSH_TIME_LIMIT = 200;	//20 seconds - this script runs every 100ms so 200 = 20 seconds
	if (Primary_CurrentPowerState == 3 && Primary_WaterFlowRateGPM > 0) //3 = ON in FLUSH - this is only a 3 when the system is actually running.
	{
		FlushPerfomingTimer++;	//This timer increments every 100ms

		if (FlushPerfomingTimer > FLUSH_TIME_LIMIT)
		{
			FlushPerfomingTimer = 0;
			TypeFoamInLines 	= NO_FOAM_INLINE;	//The lines are now cleared.
			sendEvent(EventIDs.Warning_FlushIsComplete);
			
			//These items only need to be performed if a user tried to run the system before flushing.
			if (InformedUser_FoamFlushRequired == true)
			{
				InformedUser_FoamFlushRequired = false;	//clear the foam flush requirement
				TurnFoamSystem_OFF(0);
			}		
		}
	}
	else
	{
		FlushPerfomingTimer = 0;
	} 
	*/  
}








/***********************************************************************************************
	CheckForRunningTankChange()
	
	This function is only applicable if the tank selector is an MST/MDT type.  It checks for 
	a tank selection change (reported by the DC Motor Driver) and if there is one it attempts
	to set the correct operation screen - TANK A, TANK B, or FLUSH.
	
	If the one of the foam tanks is running and the user attempts to select the other tank it
	causes a "must go to flush first" warning.
***********************************************************************************************/
uint LastCurrentTankSelected = 5;
bool InitializeCurrentTankSelected = true;
uint InitializeTimer = 0;
uint ADT_AutomaticChangeTimer = 0;
void CheckForRunningTankChange()
{
	/*	If we are not on the start page and we are using a manual tank selector (0) then 
		update the current operating page based on the reported tank selected value.
	*/
	/*	The current power state should be 1 of 4 values:
		0 = OFF
		1 = ON in Foam A
		2 = ON in Foam B
		3 = ON in FLUSH
	*/
	uint Primary_CurrentPowerState 		= 0;
	uint Secondary_CurrentPowerState 	= 0;
	smRead(VariableIDs.Primary_FoamLogix_Enabled, Primary_CurrentPowerState);
	smRead(VariableIDs.Secondary_FoamLogix_Enabled, Secondary_CurrentPowerState);
		
	
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT UpdateFoamLogixBackground.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	uint CurrentTankSelected;
	smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	
	uint OnPresetStartPage;
	smRead(VariableIDs.OnPresetStartPage, OnPresetStartPage);
	
	uint FoamTankSelectorType;
	smRead(VariableIDs.FLX_FoamTankSelectorType, FoamTankSelectorType);
	
	/*	Initalize the script variable "LastCurrentTankSelected" to the system variable
		"CurrentTankSelected" the first time this script is run.  These variables are
		used to determine when the manual tank selector has changed state.
	*/
	if (InitializeCurrentTankSelected == true)
	{
		if (++InitializeTimer > 50)
		{
			LastCurrentTankSelected = CurrentTankSelected;
			InitializeCurrentTankSelected = false;
		}
	}
	
	
	 
	/*	If we are on the operation page (OnPresetStartPage = 0) and we are using a MST/MDT type tank
		selector (FoamTankSelectorType = 0 or 1).
		*****SmartFOAM currently does not support ADT internally so all foam selectors are treated alike.  Comment out foam selector type below
	*/
//	if (OnPresetStartPage == 0)	// && (FoamTankSelectorType == 0 || FoamTankSelectorType == 1))
//	{
		
		if (LastCurrentTankSelected != CurrentTankSelected)
		{
			//sendEvent(EventIDs.Clear_Warning_FlushMustBePerformed);	REMOVED 2-19-2016 because this caused the MENU layer to change to clear when the tank select was changed

			/*	If there is foam in the line then check to see if we have selected the opposite foam tank - if so, we
				need to stop operation. If we have selected FLUSH operation then we don't need to stop.
			*/
			if (TypeFoamInLines != NO_FOAM_INLINE)
			{
				if ( CurrentTankSelected != TypeFoamInLines && CurrentTankSelected != 0)
				{
					TurnFoamSystem_OFF(0);
					sendEvent(EventIDs.Warning_ShowFlushRequired);
					InformedUser_FoamFlushRequired = true;
				}
			}
				

			
			/*	If we are running in FLUSH and then swap over to tank A or tank B - update the current power state accordingly.
			*/
			if (Primary_CurrentPowerState == 3)
			{
				if (TypeFoamInLines == NO_FOAM_INLINE)
				{
					if (CurrentTankSelected == 1)	//B
					{
						Primary_CurrentPowerState = 2;	//2 = ON in B
						smWrite(VariableIDs.Primary_FoamLogix_Enabled, Primary_CurrentPowerState);
					}
					if (CurrentTankSelected == 2)	//A
					{
						Primary_CurrentPowerState = 1;	//1 = ON in A
						smWrite(VariableIDs.Primary_FoamLogix_Enabled, Primary_CurrentPowerState);
					}
				}
				else
				{
					if ( CurrentTankSelected != TypeFoamInLines && CurrentTankSelected != 0)
					{
						TurnFoamSystem_OFF(0);
						sendEvent(EventIDs.Warning_ShowFlushRequired);
						InformedUser_FoamFlushRequired = true;
					}	
				}
			}
			
			LastCurrentTankSelected = CurrentTankSelected;
		} 
}

//-----ScriptName=SCRIPT_PrimarySimFlow_INCR
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_PrimarySimFlow_INCR for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_PrimarySimFlow_INCR () 
{
   	uint SimulatedFlowState;
   
   	smRead(VariableIDs.Primary_SimulatedWaterFlow_Enabled, SimulatedFlowState);
   
   	if (SimulatedFlowState == 1)
   	{
   		sendEvent(EventIDs.Calculate_Primary_SimulatedFlowGPM_Equals_INCR);	
   	}
}

//-----ScriptName=SCRIPT_PrimarySimFlow_DECR
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_PrimarySimFlow_DECR for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_PrimarySimFlow_DECR () 
{
	   uint SimulatedFlowState;
   
   	smRead(VariableIDs.Primary_SimulatedWaterFlow_Enabled, SimulatedFlowState);
   
   	if (SimulatedFlowState == 1)
   	{
   		sendEvent(EventIDs.Calculate_Primary_SimulatedFlowGPM_Equals_DECR);	
   	}    
}

//-----ScriptName=SCRIPT_UpdatePerformanceChart
/**************************************************************************************
Update the Performance chart based on the motor/pump type selected.

This also sets the LIMP HOME flow rate variable.  This is used when the system can't
detect the foam flow pulses anymore - like the sensor has failed - and so it then uses
the look-up table to determine PWM versus flowrate which had been hand tabulated for
different motor/pumps every 5% at 150 PSI.
***************************************************************************************/
//double[] LimpHome_FlowRate(20);
//double[] LimpHome_FlowRate_2(20);

double[][] LimpHome_FlowRate = {  	{0.05,  0.15,  0.34,  0.56, 0.88, 1.21, 1.57, 1.81, 2.10, 2.25, 2.46, 2.64, 2.80, 2.85, 2.95, 3.04, 3.09, 3.18, 3.23, 3.30},
								    {0.05,  0.15,  0.34,  0.56, 0.88, 1.21, 1.57, 1.81, 2.10, 2.25, 2.46, 2.64, 2.80, 2.85, 2.95, 3.04, 3.09, 3.18, 3.23, 3.30} 	};


void SCRIPT_UpdatePerformanceChart () 
{
  	uint PumpModel;
	smRead(VariableIDs.FLX_PumpModel, PumpModel);
	
	uint PumpModel_2;
	smRead(VariableIDs.FLX_PumpModel_2, PumpModel_2);
	
	uint UnitOfMeasure;
	smRead(VariableIDs.FLX_UnitOfMeasure, UnitOfMeasure);
	
	
	UpdateLimpHomeTable(PumpModel, 0, UnitOfMeasure);
	UpdateLimpHomeTable(PumpModel_2, 1, UnitOfMeasure);
	
	
	if (UnitOfMeasure == 0)	//English units
	{
		switch (PumpModel)
		{
			case 0: //1.7 12V
			case 6:	if (PumpModel == 0)
					{	
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 1.7AHP, 12V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 1.7AHP, 12V");
					}
					else
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 1.7AHP, 24V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 1.7AHP, 24V");
					}
						
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01, "1,700 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02, "850 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03, "567 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05, "340 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10, "170 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30, "57 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60, "28 GPM");
					
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01_VERT, "1,700 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02_VERT, "850 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03_VERT, "567 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05_VERT, "340 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10_VERT, "170 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30_VERT, "57 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60_VERT, "28 GPM");
					
					break;
			case 1: //2.1 12V
			case 7:	if (PumpModel == 1)
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 2.1A, 12V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 2.1A, 12V");
					}
					else
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 2.1A, 24V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 2.1A, 24V");
					}
						
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01, "2,100 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02, "1,050 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03, "700 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05, "420 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10, "210 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30, "70 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60, "35 GPM");
					
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01_VERT, "2,100 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02_VERT, "1,050 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03_VERT, "700 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05_VERT, "420 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10_VERT, "210 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30_VERT, "70 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60_VERT, "35 GPM");
					
					break;
			case 2: //3.3 12V
			case 8:	if (PumpModel == 2)
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 3.3, 12V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 3.3, 12V");
					}
					else
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 3.3, 24V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 3.3, 24V");
					}
						
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01, "3,300 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02, "1,650 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03, "1,100 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05, "660 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10, "330 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30, "110 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60, "55 GPM");
					
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01_VERT, "3,300 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02_VERT, "1,650 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03_VERT, "1,100 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05_VERT, "660 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10_VERT, "330 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30_VERT, "110 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60_VERT, "55 GPM");

					break;
			case 3: //5.0 12V
			case 9:	if (PumpModel == 3)
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 5.0, 12V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 5.0, 12V");
					}
					else
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 5.0, 24V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 5.0, 24V");
					}
						
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01, "5,000 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02, "2,500 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03, "1,667 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05, "1,000 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10, "500 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30, "167 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60, "83 GPM");
					
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01_VERT, "5,000 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02_VERT, "2,500 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03_VERT, "1,667 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05_VERT, "1,000 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10_VERT, "500 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30_VERT, "167 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60_VERT, "83 GPM");

					break;
			case 4: //6.5 12V
			case 10:if (PumpModel == 4)
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 6.5, 12V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 6.5, 12V");
					}
					else
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 6.5, 24V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 6.5, 24V");
					}
						
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01, "6,500 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02, "3,250 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03, "2,167 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05, "1,300 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10, "650 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30, "217 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60, "108 GPM");
					
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01_VERT, "6,500 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02_VERT, "3,250 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03_VERT, "2,167 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05_VERT, "1,300 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10_VERT, "650 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30_VERT, "217 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60_VERT, "108 GPM");

					break;
			case 5: //Dual 6.5 (13) 12V
			case 11:if (PumpModel == 5)
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model Dual 6.5, 12V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model Dual 6.5, 12V");
					}
					else
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model Dual 6.5, 24V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model Dual 6.5, 24V");
					}
						
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01, "13,000 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02, "6,500 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03, "4,333 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05, "2,600 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10, "1,300 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30, "433 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60, "217 GPM");
					
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01_VERT, "13,000 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02_VERT, "6,500 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03_VERT, "4,333 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05_VERT, "2,600 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10_VERT, "1,300 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30_VERT, "433 GPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60_VERT, "217 GPM");

					break;	
		}
	}
	else					//Metric units
	{
		switch (PumpModel)
		{
			case 0: //1.7 12V
			case 6:	if (PumpModel == 0)
					{	
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 1.7AHP, 12V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 1.7AHP, 12V");
					}
					else
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 1.7AHP, 24V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 1.7AHP, 24V");
					}
						
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01, "6,435 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02, "3,217 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03, "2,145 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05, "1,287 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10, "643 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30, "214 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60, "107 LPM");
					
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01_VERT, "6,435 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02_VERT, "3,217 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03_VERT, "2,145 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05_VERT, "1,287 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10_VERT, "643 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30_VERT, "214 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60_VERT, "107 LPM");

					break;
			case 1: //2.1 12V
			case 7:	if (PumpModel == 1)
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 2.1A, 12V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 2.1A, 12V");
					}
					else
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 2.1A, 24V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 2.1A, 24V");
					}
						
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01, "7,949 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02, "3,975 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03, "2,650 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05, "1,590 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10, "795 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30, "265 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60, "132 LPM");
					
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01_VERT, "7,949 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02_VERT, "3,975 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03_VERT, "2,650 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05_VERT, "1,590 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10_VERT, "795 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30_VERT, "265 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60_VERT, "132 LPM");

					break;
			case 2: //3.3 12V
			case 8:	if (PumpModel == 2)
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 3.3, 12V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 3.3, 12V");
					}
					else
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 3.3, 24V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 3.3, 24V");
					}
						
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01, "12,492 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02, "6,246 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03, "4,164 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05, "2,498 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10, "1,249 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30, "416 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60, "208 LPM");
					
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01_VERT, "12,492 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02_VERT, "6,246 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03_VERT, "4,164 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05_VERT, "2,498 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10_VERT, "1,249 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30_VERT, "416 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60_VERT, "208 LPM");

					break;
			case 3: //5.0 12V
			case 9:	if (PumpModel == 3)
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 5.0, 12V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 5.0, 12V");
					}
					else
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 5.0, 24V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 5.0, 24V");
					}
						
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01, "18,927 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02, "9,463 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03, "6,309 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05, "3,785 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10, "1,892 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30, "630 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60, "315 LPM");
					
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01_VERT, "18,927 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02_VERT, "9,463 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03_VERT, "6,309 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05_VERT, "3,785 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10_VERT, "1,892 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30_VERT, "630 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60_VERT, "315 LPM");
					
					break;
			case 4: //6.5 12V
			case 10:if (PumpModel == 4)
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 6.5, 12V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 6.5, 12V");
					}
					else
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model 6.5, 24V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model 6.5, 24V");
					}
						
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01, "24,605 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02, "12,302 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03, "8,202 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05, "4,921 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10, "2,460 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30, "820 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60, "410 LPM");
					
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01_VERT, "24,605 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02_VERT, "12,302 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03_VERT, "8,202 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05_VERT, "4,921 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10_VERT, "2,460 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30_VERT, "820 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60_VERT, "410 LPM");

					break;
			case 5: //Dual 6.5 (13) 12V
			case 11:if (PumpModel == 5)
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model Dual 6.5, 12V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model Dual 6.5, 12V");
					}
					else
					{
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel, "Model Dual 6.5, 24V");
						setTextWidgetText(ScreenObjectIDs.Text_PumpModel_VERT, "Model Dual 6.5, 24V");
					}
					
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01, "49,210 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02, "24,605 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03, "16,403 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05, "9,842 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10, "4,921 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30, "1,640 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60, "820 LPM");
					
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_01_VERT, "49,210 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_02_VERT, "24,605 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_03_VERT, "16,403 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_05_VERT, "9,842 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_10_VERT, "4,921 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_30_VERT, "1,640 LPM");
					setTextWidgetText(ScreenObjectIDs.Text_FlowRate_60_VERT, "820 LPM");

					break;	
		}
	}   
}





void UpdateLimpHomeTable(uint PumpModel, uint PumpNumber, uint UnitOfMeasure)
{
	if (UnitOfMeasure == 0)	//English units
	{
		switch (PumpModel)
		{
			case 0: //1.7 12V
			case 6:		
					/*	Set the "limp home" table for the 1.7 motor/pump
					*/	
					/* 5% */	LimpHome_FlowRate[PumpNumber][0] = 0.03;
					/* 10% */	LimpHome_FlowRate[PumpNumber][1] = 0.08;
					/* 15% */	LimpHome_FlowRate[PumpNumber][2] = 0.18;
					/* 20% */	LimpHome_FlowRate[PumpNumber][3] = 0.29;
					/* 25% */	LimpHome_FlowRate[PumpNumber][4] = 0.45;
					/* 30% */	LimpHome_FlowRate[PumpNumber][5] = 0.62;
					/* 35% */	LimpHome_FlowRate[PumpNumber][6] = 0.81;
					/* 40% */	LimpHome_FlowRate[PumpNumber][7] = 0.93;
					/* 45% */	LimpHome_FlowRate[PumpNumber][8] = 1.08;
					/* 50% */	LimpHome_FlowRate[PumpNumber][9] = 1.16;
					/* 55% */	LimpHome_FlowRate[PumpNumber][10] = 1.27;
					/* 60% */	LimpHome_FlowRate[PumpNumber][11] = 1.36;
					/* 65% */	LimpHome_FlowRate[PumpNumber][12] = 1.44;
					/* 70% */	LimpHome_FlowRate[PumpNumber][13] = 1.47;
					/* 75% */	LimpHome_FlowRate[PumpNumber][14] = 1.52;
					/* 80% */	LimpHome_FlowRate[PumpNumber][15] = 1.57;
					/* 85% */	LimpHome_FlowRate[PumpNumber][16] = 1.59;
					/* 90% */	LimpHome_FlowRate[PumpNumber][17] = 1.64;
					/* 95% */	LimpHome_FlowRate[PumpNumber][18] = 1.66;
					/* 100% */	LimpHome_FlowRate[PumpNumber][19] = 1.70;
					break;
			case 1: //2.1 12V
			case 7:						
					/*	Set the "limp home" table for the 2.1 motor/pump
					*/	
					/* 5% */	LimpHome_FlowRate[PumpNumber][0] = 0.03;
					/* 10% */	LimpHome_FlowRate[PumpNumber][1] = 0.10;
					/* 15% */	LimpHome_FlowRate[PumpNumber][2] = 0.22;
					/* 20% */	LimpHome_FlowRate[PumpNumber][3] = 0.36;
					/* 25% */	LimpHome_FlowRate[PumpNumber][4] = 0.56;
					/* 30% */	LimpHome_FlowRate[PumpNumber][5] = 0.77;
					/* 35% */	LimpHome_FlowRate[PumpNumber][6] = 1.00;
					/* 40% */	LimpHome_FlowRate[PumpNumber][7] = 1.15;
					/* 45% */	LimpHome_FlowRate[PumpNumber][8] = 1.34;
					/* 50% */	LimpHome_FlowRate[PumpNumber][9] = 1.43;
					/* 55% */	LimpHome_FlowRate[PumpNumber][10] = 1.57;
					/* 60% */	LimpHome_FlowRate[PumpNumber][11] = 1.68;
					/* 65% */	LimpHome_FlowRate[PumpNumber][12] = 1.76;
					/* 70% */	LimpHome_FlowRate[PumpNumber][13] = 1.81;
					/* 75% */	LimpHome_FlowRate[PumpNumber][14] = 1.88;
					/* 80% */	LimpHome_FlowRate[PumpNumber][15] = 1.93;
					/* 85% */	LimpHome_FlowRate[PumpNumber][16] = 1.97;
					/* 90% */	LimpHome_FlowRate[PumpNumber][17] = 2.02;
					/* 95% */	LimpHome_FlowRate[PumpNumber][18] = 2.06;
					/* 100% */	LimpHome_FlowRate[PumpNumber][19] = 2.10;
					break;
			case 2: //3.3 12V
			case 8:	
				
					/*	Set the "limp home" table for the 3.3 motor/pump
					*/	
					/* 5% */	LimpHome_FlowRate[PumpNumber][0] = 0.05;
					/* 10% */	LimpHome_FlowRate[PumpNumber][1] = 0.15;
					/* 15% */	LimpHome_FlowRate[PumpNumber][2] = 0.34;
					/* 20% */	LimpHome_FlowRate[PumpNumber][3] = 0.56;
					/* 25% */	LimpHome_FlowRate[PumpNumber][4] = 0.88;
					/* 30% */	LimpHome_FlowRate[PumpNumber][5] = 1.21;
					/* 35% */	LimpHome_FlowRate[PumpNumber][6] = 1.57;
					/* 40% */	LimpHome_FlowRate[PumpNumber][7] = 1.81;
					/* 45% */	LimpHome_FlowRate[PumpNumber][8] = 2.10;
					/* 50% */	LimpHome_FlowRate[PumpNumber][9] = 2.25;
					/* 55% */	LimpHome_FlowRate[PumpNumber][10] = 2.46;
					/* 60% */	LimpHome_FlowRate[PumpNumber][11] = 2.64;
					/* 65% */	LimpHome_FlowRate[PumpNumber][12] = 2.80;
					/* 70% */	LimpHome_FlowRate[PumpNumber][13] = 2.85;
					/* 75% */	LimpHome_FlowRate[PumpNumber][14] = 2.95;
					/* 80% */	LimpHome_FlowRate[PumpNumber][15] = 3.04;
					/* 85% */	LimpHome_FlowRate[PumpNumber][16] = 3.09;
					/* 90% */	LimpHome_FlowRate[PumpNumber][17] = 3.18;
					/* 95% */	LimpHome_FlowRate[PumpNumber][18] = 3.23;
					/* 100% */	LimpHome_FlowRate[PumpNumber][19] = 3.30;
					break;
			case 3: //5.0 12V
			case 9:	
					
					/*	Set the "limp home" table for the 5.0 motor/pump
					*/	
					/* 5% */	LimpHome_FlowRate[PumpNumber][0] = 0.08;
					/* 10% */	LimpHome_FlowRate[PumpNumber][1] = 0.23;
					/* 15% */	LimpHome_FlowRate[PumpNumber][2] = 0.52;
					/* 20% */	LimpHome_FlowRate[PumpNumber][3] = 0.85;
					/* 25% */	LimpHome_FlowRate[PumpNumber][4] = 1.33;
					/* 30% */	LimpHome_FlowRate[PumpNumber][5] = 1.83;
					/* 35% */	LimpHome_FlowRate[PumpNumber][6] = 2.38;
					/* 40% */	LimpHome_FlowRate[PumpNumber][7] = 2.74;
					/* 45% */	LimpHome_FlowRate[PumpNumber][8] = 3.18;
					/* 50% */	LimpHome_FlowRate[PumpNumber][9] = 3.41;
					/* 55% */	LimpHome_FlowRate[PumpNumber][10] = 3.73;
					/* 60% */	LimpHome_FlowRate[PumpNumber][11] = 4.00;
					/* 65% */	LimpHome_FlowRate[PumpNumber][12] = 4.24;
					/* 70% */	LimpHome_FlowRate[PumpNumber][13] = 4.32;
					/* 75% */	LimpHome_FlowRate[PumpNumber][14] = 4.47;
					/* 80% */	LimpHome_FlowRate[PumpNumber][15] = 4.61;
					/* 85% */	LimpHome_FlowRate[PumpNumber][16] = 4.68;
					/* 90% */	LimpHome_FlowRate[PumpNumber][17] = 4.82;
					/* 95% */	LimpHome_FlowRate[PumpNumber][18] = 4.89;
					/* 100% */	LimpHome_FlowRate[PumpNumber][19] = 5.00;
					break;
			case 4: //6.5 12V
			case 10:
					
					/*	Set the "limp home" table for the 6.5 motor/pump
					*/	
					/* 5% */	LimpHome_FlowRate[PumpNumber][0] = 0.10;
					/* 10% */	LimpHome_FlowRate[PumpNumber][1] = 0.30;
					/* 15% */	LimpHome_FlowRate[PumpNumber][2] = 0.67;
					/* 20% */	LimpHome_FlowRate[PumpNumber][3] = 1.10;
					/* 25% */	LimpHome_FlowRate[PumpNumber][4] = 1.73;
					/* 30% */	LimpHome_FlowRate[PumpNumber][5] = 2.38;
					/* 35% */	LimpHome_FlowRate[PumpNumber][6] = 3.09;
					/* 40% */	LimpHome_FlowRate[PumpNumber][7] = 3.57;
					/* 45% */	LimpHome_FlowRate[PumpNumber][8] = 4.14;
					/* 50% */	LimpHome_FlowRate[PumpNumber][9] = 4.43;
					/* 55% */	LimpHome_FlowRate[PumpNumber][10] = 4.85;
					/* 60% */	LimpHome_FlowRate[PumpNumber][11] = 5.20;
					/* 65% */	LimpHome_FlowRate[PumpNumber][12] = 5.52;
					/* 70% */	LimpHome_FlowRate[PumpNumber][13] = 5.61;
					/* 75% */	LimpHome_FlowRate[PumpNumber][14] = 5.81;
					/* 80% */	LimpHome_FlowRate[PumpNumber][15] = 5.99;
					/* 85% */	LimpHome_FlowRate[PumpNumber][16] = 6.09;
					/* 90% */	LimpHome_FlowRate[PumpNumber][17] = 6.26;
					/* 95% */	LimpHome_FlowRate[PumpNumber][18] = 6.36;
					/* 100% */	LimpHome_FlowRate[PumpNumber][19] = 6.50;
					break;
			case 5: //Dual 6.5 (13) 12V
			case 11:
					
					/*	Set the "limp home" table for the 6.5 motor/pump
					*/	
					/* 5% */	LimpHome_FlowRate[PumpNumber][0] = 0.10;
					/* 10% */	LimpHome_FlowRate[PumpNumber][1] = 0.30;
					/* 15% */	LimpHome_FlowRate[PumpNumber][2] = 0.67;
					/* 20% */	LimpHome_FlowRate[PumpNumber][3] = 1.10;
					/* 25% */	LimpHome_FlowRate[PumpNumber][4] = 1.73;
					/* 30% */	LimpHome_FlowRate[PumpNumber][5] = 2.38;
					/* 35% */	LimpHome_FlowRate[PumpNumber][6] = 3.09;
					/* 40% */	LimpHome_FlowRate[PumpNumber][7] = 3.57;
					/* 45% */	LimpHome_FlowRate[PumpNumber][8] = 4.14;
					/* 50% */	LimpHome_FlowRate[PumpNumber][9] = 4.43;
					/* 55% */	LimpHome_FlowRate[PumpNumber][10] = 4.85;
					/* 60% */	LimpHome_FlowRate[PumpNumber][11] = 5.20;
					/* 65% */	LimpHome_FlowRate[PumpNumber][12] = 5.52;
					/* 70% */	LimpHome_FlowRate[PumpNumber][13] = 5.61;
					/* 75% */	LimpHome_FlowRate[PumpNumber][14] = 5.81;
					/* 80% */	LimpHome_FlowRate[PumpNumber][15] = 5.99;
					/* 85% */	LimpHome_FlowRate[PumpNumber][16] = 6.09;
					/* 90% */	LimpHome_FlowRate[PumpNumber][17] = 6.26;
					/* 95% */	LimpHome_FlowRate[PumpNumber][18] = 6.36;
					/* 100% */	LimpHome_FlowRate[PumpNumber][19] = 6.50;
					break;	
		}
	}
	else					//Metric units
	{
		switch (PumpModel)
		{
			case 0: //1.7 12V
			case 6:	
					/*	Set the "limp home" table for the 1.7 motor/pump
					*/	
					/* 5% */	LimpHome_FlowRate[PumpNumber][0] = 0.03;
					/* 10% */	LimpHome_FlowRate[PumpNumber][1] = 0.08;
					/* 15% */	LimpHome_FlowRate[PumpNumber][2] = 0.18;
					/* 20% */	LimpHome_FlowRate[PumpNumber][3] = 0.29;
					/* 25% */	LimpHome_FlowRate[PumpNumber][4] = 0.45;
					/* 30% */	LimpHome_FlowRate[PumpNumber][5] = 0.62;
					/* 35% */	LimpHome_FlowRate[PumpNumber][6] = 0.81;
					/* 40% */	LimpHome_FlowRate[PumpNumber][7] = 0.93;
					/* 45% */	LimpHome_FlowRate[PumpNumber][8] = 1.08;
					/* 50% */	LimpHome_FlowRate[PumpNumber][9] = 1.16;
					/* 55% */	LimpHome_FlowRate[PumpNumber][10] = 1.27;
					/* 60% */	LimpHome_FlowRate[PumpNumber][11] = 1.36;
					/* 65% */	LimpHome_FlowRate[PumpNumber][12] = 1.44;
					/* 70% */	LimpHome_FlowRate[PumpNumber][13] = 1.47;
					/* 75% */	LimpHome_FlowRate[PumpNumber][14] = 1.52;
					/* 80% */	LimpHome_FlowRate[PumpNumber][15] = 1.57;
					/* 85% */	LimpHome_FlowRate[PumpNumber][16] = 1.59;
					/* 90% */	LimpHome_FlowRate[PumpNumber][17] = 1.64;
					/* 95% */	LimpHome_FlowRate[PumpNumber][18] = 1.66;
					/* 100% */	LimpHome_FlowRate[PumpNumber][19] = 1.70;
					break;
			case 1: //2.1 12V
			case 7:	
					/*	Set the "limp home" table for the 2.1 motor/pump
					*/	
					/* 5% */	LimpHome_FlowRate[PumpNumber][0] = 0.03;
					/* 10% */	LimpHome_FlowRate[PumpNumber][1] = 0.10;
					/* 15% */	LimpHome_FlowRate[PumpNumber][2] = 0.22;
					/* 20% */	LimpHome_FlowRate[PumpNumber][3] = 0.36;
					/* 25% */	LimpHome_FlowRate[PumpNumber][4] = 0.56;
					/* 30% */	LimpHome_FlowRate[PumpNumber][5] = 0.77;
					/* 35% */	LimpHome_FlowRate[PumpNumber][6] = 1.00;
					/* 40% */	LimpHome_FlowRate[PumpNumber][7] = 1.15;
					/* 45% */	LimpHome_FlowRate[PumpNumber][8] = 1.34;
					/* 50% */	LimpHome_FlowRate[PumpNumber][9] = 1.43;
					/* 55% */	LimpHome_FlowRate[PumpNumber][10] = 1.57;
					/* 60% */	LimpHome_FlowRate[PumpNumber][11] = 1.68;
					/* 65% */	LimpHome_FlowRate[PumpNumber][12] = 1.76;
					/* 70% */	LimpHome_FlowRate[PumpNumber][13] = 1.81;
					/* 75% */	LimpHome_FlowRate[PumpNumber][14] = 1.88;
					/* 80% */	LimpHome_FlowRate[PumpNumber][15] = 1.93;
					/* 85% */	LimpHome_FlowRate[PumpNumber][16] = 1.97;
					/* 90% */	LimpHome_FlowRate[PumpNumber][17] = 2.02;
					/* 95% */	LimpHome_FlowRate[PumpNumber][18] = 2.06;
					/* 100% */	LimpHome_FlowRate[PumpNumber][19] = 2.10;
					break;
			case 2: //3.3 12V
			case 8:	
					/*	Set the "limp home" table for the 3.3 motor/pump
					*/	
					/* 5% */	LimpHome_FlowRate[PumpNumber][0] = 0.05;
					/* 10% */	LimpHome_FlowRate[PumpNumber][1] = 0.15;
					/* 15% */	LimpHome_FlowRate[PumpNumber][2] = 0.34;
					/* 20% */	LimpHome_FlowRate[PumpNumber][3] = 0.56;
					/* 25% */	LimpHome_FlowRate[PumpNumber][4] = 0.88;
					/* 30% */	LimpHome_FlowRate[PumpNumber][5] = 1.21;
					/* 35% */	LimpHome_FlowRate[PumpNumber][6] = 1.57;
					/* 40% */	LimpHome_FlowRate[PumpNumber][7] = 1.81;
					/* 45% */	LimpHome_FlowRate[PumpNumber][8] = 2.10;
					/* 50% */	LimpHome_FlowRate[PumpNumber][9] = 2.25;
					/* 55% */	LimpHome_FlowRate[PumpNumber][10] = 2.46;
					/* 60% */	LimpHome_FlowRate[PumpNumber][11] = 2.64;
					/* 65% */	LimpHome_FlowRate[PumpNumber][12] = 2.80;
					/* 70% */	LimpHome_FlowRate[PumpNumber][13] = 2.85;
					/* 75% */	LimpHome_FlowRate[PumpNumber][14] = 2.95;
					/* 80% */	LimpHome_FlowRate[PumpNumber][15] = 3.04;
					/* 85% */	LimpHome_FlowRate[PumpNumber][16] = 3.09;
					/* 90% */	LimpHome_FlowRate[PumpNumber][17] = 3.18;
					/* 95% */	LimpHome_FlowRate[PumpNumber][18] = 3.23;
					/* 100% */	LimpHome_FlowRate[PumpNumber][19] = 3.30;
					break;
			case 3: //5.0 12V
			case 9:	
					/*	Set the "limp home" table for the 5.0 motor/pump
					*/	
					/* 5% */	LimpHome_FlowRate[PumpNumber][0] = 0.08;
					/* 10% */	LimpHome_FlowRate[PumpNumber][1] = 0.23;
					/* 15% */	LimpHome_FlowRate[PumpNumber][2] = 0.52;
					/* 20% */	LimpHome_FlowRate[PumpNumber][3] = 0.85;
					/* 25% */	LimpHome_FlowRate[PumpNumber][4] = 1.33;
					/* 30% */	LimpHome_FlowRate[PumpNumber][5] = 1.83;
					/* 35% */	LimpHome_FlowRate[PumpNumber][6] = 2.38;
					/* 40% */	LimpHome_FlowRate[PumpNumber][7] = 2.74;
					/* 45% */	LimpHome_FlowRate[PumpNumber][8] = 3.18;
					/* 50% */	LimpHome_FlowRate[PumpNumber][9] = 3.41;
					/* 55% */	LimpHome_FlowRate[PumpNumber][10] = 3.73;
					/* 60% */	LimpHome_FlowRate[PumpNumber][11] = 4.00;
					/* 65% */	LimpHome_FlowRate[PumpNumber][12] = 4.24;
					/* 70% */	LimpHome_FlowRate[PumpNumber][13] = 4.32;
					/* 75% */	LimpHome_FlowRate[PumpNumber][14] = 4.47;
					/* 80% */	LimpHome_FlowRate[PumpNumber][15] = 4.61;
					/* 85% */	LimpHome_FlowRate[PumpNumber][16] = 4.68;
					/* 90% */	LimpHome_FlowRate[PumpNumber][17] = 4.82;
					/* 95% */	LimpHome_FlowRate[PumpNumber][18] = 4.89;
					/* 100% */	LimpHome_FlowRate[PumpNumber][19] = 5.00;
					break;
			case 4: //6.5 12V
			case 10:
					/*	Set the "limp home" table for the 6.5 motor/pump
					*/	
					/* 5% */	LimpHome_FlowRate[PumpNumber][0] = 0.10;
					/* 10% */	LimpHome_FlowRate[PumpNumber][1] = 0.30;
					/* 15% */	LimpHome_FlowRate[PumpNumber][2] = 0.67;
					/* 20% */	LimpHome_FlowRate[PumpNumber][3] = 1.10;
					/* 25% */	LimpHome_FlowRate[PumpNumber][4] = 1.73;
					/* 30% */	LimpHome_FlowRate[PumpNumber][5] = 2.38;
					/* 35% */	LimpHome_FlowRate[PumpNumber][6] = 3.09;
					/* 40% */	LimpHome_FlowRate[PumpNumber][7] = 3.57;
					/* 45% */	LimpHome_FlowRate[PumpNumber][8] = 4.14;
					/* 50% */	LimpHome_FlowRate[PumpNumber][9] = 4.43;
					/* 55% */	LimpHome_FlowRate[PumpNumber][10] = 4.85;
					/* 60% */	LimpHome_FlowRate[PumpNumber][11] = 5.20;
					/* 65% */	LimpHome_FlowRate[PumpNumber][12] = 5.52;
					/* 70% */	LimpHome_FlowRate[PumpNumber][13] = 5.61;
					/* 75% */	LimpHome_FlowRate[PumpNumber][14] = 5.81;
					/* 80% */	LimpHome_FlowRate[PumpNumber][15] = 5.99;
					/* 85% */	LimpHome_FlowRate[PumpNumber][16] = 6.09;
					/* 90% */	LimpHome_FlowRate[PumpNumber][17] = 6.26;
					/* 95% */	LimpHome_FlowRate[PumpNumber][18] = 6.36;
					/* 100% */	LimpHome_FlowRate[PumpNumber][19] = 6.50;
					break;
			case 5: //Dual 6.5 (13) 12V
			case 11:
					/*	Set the "limp home" table for the 6.5 motor/pump
					*/	
					/* 5% */	LimpHome_FlowRate[PumpNumber][0] = 0.10;
					/* 10% */	LimpHome_FlowRate[PumpNumber][1] = 0.30;
					/* 15% */	LimpHome_FlowRate[PumpNumber][2] = 0.67;
					/* 20% */	LimpHome_FlowRate[PumpNumber][3] = 1.10;
					/* 25% */	LimpHome_FlowRate[PumpNumber][4] = 1.73;
					/* 30% */	LimpHome_FlowRate[PumpNumber][5] = 2.38;
					/* 35% */	LimpHome_FlowRate[PumpNumber][6] = 3.09;
					/* 40% */	LimpHome_FlowRate[PumpNumber][7] = 3.57;
					/* 45% */	LimpHome_FlowRate[PumpNumber][8] = 4.14;
					/* 50% */	LimpHome_FlowRate[PumpNumber][9] = 4.43;
					/* 55% */	LimpHome_FlowRate[PumpNumber][10] = 4.85;
					/* 60% */	LimpHome_FlowRate[PumpNumber][11] = 5.20;
					/* 65% */	LimpHome_FlowRate[PumpNumber][12] = 5.52;
					/* 70% */	LimpHome_FlowRate[PumpNumber][13] = 5.61;
					/* 75% */	LimpHome_FlowRate[PumpNumber][14] = 5.81;
					/* 80% */	LimpHome_FlowRate[PumpNumber][15] = 5.99;
					/* 85% */	LimpHome_FlowRate[PumpNumber][16] = 6.09;
					/* 90% */	LimpHome_FlowRate[PumpNumber][17] = 6.26;
					/* 95% */	LimpHome_FlowRate[PumpNumber][18] = 6.36;
					/* 100% */	LimpHome_FlowRate[PumpNumber][19] = 6.50;
					break;	
		}
	}
}

//-----ScriptName=SCRIPT_FoamTutorialStep_3to4
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_FoamTutorialStep_3to4 for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_FoamTutorialStep_3to4 () 
{
   	//This script determines which step 4 foam tutorial to show based on the number of pumps in the system
   
   	uint NumberOfPumps;
   	smRead(VariableIDs.FLX_NumberOfPumps, NumberOfPumps);
   
   	if (NumberOfPumps == 1)
   	{
   		sendEvent(EventIDs.Event_GOTO_FoamTutorial_4_single);		
   	}
   	else
   	{
		sendEvent(EventIDs.Event_GOTO_FoamTutorial_4_dual);
   	}
}

//-----ScriptName=SCRIPT_GotoLookupChart
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_GotoLookupChart for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_GotoLookupChart () 
{
  	uint UnitOfMeasure;
	smRead(VariableIDs.FLX_UnitOfMeasure, UnitOfMeasure);
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation); 
	
	if (UnitOfMeasure == 0)	//English
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_LookupChart_English);
		else
			sendEvent(EventIDs.Event_GOTO_LookupChart_English_VERT);
	}
	else
	{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_LookupChart_Metric);
		else
			sendEvent(EventIDs.Event_GOTO_LookupChart_Metric_VERT);
	} 
	
	//Update the current values
	sendEvent(EventIDs.SCRIPT_SmoothBoreNozzleCalculation_GPM); 
	sendEvent(EventIDs.SCRIPT_SmoothBoreNozzleCalculation_LPM); 
	sendEvent(EventIDs.Calculate_PressToSetValue_FromCalculator_Equals_TRUE);
}

//-----ScriptName=SCRIPT_EvaluateA2CModuleSetup
/*
	This script is fired when a message from 0x18EF1E41 (A2C module) is received.
	
	If the value of that message's bytes 4 (ch 0) and 5 (ch 1) are not correct then
	we will load up bytes 5-7 of the transmit message to set the module correctly.
*/
void SCRIPT_EvaluateA2CModuleSetup () 
{
 	uint Configuration_Channel_0; 
	uint Configuration_Channel_1;
	
	smRead(VariableIDs.A2CModuleRX_Byte_4, Configuration_Channel_0);
	smRead(VariableIDs.A2CModuleRX_Byte_5, Configuration_Channel_1);
	
	bool WorkingOnCH_0 = false;
	//bool WorkingOnCH_1 = false;
	
	
	/*
		Channel configuration definition:
		
		bit	7	6	5	4	3	2	1	0
			|	|	|	|	|	|	|	|_	0-5 Volt input
			|	|	|	|	|	|	|_____	4-20 ma input
			|	|	|	|	|	|_________	Thermistor input
			|	|	|	|	|_____________	Frequency input	
			|	|	|	|_________________	Reference voltage enabled
			|	|	|_____________________	5 volt reference
			|	|_________________________	9 volt reference
			|_____________________________	not used

		For SmartFOAM we want channels 0 and 1 to be set up for frequency and 9V reference
		so we should be looking for the value 0101 1000 (0x58).  If we don't have that
		value for each channel then we need to set up a transmit message to set it correctly.	
	*/
	if (Configuration_Channel_0 != 0x58)
	{
		/*	To set the channels we have to load byte 5, 6, and 7 of the message 0x18EF411E
			to 0x51, 0x23, and 0x24 respectively.
			0x51 = channel 0 set command (0x52 is channel 1 set command)
			0x23 = valid check number - this will always be 0x23
			0x24 = set 9V reference and frequency input
				high nibble is reference (0 = disabled, 1 = 5V, 2 = 9V)
				low nibble is input type (1 = 0-5v, 2 = 4-20ma, 3 = thermistor, 4 = frequency)
		*/
		uint Bytes4567_Value = 0x24235100;
		
		smWrite(VariableIDs.A2CModule_Bytes4_7, Bytes4567_Value); 
		sendEvent(EventIDs.Event_TransmitA2CModule_Setup);
		WorkingOnCH_0 = true;
	}


	
	if (Configuration_Channel_1 != 0x58 && WorkingOnCH_0 == false)
	{
		/*	To set the channels we have to load byte 5, 6, and 7 of the message 0x18EF411E
			to 0x52, 0x23, and 0x24 respectively.
			0x52 = channel 1 set command (0x51 is channel 0 set command)
			0x23 = valid check number - this will always be 0x23
			0x24 = set 9V reference and frequency input
				high nibble is reference (0 = disabled, 1 = 5V, 2 = 9V)
				low nibble is input type (1 = 0-5v, 2 = 4-20ma, 3 = thermistor, 4 = frequency)
		*/
		uint Bytes4567_Value = 0x24235200;
		
		smWrite(VariableIDs.A2CModule_Bytes4_7, Bytes4567_Value); 
		sendEvent(EventIDs.Event_TransmitA2CModule_Setup);
		//WorkingOnCH_1 = true;
	}


	
	if (Configuration_Channel_0 == 0x58 && Configuration_Channel_1 == 0x58)
	{
		smWrite(VariableIDs.A2CModule_Bytes4_7, 0);
	}
	  
}

//-----ScriptName=SCRIPT_ChaseThailandSelector
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_ChaseThailandSelector for main script method
//---------------------------------------------------------------------------------------------
bool InitChaseThailand = true;
void SCRIPT_ChaseThailandSelector () 
{
  	uint FoamSystemDisplayBackground;
	smRead(VariableIDs.FLX_Display_Type, FoamSystemDisplayBackground);
	last_FoamSystemDisplayBackground = FoamSystemDisplayBackground; 
	
	if (FoamSystemDisplayBackground == 3)
	{		
		//First time through - make sure that we don't show the preset screen
		if (InitChaseThailand == true)
		{
			sendEvent(EventIDs.SCRIPT_GOTO_OperatingScreen);
			InitChaseThailand = false;
		}
	} 
}

//-----ScriptName=SCRIPT_CheckForChaseThailandSelection
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_CheckForChaseThailandSelection for main script method
//---------------------------------------------------------------------------------------------
uint last_FoamSystemDisplayBackground = 0;	//This variable is initialized in "SCRIPT_ChaseThailandSelector" 
void SCRIPT_CheckForChaseThailandSelection () 
{
	/*	This script is fired by the USER menu save.
		If the user has changed to the Chase Thailand type screen display
		then we want to set the number of pumps to 2 - because the original
		Chase Thailand system utilized 2 6.5 foam pumps.
		
		We can check this because the foam background
	*/ 
	
	uint FoamSystemDisplayBackground;
	smRead(VariableIDs.FLX_Display_Type, FoamSystemDisplayBackground);
	
	if (last_FoamSystemDisplayBackground != 3 && FoamSystemDisplayBackground == 3)
	{
		smWrite(VariableIDs.FLX_NumberOfPumps, 2);				//Set to two pumps - use "tmp" variables because the "save" event uses these to update the new variable
		smWrite(VariableIDs.FoamSystemControlType, 1);			//set to single system with dual pumps
		
		sendEvent(EventIDs.Event_Set_Temp_OEM_Variables);			//Must make sure we have the current variables before we try to save anything new.
		sendEvent(EventIDs.Event_Set_Temp_Operating_Variables);		//These populate the "tmp" variables with their saved counterparts.
	
		
		smWrite(VariableIDs.tmp_FLX_NumberOfPumps, 2);				//Set to two pumps - use "tmp" variables because the "save" event uses these to update the new variable
		smWrite(VariableIDs.tmp_FoamSystemControlType, 1);			//set to single system with dual pumps
		
		
		
		sendEvent(EventIDs.Event_Save_Operating_Variables);	
		sendEvent(EventIDs.Event_Save_OEM_Variables);
		last_FoamSystemDisplayBackground = FoamSystemDisplayBackground;
	}   
}

//-----ScriptName=SCRIPT CalibrateWaterFlow2
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_CalibrateWaterFlow2 for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_CalibrateWaterFlow2 () 
{
 	uint Low_WaterCalValue;
	uint High_WaterCalValue;
	uint Low_WaterRawPulses;
	uint High_WaterRawPulses;
	
	uint DifferenceOfCalValues;
	double f_2PointSlope;
	double f_2PointOffset;
	uint AllowSave = 0;
	
	/*	Only calibrate if we have values that are in range.
		For example - High_WaterCalValue must be greater than Low_WaterCalValue.
					- High_WaterRawPulses and Low_WaterRawPulses must be greater than 0
					- High_WaterRawPulses must be greater than Low_WaterRawPulses.
	*/
	
	smRead(VariableIDs.WaterFlowCalibration_HighValue_2, High_WaterCalValue);
	smRead(VariableIDs.WaterFlowCalibration_LowValue_2, Low_WaterCalValue);
	
	smRead(VariableIDs.WaterFlowCalibration_HighRawPulses_2, High_WaterRawPulses);
	smRead(VariableIDs.WaterFlowCalibration_LowRawPulses_2, Low_WaterRawPulses);
	
	/*	The system variable AllowSaveOfWaterFlowVariables has a calculation event that evaluates the
		variable conditions:
							- High_WaterCalValue must be greater than Low_WaterCalValue.
							- High_WaterCalValue must be >= 50
							- Low_WaterCalValue must be >= 10
							- High_WaterRawPulses and Low_WaterRawPulses must be greater than 0
							- High_WaterRawPulses must be greater than Low_WaterRawPulses.
	*/
	smRead(VariableIDs.AllowSaveOfWaterFlowVariables, AllowSave);
	
	if (AllowSave != 0)
	{
		DifferenceOfCalValues = High_WaterCalValue - Low_WaterCalValue;
 
		f_2PointSlope = High_WaterRawPulses;
		f_2PointSlope = f_2PointSlope - Low_WaterRawPulses;
		f_2PointSlope = DifferenceOfCalValues / f_2PointSlope;		//subtraction of the two point value: High GPM - Low GPM
	
		f_2PointOffset = f_2PointSlope;
		f_2PointOffset = f_2PointOffset * High_WaterRawPulses;
		f_2PointOffset = High_WaterCalValue - f_2PointOffset;
		
		smWrite(VariableIDs.Secondary_WaterCal_2PointOffset, f_2PointOffset);	 
		smWrite(VariableIDs.Secondary_WaterCal_2PointSlope, f_2PointSlope);
		
		sendEvent(EventIDs.Event_Save_Water_Flow_Variables_2);
	}
	else
	{
		//report error
	}   
}

//-----ScriptName=SCRIPT_PrimaryCalibrationValue_ButtonCheck
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_PrimaryCalibrationValue_ButtonCheck for main script method
//---------------------------------------------------------------------------------------------
uint PrimaryFoamCalButtonTimer = 0;
void SCRIPT_PrimaryCalibrationValue_ButtonCheck () 
{
	double 	TempScratchPad = 0;
	uint8 	CurrentTankSelected;
	uint 	TotalFoamPulses;	
	uint 	Primary_FoamCalFactor_TankA;
	uint 	Primary_FoamCalFactor_TankB;
	double  PulseFactorFudge;
	smRead(VariableIDs.FLX_PulseFactor, PulseFactorFudge);
	
 	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Primary_FoamSystemTankSelection, CurrentTankSelected);
	
	/*	Get access to the multipler value for foam A and B.  This is the value that
		the raw foam pulses from the DC motor driver will be multiplied by to
		get the flow rate of foam in GPM.
	*/ 
	smRead(VariableIDs.Primary_FoamFlowCalculationValue_A, Primary_FoamCalFactor_TankA); 
	smRead(VariableIDs.Primary_FoamFlowCalculationValue_B, Primary_FoamCalFactor_TankB); 
	
	/*	0 = OFF
		1 = INCREASE PRESSED
		2 = DECREASE PRESSED
	*/
	uint PrimaryCalButtonState;
	smRead(VariableIDs.Primary_FoamCal_Button, PrimaryCalButtonState);
	
	uint ChangeValue = 1;
	
	if (PrimaryCalButtonState != 0)
	{
		PrimaryFoamCalButtonTimer++;
		if (PrimaryFoamCalButtonTimer > 500)
		{
			PrimaryFoamCalButtonTimer = 500;
		}
		
		ChangeValue = (PrimaryFoamCalButtonTimer / 10) + 1;
		
		/*	Calculate the GPM rate of foam from the raw value received from the DC motor driver
			making sure to use the correct calibration multiplier value.
			Put this value into the Primary_FoamFlowGPM.
			Load the total foam flow data from the DC Motor Driver CAN message
			into the appropriate system variable for total foam pulses.
		*/
		switch (CurrentTankSelected)
		{
			case 1:	//tank B
					if (PrimaryCalButtonState == 2)
					{
						if (Primary_FoamCalFactor_TankB < 99999)
						{
							Primary_FoamCalFactor_TankB = Primary_FoamCalFactor_TankB + ChangeValue;
							smWrite(VariableIDs.Primary_FoamFlowCalculationValue_B, Primary_FoamCalFactor_TankB);
						}
					}
					else
					{
						if (Primary_FoamCalFactor_TankB > 10)
						{
							Primary_FoamCalFactor_TankB = Primary_FoamCalFactor_TankB - ChangeValue;
							smWrite(VariableIDs.Primary_FoamFlowCalculationValue_B, Primary_FoamCalFactor_TankB);
						}
					}
					
					smRead(VariableIDs.FoamPrimary_TotalFoamPulses, TotalFoamPulses);
					TempScratchPad = TotalFoamPulses;
					/*	We need to adjust the total foam flowed pulses received from the DC motor driver to account for the
						original "fudge" factor.
					*/
					TempScratchPad = TempScratchPad * 0.940767;
					TempScratchPad = TempScratchPad * PulseFactorFudge;
					
					TempScratchPad = TempScratchPad / Primary_FoamCalFactor_TankB;	
					smWrite(VariableIDs.Primary_TotalFoamGallons_B, TempScratchPad);
					break;
					
			case 2:	//tank A
					if (PrimaryCalButtonState == 2)
					{
						if (Primary_FoamCalFactor_TankA < 99999)
						{
							Primary_FoamCalFactor_TankA = Primary_FoamCalFactor_TankA + ChangeValue;
							smWrite(VariableIDs.Primary_FoamFlowCalculationValue_A, Primary_FoamCalFactor_TankA);
						}
					}
					else
					{
						if (Primary_FoamCalFactor_TankA > 10)
						{
							Primary_FoamCalFactor_TankA = Primary_FoamCalFactor_TankA - ChangeValue;
							smWrite(VariableIDs.Primary_FoamFlowCalculationValue_A, Primary_FoamCalFactor_TankA); 
						}
					}
					
					smRead(VariableIDs.FoamPrimary_TotalFoamPulses, TotalFoamPulses);
					TempScratchPad = TotalFoamPulses;
					/*	We need to adjust the total foam flowed pulses received from the DC motor driver to account for the
						original "fudge" factor.
					*/
					TempScratchPad = TempScratchPad * 0.940767;
					TempScratchPad = TempScratchPad * PulseFactorFudge;
					
					TempScratchPad = TempScratchPad / Primary_FoamCalFactor_TankA;
					smWrite(VariableIDs.Primary_TotalFoamGallons_A, TempScratchPad);
					break;
		}
	} 
	else
	{
		PrimaryFoamCalButtonTimer = 0;
	}   
}

//-----ScriptName=SCRIPT_SecondaryCalibrationValue_ButtonCheck
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_SecondaryCalibrationValue_ButtonCheck for main script method
//---------------------------------------------------------------------------------------------
uint SecondaryFoamCalButtonTimer = 0;
void SCRIPT_SecondaryCalibrationValue_ButtonCheck () 
{
	double 	TempScratchPad = 0;
	uint8 	CurrentTankSelected;
	uint 	TotalFoamPulses;	
	uint 	Secondary_FoamCalFactor_TankA;
	uint 	Secondary_FoamCalFactor_TankB;
	double  PulseFactorFudge;
	smRead(VariableIDs.FLX_PulseFactor, PulseFactorFudge);
	
 	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Secondary_FoamSystemTankSelection, CurrentTankSelected);
	
	/*	Get access to the multipler value for foam A and B.  This is the value that
		the raw foam pulses from the DC motor driver will be multiplied by to
		get the flow rate of foam in GPM.
	*/ 
	smRead(VariableIDs.Secondary_FoamFlowCalculationValue_A, Secondary_FoamCalFactor_TankA); 
	smRead(VariableIDs.Secondary_FoamFlowCalculationValue_B, Secondary_FoamCalFactor_TankB); 
	
	/*	0 = OFF
		1 = INCREASE PRESSED
		2 = DECREASE PRESSED
	*/
	uint SecondaryCalButtonState;
	smRead(VariableIDs.Secondary_FoamCal_Button, SecondaryCalButtonState);
	
	uint ChangeValue = 1;
	
	if (SecondaryCalButtonState != 0)
	{
		SecondaryFoamCalButtonTimer++;
		if (SecondaryFoamCalButtonTimer > 500)
		{
			SecondaryFoamCalButtonTimer = 500;
		}
		
		ChangeValue = (SecondaryFoamCalButtonTimer / 10) + 1;
		
		/*	Calculate the GPM rate of foam from the raw value received from the DC motor driver
			making sure to use the correct calibration multiplier value.
			Put this value into the Primary_FoamFlowGPM.
			Load the total foam flow data from the DC Motor Driver CAN message
			into the appropriate system variable for total foam pulses.
		*/
		switch (CurrentTankSelected)
		{
			case 1:	//tank B
					if (SecondaryCalButtonState == 2)
					{
						if (Secondary_FoamCalFactor_TankB < 99999)
						{
							Secondary_FoamCalFactor_TankB = Secondary_FoamCalFactor_TankB + ChangeValue;
							smWrite(VariableIDs.Secondary_FoamFlowCalculationValue_B, Secondary_FoamCalFactor_TankB);
						}
					}
					else
					{
						if (Secondary_FoamCalFactor_TankB > 10)
						{
							Secondary_FoamCalFactor_TankB = Secondary_FoamCalFactor_TankB - ChangeValue;
							smWrite(VariableIDs.Secondary_FoamFlowCalculationValue_B, Secondary_FoamCalFactor_TankB);
						}
					}
					
					smRead(VariableIDs.FoamSecondary_TotalFoamPulses, TotalFoamPulses);
					TempScratchPad = TotalFoamPulses;
					/*	We need to adjust the total foam flowed pulses received from the DC motor driver to account for the
						original "fudge" factor.
					*/
					TempScratchPad = TempScratchPad * 0.940767;
					TempScratchPad = TempScratchPad * PulseFactorFudge;
					
					TempScratchPad = TempScratchPad / Secondary_FoamCalFactor_TankB;	
					smWrite(VariableIDs.Secondary_TotalFoamGallons_B, TempScratchPad);
					break;
					
			case 2:	//tank A
					if (SecondaryCalButtonState == 2)
					{
						if (Secondary_FoamCalFactor_TankA < 99999)
						{
							Secondary_FoamCalFactor_TankA = Secondary_FoamCalFactor_TankA + ChangeValue;
							smWrite(VariableIDs.Secondary_FoamFlowCalculationValue_A, Secondary_FoamCalFactor_TankA);
						}
					}
					else
					{
						if (Secondary_FoamCalFactor_TankA > 10)
						{
							Secondary_FoamCalFactor_TankA = Secondary_FoamCalFactor_TankA - ChangeValue;
							smWrite(VariableIDs.Secondary_FoamFlowCalculationValue_A, Secondary_FoamCalFactor_TankA); 
						}
					}
					
					smRead(VariableIDs.FoamSecondary_TotalFoamPulses, TotalFoamPulses);
					TempScratchPad = TotalFoamPulses;
					/*	We need to adjust the total foam flowed pulses received from the DC motor driver to account for the
						original "fudge" factor.
					*/
					TempScratchPad = TempScratchPad * 0.940767;
					TempScratchPad = TempScratchPad * PulseFactorFudge;
					
					TempScratchPad = TempScratchPad / Secondary_FoamCalFactor_TankA;
					smWrite(VariableIDs.Secondary_TotalFoamGallons_A, TempScratchPad);
					break;
		}
	} 
	else
	{
		SecondaryFoamCalButtonTimer = 0;
	}   
}

//-----ScriptName=SCRIPT_GOTO_PrimePump
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_GOTO_PrimePump for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_GOTO_PrimePump () 
{
  	uint PumpModel;
	smRead(VariableIDs.FLX_PumpModel, PumpModel);
	
	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation); 

	//Version 1.19 - allow any pump to prime - this used to prevent the 1.7 and 2.1 pump from using prime.
	//if ((PumpModel >= 2 && PumpModel <= 5) || PumpModel >= 8) 
	//{
		if (DisplayOrientation == 0)
			sendEvent(EventIDs.Event_GOTO_Prime_Pump);
		else
			sendEvent(EventIDs.Event_GOTO_Prime_Pump_VERT);	
	//} 
}	

//-----ScriptName=SCRIPT_CheckMotorDriverStatus
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_CheckMotorDriverStatus for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_CheckMotorDriverStatus () 
{
 	bool PrimaryFoamStatus;
	bool SecondaryFoamStatus;
	
	uint PrimaryTankSelected;
	uint SecondaryTankSelected;
	
	PrimaryFoamStatus = smRead(VariableIDs.FoamPrimary_FoamTankSelected, PrimaryTankSelected);  
	SecondaryFoamStatus = smRead(VariableIDs.FoamSecondary_FoamTankSelected, SecondaryTankSelected);
	
	if (PrimaryFoamStatus == true)
		smWrite(VariableIDs.FoamPrimary_Status, 1);
	else
		smWrite(VariableIDs.FoamPrimary_Status, 0);	
		
	if (SecondaryFoamStatus == true)
		smWrite(VariableIDs.FoamSecondary_Status, 1);
	else
		smWrite(VariableIDs.FoamSecondary_Status, 0);
		
		
	uint PrimaryTempCritical;
	uint PrimaryTempWarning;
	smRead(VariableIDs.FoamPrimary_TemperatureCritical, PrimaryTempCritical); 
	smRead(VariableIDs.FoamPrimary_TemperatureWarning, PrimaryTempWarning);	
	
	if (PrimaryTempWarning == 0 && PrimaryTempCritical == 0)
	{
		smWrite(VariableIDs.FoamPrimary_TemparatureNormal, 1);
	}
	else
	{
		smWrite(VariableIDs.FoamPrimary_TemparatureNormal, 0);
	}
	
	uint SecondaryTempCritical;
	uint SecondaryTempWarning;
	smRead(VariableIDs.FoamSecondary_TemperatureCritical, SecondaryTempCritical); 
	smRead(VariableIDs.FoamSecondary_TemperatureWarning, SecondaryTempWarning);	
	
	if (SecondaryTempWarning == 0 && SecondaryTempCritical == 0)
	{
		smWrite(VariableIDs.FoamSecondary_TemparatureNormal, 1);
	}
	else
	{
		smWrite(VariableIDs.FoamSecondary_TemparatureNormal, 0);
	}
	
	 
}

//-----ScriptName=SCRIPT_CheckSimFlowValueChange
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_CheckSimFlowValueChange for main script method
//---------------------------------------------------------------------------------------------
uint PrimarySimulatedFlowChangeTimer = 0;
uint ChangeValue = 1;
void SCRIPT_CheckSimFlowValueChange () 
{
 	uint SimulatedFlowActive;
	smRead(VariableIDs.Primary_SimulatedWaterFlow_Enabled, SimulatedFlowActive);
	
	uint SimFlowPageButton;
	smRead(VariableIDs.Primary_SimFlowValue_Button_Setup, SimFlowPageButton);
	

	uint SimulatedFlowValue_Button = 0;
	
	if (SimFlowPageButton != 0)				//If are here because we are on the sim flow setup page - use its button for the sim flow button request
	{
		smRead(VariableIDs.Primary_SimFlowValue_Button_Setup, SimulatedFlowValue_Button);
	}
	else if (SimulatedFlowActive == 1)		//If we are here because we are in simulated flow
	{	
		smRead(VariableIDs.Primary_SimFlowValue_Button, SimulatedFlowValue_Button);	
	}
	
	uint SimulateFlowValue;
	smRead(VariableIDs.Primary_SimulatedFlowGPM, SimulateFlowValue);
	 

	if (SimulatedFlowValue_Button != 0)
	{
		PrimarySimulatedFlowChangeTimer++;
		if (PrimarySimulatedFlowChangeTimer < 40)
		{
			ChangeValue = 1;
		}
		if (PrimarySimulatedFlowChangeTimer >= 40)
		{
			ChangeValue = 10;
		}
		if (PrimarySimulatedFlowChangeTimer >= 100)
		{
			ChangeValue = 100;
			PrimarySimulatedFlowChangeTimer = 100;
		}
	
	}
	else
	{
		ChangeValue = 1;
		PrimarySimulatedFlowChangeTimer = 0;
	}
	
	if (SimulatedFlowValue_Button == 1)	//INCREASE 
	{
		if (SimulateFlowValue < (8000 -  ChangeValue))
		{
			SimulateFlowValue = SimulateFlowValue + ChangeValue;
			smWrite(VariableIDs.Primary_SimulatedFlowGPM, SimulateFlowValue);
		}
		else
		{
			smWrite(VariableIDs.Primary_SimulatedFlowGPM, 8000);
		}	
	}
	if (SimulatedFlowValue_Button == 2)	//DECREASE 
	{
		if (SimulateFlowValue > (5 + ChangeValue))
		{
			SimulateFlowValue = SimulateFlowValue - ChangeValue;
			smWrite(VariableIDs.Primary_SimulatedFlowGPM, SimulateFlowValue);
		}
		else
		{
			smWrite(VariableIDs.Primary_SimulatedFlowGPM, 5);
		}
	}   
}

//-----ScriptName=SCRIPT_GotoMenu
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_GotoMenu for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_GotoMenu () 
{
 	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);
	
	if (DisplayOrientation == 0)
		SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Menu_Layer_Menu_View);
	else
		SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Menu_Layer_VERTICAL_Menu_View);	
   	
}

//-----ScriptName=SCRIPT_GotoPasswordEntry
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_GotoPasswordEntry for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_GotoPasswordEntry () 
{
 	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);
	
	if (DisplayOrientation == 0)
		SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Popup_Layer_Default_Password_Entry_Screen_View);
	else
		SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Popup_Layer_Default_VERTICAL_Password_Entry_Screen_View); 
		
	SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Menu_Layer_Default_Menu_Layer_CLEAR_View); 
	smWrite(VariableIDs.PasswordValue, 0); 
	smWrite(VariableIDs.PasswordAttemptWarning, 0);
	smWrite(VariableIDs.PasswordInvalid, 0);
}

//-----ScriptName=SCRIPT_GotoSimulatedWaterFlow
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_GotoSimulatedWaterFlow for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_GotoSimulatedWaterFlow () 
{
 	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation); 
	
	if (DisplayOrientation == 0)
		SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Menu_Layer_Default_Simulated_Water_Flow_Setup_View);
	else
	 	SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Menu_Layer_Default_VERTICAL_Simulated_Water_Flow_Setup_View);  
}

//-----ScriptName=SCRIPT_GotoPerformanceChart
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_GotoPerformanceChart for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_GotoPerformanceChart () 
{
 	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);
	
	if (DisplayOrientation == 0)
		sendEvent(EventIDs.Event_GOTO_PerformanceChart);
	else
		sendEvent(EventIDs.Event_GOTO_PerformanceChart_VERT);   
}

//-----ScriptName=SCRIPT_GotoMaintenancePage
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_GotoMaintenancePage for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_GotoMaintenancePage () 
{
 	uint DisplayOrientation;
	smRead(VariableIDs.FLX_DisplayOrientation, DisplayOrientation);
	
	if (DisplayOrientation == 0)
		SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Menu_Layer_Default_Maintenance_Interval_View);
	else
		SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Menu_Layer_Default_VERTICAL_Maintenance_Interval_View);  
		
	SendAction(ApplicationIDs.UIApp, ActionIDs.UIApp_Show_View, PageViewIDs.Popup_Layer_Popup_Page_CLEAR_View); 
}

//-----ScriptName=SCRIPT_EvaluateLogoChoiceChange
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_EvaluateLogoChoiceChange for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_EvaluateLogoChoiceChange () 
{
  	uint CurrentLogoChoice;
	smRead(VariableIDs.LogoChoice, CurrentLogoChoice);
	
	uint NewLogoChoice;
	smRead(VariableIDs.tmp_LogoChoice, NewLogoChoice); 
	
	if (CurrentLogoChoice != NewLogoChoice)
	{	
		smWrite(VariableIDs.LogoChoice, NewLogoChoice);	
		sendEvent(EventIDs.Event_RestartSystem);
	}
}

//-----ScriptName=SCRIPT_MotorType_INCR
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_MotorType_INCR for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_MotorType_INCR () 
{
 	uint MotorType;
	smRead(VariableIDs.tmp_MotorController_Type, MotorType);
	
	MotorType++;
		
	if (MotorType > 8 || MotorType < 1)
		MotorType = 1;  
	
	smWrite(VariableIDs.tmp_MotorController_Type, MotorType);  
}

//-----ScriptName=SCRIPT_MotorType_DECR
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_MotorType_DECR for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_MotorType_DECR () 
{
 	uint MotorType;
	smRead(VariableIDs.tmp_MotorController_Type, MotorType);
	
	MotorType--;
		
	if (MotorType > 8 || MotorType < 1)
		MotorType = 8; 
	
	smWrite(VariableIDs.tmp_MotorController_Type, MotorType);  
}

//-----ScriptName=SCRIPT_TankSelect_Button_1
/*	
	The top button of the tank selector pair was pressed.  The button function is dependent on the state of the FoamPrimary_FoamTankSelected system variable:
	
	0 = select A
	1 = select A
	2 = select B
*/
uint EDT_ADT_Request = 0;
void SCRIPT_TankSelect_Button_1 () 
{

	//Foam selector type (0 = none, 1 = mst, 2 = mdtII, 3 = adt
	uint FoamSelector;
	smRead(VariableIDs.tmp_FLX_FoamTankSelectorType, FoamSelector);
	
	//Only perform the rest if we have an ADT/EDT
	if (FoamSelector >= 3)
	{
				
	 	/*	Get the currently selected foam tank.
			0 = none (FLUSH)
			1 = B
			2 = A
			3 = Tank select mismatch
		*/
		uint CurrentTankSelected;
		smRead(VariableIDs.FoamPrimary_FoamTankSelected, CurrentTankSelected);
		
		/*	The current power state should be 1 of 4 values:
			0 = OFF
			1 = ON in Foam A
			2 = ON in Foam B
			3 = ON in FLUSH
		*/
		uint CurrentPowerState;
		smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
		
		// TypeFoamInLines;	//0 = none, 1 = B, 2 = A
	
		switch (CurrentTankSelected)
		{
			case 0:	//SELECT flush - then select tank A
					if (TypeFoamInLines == 1)	//IF there is B foam then we have to do a flush
					{
						EDT_ADT_Request = 1;
						smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
						smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
						/*	if the power is OFF inform the operator that we are switching to flush since the system will not automatically perform
							the changeover because the foam system is not currently enabled.
						*/
						if (CurrentPowerState == 0)	
						{
							sendEvent(EventIDs.Warning_AutoFlushInitiated);
						}
						else
						{
							sendEvent(EventIDs.Warnings_ClearMenuPageWarnings);
						}
					}
					else
					{
						//Just select the deired tank
						smWrite(VariableIDs.FreeFormCAN_TankA_Select, 1);
						smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
						sendEvent(EventIDs.Warnings_ClearMenuPageWarnings);
					}
					break;
			case 1:	//SELECT flush - then select tank A
					if (TypeFoamInLines == 1)	//IF there is B foam then we have to do a flush
					{
						EDT_ADT_Request = 1;
						smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
						smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
						/*	if the power is OFF inform the operator that we are switching to flush since the system will not automatically perform
							the changeover because the foam system is not currently enabled.
						*/
						if (CurrentPowerState == 0)	
						{
							sendEvent(EventIDs.Warning_AutoFlushInitiated);
						}
						else
						{
							sendEvent(EventIDs.Warnings_ClearMenuPageWarnings);
						}
					}
					else
					{
						//Just select the deired tank
						smWrite(VariableIDs.FreeFormCAN_TankA_Select, 1);
						smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
						sendEvent(EventIDs.Warnings_ClearMenuPageWarnings);
					}
					break;
			case 2:	//SELECT flush - then select tank B
					if (TypeFoamInLines == 2)	//IF there is A foam then we have to do a flush
					{
						EDT_ADT_Request = 2;
						smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
						smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
						/*	if the power is OFF inform the operator that we are switching to flush since the system will not automatically perform
							the changeover because the foam system is not currently enabled.
						*/
						if (CurrentPowerState == 0)	
						{
							sendEvent(EventIDs.Warning_AutoFlushInitiated);
						}
						else
						{
							sendEvent(EventIDs.Warnings_ClearMenuPageWarnings);
						}
					}
					else
					{
						//Just select the deired tank
						smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
						smWrite(VariableIDs.FreeFormCAN_TankB_Select, 1);
						sendEvent(EventIDs.Warnings_ClearMenuPageWarnings);
					}
					break;
			case 3:	//Problem - don't do anything
					smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
					smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
					break;
		}
		
		sendEvent(EventIDs.Event_TransmitOutputControlMessage);
		sendEvent(EventIDs.Event_SaveEDT_Positions);
	}
}

//-----ScriptName=SCRIPT_TankSelect_Button_2
/*	
	The top button of the tank selector pair was pressed.  The button function is dependent on the state of the FoamPrimary_FoamTankSelected system variable:
	
	0 = select B
	1 = select FLUSH
	2 = select FLUSH
*/
void SCRIPT_TankSelect_Button_2 () 
{

	//Foam selector type (0 = none, 1 = mst, 2 = mdtII, 3 = adt
	uint FoamSelector;
	smRead(VariableIDs.tmp_FLX_FoamTankSelectorType, FoamSelector);
	
	//Only perform the rest if we have an ADT/EDT
	if (FoamSelector >= 3)
	{
	 	/*	Get the currently selected foam tank.
			0 = none (FLUSH)
			1 = B
			2 = A
			3 = Tank select mismatch
		*/
		uint CurrentTankSelected;
		smRead(VariableIDs.FoamPrimary_FoamTankSelected, CurrentTankSelected);
		
	
		/*	The current power state should be 1 of 4 values:
			0 = OFF
			1 = ON in Foam A
			2 = ON in Foam B
			3 = ON in FLUSH
		*/
		uint CurrentPowerState;
		smRead(VariableIDs.Primary_FoamLogix_Enabled, CurrentPowerState);
		
	
		switch (CurrentTankSelected)
		{
			case 0:	//SELECT flush - then select tank B
					if (TypeFoamInLines == 2)	//IF there is A foam then we have to do a flush
					{
						EDT_ADT_Request = 2;
						smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
						smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
						/*	if the power is OFF inform the operator that we are switching to flush since the system will not automatically perform
							the changeover because the foam system is not currently enabled.
						*/
						if (CurrentPowerState == 0)	
						{
							sendEvent(EventIDs.Warning_AutoFlushInitiated);
						}
						else
						{
							sendEvent(EventIDs.Warnings_ClearMenuPageWarnings);
						}
					}
					else
					{
						//Just select the deired tank
						smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
						smWrite(VariableIDs.FreeFormCAN_TankB_Select, 1);
					}
					break;
			case 1:	//SELECT flush
					EDT_ADT_Request = 0;
					smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
					smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
					sendEvent(EventIDs.Warnings_ClearMenuPageWarnings);
					break;
			case 2:	//SELECT flush
					EDT_ADT_Request = 0;
					smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
					smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
					sendEvent(EventIDs.Warnings_ClearMenuPageWarnings);
					break;
			case 3:	//Problem - don't do anything
					smWrite(VariableIDs.FreeFormCAN_TankA_Select, 0);
					smWrite(VariableIDs.FreeFormCAN_TankB_Select, 0);
					break;
		}
		
		sendEvent(EventIDs.Event_TransmitOutputControlMessage);
		sendEvent(EventIDs.Event_SaveEDT_Positions);
	}
}

//-----ScriptName=SCRIPT_UpdateRemoteData
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_UpdateRemoteData for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_UpdateRemoteData () 
{
 	uint WaterFlowLPM;
	uint TotalWaterLiters;
	uint TotalFoamLiters;
	uint TankSelected;
	uint PowerState;
	uint TankALevel;
	uint TankBLevel;
	uint SystemPowerState;
	uint FoamPercentage;
	float TempData;
	
	uint UnitOfMeasure;
	smRead(VariableIDs.FLX_UnitOfMeasure, UnitOfMeasure);
	

	
/*	PRIMARY system calculations ****************************************************************************************
*/
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Primary_FoamSystemTankSelection, TankSelected);
	switch(TankSelected)
	{
		case 0:	//FLUSH
				smWrite(VariableIDs.RemoteOperationPrimary_TankSelected, 2);
				smWrite(VariableIDs.RemoteOperationPrimary_TotalFoamLiters, 0);
				smWrite(VariableIDs.RemoteOperationPrimary_FoamPercentage, 0);
				break;
		case 1:	//TANK B
				smWrite(VariableIDs.RemoteOperationPrimary_TankSelected, 1);
				smRead(VariableIDs.Primary_TotalFoamGallons_B, TempData);
				if (UnitOfMeasure == 0)	//If we are in GPM then convert to LPM before sending
				{
					TempData = TempData * 37.854118;
				}
				else
				{
					TempData = TempData * 10;	//Move decimal
				}
				TotalFoamLiters = TempData;
				smWrite(VariableIDs.RemoteOperationPrimary_TotalFoamLiters, TotalFoamLiters);
				smRead(VariableIDs.Primary_FoamPercentage_TankB, TempData);
				TempData = TempData * 10;
				FoamPercentage = TempData;
				smWrite(VariableIDs.RemoteOperationPrimary_FoamPercentage, FoamPercentage);
				break;
		case 2:	//TANK A
				smWrite(VariableIDs.RemoteOperationPrimary_TankSelected, 0);
				smRead(VariableIDs.Primary_TotalFoamGallons_A, TempData);
				if (UnitOfMeasure == 0)	//If we are in GPM then convert to LPM before sending
				{
					TempData = TempData * 37.854118;
				}
				else
				{
					TempData = TempData * 10;	//Move decimal
				}
				TotalFoamLiters = TempData;
				smWrite(VariableIDs.RemoteOperationPrimary_TotalFoamLiters, TotalFoamLiters);
				smRead(VariableIDs.Primary_FoamPercentage_TankA, TempData);
				TempData = TempData * 10;
				FoamPercentage = TempData;
				smWrite(VariableIDs.RemoteOperationPrimary_FoamPercentage, FoamPercentage);
				break;
		case 3:	//ERROR
				smWrite(VariableIDs.RemoteOperationPrimary_TankSelected, 3);
				smWrite(VariableIDs.RemoteOperationPrimary_TotalFoamLiters, 0);
				smWrite(VariableIDs.RemoteOperationPrimary_FoamPercentage, 0);
				break;
	}
	/*	Get the current state of the FoamLogix system.
		0  = OFF, 1 = ON in B, 2 = ON in A, 3 = mismatch
	*/
	smRead(VariableIDs.Primary_FoamLogix_Enabled, PowerState);
	switch (PowerState)
	{
		case 0:	smWrite(VariableIDs.RemoteOperationPrimary_SystemPowerState, 0);	
				break;
		case 1:	//Fall through (on in A and on in B are both ON conditions)
		case 2:	smWrite(VariableIDs.RemoteOperationPrimary_SystemPowerState, 1);	
				break;
		case 3: smWrite(VariableIDs.RemoteOperationPrimary_SystemPowerState, 2);	
				break;
	}
	
	smRead(VariableIDs.Primary_WaterFlowRate_GPM, TempData);
	if (UnitOfMeasure == 0)	//If we are in GPM then convert to LPM before sending
	{
		TempData = TempData * 3.7854118;
	}
	WaterFlowLPM = TempData;
	smWrite(VariableIDs.RemoteOperationPrimary_WaterFlowLPM, WaterFlowLPM);
	
	smRead(VariableIDs.Primary_TotalWaterGallons, TempData);
	if (UnitOfMeasure == 0)	//If we are in GPM then convert to LPM before sending
	{
		TempData = TempData * 37.854118;
	}
	else
	{
		TempData = TempData * 10;	//Move decimal
	}
	TotalWaterLiters = TempData;
	smWrite(VariableIDs.RemoteOperationPrimary_TotalWaterLiters, TotalWaterLiters);
	
	smRead(VariableIDs.FoamPrimary_Foam_A_Low, TankALevel);
	smWrite(VariableIDs.RemoteOperationPrimary_TankALevel, TankALevel);
	smRead(VariableIDs.FoamPrimary_Foam_B_Low, TankBLevel);
	smWrite(VariableIDs.RemoteOperationPrimary_TankBLevel, TankBLevel);
	
	
/*	SECONDARY system calculations ****************************************************************************************
*/
	/*	Get the currently selected foam tank.  This variable is updated in the SCRIPT CalculateMotorPWM.
		0 = none (FLUSH)
		1 = B
		2 = A
		3 = Tank select mismatch
	*/
	smRead(VariableIDs.Secondary_FoamSystemTankSelection, TankSelected);
	switch(TankSelected)
	{
		case 0:	//FLUSH
				smWrite(VariableIDs.RemoteOperationSecondary_TankSelected, 2);
				smWrite(VariableIDs.RemoteOperationSecondary_TotalFoamLiters, 0);
				smWrite(VariableIDs.RemoteOperationSecondary_FoamPercentage, 0);
				break;
		case 1:	//TANK B
				smWrite(VariableIDs.RemoteOperationSecondary_TankSelected, 1);
				smRead(VariableIDs.Secondary_TotalFoamGallons_B, TempData);
				if (UnitOfMeasure == 0)	//If we are in GPM then convert to LPM before sending
				{
					TempData = TempData * 37.854118;
				}
				else
				{
					TempData = TempData * 10;	//Move decimal
				}
				TotalFoamLiters = TempData;
				smWrite(VariableIDs.RemoteOperationSecondary_TotalFoamLiters, TotalFoamLiters);
				smRead(VariableIDs.Secondary_FoamPercentage_TankB, TempData);
				TempData = TempData * 10;
				FoamPercentage = TempData;
				smWrite(VariableIDs.RemoteOperationSecondary_FoamPercentage, FoamPercentage);
				break;
		case 2:	//TANK A
				smWrite(VariableIDs.RemoteOperationSecondary_TankSelected, 0);
				smRead(VariableIDs.Secondary_TotalFoamGallons_A, TempData);
				if (UnitOfMeasure == 0)	//If we are in GPM then convert to LPM before sending
				{
					TempData = TempData * 37.854118;
				}
				else
				{
					TempData = TempData * 10;	//Move decimal
				}
				TotalFoamLiters = TempData;
				smWrite(VariableIDs.RemoteOperationSecondary_TotalFoamLiters, TotalFoamLiters);
				smRead(VariableIDs.Secondary_FoamPercentage_TankA, TempData);
				TempData = TempData * 10;
				FoamPercentage = TempData;
				smWrite(VariableIDs.RemoteOperationSecondary_FoamPercentage, FoamPercentage);
				break;
		case 3:	//ERROR
				smWrite(VariableIDs.RemoteOperationSecondary_TankSelected, 3);
				smWrite(VariableIDs.RemoteOperationSecondary_TotalFoamLiters, 0);
				smWrite(VariableIDs.RemoteOperationSecondary_FoamPercentage, 0);
				break;
	}
	/*	Get the current state of the FoamLogix system.
		0  = OFF, 1 = ON in B, 2 = ON in A, 3 = mismatch
	*/
	smRead(VariableIDs.Secondary_FoamLogix_Enabled, PowerState);
	switch (PowerState)
	{
		case 0:	smWrite(VariableIDs.RemoteOperationSecondary_SystemPowerState, 0);	
				break;
		case 1:	//Fall through (on in A and on in B are both ON conditions)
		case 2:	smWrite(VariableIDs.RemoteOperationSecondary_SystemPowerState, 1);	
				break;
		case 3: smWrite(VariableIDs.RemoteOperationSecondary_SystemPowerState, 2);	
				break;
	}
	
	smRead(VariableIDs.Secondary_WaterFlowRate_GPM, TempData);
	if (UnitOfMeasure == 0)	//If we are in GPM then convert to LPM before sending
	{
		TempData = TempData * 3.7854118;
	}
	WaterFlowLPM = TempData;
	smWrite(VariableIDs.RemoteOperationSecondary_WaterFlowLPM, WaterFlowLPM);
	
	smRead(VariableIDs.Secondary_TotalWaterGallons, TempData);
	if (UnitOfMeasure == 0)	//If we are in GPM then convert to LPM before sending
	{
		TempData = TempData * 37.854118;
	}
	else
	{
		TempData = TempData * 10;	//Move decimal
	}
	TotalWaterLiters = TempData;
	smWrite(VariableIDs.RemoteOperationSecondary_TotalWaterLiters, TotalWaterLiters);
	
	smRead(VariableIDs.FoamSecondary_Foam_A_Low, TankALevel);
	smWrite(VariableIDs.RemoteOperationSecondary_TankALevel, TankALevel);
	smRead(VariableIDs.FoamSecondary_Foam_B_Low, TankBLevel);
	smWrite(VariableIDs.RemoteOperationSecondary_TankBLevel, TankBLevel);
	
	
	
	sendEvent(EventIDs.Event_TransmitRemoteCAN);
	   
}

//-----ScriptName=SCRIPT_RemoteRequest_Primary
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_RemoteRequest_Primary for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_RemoteRequest_Primary () 
{
 	uint INCR_Request;
	uint DECR_Request;
	uint ClearTotalWater;
	uint ClearTotalFoam;
	uint PowerRequest;
	uint PrimePump;
	
	smRead(VariableIDs.RemoteOperationPrimary_ClearTotalWater, ClearTotalWater);
	smRead(VariableIDs.RemoteOperationPrimary_ClearTotalFoam, ClearTotalFoam);
	smRead(VariableIDs.RemoteOperationPrimary_DecreaseRequest, DECR_Request);
	smRead(VariableIDs.RemoteOperationPrimary_IncreaseRequest, INCR_Request);
	smRead(VariableIDs.RemoteOperationPrimary_PowerRequest, PowerRequest);
	smRead(VariableIDs.RemoteOperationPrimary_PrimePump, PrimePump);
	
	
	switch (PrimePump)
	{
		case 0:	sendEvent(EventIDs.Calculate_PrimeFoamPump_Power_Equals_OFF);	
				break;
		case 1:	sendEvent(EventIDs.Calculate_PrimeFoamPump_Power_Equals_ON);
				break;
		case 2:	sendEvent(EventIDs.Calculate_PrimeFoamPump_Power_Equals_OFF);
				break;
		case 3:	sendEvent(EventIDs.Calculate_PrimeFoamPump_Power_Equals_OFF);
				break;
	}
	
	
	switch (PowerRequest)
	{
		case 0:	TurnFoamSystem_OFF(0);	
				break;
		case 1:	TurnFoamSystem_ON(0);
				break;
		case 2:	//Not used
				break;
		case 3:	//Not used
				break;
	}
	
	switch (ClearTotalWater)
	{
		case 0:	//Stasis	
				break;
		case 1:	sendEvent(EventIDs.Calculate_Primary_TotalWaterGallons_Equals_CLEAR);
				break;
		case 2:	//Not used
				break;
		case 3:	//Not used
				break;
	}
	
	switch (ClearTotalFoam)
	{
		case 0:	//Statis	
				break;
		case 1:	smWrite(VariableIDs.FoamPrimary_Command, 3);	//clear total foam flow value.
				sendEvent(EventIDs.Event_StartPrimaryFoamClearCommandTimer);
				smWrite(VariableIDs.Primary_TotalFoamGallons_A, 0);
				smWrite(VariableIDs.Primary_TotalFoamGallons_B, 0);
				break;
		case 2:	//Not used
				break;
		case 3:	//Not used
				break;
	}
	
	switch (INCR_Request)
	{
		case 0:	//Stasis	
				break;
		case 1:	sendEvent(EventIDs.SCRIPT_Primary_FoamPercentage_Increase);
				break;
		case 2:	//Not used
				break;
		case 3:	//Not used
				break;
	}
	
	switch (DECR_Request)
	{
		case 0:	//Stasis	
				break;
		case 1:	sendEvent(EventIDs.SCRIPT_Primary_FoamPercentage_Decrease);
				break;
		case 2:	//Not used
				break;
		case 3:	//Not used
				break;
	}
		   
}

//-----ScriptName=SCRIPT_RemoteRequest_Secondary
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_RemoteRequest_Secondary for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_RemoteRequest_Secondary () 
{
 	uint INCR_Request;
	uint DECR_Request;
	uint ClearTotalWater;
	uint ClearTotalFoam;
	uint PowerRequest;
	uint PrimePump;
	
	smRead(VariableIDs.RemoteOperationSecondary_ClearTotalWater, ClearTotalWater);
	smRead(VariableIDs.RemoteOperationSecondary_ClearTotalFoam, ClearTotalFoam);
	smRead(VariableIDs.RemoteOperationSecondary_DecreaseRequest, DECR_Request);
	smRead(VariableIDs.RemoteOperationSecondary_IncreaseRequest, INCR_Request);
	smRead(VariableIDs.RemoteOperationSecondary_PowerRequest, PowerRequest);
	smRead(VariableIDs.RemoteOperationSecondary_PrimePump, PrimePump);
	
	
	switch (PrimePump)
	{
		case 0:	sendEvent(EventIDs.Calculate_PrimeFoamPump_Power_2_Equals_OFF);	
				break;
		case 1:	sendEvent(EventIDs.Calculate_PrimeFoamPump_Power_2_Equals_ON);
				break;
		case 2:	sendEvent(EventIDs.Calculate_PrimeFoamPump_Power_2_Equals_OFF);
				break;
		case 3:	sendEvent(EventIDs.Calculate_PrimeFoamPump_Power_2_Equals_OFF);
				break;
	}
	
	switch (PowerRequest)
	{
		case 0:	TurnFoamSystem_OFF(1);	
				break;
		case 1:	TurnFoamSystem_ON(1);
				break;
		case 2:	//Not used
				break;
		case 3:	//Not used
				break;
	}
	
	switch (ClearTotalWater)
	{
		case 0:	//Stasis	
				break;
		case 1:	sendEvent(EventIDs.Calculate_Secondary_TotalWaterGallons_Equals_CLEAR);
				break;
		case 2:	//Not used
				break;
		case 3:	//Not used
				break;
	}
	
	switch (ClearTotalFoam)
	{
		case 0:	//Statis	
				break;
		case 1:	smWrite(VariableIDs.FoamSecondary_Command, 3);	//clear total foam flow value.
				sendEvent(EventIDs.Event_StartSecondaryFoamClearCommandTimer);
				smWrite(VariableIDs.Secondary_TotalFoamGallons_A, 0);
				smWrite(VariableIDs.Secondary_TotalFoamGallons_B, 0);
				break;
		case 2:	//Not used
				break;
		case 3:	//Not used
				break;
	}
	
	switch (INCR_Request)
	{
		case 0:	//Stasis	
				break;
		case 1:	sendEvent(EventIDs.SCRIPT_Secondary_FoamPercentage_Increase);
				break;
		case 2:	//Not used
				break;
		case 3:	//Not used
				break;
	}
	
	switch (DECR_Request)
	{
		case 0:	//Stasis	
				break;
		case 1:	sendEvent(EventIDs.SCRIPT_Secondary_FoamPercentage_Decrease);
				break;
		case 2:	//Not used
				break;
		case 3:	//Not used
				break;
	}   
}

//-----ScriptName=SCRIPT_MotorType2_DECR
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_MotorType2_DECR for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_MotorType2_DECR () 
{
 	 uint MotorType;
	smRead(VariableIDs.tmp_MotorController_Type_2, MotorType);
	
	MotorType--;
		
	if (MotorType > 8 || MotorType < 1)
		MotorType = 8; 
	
	smWrite(VariableIDs.tmp_MotorController_Type_2, MotorType);    
}

//-----ScriptName=SCRIPT_MotorType2_INCR
//---------------------------------------------------------------------------------------------
// Murphy Scripting
// - Leave EventName as SCRIPT_MotorType2_INCR for main script method
//---------------------------------------------------------------------------------------------
void SCRIPT_MotorType2_INCR () 
{
  	uint MotorType;
	smRead(VariableIDs.tmp_MotorController_Type_2, MotorType);
	
	MotorType++;
		
	if (MotorType > 8 || MotorType < 1)
		MotorType = 1;  
	
	smWrite(VariableIDs.tmp_MotorController_Type_2, MotorType);  
}



_ModbusMappingGroupIDs ModbusMappingGroupIDs;
class _ModbusMappingGroupIDs
{

   _ModbusMappingGroupIDs () 
   {

   }
}

_ModbusPort ModbusPort;
class _ModbusPort
{

   _ModbusPort () 
   {

   }
}

_VariableIDs VariableIDs;
class _VariableIDs
{  uint Primary_WaterCal_2PointOffset;
  uint Primary_WaterCal_2PointSlope;
  uint Primary_SimulatedWaterFlow_Enabled;
  uint FLX_WaterFlowSource;
  uint Primary_RawWaterFlowPulses;
  uint Primary_RawWaterFlowPulses_NewMotorDriver;
  uint Primary_SimulatedFlowGPM;
  uint Primary_WaterFlowRate_GPM;
  uint FLX_AutoEnable;
  uint Primary_FoamLogix_Enabled;
  uint Primary_FoamSystemTankSelection;
  uint BIT_WaterFlowActive;
  uint Primary_TotalWaterGallons;
  uint WaterFlow_NumberBuffers;
  uint Primary_RawWaterFlowRate_Buffered;
  uint FLX_PulseFactor;
  uint Primary_FoamFlowCalculationValue_A;
  uint Primary_FoamFlowCalculationValue_B;
  uint PrimeFoamPump_Power;
  uint DebuuginModeActive;
  uint FoamPrimary_RawFoamFlowRate;
  uint RawFoamFlow_Primary;
  uint Primary_FoamFlowRateGPM;
  uint Primary_TotalFoamGallons_A;
  uint Primary_TotalFoamGallons_B;
  uint FoamPrimary_TotalFoamPulses;
  uint FoamSystemControlType;
  uint Secondary_WaterFlowRate_GPM;
  uint Primary_MotorPWM_Request;
  uint Secondary_MotorPWM_Request;
  uint FLX_NumberOfPumps;
  uint PrimeFoamPump_Power_2;
  uint Secondary_FoamLogix_Enabled;
  uint Calibration_PrimaryMotorEnable;
  uint Calibration_SecondaryMotorEnable;
  uint FreeFormCAN_AutoBypassOutput_1;
  uint FreeFormCAN_AutoBypassOutput_2;
  uint FLX_InitialFastStartSpeed;
  uint FoamSecondary_RawFoamFlowRate;
  uint Secondary_FoamFlowRateGPM;
  uint Displayed_WaterFlow_GPM;
  uint Secondary_TotalWaterGallons;
  uint Displayed_TotalWaterFlowed;
  uint BIT_FoamLogixEnabled;
  uint FLX_FlushTime;
  uint FoamPrimary_TemperatureCritical;
  uint FoamSecondary_TemperatureCritical;
  uint FLX_PrimeLostStart;
  uint LimpHomeMode_Active_Motor_1;
  uint LimpHomeMode_Active_Motor_2;
  uint Primary_FoamPercentage_TankB;
  uint Primary_FoamPercentage_TankA;
  uint Primary_FoamFlowRateGPM_Requested;
  uint DEBUG_1;
  uint DEBUG_3;
  uint FLX_MinimumFoamFlowRate;
  uint FLX_LowFlowTransitionHysteresis;
  uint DEBUG_5;
  uint FLX_UnitOfMeasure;
  uint FLX_LowFoamTime_OFF;
  uint FLX_LowFoamTime_ON;
  uint FLX_InitialFastStartTime;
  uint DEBUG_7;
  uint FlushTimeRemaining_Primary;
  uint FreeFormCAN_TankA_Select;
  uint FreeFormCAN_TankB_Select;
  uint CurrentFoamTypeInLine;
  uint Secondary_DebugMotorPWM;
  uint Primary_DebugMotorPWM;
  uint HasSystemBeenRun;
  uint BarGraph_PWM_Value_1;
  uint BarGraph_PWM_Value_2;
  uint FoamPrimeStatus;
  uint SumFoamFlowRate_GPM;
  uint FoamSecondary_FoamTankSelected;
  uint Secondary_FoamPercentage_TankB;
  uint Secondary_FoamPercentage_TankA;
  uint Secondary_FoamFlowRateGPM_Requested;
  uint DEBUG_2;
  uint DEBUG_4;
  uint DEBUG_6;
  uint FlushTimeRemaining_Secondary;
  uint FoamPrimary_Foam_A_Low;
  uint FoamPrimary_Foam_B_Low;
  uint FLX_PrimeRampSpeed;
  uint FLX_PrimeLostTime;
  uint FoamPrimeLoss_CountdownTimer_Motor_1;
  uint FoamPrimeLoss_CountdownTimer_Motor_2;
  uint FLX_Sensitivity;
  uint FLX_Gain;
  uint FLX_FoamCalibrationSpeed;
  uint FoamPrimary_Command;
  uint FoamSecondary_Command;
  uint FoamLogix_Active_Display;
  uint WaterFlowCalibration_HighValue;
  uint WaterFlowCalibration_LowValue;
  uint WaterFlowCalibration_HighRawPulses;
  uint WaterFlowCalibration_LowRawPulses;
  uint AllowSaveOfWaterFlowVariables;
  uint PasswordValue;
  uint FLX_DisplayOrientation;
  uint SelectedOperationVariable;
  uint SelectedUserVariable;
  uint SelectedOEMVariable;
  uint Secondary_FoamFlowCalculationValue_A;
  uint Secondary_FoamFlowCalculationValue_B;
  uint RawFoamFlow_Secondary;
  uint Secondary_TotalFoamGallons_A;
  uint Secondary_TotalFoamGallons_B;
  uint FoamSecondary_TotalFoamPulses;
  uint FoamPrimary_FoamTankSelected;
  uint Secondary_FoamSystemTankSelection;
  uint FoamPrimary_TemperatureWarning;
  uint FoamSecondary_TemperatureWarning;
  uint FLX_WarningInhibitTime;
  uint FoamPump_1_LostPrime;
  uint FoamPump_2_LostPrime;
  uint FoamPrimary_DriverTemperature;
  uint FoamSecondary_DriverTemperature;
  uint RemoteOperation_FoamLogixWarningStatus;
  uint BIT_SelectedFoamTankIsLow;
  uint BIT_FlushIsActive;
  uint BIT_NoPrimeIsActive;
  uint CalibrationMode;
  uint ShowingCurrentWarning;
  uint tmp_FLX_Gain;
  uint tmp_FLX_Sensitivity;
  uint tmp_FLX_AlgorithmDiv;
  uint tmp_FLX_PrimeLostStart;
  uint tmp_FLX_PrimeRampSpeed;
  uint tmp_FLX_PrimeLostTime;
  uint tmp_FLX_FoamCalibrationSpeed;
  uint tmp_FLX_InitialFastStartSpeed;
  uint tmp_FLX_InitialFastStartTime;
  uint tmp_FLX_NumberOfPumps;
  uint tmp_FLX_MinimumFoamFlowRate;
  uint tmp_FLX_LowFlowTransitionHysteresis;
  uint tmp_FLX_WarningInhibitTime;
  uint tmp_FLX_VoltageTweak;
  uint tmp_FLX_LowFoamTime_OFF;
  uint tmp_FLX_LowFoamTime_ON;
  uint tmp_FLX_PulseFactor;
  uint tmpWaterFlow_NumberBuffers;
  uint SavingStep;
  uint RemoteActivationPreset;
  uint IO_RemoteActivationRequest;
  uint RemoteActivationFromSPS;
  uint LookupChart_NozzleSize_INCHES;
  uint LookupChart_NozzlePressure_PSI;
  uint LookupChart_GPM;
  uint LookupChart_NozzleSize_MILLIMETERS;
  uint LookupChart_NozzlePressure_BAR;
  uint LookupChart_LPM;
  uint IO_Battery_Voltage;
  uint FLX_VoltageTweak;
  uint FLX_LowVoltage_12V;
  uint FLX_LowVoltage_24V;
  uint SystemVoltage;
  uint VoltageIsLow;
  uint DemoModeActive;
  uint Demo_TankSelect;
  uint Secondary_RawWaterFlowPulses;
  uint Secondary_RawWaterFlowPulses_NewMotorDriver;
  uint FoamPrimary_MotorDutyCycle;
  uint FLX_FoamTankSelectorType;
  uint FoamSecondary_MotorDutyCycle;
  uint tmp_FoamSystemControlType;
  uint tmp_FLX_PumpModel;
  uint tmp_FLX_PumpModel_2;
  uint tmp_FLX_LowVoltage_12V;
  uint tmp_FLX_LowVoltage_24V;
  uint tmp_FoamLogix_AllowHomeButton;
  uint tmp_FLX_FoamTankSelectorType;
  uint tmp_FLX_MaxFoamPercent_A;
  uint tmp_FLX_MaxFoamPercent_B;
  uint tmp_FLX_WaterFlowSource;
  uint tmp_FLX_FlushTime;
  uint tmp_FLX_AutoEnable;
  uint FLX_Display_Type;
  uint OnPresetStartPage;
  uint Secondary_WaterCal_2PointOffset;
  uint Secondary_WaterCal_2PointSlope;
  uint Secondary_SimulatedWaterFlow_Enabled;
  uint Secondary_RawWaterFlowRate_Buffered;
  uint ShowUSBSaving;
  uint BarColor_Preset_1;
  uint BarColor_Preset_2;
  uint BarColor_Preset_3;
  uint BarColor_Preset_4;
  uint BarColor_Preset_5;
  uint BarColor_Preset_6;
  uint ButtonPreset_1;
  uint ButtonPreset_2;
  uint ButtonPreset_3;
  uint ButtonPreset_4;
  uint ButtonPreset_5;
  uint ButtonPreset_6;
  uint PresetButtonHoldTimeRemaining;
  uint BarIcon_Preset_1;
  uint BarIcon_Preset_2;
  uint BarIcon_Preset_3;
  uint BarIcon_Preset_4;
  uint BarIcon_Preset_5;
  uint BarIcon_Preset_6;
  uint ActivePreset_Icon;
  uint ActivePreset;
  uint FoamTank_Preset_1;
  uint FoamPercentage_Preset_1;
  uint FoamSystem_Preset_1;
  uint FoamTank_Preset_2;
  uint FoamPercentage_Preset_2;
  uint FoamSystem_Preset_2;
  uint FoamTank_Preset_3;
  uint FoamPercentage_Preset_3;
  uint FoamSystem_Preset_3;
  uint FoamTank_Preset_4;
  uint FoamPercentage_Preset_4;
  uint FoamSystem_Preset_4;
  uint FoamTank_Preset_5;
  uint FoamPercentage_Preset_5;
  uint FoamSystem_Preset_5;
  uint FoamTank_Preset_6;
  uint FoamPercentage_Preset_6;
  uint FoamSystem_Preset_6;
  uint TypePresetText;
  uint tmp_BarColor_Preset;
  uint tmp_BarIcon_Preset;
  uint tmp_FoamTank_Preset;
  uint tmp_FoamPercentage_Preset;
  uint tmp_FoamSystem_Preset;
  uint tmp_RemotePresetActivation_state;
  uint Num_Languages_Available;
  uint LanguageSelection;
  uint tmp_FLX_UnitOfMeasure;
  uint tmp_FLX_FoamA_Color;
  uint tmp_FLX_FoamB_Color;
  uint tmp_FLX_Display_Type;
  uint tmp_FLX_DisplayOrientation;
  uint tmp_LogoChoice;
  uint RealTimeClock_Hour;
  uint RealTimeClock_Minute;
  uint RealTimeClock_Second;
  uint RealTimeClock_Year;
  uint RealTimeClock_Month;
  uint RealTimeClock_Day;
  uint FoamLogix_Enabled_Bar_ind;
  uint FoamLogix_Decimal_0;
  uint FoamLogix_Decimal_1;
  uint FoamLogix_Digit_4;
  uint FoamLogix_Digit_3;
  uint FoamLogix_Digit_2;
  uint FoamLogix_Digit_1;
  uint FoamLogix_Digit_0;
  uint FoamLogix_AllowHomeButton;
  uint KnobIsTouched;
  uint Touch_Current_X;
  uint Touch_Current_Y;
  uint RemoteOperation_Byte_0;
  uint RemoteOperation_Byte_4;
  uint MaintenanceItemsDue;
  uint DateInterval_1M_Day;
  uint DateInterval_1M_Month;
  uint DateInterval_1M_Year;
  uint DateInterval_2M_Day;
  uint DateInterval_2M_Month;
  uint DateInterval_2M_Year;
  uint DateInterval_12M_Day;
  uint DateInterval_12M_Month;
  uint DateInterval_12M_Year;
  uint DateInterval_1M_NumDays;
  uint DateInterval_2M_NumDays;
  uint DateInterval_12M_NumDays;
  uint FLX_PumpModel;
  uint FLX_PumpModel_2;
  uint A2CModuleRX_Byte_4;
  uint A2CModuleRX_Byte_5;
  uint A2CModule_Bytes4_7;
  uint WaterFlowCalibration_HighValue_2;
  uint WaterFlowCalibration_LowValue_2;
  uint WaterFlowCalibration_HighRawPulses_2;
  uint WaterFlowCalibration_LowRawPulses_2;
  uint Primary_FoamCal_Button;
  uint Secondary_FoamCal_Button;
  uint FoamPrimary_Status;
  uint FoamSecondary_Status;
  uint FoamPrimary_TemparatureNormal;
  uint FoamSecondary_TemparatureNormal;
  uint Primary_SimFlowValue_Button_Setup;
  uint Primary_SimFlowValue_Button;
  uint PasswordAttemptWarning;
  uint PasswordInvalid;
  uint LogoChoice;
  uint tmp_MotorController_Type;
  uint RemoteOperationPrimary_TankSelected;
  uint RemoteOperationPrimary_TotalFoamLiters;
  uint RemoteOperationPrimary_FoamPercentage;
  uint RemoteOperationPrimary_SystemPowerState;
  uint RemoteOperationPrimary_WaterFlowLPM;
  uint RemoteOperationPrimary_TotalWaterLiters;
  uint RemoteOperationPrimary_TankALevel;
  uint RemoteOperationPrimary_TankBLevel;
  uint RemoteOperationSecondary_TankSelected;
  uint RemoteOperationSecondary_TotalFoamLiters;
  uint RemoteOperationSecondary_FoamPercentage;
  uint RemoteOperationSecondary_SystemPowerState;
  uint RemoteOperationSecondary_WaterFlowLPM;
  uint RemoteOperationSecondary_TotalWaterLiters;
  uint FoamSecondary_Foam_A_Low;
  uint RemoteOperationSecondary_TankALevel;
  uint FoamSecondary_Foam_B_Low;
  uint RemoteOperationSecondary_TankBLevel;
  uint RemoteOperationPrimary_ClearTotalWater;
  uint RemoteOperationPrimary_ClearTotalFoam;
  uint RemoteOperationPrimary_DecreaseRequest;
  uint RemoteOperationPrimary_IncreaseRequest;
  uint RemoteOperationPrimary_PowerRequest;
  uint RemoteOperationPrimary_PrimePump;
  uint RemoteOperationSecondary_ClearTotalWater;
  uint RemoteOperationSecondary_ClearTotalFoam;
  uint RemoteOperationSecondary_DecreaseRequest;
  uint RemoteOperationSecondary_IncreaseRequest;
  uint RemoteOperationSecondary_PowerRequest;
  uint RemoteOperationSecondary_PrimePump;
  uint tmp_MotorController_Type_2;


   _VariableIDs () 
   {
	Primary_WaterCal_2PointOffset = 414;
	Primary_WaterCal_2PointSlope = 415;
	Primary_SimulatedWaterFlow_Enabled = 134;
	FLX_WaterFlowSource = 501;
	Primary_RawWaterFlowPulses = 528;
	Primary_RawWaterFlowPulses_NewMotorDriver = 545;
	Primary_SimulatedFlowGPM = 409;
	Primary_WaterFlowRate_GPM = 115;
	FLX_AutoEnable = 505;
	Primary_FoamLogix_Enabled = 124;
	Primary_FoamSystemTankSelection = 172;
	BIT_WaterFlowActive = 177;
	Primary_TotalWaterGallons = 117;
	WaterFlow_NumberBuffers = 557;
	Primary_RawWaterFlowRate_Buffered = 360;
	FLX_PulseFactor = 434;
	Primary_FoamFlowCalculationValue_A = 405;
	Primary_FoamFlowCalculationValue_B = 410;
	PrimeFoamPump_Power = 272;
	DebuuginModeActive = 280;
	FoamPrimary_RawFoamFlowRate = 524;
	RawFoamFlow_Primary = 274;
	Primary_FoamFlowRateGPM = 114;
	Primary_TotalFoamGallons_A = 112;
	Primary_TotalFoamGallons_B = 113;
	FoamPrimary_TotalFoamPulses = 525;
	FoamSystemControlType = 435;
	Secondary_WaterFlowRate_GPM = 208;
	Primary_MotorPWM_Request = 120;
	Secondary_MotorPWM_Request = 140;
	FLX_NumberOfPumps = 428;
	PrimeFoamPump_Power_2 = 273;
	Secondary_FoamLogix_Enabled = 211;
	Calibration_PrimaryMotorEnable = 132;
	Calibration_SecondaryMotorEnable = 169;
	FreeFormCAN_AutoBypassOutput_1 = 547;
	FreeFormCAN_AutoBypassOutput_2 = 548;
	FLX_InitialFastStartSpeed = 424;
	FoamSecondary_RawFoamFlowRate = 534;
	Secondary_FoamFlowRateGPM = 148;
	Displayed_WaterFlow_GPM = 270;
	Secondary_TotalWaterGallons = 207;
	Displayed_TotalWaterFlowed = 271;
	BIT_FoamLogixEnabled = 178;
	FLX_FlushTime = 502;
	FoamPrimary_TemperatureCritical = 523;
	FoamSecondary_TemperatureCritical = 533;
	FLX_PrimeLostStart = 426;
	LimpHomeMode_Active_Motor_1 = 285;
	LimpHomeMode_Active_Motor_2 = 286;
	Primary_FoamPercentage_TankB = 408;
	Primary_FoamPercentage_TankA = 407;
	Primary_FoamFlowRateGPM_Requested = 129;
	DEBUG_1 = 116;
	DEBUG_3 = 122;
	FLX_MinimumFoamFlowRate = 427;
	FLX_LowFlowTransitionHysteresis = 433;
	DEBUG_5 = 364;
	FLX_UnitOfMeasure = 440;
	FLX_LowFoamTime_OFF = 476;
	FLX_LowFoamTime_ON = 475;
	FLX_InitialFastStartTime = 425;
	DEBUG_7 = 366;
	FlushTimeRemaining_Primary = 263;
	FreeFormCAN_TankA_Select = 549;
	FreeFormCAN_TankB_Select = 550;
	CurrentFoamTypeInLine = 389;
	Secondary_DebugMotorPWM = 282;
	Primary_DebugMotorPWM = 281;
	HasSystemBeenRun = 503;
	BarGraph_PWM_Value_1 = 232;
	BarGraph_PWM_Value_2 = 233;
	FoamPrimeStatus = 130;
	SumFoamFlowRate_GPM = 183;
	FoamSecondary_FoamTankSelected = 530;
	Secondary_FoamPercentage_TankB = 437;
	Secondary_FoamPercentage_TankA = 436;
	Secondary_FoamFlowRateGPM_Requested = 212;
	DEBUG_2 = 121;
	DEBUG_4 = 123;
	DEBUG_6 = 365;
	FlushTimeRemaining_Secondary = 287;
	FoamPrimary_Foam_A_Low = 527;
	FoamPrimary_Foam_B_Low = 519;
	FLX_PrimeRampSpeed = 421;
	FLX_PrimeLostTime = 422;
	FoamPrimeLoss_CountdownTimer_Motor_1 = 131;
	FoamPrimeLoss_CountdownTimer_Motor_2 = 153;
	FLX_Sensitivity = 420;
	FLX_Gain = 419;
	FLX_FoamCalibrationSpeed = 423;
	FoamPrimary_Command = 126;
	FoamSecondary_Command = 145;
	FoamLogix_Active_Display = 241;
	WaterFlowCalibration_HighValue = 412;
	WaterFlowCalibration_LowValue = 413;
	WaterFlowCalibration_HighRawPulses = 416;
	WaterFlowCalibration_LowRawPulses = 417;
	AllowSaveOfWaterFlowVariables = 154;
	PasswordValue = 136;
	FLX_DisplayOrientation = 498;
	SelectedOperationVariable = 155;
	SelectedUserVariable = 235;
	SelectedOEMVariable = 205;
	Secondary_FoamFlowCalculationValue_A = 411;
	Secondary_FoamFlowCalculationValue_B = 406;
	RawFoamFlow_Secondary = 275;
	Secondary_TotalFoamGallons_A = 149;
	Secondary_TotalFoamGallons_B = 150;
	FoamSecondary_TotalFoamPulses = 535;
	FoamPrimary_FoamTankSelected = 520;
	Secondary_FoamSystemTankSelection = 210;
	FoamPrimary_TemperatureWarning = 522;
	FoamSecondary_TemperatureWarning = 532;
	FLX_WarningInhibitTime = 418;
	FoamPump_1_LostPrime = 170;
	FoamPump_2_LostPrime = 171;
	FoamPrimary_DriverTemperature = 526;
	FoamSecondary_DriverTemperature = 536;
	RemoteOperation_FoamLogixWarningStatus = 553;
	BIT_SelectedFoamTankIsLow = 180;
	BIT_FlushIsActive = 181;
	BIT_NoPrimeIsActive = 182;
	CalibrationMode = 152;
	ShowingCurrentWarning = 151;
	tmp_FLX_Gain = 159;
	tmp_FLX_Sensitivity = 165;
	tmp_FLX_AlgorithmDiv = 185;
	tmp_FLX_PrimeLostStart = 162;
	tmp_FLX_PrimeRampSpeed = 164;
	tmp_FLX_PrimeLostTime = 163;
	tmp_FLX_FoamCalibrationSpeed = 158;
	tmp_FLX_InitialFastStartSpeed = 160;
	tmp_FLX_InitialFastStartTime = 161;
	tmp_FLX_NumberOfPumps = 174;
	tmp_FLX_MinimumFoamFlowRate = 173;
	tmp_FLX_LowFlowTransitionHysteresis = 200;
	tmp_FLX_WarningInhibitTime = 166;
	tmp_FLX_VoltageTweak = 195;
	tmp_FLX_LowFoamTime_OFF = 238;
	tmp_FLX_LowFoamTime_ON = 239;
	tmp_FLX_PulseFactor = 204;
	tmpWaterFlow_NumberBuffers = 555;
	SavingStep = 168;
	RemoteActivationPreset = 474;
	IO_RemoteActivationRequest = 538;
	RemoteActivationFromSPS = 184;
	LookupChart_NozzleSize_INCHES = 187;
	LookupChart_NozzlePressure_PSI = 186;
	LookupChart_GPM = 188;
	LookupChart_NozzleSize_MILLIMETERS = 192;
	LookupChart_NozzlePressure_BAR = 191;
	LookupChart_LPM = 190;
	IO_Battery_Voltage = 539;
	FLX_VoltageTweak = 430;
	FLX_LowVoltage_12V = 431;
	FLX_LowVoltage_24V = 432;
	SystemVoltage = 196;
	VoltageIsLow = 194;
	DemoModeActive = 472;
	Demo_TankSelect = 236;
	Secondary_RawWaterFlowPulses = 542;
	Secondary_RawWaterFlowPulses_NewMotorDriver = 546;
	FoamPrimary_MotorDutyCycle = 119;
	FLX_FoamTankSelectorType = 465;
	FoamSecondary_MotorDutyCycle = 143;
	tmp_FoamSystemControlType = 206;
	tmp_FLX_PumpModel = 264;
	tmp_FLX_PumpModel_2 = 362;
	tmp_FLX_LowVoltage_12V = 197;
	tmp_FLX_LowVoltage_24V = 198;
	tmp_FoamLogix_AllowHomeButton = 251;
	tmp_FLX_FoamTankSelectorType = 229;
	tmp_FLX_MaxFoamPercent_A = 346;
	tmp_FLX_MaxFoamPercent_B = 347;
	tmp_FLX_WaterFlowSource = 349;
	tmp_FLX_FlushTime = 352;
	tmp_FLX_AutoEnable = 363;
	FLX_Display_Type = 477;
	OnPresetStartPage = 230;
	Secondary_WaterCal_2PointOffset = 438;
	Secondary_WaterCal_2PointSlope = 439;
	Secondary_SimulatedWaterFlow_Enabled = 209;
	Secondary_RawWaterFlowRate_Buffered = 361;
	ShowUSBSaving = 214;
	BarColor_Preset_1 = 441;
	BarColor_Preset_2 = 442;
	BarColor_Preset_3 = 443;
	BarColor_Preset_4 = 444;
	BarColor_Preset_5 = 445;
	BarColor_Preset_6 = 446;
	ButtonPreset_1 = 216;
	ButtonPreset_2 = 217;
	ButtonPreset_3 = 218;
	ButtonPreset_4 = 219;
	ButtonPreset_5 = 220;
	ButtonPreset_6 = 221;
	PresetButtonHoldTimeRemaining = 558;
	BarIcon_Preset_1 = 447;
	BarIcon_Preset_2 = 448;
	BarIcon_Preset_3 = 449;
	BarIcon_Preset_4 = 450;
	BarIcon_Preset_5 = 451;
	BarIcon_Preset_6 = 452;
	ActivePreset_Icon = 228;
	ActivePreset = 226;
	FoamTank_Preset_1 = 453;
	FoamPercentage_Preset_1 = 454;
	FoamSystem_Preset_1 = 466;
	FoamTank_Preset_2 = 456;
	FoamPercentage_Preset_2 = 455;
	FoamSystem_Preset_2 = 467;
	FoamTank_Preset_3 = 461;
	FoamPercentage_Preset_3 = 457;
	FoamSystem_Preset_3 = 468;
	FoamTank_Preset_4 = 462;
	FoamPercentage_Preset_4 = 458;
	FoamSystem_Preset_4 = 469;
	FoamTank_Preset_5 = 463;
	FoamPercentage_Preset_5 = 459;
	FoamSystem_Preset_5 = 470;
	FoamTank_Preset_6 = 464;
	FoamPercentage_Preset_6 = 460;
	FoamSystem_Preset_6 = 471;
	TypePresetText = 222;
	tmp_BarColor_Preset = 223;
	tmp_BarIcon_Preset = 224;
	tmp_FoamTank_Preset = 387;
	tmp_FoamPercentage_Preset = 386;
	tmp_FoamSystem_Preset = 231;
	tmp_RemotePresetActivation_state = 237;
	Num_Languages_Available = 176;
	LanguageSelection = 403;
	tmp_FLX_UnitOfMeasure = 213;
	tmp_FLX_FoamA_Color = 265;
	tmp_FLX_FoamB_Color = 234;
	tmp_FLX_Display_Type = 240;
	tmp_FLX_DisplayOrientation = 288;
	tmp_LogoChoice = 289;
	RealTimeClock_Hour = 53;
	RealTimeClock_Minute = 52;
	RealTimeClock_Second = 51;
	RealTimeClock_Year = 56;
	RealTimeClock_Month = 55;
	RealTimeClock_Day = 54;
	FoamLogix_Enabled_Bar_ind = 247;
	FoamLogix_Decimal_0 = 248;
	FoamLogix_Decimal_1 = 249;
	FoamLogix_Digit_4 = 246;
	FoamLogix_Digit_3 = 245;
	FoamLogix_Digit_2 = 244;
	FoamLogix_Digit_1 = 243;
	FoamLogix_Digit_0 = 242;
	FoamLogix_AllowHomeButton = 478;
	KnobIsTouched = 252;
	Touch_Current_X = 89;
	Touch_Current_Y = 90;
	RemoteOperation_Byte_0 = 253;
	RemoteOperation_Byte_4 = 257;
	MaintenanceItemsDue = 487;
	DateInterval_1M_Day = 488;
	DateInterval_1M_Month = 489;
	DateInterval_1M_Year = 490;
	DateInterval_2M_Day = 481;
	DateInterval_2M_Month = 480;
	DateInterval_2M_Year = 479;
	DateInterval_12M_Day = 484;
	DateInterval_12M_Month = 482;
	DateInterval_12M_Year = 483;
	DateInterval_1M_NumDays = 491;
	DateInterval_2M_NumDays = 485;
	DateInterval_12M_NumDays = 486;
	FLX_PumpModel = 492;
	FLX_PumpModel_2 = 504;
	A2CModuleRX_Byte_4 = 543;
	A2CModuleRX_Byte_5 = 544;
	A2CModule_Bytes4_7 = 541;
	WaterFlowCalibration_HighValue_2 = 495;
	WaterFlowCalibration_LowValue_2 = 497;
	WaterFlowCalibration_HighRawPulses_2 = 494;
	WaterFlowCalibration_LowRawPulses_2 = 496;
	Primary_FoamCal_Button = 268;
	Secondary_FoamCal_Button = 269;
	FoamPrimary_Status = 276;
	FoamSecondary_Status = 277;
	FoamPrimary_TemparatureNormal = 278;
	FoamSecondary_TemparatureNormal = 279;
	Primary_SimFlowValue_Button_Setup = 284;
	Primary_SimFlowValue_Button = 283;
	PasswordAttemptWarning = 138;
	PasswordInvalid = 137;
	LogoChoice = 404;
	tmp_MotorController_Type = 345;
	RemoteOperationPrimary_TankSelected = 567;
	RemoteOperationPrimary_TotalFoamLiters = 562;
	RemoteOperationPrimary_FoamPercentage = 565;
	RemoteOperationPrimary_SystemPowerState = 566;
	RemoteOperationPrimary_WaterFlowLPM = 559;
	RemoteOperationPrimary_TotalWaterLiters = 561;
	RemoteOperationPrimary_TankALevel = 568;
	RemoteOperationPrimary_TankBLevel = 569;
	RemoteOperationSecondary_TankSelected = 572;
	RemoteOperationSecondary_TotalFoamLiters = 563;
	RemoteOperationSecondary_FoamPercentage = 574;
	RemoteOperationSecondary_SystemPowerState = 573;
	RemoteOperationSecondary_WaterFlowLPM = 560;
	RemoteOperationSecondary_TotalWaterLiters = 564;
	FoamSecondary_Foam_A_Low = 537;
	RemoteOperationSecondary_TankALevel = 571;
	FoamSecondary_Foam_B_Low = 529;
	RemoteOperationSecondary_TankBLevel = 570;
	RemoteOperationPrimary_ClearTotalWater = 578;
	RemoteOperationPrimary_ClearTotalFoam = 579;
	RemoteOperationPrimary_DecreaseRequest = 577;
	RemoteOperationPrimary_IncreaseRequest = 576;
	RemoteOperationPrimary_PowerRequest = 575;
	RemoteOperationPrimary_PrimePump = 584;
	RemoteOperationSecondary_ClearTotalWater = 583;
	RemoteOperationSecondary_ClearTotalFoam = 357;
	RemoteOperationSecondary_DecreaseRequest = 582;
	RemoteOperationSecondary_IncreaseRequest = 581;
	RemoteOperationSecondary_PowerRequest = 580;
	RemoteOperationSecondary_PrimePump = 585;
	tmp_MotorController_Type_2 = 369;

   }
}

_ListDataType ListDataType;
class _ListDataType
{

   _ListDataType () 
   {

   }
}

_EventIDs EventIDs;
class _EventIDs
{  uint SCRIPT_UpdatePerformanceChart;
  uint Event_DataLoggingStart;
  uint Event_DataLoggingStop;
  uint SCRIPT_FoamLogixPowerButton_Primary;
  uint Event_TransmitOutputControlMessage;
  uint Warning_FlushIsComplete;
  uint SCRIPT_FoamLogixPowerButton_Secondary;
  uint WARNING_Popup_No_Foam_A;
  uint WARNING_Popup_No_Foam_B;
  uint WARNING_Popup_Limp_Home_Mode_1;
  uint Warnings_ClearMenuPageWarnings;
  uint Warning_ShowFlushRequired;
  uint Event_StartPrimaryFoamClearCommandTimer;
  uint Event_StartSecondaryFoamClearCommandTimer;
  uint TR_Show_Saving_Widget;
  uint Calculate_Primary_FoamPercentage_TankB_Equals_INCR;
  uint Calculate_Primary_FoamPercentage_TankA_Equals_INCR;
  uint Calculate_Primary_FoamPercentage_TankB_Equals_DECR;
  uint Calculate_Primary_FoamPercentage_TankA_Equals_DECR;
  uint Event_Save_Water_Flow_Variables;
  uint TR_Clear_Information_Text;
  uint Event_GOTO_MotorControllerSetup_HORIZONTAL;
  uint Event_GOTO_MotorControllerSetup_VERTICAL;
  uint Event_GOTO_MotorControllerSetup2_HORIZONTAL;
  uint Event_GOTO_MotorControllerSetup2_VERTICAL;
  uint Event_GOTO_MotorControllerDebug;
  uint Event_LoadAllDefaults;
  uint Calculate_MiantenancePasswordEntered_Equals_TRUE;
  uint Event_GOTO_Foam_Calibration_Minus_Single_Motor;
  uint Event_GOTO_Foam_Calibration_Minus_Single_Motor_VERT;
  uint Event_GOTO_Foam_Calibration_Minus_Dual_Motor_1;
  uint Event_GOTO_Foam_Calibration_Minus_Dual_Motor_1_VERT;
  uint Event_GOTO_Water_Calibration;
  uint Event_GOTO_Water_Calibration_VERT;
  uint Event_GOTO_Water_Calibration_2;
  uint Event_GOTO_Water_Calibration_2_VERT;
  uint Event_GOTO_Direct_Water_Calibration;
  uint Event_GOTO_Direct_Water_Calibration_VERT;
  uint Event_GOTO_Direct_Water_Calibration_2;
  uint Event_GOTO_Direct_Water_Calibration_2_VERT;
  uint Event_GOTO_Operation_Variables_Menu_Start;
  uint Event_GOTO_Operation_Variables_Menu_Start_VERT;
  uint Event_GOTO_User_Menu;
  uint Event_GOTO_User_Menu_VERT;
  uint Event_GOTO_OEM_Menu_1;
  uint Event_GOTO_OEM_Menu_1_VERT;
  uint Event_GOTO_Preset_Configuration;
  uint Event_GOTO_Preset_Configuration_VERT;
  uint Event_GOTO_Datalogger;
  uint Event_GOTO_Datalogger_VERT;
  uint Calculate_DemoModeActive_Equals_TOGGLE;
  uint eventShowInvalidPassword;
  uint WARNING_Popups_Cleared;
  uint WARNING_Popup_Low_Foam_A;
  uint WARNING_Popup_Low_Foam_B;
  uint WARNING_Popup_Tank_In_FLUSH;
  uint WARNING_Popup_Motor_1_regain_prime;
  uint WARNING_Popup_Motor_1_lost_prime;
  uint WARNING_Popup_Motor_2_regain_prime;
  uint WARNING_Popup_Motor_2_lost_prime;
  uint WARNING_Popup_Comm_Loss_Pump_1;
  uint WARNING_Popup_Comm_Loss_Pump_2;
  uint WARNING_Popup_Tank_Select_Mismatch;
  uint WARNING_Popup_Motor_1_getting_hot;
  uint WARNING_Popup_Motor_2_getting_hot;
  uint WARNING_Popup_Motor_1_shutdown_due_to_heat;
  uint WARNING_Popup_Motor_2_shutdown_due_to_heat;
  uint Calculate_SelectedOperationVariable_Equals_PREVIOUS;
  uint Event_GOTO_Operation_Variables_Menu_1;
  uint Event_GOTO_Operation_Variables_Menu_1_VERT;
  uint Calculate_SelectedOperationVariable_Equals_NEXT;
  uint Event_GOTO_Operation_Variables_Menu_2;
  uint Event_GOTO_Operation_Variables_Menu_2_VERT;
  uint Event_Remote_LED_OFF;
  uint Event_Remote_LED_ON;
  uint Calculate_SelectedOEMVariableEquals_NEXT;
  uint Event_GOTO_OEM_Menu_2;
  uint Event_GOTO_OEM_Menu_2_VERT;
  uint Calculate_SelectedOEMVariableEquals_PREVIOUS;
  uint Calculate_Secondary_FoamPercentage_TankB_Equals_DECR;
  uint Calculate_Secondary_FoamPercentage_TankA_Equals_DECR;
  uint Calculate_Secondary_FoamPercentage_TankB_Equals_INCR;
  uint Calculate_Secondary_FoamPercentage_TankA_Equals_INCR;
  uint Calculate_FlashActivePresetBar_Equals_TOGGLE;
  uint Event_SaveEDT_Positions;
  uint SCRIPT_GOTO_OperatingScreen;
  uint Event_WarningSelectTankA;
  uint Event_WarningSelectTankB;
  uint Event_GOTO_PresetConfiguration_Disabled;
  uint Event_GOTO_PresetConfiguration_Disabled_VERT;
  uint Event_GOTO_PresetConfiguration_Enabled;
  uint Event_GOTO_PresetConfiguration_Enabled_VERT;
  uint SAVE_Preset_1;
  uint SAVE_Preset_2;
  uint SAVE_Preset_3;
  uint SAVE_Preset_4;
  uint SAVE_Preset_5;
  uint SAVE_Preset_6;
  uint Calculate_SelectedUserVariableEquals_NEXT;
  uint Calculate_SelectedUserVariableEquals_PREVIOUS;
  uint Event_SaveFoamSystemControlType;
  uint Event_GOTO_Preset_Page;
  uint SCRIPT_Primary_FoamPercentage_Increase;
  uint SCRIPT_Primary_FoamPercentage_Decrease;
  uint Event_Zeroize_Primary_Total_Water_Flowed;
  uint SCRIPT_ZeroizeFoamTotal;
  uint Calculate_Primary_SimulatedWaterFlow_Enabled_Equals_ON;
  uint Calculate_Primary_SimulatedWaterFlow_Enabled_Equals_OFF;
  uint SCRIPT_Reset_1M_Interval;
  uint SCRIPT_Reset_2M_Interval;
  uint SCRIPT_Reset_12M_Interval;
  uint Event_Save_2M_Interval;
  uint Event_Save_12M_Interval;
  uint Event_Save_1M_Interval;
  uint Clear_Warning_FlushMustBePerformed;
  uint Calculate_Primary_SimulatedFlowGPM_Equals_INCR;
  uint Calculate_Primary_SimulatedFlowGPM_Equals_DECR;
  uint Event_GOTO_FoamTutorial_4_single;
  uint Event_GOTO_FoamTutorial_4_dual;
  uint Event_GOTO_LookupChart_English;
  uint Event_GOTO_LookupChart_English_VERT;
  uint Event_GOTO_LookupChart_Metric;
  uint Event_GOTO_LookupChart_Metric_VERT;
  uint SCRIPT_SmoothBoreNozzleCalculation_GPM;
  uint SCRIPT_SmoothBoreNozzleCalculation_LPM;
  uint Calculate_PressToSetValue_FromCalculator_Equals_TRUE;
  uint Event_TransmitA2CModule_Setup;
  uint Event_Set_Temp_OEM_Variables;
  uint Event_Set_Temp_Operating_Variables;
  uint Event_Save_Operating_Variables;
  uint Event_Save_OEM_Variables;
  uint Event_Save_Water_Flow_Variables_2;
  uint Event_GOTO_Prime_Pump;
  uint Event_GOTO_Prime_Pump_VERT;
  uint Event_GOTO_PerformanceChart;
  uint Event_GOTO_PerformanceChart_VERT;
  uint Event_RestartSystem;
  uint Warning_AutoFlushInitiated;
  uint Event_TransmitRemoteCAN;
  uint Calculate_PrimeFoamPump_Power_Equals_OFF;
  uint Calculate_PrimeFoamPump_Power_Equals_ON;
  uint Calculate_Primary_TotalWaterGallons_Equals_CLEAR;
  uint Calculate_PrimeFoamPump_Power_2_Equals_OFF;
  uint Calculate_PrimeFoamPump_Power_2_Equals_ON;
  uint Calculate_Secondary_TotalWaterGallons_Equals_CLEAR;
  uint SCRIPT_Secondary_FoamPercentage_Increase;
  uint SCRIPT_Secondary_FoamPercentage_Decrease;


   _EventIDs () 
   {
	SCRIPT_UpdatePerformanceChart = 1022;
	Event_DataLoggingStart = 842;
	Event_DataLoggingStop = 843;
	SCRIPT_FoamLogixPowerButton_Primary = 617;
	Event_TransmitOutputControlMessage = 1183;
	Warning_FlushIsComplete = 1015;
	SCRIPT_FoamLogixPowerButton_Secondary = 838;
	WARNING_Popup_No_Foam_A = 812;
	WARNING_Popup_No_Foam_B = 813;
	WARNING_Popup_Limp_Home_Mode_1 = 1107;
	Warnings_ClearMenuPageWarnings = 1187;
	Warning_ShowFlushRequired = 1014;
	Event_StartPrimaryFoamClearCommandTimer = 626;
	Event_StartSecondaryFoamClearCommandTimer = 733;
	TR_Show_Saving_Widget = 724;
	Calculate_Primary_FoamPercentage_TankB_Equals_INCR = 632;
	Calculate_Primary_FoamPercentage_TankA_Equals_INCR = 614;
	Calculate_Primary_FoamPercentage_TankB_Equals_DECR = 633;
	Calculate_Primary_FoamPercentage_TankA_Equals_DECR = 615;
	Event_Save_Water_Flow_Variables = 692;
	TR_Clear_Information_Text = 673;
	Event_GOTO_MotorControllerSetup_HORIZONTAL = 1172;
	Event_GOTO_MotorControllerSetup_VERTICAL = 1173;
	Event_GOTO_MotorControllerSetup2_HORIZONTAL = 1211;
	Event_GOTO_MotorControllerSetup2_VERTICAL = 1212;
	Event_GOTO_MotorControllerDebug = 1090;
	Event_LoadAllDefaults = 1048;
	Calculate_MiantenancePasswordEntered_Equals_TRUE = 1008;
	Event_GOTO_Foam_Calibration_Minus_Single_Motor = 1019;
	Event_GOTO_Foam_Calibration_Minus_Single_Motor_VERT = 1116;
	Event_GOTO_Foam_Calibration_Minus_Dual_Motor_1 = 620;
	Event_GOTO_Foam_Calibration_Minus_Dual_Motor_1_VERT = 1114;
	Event_GOTO_Water_Calibration = 643;
	Event_GOTO_Water_Calibration_VERT = 1128;
	Event_GOTO_Water_Calibration_2 = 1067;
	Event_GOTO_Water_Calibration_2_VERT = 1127;
	Event_GOTO_Direct_Water_Calibration = 804;
	Event_GOTO_Direct_Water_Calibration_VERT = 1113;
	Event_GOTO_Direct_Water_Calibration_2 = 1080;
	Event_GOTO_Direct_Water_Calibration_2_VERT = 1112;
	Event_GOTO_Operation_Variables_Menu_Start = 745;
	Event_GOTO_Operation_Variables_Menu_Start_VERT = 1120;
	Event_GOTO_User_Menu = 757;
	Event_GOTO_User_Menu_VERT = 1126;
	Event_GOTO_OEM_Menu_1 = 827;
	Event_GOTO_OEM_Menu_1_VERT = 1117;
	Event_GOTO_Preset_Configuration = 935;
	Event_GOTO_Preset_Configuration_VERT = 1121;
	Event_GOTO_Datalogger = 949;
	Event_GOTO_Datalogger_VERT = 1111;
	Calculate_DemoModeActive_Equals_TOGGLE = 931;
	eventShowInvalidPassword = 670;
	WARNING_Popups_Cleared = 685;
	WARNING_Popup_Low_Foam_A = 683;
	WARNING_Popup_Low_Foam_B = 684;
	WARNING_Popup_Tank_In_FLUSH = 689;
	WARNING_Popup_Motor_1_regain_prime = 687;
	WARNING_Popup_Motor_1_lost_prime = 688;
	WARNING_Popup_Motor_2_regain_prime = 691;
	WARNING_Popup_Motor_2_lost_prime = 690;
	WARNING_Popup_Comm_Loss_Pump_1 = 736;
	WARNING_Popup_Comm_Loss_Pump_2 = 737;
	WARNING_Popup_Tank_Select_Mismatch = 739;
	WARNING_Popup_Motor_1_getting_hot = 794;
	WARNING_Popup_Motor_2_getting_hot = 795;
	WARNING_Popup_Motor_1_shutdown_due_to_heat = 796;
	WARNING_Popup_Motor_2_shutdown_due_to_heat = 797;
	Calculate_SelectedOperationVariable_Equals_PREVIOUS = 696;
	Event_GOTO_Operation_Variables_Menu_1 = 697;
	Event_GOTO_Operation_Variables_Menu_1_VERT = 1118;
	Calculate_SelectedOperationVariable_Equals_NEXT = 695;
	Event_GOTO_Operation_Variables_Menu_2 = 744;
	Event_GOTO_Operation_Variables_Menu_2_VERT = 1119;
	Event_Remote_LED_OFF = 749;
	Event_Remote_LED_ON = 750;
	Calculate_SelectedOEMVariableEquals_NEXT = 820;
	Event_GOTO_OEM_Menu_2 = 1178;
	Event_GOTO_OEM_Menu_2_VERT = 1179;
	Calculate_SelectedOEMVariableEquals_PREVIOUS = 821;
	Calculate_Secondary_FoamPercentage_TankB_Equals_DECR = 837;
	Calculate_Secondary_FoamPercentage_TankA_Equals_DECR = 835;
	Calculate_Secondary_FoamPercentage_TankB_Equals_INCR = 836;
	Calculate_Secondary_FoamPercentage_TankA_Equals_INCR = 834;
	Calculate_FlashActivePresetBar_Equals_TOGGLE = 893;
	Event_SaveEDT_Positions = 1185;
	SCRIPT_GOTO_OperatingScreen = 828;
	Event_WarningSelectTankA = 872;
	Event_WarningSelectTankB = 873;
	Event_GOTO_PresetConfiguration_Disabled = 934;
	Event_GOTO_PresetConfiguration_Disabled_VERT = 1123;
	Event_GOTO_PresetConfiguration_Enabled = 933;
	Event_GOTO_PresetConfiguration_Enabled_VERT = 1124;
	SAVE_Preset_1 = 857;
	SAVE_Preset_2 = 858;
	SAVE_Preset_3 = 863;
	SAVE_Preset_4 = 864;
	SAVE_Preset_5 = 865;
	SAVE_Preset_6 = 866;
	Calculate_SelectedUserVariableEquals_NEXT = 941;
	Calculate_SelectedUserVariableEquals_PREVIOUS = 942;
	Event_SaveFoamSystemControlType = 986;
	Event_GOTO_Preset_Page = 990;
	SCRIPT_Primary_FoamPercentage_Increase = 630;
	SCRIPT_Primary_FoamPercentage_Decrease = 631;
	Event_Zeroize_Primary_Total_Water_Flowed = 738;
	SCRIPT_ZeroizeFoamTotal = 629;
	Calculate_Primary_SimulatedWaterFlow_Enabled_Equals_ON = 996;
	Calculate_Primary_SimulatedWaterFlow_Enabled_Equals_OFF = 997;
	SCRIPT_Reset_1M_Interval = 1006;
	SCRIPT_Reset_2M_Interval = 1001;
	SCRIPT_Reset_12M_Interval = 1002;
	Event_Save_2M_Interval = 999;
	Event_Save_12M_Interval = 1000;
	Event_Save_1M_Interval = 1007;
	Clear_Warning_FlushMustBePerformed = 1017;
	Calculate_Primary_SimulatedFlowGPM_Equals_INCR = 637;
	Calculate_Primary_SimulatedFlowGPM_Equals_DECR = 638;
	Event_GOTO_FoamTutorial_4_single = 1027;
	Event_GOTO_FoamTutorial_4_dual = 1028;
	Event_GOTO_LookupChart_English = 1030;
	Event_GOTO_LookupChart_English_VERT = 1144;
	Event_GOTO_LookupChart_Metric = 1029;
	Event_GOTO_LookupChart_Metric_VERT = 1145;
	SCRIPT_SmoothBoreNozzleCalculation_GPM = 774;
	SCRIPT_SmoothBoreNozzleCalculation_LPM = 785;
	Calculate_PressToSetValue_FromCalculator_Equals_TRUE = 1034;
	Event_TransmitA2CModule_Setup = 1044;
	Event_Set_Temp_OEM_Variables = 826;
	Event_Set_Temp_Operating_Variables = 720;
	Event_Save_Operating_Variables = 721;
	Event_Save_OEM_Variables = 825;
	Event_Save_Water_Flow_Variables_2 = 1050;
	Event_GOTO_Prime_Pump = 1085;
	Event_GOTO_Prime_Pump_VERT = 1139;
	Event_GOTO_PerformanceChart = 1025;
	Event_GOTO_PerformanceChart_VERT = 1140;
	Event_RestartSystem = 1136;
	Warning_AutoFlushInitiated = 1186;
	Event_TransmitRemoteCAN = 1191;
	Calculate_PrimeFoamPump_Power_Equals_OFF = 1082;
	Calculate_PrimeFoamPump_Power_Equals_ON = 1081;
	Calculate_Primary_TotalWaterGallons_Equals_CLEAR = 647;
	Calculate_PrimeFoamPump_Power_2_Equals_OFF = 1087;
	Calculate_PrimeFoamPump_Power_2_Equals_ON = 1086;
	Calculate_Secondary_TotalWaterGallons_Equals_CLEAR = 829;
	SCRIPT_Secondary_FoamPercentage_Increase = 841;
	SCRIPT_Secondary_FoamPercentage_Decrease = 840;

   }
}

_ActionIDs ActionIDs;
class _ActionIDs
{  uint UIApp_Show_View;


   _ActionIDs () 
   {
	UIApp_Show_View = 1937;

   }
}

_ApplicationIDs ApplicationIDs;
class _ApplicationIDs
{  uint UIApp;


   _ApplicationIDs () 
   {
	UIApp = 1936;

   }
}

_ActionData ActionData;
class _ActionData
{

   _ActionData () 
   {

   }
}

_VariableEnumerations VariableEnumerations;
class _VariableEnumerations
{

   _VariableEnumerations () 
   {

   }
}

_TimerIDs TimerIDs;
class _TimerIDs
{

   _TimerIDs () 
   {

   }
}

_CustomListColumn CustomListColumn;
class _CustomListColumn
{

   _CustomListColumn () 
   {

   }
}

_PersistentVariableIDs PersistentVariableIDs;
class _PersistentVariableIDs
{

   _PersistentVariableIDs () 
   {

   }
}

_IOAnalogOutputIDs IOAnalogOutputIDs;
class _IOAnalogOutputIDs
{

   _IOAnalogOutputIDs () 
   {

   }
}

_IODigitalOutputIDs IODigitalOutputIDs;
class _IODigitalOutputIDs
{

   _IODigitalOutputIDs () 
   {

   }
}

_IORelayIDs IORelayIDs;
class _IORelayIDs
{

   _IORelayIDs () 
   {

   }
}

_IOFrequencyOutputIDs IOFrequencyOutputIDs;
class _IOFrequencyOutputIDs
{

   _IOFrequencyOutputIDs () 
   {

   }
}

_IORelayFunctionIDs IORelayFunctionIDs;
class _IORelayFunctionIDs
{

   _IORelayFunctionIDs () 
   {

   }
}

_SnapshotLoggerIDs SnapshotLoggerIDs;
class _SnapshotLoggerIDs
{

   _SnapshotLoggerIDs () 
   {

   }
}

_CANMessageInfoType CANMessageInfoType;
class _CANMessageInfoType
{

   _CANMessageInfoType () 
   {

   }
}

_FreeFormCANIDs FreeFormCANIDs;
class _FreeFormCANIDs
{

   _FreeFormCANIDs () 
   {

   }
}

_DMMemoryMappingIDs DMMemoryMappingIDs;
class _DMMemoryMappingIDs
{

   _DMMemoryMappingIDs () 
   {

   }
}

_CANPortIDs CANPortIDs;
class _CANPortIDs
{

   _CANPortIDs () 
   {

   }
}

_DMStringIDs DMStringIDs;
class _DMStringIDs
{

   _DMStringIDs () 
   {

   }
}

_PageViewIDs PageViewIDs;
class _PageViewIDs
{  uint Menu_Layer_Default_Maintenance_Interval_View;
  uint Menu_Layer_Default_VERTICAL_Maintenance_Interval_View;
  uint Popup_Layer_Popup_Page_CLEAR_View;
  uint Main_Layer_Default_SmartFOAM_Minus_Single_View;
  uint Main_Layer_Default_VERTICAL_SmartFOAM_Minus_Single_View;
  uint Main_Layer_Default_SmartFOAM_Minus_Dual_as_Single_View;
  uint Main_Layer_Default_VERTICAL_SmartFOAM_Minus_Dual_as_Single_View;
  uint Main_Layer_Default_SmartFOAM_Minus_Dual_Seperate_View;
  uint Main_Layer_Default_FoamLogix_View;
  uint Main_Layer_Default_VERTICAL_FoamLogix_View;
  uint Main_Layer_Default_AnalogFoam_View;
  uint Main_Layer_Default_VERTICAL_AnalogFoam_View;
  uint Main_Layer_Default_Chase_Thailand_Screen_Minus_ENGLISH_View;
  uint Main_Layer_Default_Chase_Thailand_Screen_Minus_METRIC_View;
  uint Menu_Layer_Menu_View;
  uint Menu_Layer_VERTICAL_Menu_View;
  uint Popup_Layer_Default_Password_Entry_Screen_View;
  uint Popup_Layer_Default_VERTICAL_Password_Entry_Screen_View;
  uint Menu_Layer_Default_Menu_Layer_CLEAR_View;
  uint Menu_Layer_Default_Simulated_Water_Flow_Setup_View;
  uint Menu_Layer_Default_VERTICAL_Simulated_Water_Flow_Setup_View;


   _PageViewIDs () 
   {
	Menu_Layer_Default_Maintenance_Interval_View = 8974;
	Menu_Layer_Default_VERTICAL_Maintenance_Interval_View = 8993;
	Popup_Layer_Popup_Page_CLEAR_View = 12297;
	Main_Layer_Default_SmartFOAM_Minus_Single_View = 2543;
	Main_Layer_Default_VERTICAL_SmartFOAM_Minus_Single_View = 2550;
	Main_Layer_Default_SmartFOAM_Minus_Dual_as_Single_View = 2544;
	Main_Layer_Default_VERTICAL_SmartFOAM_Minus_Dual_as_Single_View = 2554;
	Main_Layer_Default_SmartFOAM_Minus_Dual_Seperate_View = 2542;
	Main_Layer_Default_FoamLogix_View = 2546;
	Main_Layer_Default_VERTICAL_FoamLogix_View = 2552;
	Main_Layer_Default_AnalogFoam_View = 2547;
	Main_Layer_Default_VERTICAL_AnalogFoam_View = 2553;
	Main_Layer_Default_Chase_Thailand_Screen_Minus_ENGLISH_View = 2549;
	Main_Layer_Default_Chase_Thailand_Screen_Minus_METRIC_View = 2548;
	Menu_Layer_Menu_View = 8962;
	Menu_Layer_VERTICAL_Menu_View = 8984;
	Popup_Layer_Default_Password_Entry_Screen_View = 12298;
	Popup_Layer_Default_VERTICAL_Password_Entry_Screen_View = 12319;
	Menu_Layer_Default_Menu_Layer_CLEAR_View = 8963;
	Menu_Layer_Default_Simulated_Water_Flow_Setup_View = 8964;
	Menu_Layer_Default_VERTICAL_Simulated_Water_Flow_Setup_View = 8985;

   }
}

_VideoInputIDs VideoInputIDs;
class _VideoInputIDs
{

   _VideoInputIDs () 
   {

   }
}

_StringIDs StringIDs;
class _StringIDs
{  uint PresetButtonText_1;
  uint PresetButtonText_2;
  uint PresetButtonText_3;
  uint PresetButtonText_4;
  uint PresetButtonText_5;
  uint PresetButtonText_6;
  uint ActivePreset_Text;
  uint test_string;
  uint PresetCurrentText;
  uint PresetConfigureTitle;
  uint DateTime_TextString;
  uint _1MonthIntervalDate;
  uint _2MonthIntervalDate;
  uint _12MonthIntervalDate;


   _StringIDs () 
   {
	PresetButtonText_1 = 476;
	PresetButtonText_2 = 477;
	PresetButtonText_3 = 478;
	PresetButtonText_4 = 479;
	PresetButtonText_5 = 480;
	PresetButtonText_6 = 481;
	ActivePreset_Text = 486;
	test_string = 482;
	PresetCurrentText = 484;
	PresetConfigureTitle = 483;
	DateTime_TextString = 487;
	_1MonthIntervalDate = 490;
	_2MonthIntervalDate = 488;
	_12MonthIntervalDate = 489;

   }
}

_ColorTypes ColorTypes;
class _ColorTypes
{

   _ColorTypes () 
   {

   }
}

_ScreenObjectIDs ScreenObjectIDs;
class _ScreenObjectIDs
{  uint Image_Saving_disk;
  uint Image_USB_Saving_disk;
  uint AnalogKnobImage;
  uint AnalogKnobImage_VERT;
  uint Text_PumpModel;
  uint Text_PumpModel_VERT;
  uint Text_FlowRate_01;
  uint Text_FlowRate_02;
  uint Text_FlowRate_03;
  uint Text_FlowRate_05;
  uint Text_FlowRate_10;
  uint Text_FlowRate_30;
  uint Text_FlowRate_60;
  uint Text_FlowRate_01_VERT;
  uint Text_FlowRate_02_VERT;
  uint Text_FlowRate_03_VERT;
  uint Text_FlowRate_05_VERT;
  uint Text_FlowRate_10_VERT;
  uint Text_FlowRate_30_VERT;
  uint Text_FlowRate_60_VERT;


   _ScreenObjectIDs () 
   {
	Image_Saving_disk = 13556;
	Image_USB_Saving_disk = 9904;
	AnalogKnobImage = 5068;
	AnalogKnobImage_VERT = 6718;
	Text_PumpModel = 9677;
	Text_PumpModel_VERT = 11212;
	Text_FlowRate_01 = 9656;
	Text_FlowRate_02 = 9659;
	Text_FlowRate_03 = 9662;
	Text_FlowRate_05 = 9665;
	Text_FlowRate_10 = 9668;
	Text_FlowRate_30 = 9671;
	Text_FlowRate_60 = 9674;
	Text_FlowRate_01_VERT = 11191;
	Text_FlowRate_02_VERT = 11194;
	Text_FlowRate_03_VERT = 11197;
	Text_FlowRate_05_VERT = 11200;
	Text_FlowRate_10_VERT = 11203;
	Text_FlowRate_30_VERT = 11206;
	Text_FlowRate_60_VERT = 11209;

   }
}

_SmartObjectIDs SmartObjectIDs;
class _SmartObjectIDs
{

   _SmartObjectIDs () 
   {

   }
}

_AnimationTypes AnimationTypes;
class _AnimationTypes
{

   _AnimationTypes () 
   {

   }
}

_PdmOutputIDs PdmOutputIDs;
class _PdmOutputIDs
{

   _PdmOutputIDs () 
   {

   }
}